<?php

defined( 'oPage' ) or die();

/** Provides a unified source for system, webserver, php and script parameters
*/

class CApp
{
    /** PHP Informationen */
    var $sFullversion;
    var $iVersion;
    var $sInterface;

    /** HTTP Informationen */
    var $sHostName;     // may be something different as servername
    var $sServerName;
    var $sProtocol;
    var $sScriptName;
    var $sPathInfo;
    var $sQueryString;
    var $sRequestUri;
    var $sRequestMethod;
    var $sAcceptEncoding;
    var $sReferer;
    var $aParameter = array();
    var $aCookie;
    var $sUserAgent;
    var $sBrowserVersion;
    var $sBrowserMajorVersion;
    var $sBrowserMinorVersion;
    var $sBrowser;
    var $sBrowserLanguage;
    var $fKonqueror = false;
    var $fAol = false;
    var $fWebTv = false;

    /** oPage Informationen
    */
    var $fDebugAllowed;
    var $fInstallAllowed;
    var $fSetupAllowed;
    var $soPage;
    var $sQuerySeparator;
    var $sParamSeparator;
    var $sHttp;
    var $sHttps;

    /** Document root as reported by the Webserver
    */
    var $sDocumentRoot;

    /** Root directory of the Web
    */
    var $sRoot;

    /** Web root (from the file system point of view), usually set to $sDocumentRoot . $sRoot
        but can be different, if $sRoot contains a link to the users home directory, like /~username/web/
        --> set the files parameter in the config array.
    */
    var $sFileRoot;

    var $fAdmin = false;
    var $fPrint = false;
    var $fDevelopment = false;
    var $fTesting = false;
    var $fStaging = false;
    var $fProduction = false;
    var $fSetup = false;

    /** Charset
    */
    var $sCharset = 'windows-1252';

    /** Timezone
    */
    var $sTimezone;

    // Interface Language
    var $aLanguage;
    var $aLanguageShort;
    var $iLanguageDefault;
    var $iLanguage;

    // Content Language
    var $aContentLanguage;
    var $aContentLanguageShort;
    var $iContentLanguageDefault;
    var $iContentLanguage;

    // Database
    var $sDatabase;
    var $sDatabasePassword;
    var $sTablePrefix;

    var $sBackUrl;

    /** Definiert fr jede Sprache den Language Postfix
        Wenn fr eine Sprache nix definiert ist, wird bei 0 nix, ansonsten die Nummer angehngt.
        @default nothing
    */
    var $aLanguagePostfix;

    /** speichert Pfad Eintrge zu Commands, etc.
    */
    var $aCommand = array();

    /** speichert Features und deren Level
    */
    var $aFeature = array();

    /** <b>Konstruktor</b><br>
    	Initialisiert die Webseite(variablen) mit wichtigen Informationen
    	@param $aConfig Array with arrays of parameters
    */
    function CApp( $aConfig )
    {
        global $HTTP_SERVER_VARS;

        // PHP Informationen holen
        $this->sFullversion = phpversion();
        $this->iVersion = substr( $this->sFullversion, 0, 1 );

        // HTTP Informationen holen
        if ( $this->GetFullversion() >= '4.0.1' )
            $this->sInterface = php_sapi_name();
        else
            $this->sInterface = 'unkown';

        if ( $this->sInterface == 'isapi' || $this->sInterface == 'apache2filter' || $this->sInterface == 'apache2handler' || $this->sInterface == 'apache' )
        {
            if ( $this->GetFullversion() > '4.1.0' )
            {
                $this->sHostName = $_SERVER[ 'HTTP_HOST' ];
                $this->sServerName = $_SERVER[ 'SERVER_NAME' ];
                $this->sReferer = $_SERVER[ 'HTTP_REFERER' ];
                $this->sDocumentRoot = $_SERVER[ 'DOCUMENT_ROOT' ];
                $this->sScriptName = $_SERVER[ 'SCRIPT_NAME' ];
                $this->sPathInfo = $_SERVER[ 'PATH_INFO' ];
                $this->sQueryString = $_SERVER[ 'QUERY_STRING' ];
                $this->sRequestMethod = $_SERVER[ 'REQUEST_METHOD' ];
                $this->sAcceptEncoding = $_SERVER[ 'HTTP_ACCEPT_ENCODING' ];
                if ( $_SERVER[ 'HTTPS' ] == 'on' )
                    $this->sProtocol = 'https://';
                else
                    $this->sProtocol = 'http://';
                $this->sBrowserLanguage = $_SERVER[ 'HTTP_ACCEPT_LANGUAGE' ];
            }
            else
            {
                $this->sHostName = $HTTP_SERVER_VARS[ 'HTTP_HOST' ];
                $this->sServerName = $HTTP_SERVER_VARS[ 'SERVER_NAME' ];
                $this->sReferer = $HTTP_SERVER_VARS[ 'HTTP_REFERER' ];
                $this->sDocumentRoot = $HTTP_SERVER_VARS[ 'DOCUMENT_ROOT' ];
                $this->sScriptName = $HTTP_SERVER_VARS[ 'SCRIPT_NAME' ];
                $this->sPathInfo = $HTTP_SERVER_VARS[ 'PATH_INFO' ];
                $this->sQueryString = $HTTP_SERVER_VARS[ 'QUERY_STRING' ];
                $this->sRequestMethod = $HTTP_SERVER_VARS[ 'REQUEST_METHOD' ];
                $this->sAcceptEncoding = $HTTP_SERVER_VARS[ 'HTTP_ACCEPT_ENCODING' ];
                if ( $HTTP_SERVER_VARS[ 'HTTPS' ] == 'on' )
                    $this->sProtocol = 'https://';
                else
                    $this->sProtocol = 'http://';
                $this->sBrowserLanguage = $HTTP_SERVER_VARS[ 'HTTP_ACCEPT_LANGUAGE' ];
            }
        }
        else if ( $this->sInterface == 'cgi' || $this->sInterface == 'cgi-fcgi' )
        {
            if ( $this->GetFullversion() > '4.1.0' )
            {
                $this->sHostName = $_SERVER[ 'HTTP_HOST' ];
                $this->sServerName = $_SERVER[ 'SERVER_NAME' ];
                $this->sReferer = $_SERVER[ 'HTTP_REFERER' ];
                $this->sDocumentRoot = $_SERVER[ 'DOCUMENT_ROOT' ];
                $this->sScriptName = $_SERVER[ 'SCRIPT_NAME' ];
                $this->sPathInfo = $_SERVER[ 'PATH_INFO' ];
                $this->sQueryString = $_SERVER[ 'QUERY_STRING' ];
                $this->sRequestMethod = $_SERVER[ 'REQUEST_METHOD' ];
                $this->sAcceptEncoding = $_SERVER[ 'HTTP_ACCEPT_ENCODING' ];
                if ( $_SERVER[ 'HTTPS' ] == 'on' )
                    $this->sProtocol = 'https://';
                else
                    $this->sProtocol = 'http://';
                $this->sBrowserLanguage = $_SERVER[ 'HTTP_ACCEPT_LANGUAGE' ];
            }
            else
            {
                // attention: in cgi mode PHP_SELF contains the script name, SCRIPT_NAME contains the name of the cgi php interpreter
                $this->sHostName = getenv( 'HTTP_HOST' );
                $this->sServerName = getenv( 'SERVER_NAME' );
                $this->sReferer = getenv( 'HTTP_REFERER' );
                $this->sDocumentRoot = getenv( 'DOCUMENT_ROOT' );
                $this->sPathInfo = $_SERVER[ 'PATH_INFO' ];
                $this->sQueryString = getenv( 'QUERY_STRING' );
                $this->sRequestMethod = getenv( 'REQUEST_METHOD' );
                $this->sAcceptEncoding = getenv( 'HTTP_ACCEPT_ENCODING' );
                $this->sScriptName = $HTTP_SERVER_VARS[ 'PHP_SELF' ];
                if ( $HTTP_SERVER_VARS[ 'HTTPS' ] == 'on' )
                    $this->sProtocol = 'https://';
                else
                    $this->sProtocol = 'http://';
                $this->sBrowserLanguage = $HTTP_SERVER_VARS[ 'HTTP_ACCEPT_LANGUAGE' ];
            }
        }
        else if ( $this->sInterface == 'cli' )
        {
            $this->sHostName = 'localhost';
            $this->sServerName = 'localhost';
            $this->sReferer = null;
            $this->sDocumentRoot = 'd:\daten\http';
            $this->sPathInfo = '';
            $this->sQueryString = null;
            $this->sRequestMethod = null;
            $this->sAcceptEncoding = null;
            if ( $this->GetFullversion() > '4.1.0' )
            {
                $this->sScriptName = $_SERVER[ 'SCRIPT_NAME' ];
                $this->sProtocol = 'http://';
                $this->sBrowserLanguage = 'de';
            }
            else
            {
                $this->sScriptName = $HTTP_SERVER_VARS[ 'PHP_SELF' ];
                $this->sProtocol = 'http://';
                $this->sBrowserLanguage = 'de';
            }
        }
        else
        {
            $this->sHostName = $GLOBALS[ 'HTTP_HOST' ];
            $this->sServerName = $GLOBALS[ 'SERVER_NAME' ];
            $this->sReferer = $GLOBALS[ 'HTTP_REFERER' ];
            $this->sDocumentRoot = $GLOBALS[ 'DOCUMENT_ROOT' ];
            $this->sScriptName = $GLOBALS[ 'SCRIPT_NAME' ];
            $this->sPathInfo = $GLOBALS[ 'PATH_INFO' ];
            $this->sQueryString = $GLOBALS[ 'QUERY_STRING' ];
            $this->sRequestMethod = $GLOBALS[ 'REQUEST_METHOD' ];
            $this->sAcceptEncoding = $GLOBALS[ 'HTTP_ACCEPT_ENCODING' ];
            if ( $GLOBALS[ 'HTTPS' ] == 'on' )
                $this->sProtocol = 'https://';
            else
                $this->sProtocol = 'http://';
            $this->sBrowserLanguage = $GLOBALS[ 'HTTP_ACCEPT_LANGUAGE' ];
        }

        // If we call a page with parameters as path (e.g. index.php/news=2/back=url)
        // QueryString will be empty and Parameters are stored in PATH_INFO
        // does not work if php runs in cgi-mode!
        if ( $this->sPathInfo != '' )
        {
            // Parse PathInfo for parameters
            $aParameters = explode( '/', substr( $this->sPathInfo, 1 ) );
            foreach( $aParameters as $sParameter )
            {
                $aParameter = explode( '=', $sParameter );
                if ( $aParameter[ 0 ] != '' )
                    $this->aParameter[ $aParameter[ 0 ] ] = urldecode( $aParameter[ 1 ] );
            }
        }

        // $aConfig is an array of arrays.
        // Key is the systems hostname
        $sHostname = php_uname( 'n' );
        if ( is_array( $aConfig[ $sHostname ] ) )
        {
            $aConfig = $aConfig[ $sHostname ];
        }
        elseif ( is_array( $aConfig[ '' ] ) )
        {
            $aConfig = $aConfig[ '' ];
        }
        else
        {
            die( __CLASS__ . ': could not detect, which configuration to use' );
        }

        switch( $aConfig[ 'mode' ] )
        {
            case null:
            case '':
            case 'production':
                $this->fProduction = true;
            break;

            case 'staging':
                $this->fStaging = true;
            break;

            case 'testing':
                $this->fTesting = true;

                // allow debug, if not defined
                if ( !isset( $aTesting[ 'debug' ] ) )
                    $aTesting[ 'debug' ] = true;
            break;

            case 'development':
                $this->fDevelopment = true;

                // allow debug, install and setup, if not defined
                if ( !isset( $aConfig[ 'debug' ] ) )
                    $aDevelopment[ 'debug' ] = true;
                if ( !isset( $aConfig[ 'install' ] ) )
                    $aConfig[ 'install' ] = true;
                if ( !isset( $aConfig[ 'setup' ] ) )
                    $aConfig[ 'setup' ] = true;
            break;

            default:
                die( __CLASS__ . ': invalid mode in config.php' );
            break;
        }

        // Check config settings
        // if ( !$this->IsPhpUnit() )
        // {
            if ( !$this->CheckConfig( $aConfig ) )
            {
                die( __CLASS__ . ': please check custom/config.php' );
            }
        // }

        // when scriptname ends with '/', remove it
        if ( substr( $this->sScriptName, -1 ) == '/' )
        {
            $this->sScriptName = substr( $this->sScriptName, 0, -1 );
        }
        $this->sScriptName = str_replace( "\\", '/', str_replace( "\\\\", '/', $this->sScriptName ) );

        // when document root ends with '/', remove it
        if ( substr( $this->sDocumentRoot, -1 ) == '/' )
        {
            $this->sDocumentRoot = substr( $this->sDocumentRoot, 0, -1 );
        }
        $this->sDocumentRoot = str_replace( "\\", '/', str_replace( "\\\\", '/', $this->sDocumentRoot ) );

        // Allow Debug?
        $this->SetDebug( $aConfig[ 'debug' ] );

        // Use Install?
        $this->SetInstall( $aConfig[ 'install' ] );

        // Use Setup?
        $this->SetSetup( $aConfig[ 'setup' ] );

        // oPage Version
        $this->SetoPage( $aConfig[ 'oPage' ] );

        // Urls
        $this->sHttp = $aConfig[ 'http' ];
        $this->sHttps = $aConfig[ 'https' ];

        $this->sQuerySeparator = $aConfig[ 'Query_Separator' ];
        if ( $this->sQuerySeparator === null )
        {
            $this->sQuerySeparator = '?';
        }
        $this->sParamSeparator = $aConfig[ 'Param_Separator' ];
        if ( $this->sParamSeparator === null )
        {
            $this->sParamSeparator = '&amp;';
        }

        // Root
        $this->sRoot = $aConfig[ 'Root' ];

        // File Root
        if ( isset( $aConfig[ 'File_Root' ] ) )
        {
            $this->sFileRoot = $aConfig[ 'File_Root' ];
        }

        // Project specific table prefix
        $this->sTablePrefix = $aConfig[ 'DB_Table_Prefix' ];

        // Error Handling
        define( 'ERROR_HANDLING', $aConfig[ 'ErrorHandling' ] );

        // Database
        define( 'DATABASE', 'db_' . $aConfig[ 'DB' ] . '.inc' );
        if ( !isset( $aConfig[ 'DB_Server' ] ) || $aConfig[ 'DB_Server' ] == '' )
            $aConfig[ 'DB_Server' ] = 'localhost';
        define( 'DATABASE_SERVER', $aConfig[ 'DB_Server' ] );
        if ( !isset( $aConfig[ 'DB_User' ] ) || $aConfig[ 'DB_User' ] == '' )
            $aConfig[ 'DB_User' ] = 'root';
        define( 'DATABASE_USER', $aConfig[ 'DB_User' ] );
        define( 'DATABASE_PASSWORD', $aConfig[ 'DB_Password' ] );
        define( 'DATABASE_DATABASE', $aConfig[ 'DB_Database' ] );
        $this->sDatabase = $aConfig[ 'DB_Database' ];
        $this->sDatabasePassword = $aConfig[ 'DB_Password' ];

        // Email
        define( 'MAIL_SERVER', $aConfig[ 'Mail_Server' ] );

        // Timezone
        $this->SetTimezone( $aConfig[ 'Timezone' ] );
        if ( function_exists( 'date_default_timezone_set' ) )
            date_default_timezone_set( $this->GetTimezone() );

        // Charset
        if ( isset( $aConfig[ 'Charset' ] ) )
            $this->SetCharset( $aConfig[ 'Charset' ] );

        // 2010.03.29 DOH   Do not touch scriptname, if under phpunit
        // if ( !$this->IsPhpUnit() )
        // {
            // ScriptName relativ zu Root erzeugen
            if ( strcasecmp( substr( $this->sScriptName, 0, strlen( $this->GetRoot() ) ), $this->GetRoot() ) == 0 )
            {
                $this->sScriptName = substr( $this->sScriptName, strlen( $this->GetRoot() ) );
            }
            else
            {
                // if ( !$this->IsPhpUnit() )
                if ( $this->sInterface != 'cli' )
                {
                    die( 'app.php: Invalid root: "' . $this->GetRoot() . '"' );
                }
            }
        // }

        // Scriptname zerlegen fr sDir
        $aPathInfo = PathInfo( $this->GetScriptName() );

        if ( $aPathInfo[ 'dirname' ] == '.' )
            $this->sDir = '';
        else
            $this->sDir = $aPathInfo[ 'dirname' ] . '/';

        $this->sRequestUri = $this->GetQueryString();
        if ( !empty( $this->sRequestUri ) )
            $this->sRequestUri = '?' . $this->sRequestUri;
        $this->sRequestUri = $this->GetScriptName() . $this->sRequestUri;

        // Detect admin mode
        $this->fAdmin = $this->DetectAdmin( $this->GetScriptName() );

        // Detect setup mode
        if ( $this->GetScriptName() == 'admin/setup.php' )
            $this->fSetup = true;
        else
            $this->fSetup = false;

        // Interface Language Werte aus Prepend bernehmen (abhngig von Admin j/n)
        if ( $this->GetAdmin() )
            $sBuffer = 'Admin_';
        else
            $sBuffer = '';
        $this->aLanguage = $aConfig[ $sBuffer . 'Language' ];
        if ( $this->aLanguage === null )
            $this->aLanguage = array( '0' => 'Deutsch' );
        $this->aLanguageShort = $aConfig[ $sBuffer . 'Language_Short' ];
        if ( $this->aLanguageShort === null )
            $this->aLanguageShort = array( '0' => 'de' );
        $this->aLanguagePostfix = $aConfig[ $sBuffer . 'Language_Postfix' ];
        if ( $this->aLanguagePostfix === null )
            $this->aLanguagePostfix = array( '0' => '' );
        $this->iLanguageDefault = $aConfig[ $sBuffer . 'Language_Default' ];
        if ( $this->iLanguageDefault === null )
            $this->iLanguageDefault = 0;

        // Content Language values from prepend
        $this->aContentLanguage = $aConfig[ 'Language' ];
        if ( $this->aContentLanguage === null )
            $this->aContentLanguage = array( '0' => 'Deutsch' );
        $this->aContentLanguageShort = $aConfig[ 'Language_Short' ];
        if ( $this->aContentLanguageShort === null )
            $this->aContentLanguageShort = array( '0' => 'de' );
        $this->aContentLanguagePostfix = $aConfig[ 'Language_Postfix' ];
        if ( $this->aContentLanguagePostfix === null )
            $this->aContentLanguagePostfix = array( '0' => '' );
        $this->iContentLanguageDefault = $aConfig[ 'Language_Default' ];
        if ( $this->iContentLanguageDefault === null )
            $this->iContentLanguageDefault = 0;

        if ( isset( $aConfig[ 'Features' ] ) )
        {
            if ( is_array( $aConfig[ 'Features' ] ) )
            {
                $this->aFeature = array_merge( $this->aFeature, $aConfig[ 'Features' ] );
            }
        }

        unset( $aConfig );

        // Overwrite Default Language with BrowserLanguage
        // Prepare Browser Language:
        // 'de-at,de;q=0.8,en-us;q=0.5,en;q=0.3'
        // If language short code is not available, GetLanguageShortId returns the old LanguageDefault
        $this->iLanguageDefault = $this->GetLanguageShortID( substr( $this->sBrowserLanguage, 0, 2 ) );
        $this->iContentLanguageDefault = $this->GetLanguageShortID( substr( $this->sBrowserLanguage, 0, 2 ), true );

        // POST bzw. GET Parameter bernehmen
        if ( $this->GetFullversion() > '4.1.0' )
        {
            foreach( $_POST as $sName => $Value )
                if ( $sName != '' )
                {
                    if ( is_array( $Value ) )
                        foreach( $Value as $sKey => $sValue )
                            $this->aParameter[ $sName ][ $sKey ] = stripslashes( $sValue );
                    else
                        $this->aParameter[ $sName ] = stripslashes( $Value );
                }
        }
        else
        {
            foreach( $GLOBALS[ 'HTTP_POST_VARS' ] as $sName => $Value )
                if ( $sName != '' )
                {
                    if ( is_array( $Value ) )
                        foreach( $Value as $sKey => $sValue )
                            $this->aParameter[ $sName ][ $sKey ] = stripslashes( $sValue );
                    else
                        $this->aParameter[ $sName ] = stripslashes( $Value );
                }
        }

        // In PHP3 sind die POST_FILES in POST_VARS bereits enthalten, in PHP4 fgen wir diese hier ein.
        if ( $this->GetFullversion() > '4.1.0' )
        {
            foreach( $_FILES as $sKey => $aFile )
            {
                if ( $aFile[ 'size' ] > 0 )
                    $this->aParameter[ $sKey ] = $aFile[ 'tmp_name' ];
                $this->aParameter[ $sKey . '_name' ] = $aFile[ 'name' ];
                $this->aParameter[ $sKey . '_type' ] = $aFile[ 'type' ];
                $this->aParameter[ $sKey . '_size' ] = $aFile[ 'size' ];
            }
        }
        else
        {
            foreach( $GLOBALS[ 'HTTP_POST_FILES' ] as $sKey => $aFile )
            {
                if ( $aFile[ 'size' ] > 0 )
                    $this->aParameter[ $sKey ] = $aFile[ 'tmp_name' ];
                $this->aParameter[ $sKey . '_name' ] = $aFile[ 'name' ];
                $this->aParameter[ $sKey . '_type' ] = $aFile[ 'type' ];
                $this->aParameter[ $sKey . '_size' ] = $aFile[ 'size' ];
            }
        }

        if ( $this->GetFullversion() > '4.1.0' )
        {
            foreach( $_GET as $sName => $sValue )
                if ( $sName != '' )
                    $this->aParameter[ $sName ] = stripslashes( $sValue );
        }
        else
        {
            foreach( $GLOBALS[ 'HTTP_GET_VARS' ] as $sName => $sValue )
                if ( $sName != '' )
                    $this->aParameter[ $sName ] = stripslashes( $sValue );
        }

        // Cookie kopieren
        if ( $this->GetFullversion() > '4.1.0' )
            $this->aCookie = $_COOKIE;
        else
            $this->aCookie = $GLOBALS[ 'HTTP_COOKIE_VARS' ];

        // Language aus Cookie lesen
        $this->SetLanguage( $this->ReadLanguage() );
        if ( $this->GetAdmin() )
            $this->SetLanguage( $this->GetLanguage( true ), true );
        else
            $this->SetLanguage( $this->GetLanguage(), true );

        // Language aus Parameter bernehmen und speichern
        $sLanguage = $this->GetParameter( 'language' );
        if ( $sLanguage != '' )
        {
            if ( strlen( $sLanguage ) > 2 )
                $iLanguage = $this->GetLanguageID( $sLanguage );
            else
                $iLanguage = $this->GetLanguageShortID( $sLanguage );

            if ( $this->GetLanguage() <> $iLanguage )
            {
                $this->WriteLanguage( $iLanguage );
                $this->SetLanguage( $iLanguage );
                if ( !$this->GetAdmin() )
                    $this->SetLanguage( $iLanguage, true );
            }
        }

        // Print Version gewnscht?
        if ( $this->GetParameter( 'print' ) == 1 )
            $this->fPrint = true;
        else
            $this->fPrint = false;

        // Back zusammenstellen
        $aParameter = $this->GetParameters();
        // Unset a lot of parameters not good for back urls, etc.
        unset( $aParameter[ session_name() ] );
        unset( $aParameter[ 'submit' ] );
        unset( $aParameter[ 'submit_logon' ] );
        unset( $aParameter[ 'submit_logoff' ] );
        unset( $aParameter[ 'timezone' ] );
        unset( $aParameter[ 'datetime' ] );
        unset( $aParameter[ 'language' ] );

        // Prepare Back Uri
        $sParameter = '';
    	foreach ( $aParameter as $sKey => $sValue)
    	{
    	    if ( $sParameter != '' )
    	        $sParameter .= '&';
    	    $sParameter = $sParameter . urlencode( $sKey ) .  '=' . urlencode( $sValue );
    	}
    	if ( $sParameter != '' )
    	    $sParameter = '?' . $sParameter;
        $this->sBackUrl = $this->GetScriptname() . $sParameter;

        // User Agent ermitteln
        $this->sUserAgent = getenv( 'HTTP_USER_AGENT' );

        // Determine Browser Name and Version
        $this->DetectBrowserAndVersion( $this->sUserAgent, $this->sBrowser, $this->sBrowserVersion );

        // Determine Major and Minor Version
        if( $this->sBrowserVersion > 0 )
        {
            $pos = strpos( $this->sBrowserVersion, '.' );
            if ($pos > 0)
            {
                $this->sBrowserMajorVersion = substr( $this->sBrowserVersion, 0, $pos);
                $this->sBrowserMinorVersion = substr( $this->sBrowserVersion, $pos, strlen( $this->sBrowserVersion ) );
            }
            else
                $this->sBrowserMajorVersion = $this->sBrowserVersion;
        }
    }

    function CheckConfig( $aConfig )
    {
        $sBuffer = '';

        if ( !isset( $aConfig[ 'http' ] ) || $aConfig[ 'http' ] == '' )
        {
            $sBuffer .= 'Edit custom/config.php: http<br/>' . "\n";
        }
        if ( !isset( $aConfig[ 'https' ] ) || $aConfig[ 'https' ] == '' )
        {
            $sBuffer .= 'Edit custom/config.php: https<br/>' . "\n";
        }

        if ( !isset( $aConfig[ 'Root' ] ) || $aConfig[ 'Root' ] == '' )
        {
            $sBuffer .= 'Edit custom/config.php: Root<br/>' . "\n";
        }

        if ( !isset( $aConfig[ 'DB' ] ) || $aConfig[ 'DB' ] == '' )
        {
            $sBuffer .= 'Edit custom/config.php: DB<br/>' . "\n";
        }
        elseif ( $aConfig[ 'DB' ] != 'mysql' )
        {
            $sBuffer .= 'Edit custom/config.php: DB: invalid value<br/>' . "\n";
        }
        if ( !isset( $aConfig[ 'DB_Database' ] ) || $aConfig[ 'DB_Database' ] == '' )
        {
            $sBuffer .= 'Edit custom/config.php: DB_Database<br/>' . "\n";
        }

        if ( !isset( $aConfig[ 'Mail_Server' ] ) || $aConfig[ 'Mail_Server' ] == '' )
        {
            $sBuffer .= 'Edit custom/config.php: Mail_Server<br/>' . "\n";
        }

        if ( $sBuffer != '' )
        {
			echo php_uname( 'n' ) . ':<br/>';
            echo $sBuffer;
            return( false );
        }

        return( true );
    }

    /** Liefert einen Output ber die aktuellen Application-Informationen
    	@output Aufgelistet werden u.a. benutzte PHP-Version, angeforderte URL's, <br>Sprache, HTTP-Variablen, usw.
    */
    function Output()
    {
        echo 'Fullversion: ' . $this->GetFullversion() . "<br>\n";
        echo 'Version: ' . $this->GetVersion() . "<br>\n";
        echo 'Interface: ' . $this->GetInterface() . "<br>\n";
        echo "<br>\n";
        echo 'ServerName: ' . $this->GetServerName() . "<br>\n";
        echo 'ScriptName: ' . $this->GetScriptName() . "<br>\n";
        echo 'PathInfo: ' . $this->GetPathInfo() . "<br>\n";
        echo 'QueryString: ' . $this->GetQueryString() . "<br>\n";
        echo 'RequestUri: ' . $this->GetRequestUri() . "<br>\n";
        echo 'RequestMethod: ' . $this->GetRequestMethod() . "<br>\n";
        echo "<br>\n";
        echo 'Root: ' . $this->GetRoot() . "<br>\n";
        echo $this->GetAdmin() ? "Admin: true<br>\n" : "Admin: false<br>\n";
        echo 'Charset: ' . $this->GetCharset() . "<br>\n";
        echo 'Interface Language: ' . $this->GetLanguageText( $this->GetLanguage() ) .  "<br>\n";
        echo 'Content Language: ' . $this->GetLanguageText( $this->GetLanguage( true ), true ) .  "<br>\n";
        echo "<br>\n";

        echo "<h1>Parameters</h1>\n";
        $this->ArrayPrint( $this->aParameter );
        echo "<br>\n";

        echo "<h1>_POST</h1>\n";
        if ( $this->GetFullversion() > '4.1.0' )
            $this->ArrayPrint( $_POST );
        else
            $this->ArrayPrint( $GLOBALS[ 'HTTP_POST_VARS' ] );
        echo "<br>\n";

        echo "<h1>_FILES</h1>\n";
        if ( $this->GetFullversion() > '4.1.0' )
            $this->ArrayPrint( $_FILES );
        else
            $this->ArrayPrint( $GLOBALS[ 'HTTP_POST_FILES' ] );
        echo "<br>\n";

        echo "<h1>_GET</h1>\n";
        if ( $this->GetFullversion() > '4.1.0' )
            $this->ArrayPrint( $_GET );
        else
            $this->ArrayPrint( $GLOBALS[ 'HTTP_GET_VARS' ] );

        echo "<h1>_SERVER</h1>\n";
        if ( $this->GetFullversion() > '4.1.0' )
            $this->ArrayPrint( $_SERVER );
        else
            $this->ArrayPrint( $GLOBALS[ 'HTTP_SERVER_VARS' ] );
        echo "<br>\n";

        echo "<h1>_SESSION</h1>\n";
        if ( $this->GetFullversion() > '4.1.0' )
            $this->ArrayPrint( $_SESSION );
        else
            $this->ArrayPrint( $GLOBALS[ 'HTTP_SESSION_VARS' ] );
        echo "<br>\n";

        echo "<h1>_COOKIE</h1>\n";
        if ( $this->GetFullversion() > '4.1.0' )
            $this->ArrayPrint( $_COOKIE );
        else
            $this->ArrayPrint( $GLOBALS[ 'HTTP_COOKIE_VARS' ] );
        echo "<br>\n";

        echo "<h1>GLOBALS</h1>\n";
        $this->ArrayPrint( $GLOBALS );
        echo "<br>\n";
    }

    /** Gibt ein Array mit seinen Schlsselwrter und deren Inhalten aus
    	@param Schlsselwrter mit Inhalten als Array
    	@output fr alle Eintrge: Schlsselwort = Inhalt
    */
    function ArrayPrint( $aArray )
    {
        if ( !is_array( $aArray ) )
            return;

        foreach( $aArray as $sKey => $sValue )
        {
            if ( is_array( $sValue ) )
            {
                if ( $sKey !== 'GLOBALS' )
                {
                    echo $sKey . ' = (Array)<br>' . "\n";
                    $this->ArrayPrint( $sValue );
                }
            }
            else
            {
                if ( is_string( $sValue ) )
                    echo $sKey . ' = ' . $sValue . '<br>' ."\n";
                else
                    echo $sKey . ' = ' . gettype( $sValue ) . '<br>' . "\n";
            }
        }
    }

    /** detect browser name and version based on useragent string
        @param $sUserAgent UserAgent string
        @param &$sBrowser  Reference to the variable where browser name should be stored
        @param &$sBrowserVersion  Reference to the variable where browser version should be stored
        @return $sBrowser . ' ' . $sBrowserVersion
    */
    function DetectBrowserAndVersion( $sUserAgent, &$sBrowser, &$sBrowserVersion )
    {
        // the order of this preg_match entries is important!

        if ( preg_match( '/MSIE ([0-9]\.[0-9a-zA-Z]{1,4})/i', $sUserAgent, $info ) || preg_match( '/Microsoft Internet Explorer ([0-9]\.[0-9a-zA-Z]{1,4})/', $sUserAgent, $info ) )
        {
            $sBrowserVersion = $info[ 1 ];
            $sBrowser = 'IE';
        }
        elseif ( preg_match( '/Mozilla\/([0-9]\.[0-9a-zA-Z]{1,4})/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = $info[ 1 ];
            $sBrowser = 'Netscape';
        }
        elseif ( preg_match( '/Opera[ \/]([0-9]\.[0-9a-zA-Z]{1,4})/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = $info[ 1 ];
            $sBrowser = 'Opera';
        }
        elseif ( preg_match( '/iCab[ \/]([0-9]\.[0-9a-zA-Z]{1,4})/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = $info[ 1 ];
            $sBrowser = 'iCab';
        }
        elseif ( preg_match( '/Lynx[ \/]([0-9]\.[0-9a-zA-Z.]{1,9})/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = $info[ 1 ];
            $sBrowser = 'Lynx';
        }
        elseif ( preg_match( '/Konqueror[ \/]([0-9]\.[0-9a-zA-Z]{1,4})/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = $info[ 1 ];
            $sBrowser = 'Konqueror';
        }
        elseif ( preg_match( '/Netscape6\/([0-9].[0-9a-zA-Z]{1,4})/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = $info[ 1 ];
            $sBrowser = 'Netscape';
        }
        elseif ( preg_match( '/aol/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = 0;
            $sBrowser = 'Aol';
        }
        elseif( preg_match( '/webtv/i', $sUserAgent, $info ) )
        {
            $sBrowserVersion = 0;
            $sBrowser = 'WebTv';
        }
        else
        {
            $sBrowserVersion = 0;
            $sBrowser = 'Unknown';
        }

        return( $sBrowser . ' ' . $sBrowserVersion );
    }

    /** returns a sitewide password, eg. for encrypting text
    */
    function GetPassword()
    {
        if ( $this->sDatabasePassword != '' )
            return( $this->sDatabasePassword );

        return( 'oPage' );
    }

    /** SetQuerySeparator used for changing the QueryParameter separator "?"
        http://www.server.com?param=value --> http://www.server.com/param=value
        @param $sValue new Value
        @return void
    */
    function SetQuerySeparator( $sValue )
    {
        $this->sQuerySeparator = $sValue;
    }

    /** GetQuerySeparator used for changing the QueryParameter separator "?"
        http://www.server.com?param=value --> http://www.server.com/param=value
        @return string
    */
    function GetQuerySeparator()
    {
        return( $this->sQuerySeparator );
    }

    /** SetParamSeparator used for changing the ParamParameter separator "&"
        http://www.server.com?param=value&param2=value --> http://www.server.com?param=value/param2=value
        @param $sValue new Value
        @return void
    */
    function SetParamSeparator( $sValue )
    {
        $this->sParamSeparator = $sValue;
    }

    /** GetParamSeparator used for changing the ParamParameter separator "&"
        http://www.server.com?param=value&param2=value --> http://www.server.com?param=value/param2=value
        @return string
    */
    function GetParamSeparator()
    {
        return( $this->sParamSeparator );
    }

    /** Get/Set charset
        @param $sCharset charset, e.g. 'windows-1252', 'iso-8859-15' or 'utf-8', default null
    */
    function GetCharset()
    {
        return( $this->sCharset );
    }
    function SetCharset( $sCharset = null )
    {
        if ( is_null( $sCharset ) )
            $sCharset = 'windows-1252';

        $this->sCharset = strtolower( $sCharset );
    }

    /** Get/Set timezone
        @param $sTimezone, eg. 'Europe/Vienna'
    */
    function GetTimezone()
    {
        return( $this->sTimezone );
    }
    function SetTimezone( $sTimezone = null )
    {
        if ( $sTimezone === '' || $sTimezone === null )
            $sTimezone = 'Europe/Vienna';

        $this->sTimezone = $sTimezone;
    }

    /** Abspeichern der Language-No. in einem Cookie am Client (Verfallsdatum: 1 Tag)
    	@param Sprachennummer als Integer
    */
    function WriteLanguage( $iLanguage, $fContent = false )
    {
        $sCookie = 'language';
        if ( $fContent )
            $sCookie = 'content_' . $sCookie;
        else
            if ( $this->GetAdmin() )
                $sCookie = 'admin_' . $sCookie;

        SetCookie( $sCookie, $iLanguage, mktime(0,0,0,12,31,2034), $this->sRoot );
    }

    /** Liest die aktuelle Language-No. vom Client
    	@return Sprachennummer als Integer
    */
    function ReadLanguage( $fContent = false )
    {
        $sCookie = 'language';
        if ( $fContent )
            $sCookie = 'content_' . $sCookie;
        else
            if ( $this->GetAdmin() )
                $sCookie = 'admin_' . $sCookie;

        $iLanguage = $this->GetCookie( $sCookie );

        if ( $fContent )
        {
            if ( $iLanguage < 0 || !isset( $this->aContentLanguage[ $iLanguage ] ) )
                $iLanguage = $this->GetLanguageDefault( $fContent );
        }
        else
            if ( $iLanguage < 0 || !isset( $this->aLanguage[ $iLanguage ] ) )
                $iLanguage = $this->GetLanguageDefault();

    	return( $iLanguage );
    }

    /** Liefert die ID der aktuellen Sprache ( 0, 1, 2 )
    	@return Sprachennummer als Integer
    */
    function GetLanguage( $fContent = false )
    {
        if ( $fContent )
            return( $this->iContentLanguage );

        return( $this->iLanguage );
    }

    /** Setzt die ID der gewnschten Sprache ( 0, 1, 2 )
    	@param Sprachennummer als Integer
    */
    function SetLanguage( $iLanguage, $fContent = false )
    {
        if ( !isset( $iLanguage ) || $iLanguage < 0 )
            $iLanguage = $this->GetLanguageDefault( $fContent );

        if ( $fContent )
            $this->iContentLanguage = $iLanguage;
        else
            $this->iLanguage = $iLanguage;
    }

    /** Liefert Postfix der jeweiligen Sprachen ( Bsp: '_1', '_2', '' )
    	@param die Sprachennummer
    	@return Postfix zur jeweiligen Sprache als String
    */
    function GetLanguagePostfix( $iLanguage = -1, $fContent = false )
    {
        if ( $iLanguage == -1 )
             $iLanguage = $this->GetLanguage( $fContent );

        if ( $fContent )
        {
            $sBuffer = $this->aContentLanguagePostfix[ $iLanguage ];
            if ( $sBuffer != '' )
                $sBuffer = '_' . $sBuffer;

            return( $sBuffer );
        }

        $sBuffer = $this->aLanguagePostfix[ $iLanguage ];
        if ( $sBuffer != '' )
            $sBuffer = '_' . $sBuffer;

        return( $sBuffer );
    }

    /** Liefert Anzahl der verfgbaren Sprachen
    	@return Anzahl der verfgbaren Sprachen als Integer
    */
    function GetLanguageCount( $fContent = false )
    {
        if ( $fContent )
        {
            if ( is_array( $this->aContentLanguage ) )
                return( count( $this->aContentLanguage ) );
        }
        else
            if ( is_array( $this->aLanguage ) )
                return( count( $this->aLanguage ) );

        return( 0 );
    }

    /** Liefert die Standardsprache
    	@return Standardsprache als Integer
    */
    function GetLanguageDefault( $fContent = false )
    {
        if ( $fContent )
            return( $this->iContentLanguageDefault );

        return( $this->iLanguageDefault );
    }

    /** Liefert ID der jeweiligen Sprache aufgrund des Namens (deutsch, english, ...)
    	@param Sprache als String
    	@return ID der Sprache als Integer
    */
    function GetLanguageID( $sLanguage, $fContent = false )
    {
        if ( $fContent )
        {
            if ( !is_array( $this->aContentLanguage ) )
            	return( $this->iContentLanguageDefault );

            foreach( $this->aContentLanguage as $sKey => $sValue )
                if ( strcasecmp( $sValue, $sLanguage ) == 0 )
        		    return( $sKey );

        	return( $this->iContentLanguageDefault );
        }

        if ( !is_array( $this->aLanguage ) )
        	return( $this->iLanguageDefault );

        foreach( $this->aLanguage as $sKey => $sValue )
            if ( strcasecmp( $sValue, $sLanguage ) == 0 )
    		    return( $sKey );

    	return( $this->iLanguageDefault );
    }

    /** Liefert ID der jeweiligen Sprache aufgrund des Krzels (de, en, ...)
    	@param Sprache als String
    	@return ID der Sprache als Integer
    */
    function GetLanguageShortID( $sLanguage, $fContent = false )
    {
        if ( $fContent )
        {
            if ( !is_array( $this->aContentLanguageShort ) )
            	return( $this->iContentLanguageDefault );

            foreach( $this->aContentLanguageShort as $sKey => $sValue )
                if ( strcasecmp( $sValue, $sLanguage ) == 0 )
        		    return( $sKey );

        	return( $this->iContentLanguageDefault );
        }

        if ( !is_array( $this->aLanguageShort ) )
        	return( $this->iLanguageDefault );

        foreach( $this->aLanguageShort as $sKey => $sValue )
            if ( strcasecmp( $sValue, $sLanguage ) == 0 )
    		    return( $sKey );

    	return( $this->iLanguageDefault );
    }

    /** Liefert den Namen der Sprache
    	Wurde keine Sprache angegeben, so wird ein Leerstring zurckgegeben
    	@param ID der Sprache als Integer
    	@return Sprache als String
    */
    function GetLanguageText( $i = -1, $fContent = false )
    {
        if ( $fContent )
        {
            if ( !is_array( $this->aContentLanguage ) )
            	return( '' );
            if ( $i == -1 )
                $i = $this->GetLanguage( $fContent );

            return( $this->aContentLanguage[ $i ] );
        }

        if ( !is_array( $this->aLanguage ) )
        	return( '' );
        if ( $i == -1 )
            $i = $this->GetLanguage( $fContent );

        return( $this->aLanguage[ $i ] );
    }

    /** Liefert das Krzel der Sprache
    	Wurde keine Sprachen angegeben, so wird ein Leerstring zurckgegeben
    	@param ID der Sprache als Integer
    	@return Krzel als String
    */
    function GetLanguageShortText( $i = -1, $fContent = false )
    {
        if ( $fContent )
        {
            if ( !is_array( $this->aContentLanguageShort ) )
            	return( '' );
            if ( $i == -1 )
                $i = $this->GetLanguage( $fContent );

            return( $this->aContentLanguageShort[ $i ] );
        }

        if ( !is_array( $this->aLanguageShort ) )
        	return( '' );
        if ( $i == -1 )
            $i = $this->GetLanguage();

        return( $this->aLanguageShort[ $i ] );
    }

    /** Returns the short text for a language identified by the language postfix
    	@param $sPostfix, default false
    	@param $fContent force content language settings, default false
    	@return short text for a language
    */
    function GetLanguageShortTextByPostfix( $sPostfix = null, $fContent = null )
    {
        if ( is_null( $sPostfix ) )
            $sPostfix = '';
        if ( is_null( $fContent ) )
            $fContent = false;

        // get right language postfix array
        if ( $fContent )
            $aPostfix = &$this->aContentLanguagePostfix;
        else
            $aPostfix = &$this->aLanguagePostfix;

        // get id for provided postfix
        $sKey = array_search( substr( $sPostfix, -1 ), $aPostfix );

        return( $this->GetLanguageShortText( $sKey, $fContent ) );
    }

    /** Returns the Language referenced by Position Index
        @param Index of Language 0, 1, 2 (this is something different than language id, which is the index of the array)
        @return Id of Language or -1
    */
    function GetLanguageIdByIndex( $iIndex, $fContent = false )
    {
        if ( $fContent )
        {
            if ( !is_array( $this->aContentLanguage ) )
                return( -1 );

            $aKeys = array_keys( $this->aContentLanguage );
        }
        else
        {
            if ( !is_array( $this->aLanguage ) )
                return( -1 );

            $aKeys = array_keys( $this->aLanguage );
        }

        return( $aKeys[ $iIndex ] );
    }

    function IsLanguage( $iLanguage, $fContent = false )
    {
        if ( $fContent )
            return( isset( $this->aContentLanguage[ $iLanguage ] ) );

        return( isset( $this->aLanguage[ $iLanguage ] ) );
    }

    /** Returns name of database
        @return string
    */
    function GetDatabaseName()
    {
        return( $this->sDatabase );
    }

    /** Returns password of database
        @return string
    */
    function GetDatabasePassword()
    {
        return( $this->sDatabasePassword );
    }

    /** Liefert den Prefix fr die Datenbank Tabellen
    	Damit ist es mglich, mehrer iPage Installation in einer DB zu implementieren
    	@return Tabellenprefix als String
    */
    function GetTablePrefix()
    {
        return( $this->sTablePrefix );
    }

    function GetDebug()
    {
        return( $this->fDebugAllowed );
    }
    function SetDebug( $fValue )
    {
        $this->fDebugAllowed = $fValue;
    }

    function GetInstall()
    {
        return( $this->fInstallAllowed );
    }
    function SetInstall( $fValue )
    {
        $this->fInstallAllowed = $fValue;
    }

    function GetSetup()
    {
        return( $this->fSetupAllowed );
    }
    function SetSetup( $fValue )
    {
        $this->fSetupAllowed = $fValue;
    }

    /** Returns oPage Version (base, std, pro)
    	@return string
    */
    function GetoPage()
    {
        return( $this->soPage );
    }
    function SetoPage( $sValue = null )
    {
        switch( $sValue )
        {
            case 'light':
            case 'base':
                $this->soPage = 'base';
            break;

            case 'standard':
            case 'std':
                $this->soPage = 'std';
            break;

            case 'professional':
            case 'pro':
                $this->soPage = 'pro';
            break;

            default:
                $this->soPage = self::DetectoPageByPath();
            break;
        }

        return( true );
    }

    /**
     * Dectects the current oPage version by path
     * return string: base, std, pro
     */
    static function DetectoPageByPath()
    {
        global $sRoot;

        if ( is_dir( $sRoot . 'include_pro' ) )
        {
            return( 'pro' );
        }

        if ( is_dir( $sRoot . 'include_std' ) )
        {
            return( 'std' );
        }

        return( 'base' );
    }

    /** Liefert die Vollversionsnummer von PHP
    	@return Vollversionsnummer als String
    */
    function GetFullversion()
    {
        return( $this->sFullversion );
    }

    /** Liefert die Versionsnummer von PHP
    	@return Versionsnummer als Integer
    */
    function GetVersion()
    {
        return( $this->iVersion );
    }

    /** Liefert das Interface
    	@return Interfaceart (unknown, isapi) als String
    */
    function GetInterface()
    {
        return( $this->sInterface );
    }

    /** Liefert den Hostname
    	@return Hostname als String
    */
    function GetHostname()
    {
        return( $this->sHostName );
    }

    /** Liefert den Servernamen
    	@return Servername als String
    */
    function GetServerName()
    {
        return( $this->sServerName );
    }

    /** Liefert den Scriptnamen
    	@return Scriptname als String
    */
    function GetScriptName()
    {
        return( $this->sScriptName );
    }

    /** Returns PathInfo
    	@return PathInfo as String
    */
    function GetPathInfo()
    {
        return( $this->sPathInfo );
    }

    /** Liefert den Querystring
    	@return Querystring als String
    */
    function GetQueryString()
    {
        return( $this->sQueryString );
    }

    /** Liefert die Request-Uri
    	@return Request-Uri als String
    */
    function GetRequestUri()
    {
        return( $this->sRequestUri );
    }

    /** Liefert den Request-Mode
    	@return Request-Mode als String
    */
    function GetRequestMethod()
    {
        return( $this->sRequestMethod );
    }

    /** Returns the accepted encondings
    	@return Request-Mode als String
    */
    function GetAcceptEncoding()
    {
        return( $this->sAcceptEncoding );
    }

    /** Liefert das Protokoll (http:// oder https://)
    	@return Protokoll as String
    */
    function GetProtocol()
    {
        return( $this->sProtocol );
    }

    /** Liefert den Referer
    	@return Referer als Url
    */
    function GetReferer()
    {
        return( $this->sReferer );
    }

    /** Liefert das Document-Root Verzeichnis
    	@return Rootverzeichnis als String
    */
    function GetDocumentRoot()
    {
        return( $this->sDocumentRoot );
    }

    /** Returns http servername
    	@return string
    */
    function GetHttp()
    {
        return( $this->sHttp );
    }

    function SetHttp( $sValue )
    {
        if ( substr( $sValue, 0, 7 ) != 'http://' && substr( $sValue, 0, 8 ) != 'https://' )
        {
            $this->sHttp = 'http://' . $sValue;
        }
        else
        {
            $this->sHttp = $sValue;
        }
    }

    /** Returns https servername
    	@return string
    */
    function GetHttps()
    {
        return( $this->sHttps );
    }

    function SetHttps( $sValue )
    {
        if ( substr( $sValue, 0, 7 ) != 'http://' && substr( $sValue, 0, 8 ) != 'https://' )
        {
            $this->sHttps = 'https://' . $sValue;
        }
        else
        {
            $this->sHttps = $sValue;
        }
    }

    /** Liefert das Root-Verzeichnis
    	@return Rootverzeichnis als String
    */
    function GetRoot()
    {
        return( $this->sRoot );
    }

    /** Returns the file root path, default Document Root + Web Root, but can be overruled.
    	@return string
    */
    function GetFileRoot()
    {
        if ( !isset( $this->sFileRoot ) )
            return( $this->sDocumentRoot . $this->sRoot );
        return( $this->sFileRoot );
    }

    /** Liefert das aktuelle Verzeichnis
    	@return Verzeichnis als String (absolut zu Root)
    */
    function GetDir()
    {
        return( $this->sDir );
    }

    /** Determines upon the provided scriptname, if oPage is in frontend or in backend mode and returns true/false accordingly
        @param $sScriptname current scriptname, e.g. return of GetScriptname()
        @return true/false
    */
    function DetectAdmin( $sScriptname )
    {
        // Activate admin mode, if path starts with 'admin/'
        if ( substr( $sScriptname, 0, 6 ) == 'admin/' )
        {
            return( true );
        }
        if ( substr( $sScriptname, 0, 10 ) == 'admin_std/' )
        {
            return( true );
        }
        if ( substr( $sScriptname, 0, 10 ) == 'admin_pro/' )
        {
            return( true );
        }

        // Activate admin mode, if path starts with 'plugin/something/admin/'
        if ( substr( $sScriptname, 0, 7 ) == 'plugin/' )
        {
            $i = strpos( $sScriptname, '/', 7 );
            if ( $i !== false && substr( $sScriptname, $i + 1, 6 ) == 'admin/' )
            {
                return( true );
            }
        }

        return( false );
    }

    /** Liefert den Zustand, ob eine Adminschnittstelle aufgerufen wird
    	@return true wenn Adminschnittstelle aufgerufen wird
    	@return false wenn keine Adminschnittstelle aufgerufen wird
    */
    function GetAdmin()
    {
        return( $this->fAdmin );
    }

    /** Liefert den Zustand, ob die Seite gedruckt werden soll
    	@return true wenn die Seite ausgedruckt wird
    	@return false wenn die Seite nicht ausgedruckt wird
    */
    function GetPrint()
    {
        return( $this->fPrint );
    }

    /** berprft, ob ein Parameter angegeben wurde
        @param Name eines Form-Parameters als String
        @return true/false
    */
    function IsParameter( $sName )
    {
        return( IsKey( $this->aParameter, $sName ) );
    }

    /** Liefert den Wert eines Parameters
    	@param Name eines Form-Parameters als String
    	@return Wert eines Form-Parameters als String
    */
    function GetParameter( $sName, $sDefault = null )
    {
        if ( isset( $this->aParameter[ $sName ] ) )
            return( $this->aParameter[ $sName ] );

        return( $sDefault );
    }

    /** Setzt den Wert eines Parameters
    	@param $sName eines Form-Parameters als String
    	@param $sValue eines Form-Parameters als String
    */
    function SetParameter( $sName, $sValue )
    {
        $this->aParameter[ $sName ] = $sValue;
    }

    /** Deletes the parameter
    	@param $sName eines Form-Parameters als String
    */
    function ClearParameter( $sName )
    {
        unset( $this->aParameter[ $sName ] );
    }

    /** Liefert ein Array mit allen Parametern
    	@return Array mit allen Parametern
    */
    function GetParameters( $sFilter = Null )
    {
        if ( $sFilter === null )
            return( $this->aParameter );

        $aParameter = array();
        foreach( $this->aParameter as $sKey => $sValue )
            if ( substr( $sKey, 0, strlen( $sFilter ) ) == $sFilter )
                $aParameter[ $sKey ] = $sValue;

        return( $aParameter );
    }

    /** Liefert ein Cookie zu einem bestimmten Namen
    	@param $sName Name eines Cookies als String
    	@return Wert eines Cookies als String
    */
    function GetCookie( $sName )
    {
        return( $this->aCookie[ $sName ] );
    }

    /** Erzeugt einen projektspezifischen absoluten Pfad durch einen
        projektspezifischen relativen Pfad
        @param $sDestination Pfad relativ zur Homepage als String
        @param $sDelimiter Trennzeichen im Pfad als String
        @return relativer Pfad + Filename als String
    */
    function PathLink( $sDestination, $sDelimiter = '/' )
    {
        $sSource = $this->GetRoot() . $this->GetScriptName();
        $sDestination = $this->GetRoot() . $sDestination;

        while( true )
        {
            $i = strpos( $sSource, $sDelimiter ) + 1;
            $j = strpos( $sDestination, $sDelimiter ) + 1;

            if ( substr( $sSource, $i - 1, 1 ) <> $sDelimiter )
                break;
            if ( substr( $sDestination, $j - 1, 1 ) <> $sDelimiter )
                break;

            if ( substr( $sSource, 0, $i ) <> substr( $sDestination, 0, $j ) )
                break;

            $sSource = substr( $sSource, $i );
            $sDestination = substr( $sDestination, $j );
        }

        $sLink = '';
        while( true )
        {
            $i = strpos( $sSource, $sDelimiter );

            if ( !$i )
                break;

            $sLink = '..' . $sDelimiter . $sLink;
            $sSource = substr( $sSource, $i + 1 );
        }

        if ( $sLink == '.' )
            $sLink = '.' . $sDelimiter;

        return( $sLink . $sDestination );
    }

    /** Liefert die Url zur vorherigen Seite
    	@return Url zur Vorseite als String
    */
    function GetBackUrl()
    {
        return( $this->sBackUrl );
    }

    /** Checks, if system is running under phpunit
        @param void
    	@return boolean
    */
    function IsPhpUnit()
    {
        if ( basename( $this->sScriptName ) != 'phpunit' )
            return( false );

        return( true );
    }

    /** Liefert den Zustand, ob es ein Developmentserver ist
    	@return true wenn es ein Developmentserver ist
    	@return false wenn es kein Developmentserver ist
    */
    function IsDevelopment()
    {
        return( $this->fDevelopment );
    }

    /** Liefert den Zustand, ob es ein Testserver ist
    	@return true wenn es ein Testserver ist
    	@return false wenn es kein Testserver ist
    */
    function IsTesting()
    {
        return( $this->fTesting );
    }

    /** Liefert den Zustand, ob es ein Stagingserver ist
    	@return true wenn es ein Stagingserver ist
    	@return false wenn es kein Stagingserver ist
    */
    function IsStaging()
    {
        return( $this->fStaging );
    }

    /** Liefert den Zustand, ob es ein Produktionsserver ist
    	@return true wenn es ein Produktionsserver ist
    	@return false wenn es kein Produktionsserver ist
    */
    function IsProduction()
    {
        return( $this->fProduction );
    }

    /** admin/setup.php running?
        @return true, when admin/setup.php is running
        @return false, when admin/setup.php is not running
    */
    function IsSetup()
    {
        return( $this->fSetup );
    }

    /** Returns true, when webserver is apache
    */
    function IsApache()
    {
        if ( $this->sInterface == 'apache2filter' || $this->sInterface == 'apache2handler' || $this->sInterface == 'apache' )
            return( true );
        else
            return( false );
    }

    /** Returns true, when webserver is iis
    */
    function IsIis()
    {
        if ( $this->sInterface == 'isapi' )
            return( true );
        if ( substr( $_SERVER[ 'SERVER_SOFTWARE' ], 0, 13 ) == 'Microsoft-IIS' )
            return( true );

        return( false );
    }

    /** Returns true, when the according apache module is loaded
        Check for Apache with IsApache() before calling this method!
        @param $sModule name of the module, eg. 'mod_rewrite'
        @return true/false
    */
    function IsApacheModule( $sModule )
    {
        if ( !function_exists( 'apache_get_modules' ) )
            return( false );
        return( in_array( $sModule, apache_get_modules() ) ? true : false );
    }

    /** Liefert den benutzen Browser zurck
    	@return Browsername als String
    */
    function GetBrowser()
    {
        return( $this->sBrowser );
    }

    /** Liefert die benutze Browserversion zurck
    	@return Browserversion als String
    */
    function GetBrowserVersion()
    {
        return( $this->sBrowserVersion );
    }

    /** Returns the major browser version
    	@return Browserversion as string
    */
    function GetBrowserMajorVersion()
    {
        return( $this->sBrowserMajorVersion );
    }

    /** Returns IP address of the client
        @return String
    */
    function GetClientIP()
    {
        return( ClientIP() );
    }

    /** Speichert ein Commando
    */
    function SetCommand( $sName, $sValue )
    {
        $this->aCommand[ $sName ] = $sValue;
    }

    /** Liefert ein Commando
    */
    function GetCommand( $sName )
    {
        return( $this->aCommand[ $sName ] );
    }

    /** Aktiviert ein Feature
        @param $sModul Name des Moduls
        @param $sName  Name des Features
        @param $sValue Wert des Features, z.B. 0 = ausgeschaltet, 1 = aktiviert
    */
    function SetFeature( $sModul, $sName, $sValue )
    {
        $this->aFeature[ $sModul ][ $sName ] = $sValue;
    }

    /** Liefert den Level eines Features
        @param $sModul Name des Moduls
        @param $sName  Name des Features
        @return -1 wenn fr das Feature kein Wert vorhanden ist, oder der Wert = null oder der Wert = '' ist.
        @return Wert wenn fr das Feature ein Wert eingetragen ist
    */
    function GetFeature( $sModul, $sName = null )
    {
        if ( !is_array( $this->aFeature[ $sModul ] ) )
        {
            return( -1 );
        }
        if ( $sName == null )
        {
            return( $this->aFeature[ $sModul ] );
        }
        if ( $this->aFeature[ $sModul ][ $sName ] === null )
        {
            return( -1 );
        }

        return( $this->aFeature[ $sModul ][ $sName ] );
    }
}
