<?php

defined( 'oPage' ) or die();

class CFilter extends CControl
{
	var $oForm;
	var $aFields = Array();

	function CFilter( $sName = 'filter' )
    {
        global $oFactory;

        $this->CControl( $sName );
		$this->oForm = $oFactory->Control( 'CForm', $sName );
	}

    function Copy( $oCopy = Null )
    {
        if ( !is_object( $oCopy ) )
        {
            $sClass = get_class( $this );
            $oCopy = new $sClass( '' );
        }
        $oCopy = CControl::Copy( $oCopy );

        $oCopy->oForm = $this->oForm->Copy();
        $oCopy->aFields = $this->aFields;

        return( $oCopy );
    }

	function AddField( $sName, $sType = null )
	{
	    if ( $sType === null )
	        $sType = 'TEXT';

		$this->aFields[ $sName ] = Array();
		if ( $sType == 'YESNO' )
		    $sType = 'TRISTATE';
		if ( $sType == 'SELECT_SWITCH' )
		    $sType = 'SELECT';

		$this->oForm->AddField( 'field_' . $sName, $sType );
	}

    function SetFieldType( $sName, $sValue )
    {
        $this->oForm->SetFieldType( 'field_' . $sName, $sValue );
    }

    function SetPrefix( $sValue )
    {
        $this->oForm->SetPrefix( $sValue );
        CControl::SetPrefix( $sValue );
    }

    function SetWidth( $sName, $sValue )
    {
        $this->oForm->SetWidth( 'field_' . $sName, $sValue );
    }

    function SetHeight( $sName, $sValue )
    {
        $this->oForm->SetHeight( 'field_' . $sName, $sValue );
    }

    /** Setzt die Image-Breite
        @param sName Feldname
        @param sValue Breite in Pixel
        @return void
    */
    function SetImageWidth( $sName, $sValue )
    {
        $this->oForm->SetImageWidth( 'field_' . $sName, $sValue );
    }

    /** Setzt die Image-Hhe
        @param sName Feldname
        @param sValue Hhe in Pixel
        @return void
    */
    function SetImageHeight( $sName, $sValue )
    {
        $this->oForm->SetImageHeight( 'field_' . $sName, $sValue );
    }

    function SetOptions( $sName, $aValues )
    {
        $this->oForm->SetOptions( 'field_' . $sName, $aValues );
    }

    function SetRelation( $sName, $oObject )
    {
        $this->aFields[ $sName ][ 'relation' ] = $oObject;
    }

    function SetMaxlength( $sName, $sValue )
    {
        // MaxLength of the field is possibly to small
        // because a filter statement can contain operators and multiple values
        // so maxlength is set to 512, which is (hopefully) enough
        $this->oForm->SetMaxlength( 'field_' . $sName, 512 );
    }

    /** Fgt ein zustzliches Suchfeld hinzu. Dieses Feld wird ebenfalls durchsucht.
        @param $sName Name des Hauptfeldes
        @param $sValue Name des zustzlichen Feldes
        @return void
    */
    function AddSearchField( $sName, $sValue )
    {
        $this->aFields[ $sName ][ 'search' ][] = $sValue;
    }

    function Action()
    {
        $fRead = false;

        // Filter neu anwenden (ist eigentlich eine Dummy-Abfrage,
        // da zumeist ein anderer Default-Submit-Button existiert,
        // der fRead = true setzt.
        if ( $this->oForm->IsSubmitted( 'action_submit' ) )
            $fRead = true;

        // Filter zurcksetzen
        if ( $this->oForm->IsSubmitted( 'action_reset' ) )
        {
            $this->Reset();
            $fRead = true;
        }

        return( $fRead );
    }

    /** Alle Filterfelder lschen
        @return void
    */
    function Reset()
    {
		foreach( $this->aFields as $sName => $aField )
			$this->oForm->SetValue( 'field_' . $sName, '' );
    }

    function OutputTpl( &$oTemplate, $sBlock = null, $sSub = null )
    {
        if ( !parent::OutputTpl( $oTemplate, $sBlock, $sSub ) )
            return( false );

		$this->oForm->OutputTpl( $oTemplate, $sBlock );
    }

    /** Returns the value of a filter field
        @param $sName Name of the field
        @return string
    */
    function GetValue( $sName )
    {
        return( $this->oForm->GetValue( 'field_' . $sName, null, false ) );
    }

    /** Bereitet den Filterwert auf.
        @param sName Name des Feldes
        @param &Filter Referenz auf ein Feld das den Wert aufnimmt (String oder Array, siehe Modul.php)
        @param &sCompare Referenz auf ein Feld, das den Compare Operator aufnimmt
        @return true/false ob der Filter verwendet werden soll.
    */
	function Get( $sName, &$Filter, &$sCompare )
	{
	    global $oApp;

        $sCompare = '';
        $Filter = $this->oForm->GetValue( 'field_' . $sName, null, false );
        if ( IsNull( $Filter ) )
            return( false );

        $oRelation = &$this->aFields[ $sName ][ 'relation' ];
        if ( is_object( $oRelation ) )
        {
            $Filter = $this->oForm->FormatValue( 'field_' . $sName, $Filter );
            $oRelation->SetFilter( $sName, $Filter );
            $oRelation->QueryData();
            $sValue = $oRelation->GetValuesString( 'id', true );
            if ( IsNull( $sValue ) )
                return( false );

            $Filter = array();
            $Filter[] = array( 'field' => 'id', 'value' => '(' . $sValue . ')', 'compare' => 'in', 'escape' => false );

            return( true );
        }
        else
        {
            // ' entfernen
            $Filter = str_replace( '\'', '', $Filter );

            // split by ' '
            $aFilter = explode( ' ', $Filter );
            $Filter = array();

            $i = 0;
            while( $i < count( $aFilter ) )
            {
                $sValue = $aFilter[ $i ];
                ++$i;
                while( $i < count( $aFilter ) )
                {
                    $sFilter1 = substr( $aFilter[ $i ], 0, 1 );
                    $sFilter1a = substr( $aFilter[ $i ], 1, 1 );
                    $sFilter2 = substr( $aFilter[ $i ], 0, 2 );
                    $sFilter2a = substr( $aFilter[ $i ], 1, 2 );
                    $sFilter3 = substr( $aFilter[ $i ], 0, 3 );
                    $sFilter3a = substr( $aFilter[ $i ], 2, 3 );

                    if
                    (
                        $sFilter3 == '!==' ||
                        ( ( $sFilter1 == '|' || $sFilter1 == '&' ) && ( $sFilter3a == '!==' ) ) ||
                        $sFilter2 == '<=' ||
                        $sFilter2 == '>=' ||
                        $sFilter2 == '!=' ||
                        ( ( $sFilter1 == '|' || $sFilter1 == '&' ) && ( $sFilter2a == '==' || $sFilter2a == '<=' || $sFilter2a == '>=' || $sFilter2a == '!=' ) ) ||
                        $sFilter1 == '<' ||
                        $sFilter1 == '>' ||
                        $sFilter1 == '=' ||
                        ( ( $sFilter1 == '|' || $sFilter1 == '&' ) && ( $sFilter1a == '<' || $sFilter1a == '>' || $sFilter1a == '=' ) ) ||
                        ( $sFilter1 == '|' || $sFilter1 == '&' )
                    )
                        break;

                    $sValue .= ' ' . $aFilter[ $i ];
                    ++$i;
                }

                // Wenn im Suchstring | oder & vorhanden ist, dann als Operator verwenden.
                if ( substr( $sValue, 0, 1 ) == '|' )
                {
                    $sOperator = 'or';
        		    $sValue = substr( $sValue, 1 );
        		}
                elseif ( substr( $sValue, 0, 1 ) == '&' )
                {
                    $sOperator = 'and';
        		    $sValue = substr( $sValue, 1 );
        		}
                else
                    $sOperator = 'and';

            	// Wenn im Suchstring ein < oder > oder = enthalten ist, dann diese auch verwenden
        	    if ( substr( $sValue, 0, 3 ) == '!==' )
        	    {
        		    $sCompare = 'NOT LIKE';
        		    $sValue = substr( $sValue, 3 );
        	    }
        	    elseif ( substr( $sValue, 0, 2 ) == '==' )
        	    {
        		    $sCompare = 'LIKE';
        		    $sValue = substr( $sValue, 2 );
        	    }
        	    elseif ( substr( $sValue, 0, 2 ) == '<=' || substr( $sValue, 0, 2 ) == '>=' || substr( $sValue, 0, 2 ) == '!=' )
        		{
        		    $sCompare = substr( $sValue, 0, 2 );
        		    $sValue = substr( $sValue, 2 );
        	    }
                elseif ( substr( $sValue, 0, 1 ) == '<' || substr( $sValue, 0, 1 ) == '>' || substr( $sValue, 0, 1 ) == '=' )
        		{
        		    $sCompare = substr( $sValue, 0, 1 );
        		    $sValue = substr( $sValue, 1 );
        	    }

                $sValue = $this->oForm->FormatValue( 'field_' . $sName, $sValue );

        		switch( $this->oForm->GetType( 'field_' . $sName ) )
                {
                    case 'NAME':
                    case 'TEXT':
                    case 'EMAIL':
                    case 'USERNAME':
                    case 'PASSWORD':
                        if ( $sCompare == '' )
        	                $sCompare = 'LIKE';

                        if ( $sCompare == 'LIKE' || $sCompare == 'NOT LIKE' )
                        {
                            // Wenn ein * angegeben ist, wird er durch ein % ersetzt: Aus xxxx* wird LIKE 'xxxx%', aus *xxxx wird LIKE '%xxxx', aus *xx*xx* wird LIKE '%xx%xx%'
                            $sValue = str_replace( '*', '%', $sValue );

                            // Wenn kein % angegeben ist, dann wird mit der Wert als Beginn verwendet: Aus xxxx wird LIKE 'xxxx%'
                            if ( strpos( $sValue, '%' ) === false )
                                if ( $oApp->GetFeature( 'ipage', 'filter_full' ) > 0 )
                                    $sValue = '%' . $sValue . '%';
                                else
                        			$sValue .= '%';
                        }
                    break;

                    case 'YESNO':
                    case 'TRISTATE':
                    default:
                        if ( $sCompare == '' )
                            $sCompare = '=';
                    break;
                }

                // '~' is a replacement for null
                if ( $sValue == '~' )
                {
                    $sValue = null;
                }

                $Filter[] = array( 'field' => $sName, 'value' => $sValue, 'compare' => $sCompare, 'operator' => $sOperator );
            }
        }

        // Wenn aField[ 'search' ] ein Array ist, dann statt dem Value-String ein Array mit Feld=>Wert erstellen
        if ( is_array( $this->aFields[ $sName ][ 'search' ] ) )
        {
            $sFilter = $Filter;
            $Filter = array();
            $Filter[] = array( 'field' => $sName, 'value' => $sFilter );
            foreach( $this->aFields[ $sName ][ 'search' ] as $sField )
                $Filter[] = array( 'field' => $sField, 'value' => $sFilter );
        }

		return( true );
	}
}
