<?php

defined( 'oPage' ) or die();

class CPager extends CControl
{
    var $iPage = 1;
    var $iPages;
    var $iNumbers = 9;

    function CPager( $sName = null )
    {
        if ( $sName === null )
            $sName = 'pager';

        $this->CControl( $sName );

        $this->AddParameterExclude( 'submit' );
        $this->AddParameterExclude( 'submit_x' );
        $this->AddParameterExclude( 'submit_y' );
        $this->AddParameterExclude( 'timezone' );
        $this->AddParameterExclude( 'datetime' );
    }

    function Copy( $oCopy = Null )
    {
        if ( !is_object( $oCopy ) )
        {
            $sClass = get_class( $this );
            $oCopy = new $sClass( '' );
        }
        $oCopy = CControl::Copy( $oCopy );

        $oCopy->iPage = $this->iPage;
        $oCopy->iPages = $this->iPages;
        $oCopy->iNumbers = $this->iNumbers;

        return( $oCopy );
    }

    function GetParameter()
    {
        global $oApp;
        $this->SetPage( $oApp->GetParameter( 'page' ) );
    }

    function SetPage( $iValue )
    {
        $this->iPage = $iValue;
        if ( $this->iPage < 1 )
            $this->iPage = 1;
    }

    function GetPage()
    {
        return( $this->iPage );
    }

    function SetPages( $iValue )
    {
        $this->iPages = $iValue;
        if ( $this->iPages < 0 )
            $this->iPages = 1;
        if ( $this->GetPage() > $this->iPages )
            $this->SetPage( $this->iPages );
    }

    function GetPages()
    {
        return( $this->iPages );
    }

    function SetNumbers( $iNumbers )
    {
        $this->iNumbers = $iNumbers;
    }

    function GetNumbers()
    {
        return( $this->iNumbers );
    }

    function OutputTpl( &$oTemplate, $sBlock = null, $sSub = null )
    {
        global $oApp;

        if ( $sBlock === null )
            $sBlock = 'main';

        if ( !parent::OutputTpl( $oTemplate, $sBlock, $sSub ) )
            return( false );

        $aVars = $oTemplate->GetVars();

        $sBlockName = $sBlock . '.' . $this->GetName() . $sSub;

        $iPage = $this->GetPage();
        $iPages = $this->GetPages();

        // Wenn nur eine Seite vorhanden ist, Pager nicht ausgeben
        if ( $iPages <= 1 )
        {
            return( true );
        }

        $sScriptname = $oApp->GetScriptname();
        $aParameter = $oApp->GetParameters();

        // Nur bestimmte Felder einschlieen
        if ( $this->GetParameterInclude() != '' )
        {
            foreach( $aParameter as $sName => $sValue )
            {
                if ( !$this->IsParameterIncluded( $sName ) )
                {
                    unset( $aParameter[ $sName ] );
                }
            }
        }

        // Bestimmte Felder ausschlieen
        if ( $this->GetParameterExclude() != '' )
        {
            foreach( $aParameter as $sName => $sValue )
            {
                if ( $this->IsParameterExcluded( $sName ) )
                {
                    unset( $aParameter[ $sName ] );
                }
            }
        }

        // Alle leeren Felder ausschlieen
        foreach( $aParameter as $sName => $sValue )
        {
            if ( IsNull( $aParameter[ $sName ] ) )
            {
                unset( $aParameter[ $sName ] );
                continue;
            }
        }

        $sQuerySeparator = $oApp->GetQuerySeparator();
        $sParamSeparator = $oApp->GetParamSeparator();

        $oTemplate->assign( 'DATA.page', $iPage );
        $oTemplate->assign( 'DATA.pages', $iPages );

        if ( $iPage > 1 )
        {
            // First und Previous nur dann anzeigen, wenn iPage > 1
            $oTemplate->assign( 'DATA.page_first', 1 );
            $aParameter[ 'page' ] = 1;
            $oTemplate->assign( 'DATA.param_first', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.first' );

            $oTemplate->assign( 'DATA.page_previous', $iPage - 1 );
            $aParameter[ 'page' ] = $iPage - 1;
            $oTemplate->assign( 'DATA.param_previous', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.previous' );
        }
        else
        {
            // First_all und Previous_all anzeigen
            $oTemplate->assign( 'DATA.page_first', 1 );
            $aParameter[ 'page' ] = 1;
            $oTemplate->assign( 'DATA.param_first', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.first_all' );

            // Previous hpft hier ans Ende
            $oTemplate->assign( 'DATA.page_previous', $iPages );
            $aParameter[ 'page' ] = $iPages;
            $oTemplate->assign( 'DATA.param_previous', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.previous_all' );
        }

        $iStart = $iPage - ceil( ( $this->GetNumbers()-1 ) / 2 );
        $iEnd = $iPage + ceil( ( $this->GetNumbers()-1 ) / 2 );
        while ( $iStart < 1 || $iEnd > $iPages )
        {
            if ( $iStart < 1 )
            {
                $iStart++;
                if ( $iEnd < $iPages )
                    $iEnd++;
            }
            if ( $iEnd > $iPages )
            {
                $iEnd--;
                if ( $iStart > 1 )
                    $iStart--;
            }
        }

        for ( $i = $iStart; $i <= $iEnd; $i++ )
        {
            $aParameter[ 'page' ] = $i;
            $oTemplate->assign( 'DATA.page_site', $i );
            $oTemplate->assign( 'DATA.param_site', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            if ( $iPage == $i )
                $oTemplate->parse( $sBlockName . '.site.active' );
            else
                $oTemplate->parse( $sBlockName . '.site.inactive' );
            $oTemplate->parse( $sBlockName . '.site' );
        }

        if ( $iPage < $iPages )
        {
            $oTemplate->assign( 'DATA.page_next', $iPage + 1 );
            $aParameter[ 'page' ] = $iPage + 1;
            $oTemplate->assign( 'DATA.param_next', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.next' );

            $oTemplate->assign( 'DATA.page_last', $iPages );
            $aParameter[ 'page' ] = $iPages;
            $oTemplate->assign( 'DATA.param_last', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.last' );
        }
        else
        {
            $oTemplate->assign( 'DATA.page_next', 1 );
            $aParameter[ 'page' ] = 1;
            $oTemplate->assign( 'DATA.param_next', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.next_all' );

            $oTemplate->assign( 'DATA.page_last', $iPages );
            $aParameter[ 'page' ] = $iPages;
            $oTemplate->assign( 'DATA.param_last', HttpParam( $aParameter, $sQuerySeparator, $sParamSeparator ) );
            $oTemplate->parse( $sBlockName . '.last_all' );
        }

        $oTemplate->parse( $sBlockName );
        $oTemplate->SetVars( $aVars );
    }
}
