<?php

defined( 'oPage' ) or die();

/**
    Formatieren von Datums und Zeit Werten.
    Wird ein ungltiges Format angegeben, wird die Verarbeitung abgebrochen.
*/

include $sRoot . 'include/adodb/adodb-time.inc.php';

class CDate
{
    /** Name of the charset, default iso-8859-1
        windows-1252
        iso-8859-1
        utf-8
    */
    var $sCharset = 'iso-8859-1';

    /** Current language
    */
    var $sLanguage;

    /** Enthlt die Wochentage in der aktuellen Sprache
    */
    var $aWeekdays;

    /** Enthlt die Monatsnamen in der aktuellen Sprache
    */
    var $aMonths;

    /** Enthlt den TimeStamp
    */
    var $iDateTime; // unix timestamp
    var $aDateTime; // array of getdate()

    /** Constructor
        @param $sLanguage de, en (default de)
        @return void
    */
    function CDate( $sLanguage = Null, $sCharset = null )
    {
        $this->SetLanguage( $sLanguage );

        if ( $sCharset !== null )
        {
            $this->sCharset = $sCharset;
        }

        $this->iDateTime = time();
        $this->aDateTime = adodb_getdate( $this->iDateTime );

        $this->aWeekdays[ 'en' ] = Array
        (
			0 => 'Sunday',
			1 => 'Monday',
			2 => 'Tuesday',
			3 => 'Wednesday',
			4 => 'Thursday',
			5 => 'Friday',
			6 => 'Saturday'
	    );
        $this->aWeekdays[ 'de' ] = Array
        (
			0 => 'Sonntag',
			1 => 'Montag',
			2 => 'Dienstag',
			3 => 'Mittwoch',
			4 => 'Donnerstag',
			5 => 'Freitag',
			6 => 'Samstag'
		);

	    $this->aMonths[ 'en' ] = Array
	    (
	        1 => 'January',
	        2 => 'February',
	        3 => 'March',
	        4 => 'April',
	        5 => 'May',
	        6 => 'June',
	        7 => 'July',
	        8 => 'August',
	        9 => 'September',
	       10 => 'October',
	       11 => 'November',
	       12 => 'December'
	    );
	    $this->aMonths[ 'de' ] = Array
	    (
	        1 => 'Jnner',
	        2 => 'Februar',
	        3 => 'Mrz',
	        4 => 'April',
	        5 => 'Mai',
	        6 => 'Juni',
	        7 => 'Juli',
	        8 => 'August',
	        9 => 'September',
	       10 => 'Oktober',
	       11 => 'November',
	       12 => 'Dezember'
	    );
    }

    /** mktime stub for adodb_mktime
    */
    function mktime( $iHour, $iMinute, $iSecond, $iMonth = 0, $iDay = 0, $iYear = 0 )
    {
        return( adodb_mktime( $iHour, $iMinute, $iSecond, $iMonth, $iDay, $iYear ) );
    }

    /** Konvertiert einen DB DateTime String in einen Timestamp
        @param $sDateTime Datums bzw. Zeitwert aus MySQL
        @return iDateTime
    */
    function FromDB( $sDateTime )
    {
        return( $this->FromString( $sDateTime, 4 ) );
    }

    /** Konvertiert einen DateTime String in einen Timestamp
        @param $sDateTime Datums bzw. Zeitwert
        @param $Format Format Parameter oder Formatstring, in dem das Datum angegeben ist.
        @return nix
    */
    function FromString( $sDateTime, $Format = 0  )
    {
        // Reset values
		$this->iDateTime = null;
		$this->aDateTime = array();

		if ( $sDateTime === null || $sDateTime === '' )
        {
			return;
        }

		$this->iDateTime = $sDateTime;
		if ( is_integer( $Format ) )
        {
            switch( $Format )
            {
                case 0:
            		list( $sDay, $sMonth, $sYear ) = preg_split( "/[^0-9]/", $sDateTime );
            		$this->iDateTime = adodb_mktime( 0, 0, 0, (int)$sMonth, (int)$sDay, (int)$sYear );
                    $this->aDateTime[ 'seconds' ] = null;
                    $this->aDateTime[ 'minutes' ] = null;
                    $this->aDateTime[ 'hours' ] = null;
                    $this->aDateTime[ 'mday' ] = $sDay;
                    $this->aDateTime[ 'mon' ] = $sMonth;
                    $this->aDateTime[ 'year' ] = $sYear;
                break;

                case 1:
            		list( $sHour, $sMinute, $sSecond ) = preg_split( "/[^0-9]/", $sDateTime );
            		$this->iDateTime = adodb_mktime( (int)$sHour, (int)$sMinute, (int)$sSecond, 0, 0, 0 );
                    $this->aDateTime[ 'seconds' ] = $sSecond;
                    $this->aDateTime[ 'minutes' ] = $sMinute;
                    $this->aDateTime[ 'hours' ] = $sHour;
                    $this->aDateTime[ 'mday' ] = null;
                    $this->aDateTime[ 'mon' ] = null;
                    $this->aDateTime[ 'year' ] = null;
                break;

                case 2:
            		list( $sDay, $sMonth, $sYear, $sHour, $sMinute, $sSecond ) = preg_split( "/[^0-9]/", $sDateTime );
            		$this->iDateTime = adodb_mktime( (int)$sHour, (int)$sMinute, (int)$sSecond, (int)$sMonth, (int)$sDay, (int)$sYear );
                    $this->aDateTime[ 'seconds' ] = $sSecond;
                    $this->aDateTime[ 'minutes' ] = $sMinute;
                    $this->aDateTime[ 'hours' ] = $sHour;
                    $this->aDateTime[ 'mday' ] = $sDay;
                    $this->aDateTime[ 'mon' ] = $sMonth;
                    $this->aDateTime[ 'year' ] = $sYear;
                break;

                case 3:
            		list( $sYear, $sMonth, $sDay ) = preg_split( '/[^0-9]/', $sDateTime );
            		$this->iDateTime = adodb_mktime( 0, 0, 0, (int)$sMonth, (int)$sDay, (int)$sYear );
                    $this->aDateTime[ 'seconds' ] = null;
                    $this->aDateTime[ 'minutes' ] = null;
                    $this->aDateTime[ 'hours' ] = null;
                    $this->aDateTime[ 'mday' ] = $sDay;
                    $this->aDateTime[ 'mon' ] = $sMonth;
                    $this->aDateTime[ 'year' ] = $sYear;
                break;

                case 4:
            		list( $sYear, $sMonth, $sDay, $sHour, $sMinute, $sSecond ) = preg_split( '/[^0-9]/', $sDateTime );
            		$this->iDateTime = adodb_mktime( (int)$sHour, (int)$sMinute, (int)$sSecond, (int)$sMonth, (int)$sDay, (int)$sYear );
                    $this->aDateTime[ 'seconds' ] = $sSecond;
                    $this->aDateTime[ 'minutes' ] = $sMinute;
                    $this->aDateTime[ 'hours' ] = $sHour;
                    $this->aDateTime[ 'mday' ] = $sDay;
                    $this->aDateTime[ 'mon' ] = $sMonth;
                    $this->aDateTime[ 'year' ] = $sYear;
                break;

                default:
                    die( 'CDate: FromString: Ungltiges Datumsformat' );
                break;
            }
        }
        else
        {
    		list( $sDay, $sMonth, $sYear, $sHour, $sMinute, $sSecond ) = preg_split( '/[^0-9]/', $sDateTime );;
    		$this->iDateTime = adodb_mktime( (int)$sHour, (int)$sMinute, (int)$sSecond, (int)$sMonth, (int)$sDay, (int)$sYear );
            $this->aDateTime[ 'seconds' ] = $sSecond;
            $this->aDateTime[ 'minutes' ] = $sMinute;
            $this->aDateTime[ 'hours' ] = $sHour;
            $this->aDateTime[ 'mday' ] = $sDay;
            $this->aDateTime[ 'mon' ] = $sMonth;
            $this->aDateTime[ 'year' ] = $sYear;
        }

        // Prepare english day
        $this->_PrepareDay();

        return( $this->iDateTime );
    }

    /** Formatiert den Datums und/oder Zeitwert nach dem angegebenen Format
        0: DD.MM.YYYY
        1: HH:MM:SS
        2: DD.MM.YYYY HH:MM:SS
        3: YYYY-MM-DD (fr Datenbank)
        4: YYYY-MM-DD HH:MM:SS (fr Datenbank)
        @param $iFormat Integer Wert fr das Format, Default = 0
        @return String
    */
    function Format( $Format = 0, $iDateTime = null, $fString = null )
    {
        if ( $iDateTime === null || $iDateTime === '' )
        {
            $iDateTime = $this->iDateTime;
            $aDateTime = &$this->aDateTime;
        }
        else
        {
            $aDateTime = adodb_getdate( $iDateTime );
            // Prepare english day
            $this->_PrepareDay();
        }

        if ( $fString === null )
        {
            $fString = false;
        }

		if ( $iDateTime === null || $iDateTime == -1 )
        {
			return;
        }

		if ( is_integer( $Format ) )
        {
            switch( $Format )
            {
                case 0:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'mday' ] . '.' . $aDateTime[ 'mon' ] . '.' . $aDateTime[ 'year' ];
                    else
                        $sBuffer = adodb_date( 'd.m.Y', $iDateTime );
                break;

                case 1:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ] . ':' . $aDateTime[ 'seconds' ];
                    else
                        $sBuffer = adodb_date( 'H:i:s', $iDateTime );
                break;

                case 2:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'mday' ] . '.' . $aDateTime[ 'mon' ] . '.' . $aDateTime[ 'year' ] . ' ' . $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ] . ':' . $aDateTime[ 'seconds' ];
                    else
                        $sBuffer = adodb_date( 'd.m.Y H:i:s', $iDateTime );
                break;

                case 3:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'year' ] . '-' . $aDateTime[ 'mon' ] . '-' . $aDateTime[ 'mday' ];
                    else
                        $sBuffer = adodb_date( 'Y-m-d', $iDateTime );
                break;

                case 4:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'year' ] . '-' . $aDateTime[ 'mon' ] . '-' . $aDateTime[ 'mday' ] . ' ' . $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ] . ':' . $aDateTime[ 'seconds' ];
                    else
                        $sBuffer = adodb_date( 'Y-m-d H:i:s', $iDateTime );
                break;

                case 5:
                    $aDate = adodb_getdate( $iDateTime );
                    if ( $fString )
                        $sBuffer = $this->aWeekdays[ $this->sLanguage ][ $aDate[ 'wday' ] ] . ', ' . $aDateTime[ 'mday' ] . '. ' . $this->aMonths[ $this->sLanguage ][ (int)$aDateTime[ 'mon' ] ] . ' ' . $aDateTime[ 'year' ] . ', ' . $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ] . ':' . $aDateTime[ 'seconds' ];
                    else
                        $sBuffer = $this->aWeekdays[ $this->sLanguage ][ $aDate[ 'wday' ] ] . ', ' . $aDate[ 'mday' ] . '. ' . $this->aMonths[ $this->sLanguage ][ $aDate[ 'mon' ] ] . ' ' . $aDate[ 'year' ] . ', ' . adodb_date( 'H:i:s', $iDateTime );
                break;

                case 6:
                    if ( $fString )
                        $sBuffer = $this->aMonths[ $this->sLanguage ][ (int)$aDateTime[ 'mon' ] ] . ' ' . $aDateTime[ 'year' ];
                    else
                    {
                        $aDate = adodb_getdate( $iDateTime );
                        $sBuffer = $this->aMonths[ $this->sLanguage ][ $aDate[ 'mon' ] ] . ' ' . $aDate[ 'year' ];
                    }
                break;

                case 7:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'year' ] . '.' . $aDateTime[ 'mon' ];
                    else
                        $sBuffer = adodb_date( 'Y.m', $iDateTime );
                break;

                case 8:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'year' ];
                    else
                        $sBuffer = adodb_date( 'Y', $iDateTime );
                break;

                case 9:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'year' ] . $aDateTime[ 'mon' ] . $aDateTime[ 'mday' ] . $aDateTime[ 'hours' ];
                    else
                        $sBuffer = adodb_date( 'YmdH', $iDateTime );
                break;

                case 10:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'mday' ] . '. ' . $aDateTime[ 'mon' ] . '. ' . $aDateTime[ 'year' ];
                    else
                        $sBuffer = adodb_date( 'j. n. Y', $iDateTime );
                break;

                case 11:
                    if ( $fString )
                        $sBuffer = '0000-' . $aDateTime[ 'mon' ] . '-' . $aDateTime[ 'mday' ] . ' ' . $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ] . ':' . $aDateTime[ 'seconds' ];
                    else
                        $sBuffer = '0000-' . adodb_date( 'm-d H:i:s', $iDateTime );
                break;

                case 12:
                    $sBuffer = $this->aWeekdays[ $this->sLanguage ][ $aDateTime[ 'wday' ] ];
                break;

                // 28. Februar 2008, 10:55
                // 28th of February 2008, 10:55
                case 13:
                    if ( $this->sLanguage == 'en' )
                        $sBuffer = $aDateTime[ 'mday' ] . ' ' . $this->aMonths[ $this->sLanguage ][ (int)$aDateTime[ 'mon' ] ] . ' ' . $aDateTime[ 'year' ] . ', ' . $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ];
                    else
                        $sBuffer = $aDateTime[ 'mday' ] . '. ' . $this->aMonths[ $this->sLanguage ][ (int)$aDateTime[ 'mon' ] ] . ' ' . $aDateTime[ 'year' ] . ', ' . $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ];
                break;

                // 2 char Weekday: Mo, Tu, ...
                case 14:
                    $sBuffer = substr( $this->aWeekdays[ $this->sLanguage ][ $aDateTime[ 'wday' ] ], 0, 2 );
                break;

                // Montag, 28. Februar 2008
                case 15:
                    $aDate = adodb_getdate( $iDateTime );
                    if ( $fString )
                        $sBuffer = $this->aWeekdays[ $this->sLanguage ][ $aDate[ 'wday' ] ] . ', ' . $aDateTime[ 'mday' ] . '. ' . $this->aMonths[ $this->sLanguage ][ (int)$aDateTime[ 'mon' ] ] . ' ' . $aDateTime[ 'year' ];
                    else
                        $sBuffer = $this->aWeekdays[ $this->sLanguage ][ $aDate[ 'wday' ] ] . ', ' . $aDate[ 'mday' ] . '. ' . $this->aMonths[ $this->sLanguage ][ $aDate[ 'mon' ] ] . ' ' . $aDate[ 'year' ];
                break;

                // 23:59
                case 16:
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'hours' ] . ':' . $aDateTime[ 'minutes' ];
                    else
                        $sBuffer = adodb_date( 'H:i', $iDateTime );
                break;

                // 28. Februar 2008
                case 17:
                    $aDate = adodb_getdate( $iDateTime );
                    if ( $fString )
                        $sBuffer = $aDateTime[ 'mday' ] . '. ' . $this->aMonths[ $this->sLanguage ][ (int)$aDateTime[ 'mon' ] ] . ' ' . $aDateTime[ 'year' ];
                    else
                        $sBuffer = $aDate[ 'mday' ] . '. ' . $this->aMonths[ $this->sLanguage ][ $aDate[ 'mon' ] ] . ' ' . $aDate[ 'year' ];
                break;

                default:
                    die( 'CDate: Format: Ungltiges Datumsformat' );
                break;
            }
        }
        else
        {
            $sBuffer = adodb_date( $Format, $iDateTime );
        }

        // If charset is utf-8, convert template from Windows to UTF-8
        if ( $this->sCharset == 'utf-8' )
        {
            $sBuffer = StringWindowsToUtf8( $sBuffer );
        }

        return( $sBuffer );
    }

    /** Formatiert den GMT Datums und/oder Zeitwert nach dem angegebenen Format
        0: DD.MM.YYYY
        1: HH:MM:SS
        2: DD.MM.YYYY HH:MM:SS
        3: YYYY-MM-DD (fr Datenbank)
        4: YYYY-MM-DD HH:MM:SS (fr Datenbank)
        @param $iFormat Integer Wert fr das Format, Default = 0
        @return String
    */
    function FormatGMT( $Format = 0, $iDateTime = null )
    {
        if ( $iDateTime === null || $iDateTime === '' )
            $iDateTime = $this->iDateTime;
		if ( $iDateTime === null || $iDateTime === '' || $iDateTime == -1 )
			return;

		if ( is_integer( $Format ) )
        {
            switch( $Format )
            {
                case 0:
                    $sBuffer = adodb_gmdate( 'd.m.Y', $iDateTime );
                break;

                case 1:
                    $sBuffer = adodb_gmdate( 'H:i:s', $iDateTime );
                break;

                case 2:
                    $sBuffer = adodb_gmdate( 'd.m.Y H:i:s', $iDateTime );
                break;

                case 3:
                    $sBuffer = adodb_gmdate( 'Y-m-d', $iDateTime );
                break;

                case 4:
                    $sBuffer = adodb_gmdate( 'Y-m-d H:i:s', $iDateTime );
                break;

                default:
                    die( 'CDate: FormatGMT: Ungltiges Datumsformat' );
                break;
            }
        }
        else
            $sBuffer = adodb_gmdate( $Format, $iDateTime );

        return( $sBuffer );
    }

    /** Converts DateTime stamps into other datetime stamps
        @param $iFormat = 0
        @param $iDateTime = null
        @return $iDateTime
    */
    function Convert( $iFormat = 0, $iDateTime = null )
    {
        if ( $iDateTime === null || $iDateTime === '' )
            $iDateTime = $this->iDateTime;
		if ( $iDateTime === null || $iDateTime == -1 )
			return;

        switch( $iFormat )
        {
            case 0:
                $aDate = adodb_getdate( $iDateTime );
                $iDateTime = adodb_mktime ( 0, 0, 0, $aDate[ 'mon' ], $aDate[ 'mday' ], $aDate[ 'year' ] );
            break;

            default:
                die( 'CDate: Format: Ungltiges Datumsformat' );
            break;
        }

        return( $iDateTime );
    }

    /** Localizes a give datetime according to the given timezone
        @param iDatetime Datetime (server side)
        @param iTimezone Timezone (client side)
        @return iDateTime (client side)
    */
    function Localize( $iDatetime, $iTimezone )
    {
        if ( $iTimezone === null || $iTimezone === '' )
            return( $iDatetime );

    	$aTimeOfDay = GetTimeOfDay();

        // Bug in PHP 4.1.2 on WINNT
        if ( PHP_VERSION == '4.1.2' && PHP_OS == 'WINNT' )
            return( $iTimezone * 60 - $aTimeOfDay[ 'minuteswest' ] + $iDatetime );

        return( ( $iTimezone - $aTimeOfDay[ 'minuteswest' ] ) * 60 + $iDatetime );
    }

    function SetDateTime( $iValue )
    {
        $this->iDateTime = $iValue;
        $this->aDateTime = adodb_getdate( $iValue );
    }

    function GetDateTime()
    {
        return( $this->iDateTime );
    }

    /** Sets the language
    	@param $sLanguage short text for the language: {de|en}, default de
    */
    function SetLanguage( $sLanguage = null )
    {
        if ( $sLanguage === null || $sLanguage === '' )
            $sLanguage = 'de';

        $this->sLanguage = $sLanguage;
    }

    /** Prepare english day
        @param none
        @return void
    */
    function _PrepareDay()
    {
        $this->aDateTime[ 'mdayx' ] = null;

        if ( $this->aDateTime[ 'mday' ] === null || $this->aDateTime[ 'mday' ] === '' )
            return;

        if ( $this->aDateTime[ 'mday' ] == 1 )
            $this->aDateTime[ 'mdayx' ] = 'st';
        elseif ( $this->aDateTime[ 'mday' ] == 2 )
            $this->aDateTime[ 'mdayx' ] = 'nd';
        elseif ( $this->aDateTime[ 'mday' ] == 3 )
            $this->aDateTime[ 'mdayx' ] = 'rd';
        else
            $this->aDateTime[ 'mdayx' ] = 'th';
    }

    /** Get month
        @param iMonth
    */
    function GetMonth( $iMonth )
    {
        $iMonth = $iMonth % 12;
        if ( $iMonth == 0 )
        {
            $iMonth = 12;
        }

        $sBuffer = $this->aMonths[ $this->sLanguage ][ $iMonth ];

        // If charset is utf-8, convert template from Windows to UTF-8
        if ( $this->sCharset == 'utf-8' )
        {
            $sBuffer = StringWindowsToUtf8( $sBuffer );
        }

        return( $sBuffer );
    }
}

if ( !function_exists( 'StringWindowsToUtf8' ) )
{
    /*
        This structure encodes the difference between ISO-8859-1 and Windows-1252,
        as a map from the UTF-8 encoding of some ISO-8859-1 control characters to
        the UTF-8 encoding of the non-control characters that Windows-1252 places
        at the equivalent code points.
        Aidan Kehoe <php-manual@parhasard.net>
    */
    global $aCp1252ToUtf8;
    $aCp1252ToUtf8 = array
    (
        "\xc2\x80" => "\xe2\x82\xac", /* EURO SIGN */
        "\xc2\x82" => "\xe2\x80\x9a", /* SINGLE LOW-9 QUOTATION MARK */
        "\xc2\x83" => "\xc6\x92",     /* LATIN SMALL LETTER F WITH HOOK */
        "\xc2\x84" => "\xe2\x80\x9e", /* DOUBLE LOW-9 QUOTATION MARK */
        "\xc2\x85" => "\xe2\x80\xa6", /* HORIZONTAL ELLIPSIS */
        "\xc2\x86" => "\xe2\x80\xa0", /* DAGGER */
        "\xc2\x87" => "\xe2\x80\xa1", /* DOUBLE DAGGER */
        "\xc2\x88" => "\xcb\x86",     /* MODIFIER LETTER CIRCUMFLEX ACCENT */
        "\xc2\x89" => "\xe2\x80\xb0", /* PER MILLE SIGN */
        "\xc2\x8a" => "\xc5\xa0",     /* LATIN CAPITAL LETTER S WITH CARON */
        "\xc2\x8b" => "\xe2\x80\xb9", /* SINGLE LEFT-POINTING ANGLE QUOTATION */
        "\xc2\x8c" => "\xc5\x92",     /* LATIN CAPITAL LIGATURE OE */
        "\xc2\x8e" => "\xc5\xbd",     /* LATIN CAPITAL LETTER Z WITH CARON */
        "\xc2\x91" => "\xe2\x80\x98", /* LEFT SINGLE QUOTATION MARK */
        "\xc2\x92" => "\xe2\x80\x99", /* RIGHT SINGLE QUOTATION MARK */
        "\xc2\x93" => "\xe2\x80\x9c", /* LEFT DOUBLE QUOTATION MARK */
        "\xc2\x94" => "\xe2\x80\x9d", /* RIGHT DOUBLE QUOTATION MARK */
        "\xc2\x95" => "\xe2\x80\xa2", /* BULLET */
        "\xc2\x96" => "\xe2\x80\x93", /* EN DASH */
        "\xc2\x97" => "\xe2\x80\x94", /* EM DASH */
        "\xc2\x98" => "\xcb\x9c",     /* SMALL TILDE */
        "\xc2\x99" => "\xe2\x84\xa2", /* TRADE MARK SIGN */
        "\xc2\x9a" => "\xc5\xa1",     /* LATIN SMALL LETTER S WITH CARON */
        "\xc2\x9b" => "\xe2\x80\xba", /* SINGLE RIGHT-POINTING ANGLE QUOTATION*/
        "\xc2\x9c" => "\xc5\x93",     /* LATIN SMALL LIGATURE OE */
        "\xc2\x9e" => "\xc5\xbe",     /* LATIN SMALL LETTER Z WITH CARON */
        "\xc2\x9f" => "\xc5\xb8"      /* LATIN CAPITAL LETTER Y WITH DIAERESIS*/
    );

    function StringWindowsToUtf8( $sBuffer )
    {
       global $aCp1252ToUtf8;
       return( strtr( utf8_encode( $sBuffer ), $aCp1252ToUtf8 ) );
    }
}
