<?php

defined( 'oPage' ) or die();

// Datenbank-Treiber inkludieren
include_once $sRoot . 'include/database/' . DATABASE;
include_once $sRoot . 'include/tools.php';

/** Db Schnittstelle zwischen PHP und MySQL
	@autor Dorn Hannes (DOH)
	@history 23.02.2001 (ALS) - SourceDoku erstellt
*/
class CDb extends DB_Sql
{
    /** Trace output
        0: no trace
        1: output with echo
        2: output with echo as html comment
        3: output with syslog
    */
    var $iTrace = 0;

    /** Explain SQL statements
        Do explain end echo result
    */
    var $fExplain = false;

    /** Version
        Database version
    */
    var $sVersion;

    /** Charset
        One of valid mysql Charsets
    */
    var $sCharset;

    /** Collation
        One of valid mysql Collations
    */
    var $sCollation;

    /** <b>Konstruktor</b><br>
    	Initialisierung der Datenbankvariablen in DB_Sql durch globale
    	Vorgaben in app.php
    */
    function CDb( $sHost = null, $sDatabase = null, $sUser = null, $sPassword = null, $sErrorHandling = null )
    {
        if( $sHost === null )
            $sHost = DATABASE_SERVER;
        if( $sDatabase === null )
            $sDatabase = DATABASE_DATABASE;
        if( $sUser === null )
            $sUser = DATABASE_USER;
        if( $sPassword === null )
            $sPassword = DATABASE_PASSWORD;
        if( $sErrorHandling === null )
            $sErrorHandling = ERROR_HANDLING;

        $this->Host = $sHost;
        $this->Database = $sDatabase;
        $this->User = $sUser;
        $this->Password = $sPassword;
        $this->Halt_On_Error = $sErrorHandling;
    }

    /** Verbindung zur Datenbank aufbauen
    	@param: optional () oder ( "Hostname", "Datenbankname", "User", "Passwort" )
    	@return: Connection - Handle
    */
    function Conn( $sHost = null, $sDatabase = null, $sUser = null, $sPassword = null )
    {
        if ( $sHost === null || $sHost == "" )
            $sHost = $this->Host;

        if ( $sDatabase === null || $sDatabase == "" )
            $sDatabase = $this->Database;

        if ( $sUser === null || $sUser == "" )
            $sUser = $this->User;

        if ( $sPassword === null || $sPassword == "" )
            $sPassword = $this->Password;

        $iHandle = DB_Sql::connect( $sDatabase, $sHost, $sUser, $sPassword );

        if ( $iHandle )
        {
            // get current mysql version
            $this->query( 'SELECT VERSION() as version' );
            $aResult = $this->Result();
            $this->sVersion = $aResult[ 0 ][ 'version' ];

            // if version >= 4.1 -> set charset
            if ( $this->sVersion >= 4.1 )
                // Set correct charset, this avoids, that MySql reencodes the data again in UTF-8
                if ( $this->sCharset !== null && $this->sCharset !== '' )
                    $this->query( 'SET NAMES \'' . $this->sCharset . '\'' );
        }

        return( $iHandle );
    }

    /** Sets a charset
        @param $sValue a valid mysql charsets like utf8, or latin1, default null (use default charset)
    */
    function SetCharset( $sValue = null )
    {
        $this->sCharset = $sValue;
    }

    /** Sets a collate
        @param $sValue a valid mysql collations like utf8_general_ci, or latin1_german1_ci, default null (use default collation)
    */
    function SetCollation( $sValue = null )
    {
        $this->sCollation = $sValue;
    }

    /** Test DB Connection
        @param $sHost hostname, default null
        @param $sDatabase database name, default null
        @param $sUser database username, default null
        @param $sPassword database password, default null
        @return true/false
    */
    function Test( $sHost = null, $sDatabase = null, $sUser = null, $sPassword = null )
    {
        $this->Halt_On_Error = 'no';
        return( $this->Conn( $sHost, $sDatabase, $sUser, $sPassword ) );
    }

    /** Schlieen der Datenbankverbindung
    	@return immer True, da mysql die Fkt. der Disconnection nicht untersttzt
    */
    function Disconnect()
    {
        // close_connection ... wird von db_mysql nicht untersttzt
        return true;
    }

    /** Create a lock
        @param $sTable table name
        @param $sId id value
        @param $iTimeout timeout in seconds, default 60
        @return true, false
    */
    function Lock( $sTable, $sId, $iTimeout = null )
    {
        if ( $iTimeout === null )
            $iTimeout = 60;

        // Get the lock
        $this->Query( 'select get_lock( \'' . addslashes( $sTable . '#' . $sId ) . '\', ' . $iTimeout . ' ) as my_lock' );
        $aResult = $this->Result();
        if ( !isset( $aResult[ 0 ] ) || $aResult[ 0 ][ 'my_lock' ] != 1 )
            return( false );

        return( true );
    }

    /** Release a lock
        @param $sTable table name
        @param $sId id value
        @return void
    */
    function Unlock( $sTable, $sId )
    {
        $this->Query( 'select release_lock( \'' . addslashes( $sTable . '#' . $sId ) . '\' )' );
    }

    /** Returns an 2 dim array with the result rows
        @param void
        @return Array
    */
    function Result()
    {
        $aResult = array();
        while( $this->next_record() )
            $aResult[] = $this->Record;

        return( $aResult );
    }

    /** Daten aus einer Tabelle lesen
    	@param $sTable Tabelle aus der die Daten gelesen werden sollen
    	@param $sFields Attribute der Tabelle - getrennt durch ','
    	@param $sCondition Bedingung - entspricht inhaltlich einer where-Klausel ohne dem reserviertem Wort 'where'
    	@param $sOrder      ... Sortierung - entspricht einer ORDER-BY-Klausel ohne 'ORDER BY'
    	@return 2-dimensionales Array: $aArray[Datenstze][Attribute]
    	@Anmerkungen Datenstze werden von 0-n indiziert, Attribute von 0-n bzw. durch %Attributname_1% - %Attributname_N%
    	Zu beachten ist, dass assoziative Arrays intern jeweils doppelt abgespeichert sind!
    	Falls ein genderter Read-Returnwert durch Update oder Insert in die Tabelle geschrieben werden soll,
    	muss das Paar %Attributname% = Wert korrekt sein. Ist blo das Paar Index = Wert gendert, so geht
    	dieser Wert verloren. Wird das Index-Paar gendert, wird die Assoziation nicht automatisch mitverndert, u.u.
    	D.h. wenn zB das 2. Array-Attribut 'name' ist:
    	aArray[0][name] = "neuer Wert" ... korrekt; aArray[0][1] = "neuer Wert" ... keine nderung bei Insert und Update
    	@Beispiele $oResult = Read ( "test_tablelle", "id, name", "id = 2", "name");
    	Liefert ein Array aus der Tabelle 'test_tabelle' wobei die Bedingung der Datenstze 'id = 2' gilt,
    	sortiert nach 'name';
    	oResult[0]["id"] ... wrde 'id' des ersten Datensatzes liefern - entspricht oResult[0][0]
    	$oResult = Read ( "test_tabelle",  "*" );
    	Liefert ein Array aller Attribute aus der Tabelle 'test_tabelle' (keine Bedingung, keine Sortierung).
    	$oResult = Read ( "test_tabelle",  "name", "", "name");
    	Liefert ein Array aller 'name' aus der Tabelle 'test_tabelle' (keine Bedingung) sortiert nach 'name'.
    	oResult[0]["name"] ... wrde 'name' des ersten Datensatzes liefern
    	oResult[1][0]      ... wrde zweiten 'name' liefern
    */
    function Read( $sTable, $sFields = Null, $sCondition = Null, $sOrder = Null, $sLimit = null, $sGroup = null, $iTrace = Null, $fExplain = null )
    {
        if ( $sFields === null )
            $sFields = '*';
        if ( $fExplain === null )
            $fExplain = false;

        $this->TraceBegin( 'DB: Read', $iTrace );

        // Select
        $sQuery = "select $sFields from $sTable";
        // Where
        if ( $sCondition !== null && $sCondition <> ''  )
            $sQuery .= " where $sCondition";
        // Group by
        if ( $sGroup !== null && $sGroup <> '' )
            $sQuery .= " group by $sGroup";
        // Order by
        if ( $sOrder !== null && $sOrder <> '' )
            $sQuery .= " order by $sOrder";
        // Limit
        if ( $sLimit !== null && $sLimit <> '' )
            $sQuery .= " limit $sLimit";

        // Do explain?
        if ( $fExplain )
        {
            echo 'Query: ' . $sQuery . "\n<br>";
            $this->query( 'explain ' . $sQuery );
            $aResult = $this->Result();
            echo '<pre>';
            print_r( $aResult );
            echo '</pre>';
        }

        // Ausfhren der Abfrage
        $this->TraceBegin( 'DB: Read: Query', $iTrace );
        $this->query( $sQuery );
        $this->TraceEnd( 'DB: Read: Query', $sQuery, $iTrace );

        // Aufbereitung der Daten in ein 2-dim Array
        $this->TraceBegin( 'DB: Read: Get Data', $iTrace );
        $aResult = $this->Result();
        $this->TraceEnd( 'DB: Read: Get Data', Null, $iTrace );

        // Freigeben des belegten Speicherplatzes
        $this->free();

        $this->TraceEnd( 'DB: Read', Null, $iTrace );

        return( $aResult );
    }

    /** Counts all the records which fit the provided filter an limit
        @param $sTable table name
        @param $sFields it's best to use 'id' as field name
        @param $sCondition where clause
        @param $sLimit
        @return integer
    */
    function CountAll( $sTable, $sFields = null, $sCondition = null, $sLimit = null, $iTrace = Null )
    {
        if ( $sFields === null )
            $sFields = 'id';

        $this->TraceBegin( 'DB: CountAll', $iTrace );

        // Basic query
        $sQuery = 'select ' . $sFields . ' from ' . $sTable;

        // Append filter
        if ( $sCondition !== null && $sCondition !== '' )
            $sQuery .= ' where ' . $sCondition;

        // Append limit
        if ( strlen( $sLimit ) > 0 )
            $sQuery .= ' limit ' . $sLimit;

        // Excute query
        $this->TraceBegin( 'DB: CountAll: Query', $iTrace );
        $this->query( $sQuery );
        $iCounter = $this->num_rows();
        $this->TraceEnd( 'DB: CountAll: Query', $sQuery, $iTrace );

        $this->TraceEnd( 'DB: CountAll', Null, $iTrace );

        return( $iCounter );
    }

    /** Datenstze in die Tabelle einfgen
    	@param $sTable Tabellenname der zu erweiternde Tabelle
    	@param $aAttribute 2-dim, teils assoziatives Array: Array[Datensatz][Attribut/Wert-Paar]
    	@param $sNameId = Null Wird $sNameId gesetzt, wird fr jede Row das Feld $sNameId mit der neuen Datensatz ID gefllt.
    	@Anmerkungen Zu beachten ist, dass assoziative Arrays intern jeweils doppelt abgespeichert sind!
    	Falls ein genderter Read-Returnwert durch Update oder Insert in die Tabelle geschrieben werden soll,
    	muss das Paar %Attributname% = Wert korrekt sein. Ist blo das Paar Index = Wert gendert, so geht
    	dieser Wert verloren - wird das Index-Paar gendert, wird die Assoziation nicht automatisch mitverndert, u.u.
    	D.h. wenn zB das 2. Array-Attribut 'name' ist:
    	aArray[0][name] = "neuer Wert" ... korrekt; aArray[0][1] = "neuer Wert" ... keine nderung bei Insert und Update
    	@return Anzahl der eingefgten Stze
    	@Beispiel Daten aus einer Tabelle lesen:
    	$a2 = array(array());
    	$a2 = Read( "tabelle", "attribut1, attribut2", "" , "attribut1" );
    	gelesene Daten in eine Tabelle einfgen:
    	$rows = Insert( "tabelle", $a2 );
    	Selbst angelegtes Array in eine Tabelle einfgen:
    	$a4 =  array( array( "id" => 20, "name" => "test0", "link" => "test0"  ),
    	array( "id" => 21, "name" => "test1", "contentwidth" => "800"  ),
    	array( "id" => "22", "name" => "test1", "moduldesign" => 1  ));
    	$rows = Insert( "modul", $a4 );
    */
    function Insert( $sTable, &$aRows, $sNameId = Null, $iTrace = Null )
    {
        $this->TraceBegin( 'DB: Insert', $iTrace );

        $iRows = 0;

        foreach( $aRows as $sKeys => $aRow )
        {
            $sAttributes = "";
            $sValues = "";

            // Attribut/Wert-Paare in Set-Statement aufspalten
            foreach( $aRow as $sKey => $sValue )
            {
                if ( strlen( $sValue ) == 0 )
                    $sValue = "null";

                if ( strtolower( $sValue ) != "null" )
                    if ( !is_integer( $sValue ) )
                        $sValue = "'" . addslashes( $sValue ) . "'";

                if ( strlen( $sAttributes ) > 0 )
                    $sAttributes .= ", ";
                $sAttributes .= $sKey;

                if ( strlen( $sValues ) > 0 )
                    $sValues .= ", ";
                $sValues .= $sValue;
            }

            // Erstellen des insert-Befehls
            $sQuery = "INSERT INTO $sTable ( $sAttributes ) VALUES ( $sValues )";

            // Ausfhren der Anfgeabfrage
            $this->TraceBegin( 'DB: Insert: Query', $iTrace );
            $oResult = $this->query( $sQuery );
            $iRows += $this->affected_rows();
            $this->TraceEnd( 'DB: Insert: Query', $sQuery, $iTrace );

            // Soll die neue ID gesetzt werden?
            if ( !IsNull( $sNameId ) )
                $aRows[ $sKeys ][ $sNameId ] = $this->lid();
        }

        $this->TraceEnd( 'DB: Insert', Null, $iTrace );

        return( $iRows );
    }

    /** Lschen von Stzen einer Tabelle
    	@param $sTable Tabelle, in der Datenstze gelscht werden sollen
    	@param $sContition optionale Bedingung
    	@return Anzahl der gelschten Stze
    	@Beispiel Lschen aller Stze aus test_tabelle, wo der Attributwert von 'attr2' gleich 'sepp' ist:
    	$rows = Delete( "test_tabelle", "attr2 = 'sepp'" );
    	Lschen aller Stze aus test_tabelle, wo der Attributwert von 'id' gleich '1' ist:
    	$rows = Delete( "test_tabelle", "id = 1" );
    */
    function Delete( $sTable, $sCondition = '', $iTrace = Null )
    {
        $this->TraceBegin( 'DB: Delete', $iTrace );

        if ( $sCondition != '' )
        {
            $sQuery = "DELETE FROM $sTable WHERE $sCondition";
            $iRows = 0;
        }
        else
        {
            $sQuery = "TRUNCATE TABLE $sTable";

            // Wenn wir keine Condition haben, dann liefert MySql bei AffectedRows immer 0
            // Daher ermitteln wir die Anzahl der Datenstze vorher.
            $aResult = $this->Read( $sTable, 'count(*) as count' );
            $iRows = $aResult[ 0 ][ 'count' ];
        }

        // Ausfhren der Lschabfrage
        $this->TraceBegin( 'DB: Delete: Query', $iTrace );
        $this->query( $sQuery );
        $this->TraceEnd( 'DB: Delete: Query', $sQuery, $iTrace );

        if ( $iRows == 0 )
            $iRows = $this->affected_rows();

        $this->TraceEnd( 'DB: Delete', Null, $iTrace );

        return( $iRows );
    }

    /** Lscht die angegebenen Datenstze in der Tabelle
    	Type: Public
    	@param $sTable Tabellenname
    	@param $aRows Datenstze
    	@param $sId Name des ID Feldes
    	@return true/false
    */
    function DeleteX( $sTable, $aRows, $sId, $iTrace = Null )
    {
        $this->TraceBegin( 'DB: DeleteX', $iTrace );

        $iRows = 0;

        foreach( $aRows as $sKey => $aRow )
        {
            $sQuery = "DELETE FROM $sTable WHERE " . $sId . "= '" . addslashes( $aRow[ $sId ] ) . "'";

            // Ausfhren der Lschabfrage
            $this->TraceBegin( 'DB: DeleteX: Query', $iTrace );
            $this->Query( $sQuery );
            $this->TraceEnd( 'DB: DeleteX: Query', $sQuery, $iTrace );

            $iRows += $this->affected_rows();
        }

        $this->TraceEnd( 'DB: DeleteX', Null, $iTrace );

        return( $iRows );
    }

    /** nderung von Tabelleninhalten
    	@param $sTable Tabelle, in der Werte gendert werden sollen
    	@param $aAttribute 2-dim, teils assoziatives Array: Array[Datensatz][Attribut/Wert-Paar]
    	@Anmerkung Zu beachten ist, dass assoziative Arrays intern jeweils doppelt abgespeichert sind!
    	Falls ein genderter Read-Returnwert durch Update oder Insert in die Tabelle geschrieben werden soll,
    	muss das Paar %Attributname% = Wert korrekt sein. Ist blo das Paar Index = Wert gendert, so geht
    	dieser Wert verloren - wird das Index-Paar gendert, wird die Assoziation nicht automatisch mitverndert, u.u.
    	D.h. wenn zB das 2. Array-Attribut 'name' ist:
    	aArray[0][name] = "neuer Wert" ... korrekt; aArray[0][1] = "neuer Wert" ... keine nderung bei Insert und Update
    	$sId        ... Attribut, dessen Wert im Array die zu ndernden Datenstze bestimmt
    	@return Anzahl der genderten Stze
    	@Beispiel Daten aus einer Tabelle lesen
    	$array2 = array(array());
    	$array2 = Read( "test_tabelle", "id, attr1, attr2", "" , "id" );
    	Daten, in Array ndern, wobei die Attribute namentlich angegeben werden sollten!!!
    	$array2[0][attr1] = "Musterseite";
    	gendertes Array in Tabelle aktualisieren
    	$rows = Update( "test_tabelle", $array2, "id" );
    */
    function Update( $sTable, $aRows, $sId, $aLeave = null, $iTrace = null )
    {
        $this->TraceBegin( 'DB: Update', $iTrace );

        $iRows = 0;
        foreach( $aRows as $aRow )
        {
            // Set-Statement eines Datensatzes
            $sSet = "";

            // ID-Wert eines Datensatzes
            $sIdValue = "";

            // Attribut/Wert-Paare in Set-Statement aufspalten
            foreach( $aRow as $sKey => $sValue )
            {
                if ( is_array( $aLeave ) )
                    if ( in_array( $sKey, $aLeave ) )
					    continue;

    			if ( $sKey == $sId )
                    $sIdValue = $sValue;
                else
                {
                    if ( strlen( $sValue ) == 0 )
                        $sValue = "null";

                    if ( strtolower( $sValue ) != "null" )
                        if ( !is_integer( $sValue ) )
                            $sValue = "'" . addslashes( $sValue ) . "'";

                    if ( strlen( $sSet ) > 0 )
                        $sSet .= ", ";
                    $sSet .= $sKey . "=" . $sValue;
                }
            }

            if ( strlen( $sIdValue ) > 0 and strlen( $sSet ) > 0)
            {
                // Erstellen des update-Befehls
                $this->TraceBegin( 'DB: Update: Query', $iTrace );
                $sQuery = "UPDATE $sTable SET $sSet WHERE $sId='$sIdValue'";
                $this->TraceEnd( 'DB: Update: Query', $sQuery, $iTrace );

                // Ausfhren der nderungsabfrage
                $this->query( $sQuery );
                $iRows += $this->affected_rows();
            }
        }

        $this->TraceEnd( 'DB: Update', Null, $iTrace );

        return( $iRows );
    }

	/** Liefert zu einer ParentId den obersten Eintrag in der Hierarchie
		@param $sTable
		@param $sIDField
		@param $sParentIDField
		@param $sID
		@return ID des oberstend Datensatzes bzw. -1
	*/
	function GetRootID( $sTable, $sIDField, $sParentIDField, $sID )
	{
		$aResult = $this->Read( $sTable, $sIDField . "," . $sParentIDField, $sIDField . "=" . $sID );
		if ( count( $aResult ) )
			if ( $aResult[0][ $sIDField ] == $aResult[0][ $sParentIDField ] || $aResult[0][ $sParentIDField ] == '' )
				return( $aResult[0][ $sIDField ] );
			else
			    return( $this->GetRootId( $sTable, $sIDField, $sParentIDField, $aResult[0][ $sParentIDField ] ) );

		return( -1 );
	}

	/** Liefert zu einer ParentId den Eintrag in der Hierarchie
		@param $sTable          Name of the Table
		@param $iParentId       Id of the Record
		@param $sID             Name of Id Field
		@param $sParentID       Name of ParentId Field
		@return Id of Parent Record
	*/
	function GetParentID( $sTable, $iParentId, $sId = 'id', $sParentId = 'parent_id' )
	{
	    // Read entry, where Id = ParentId
		$aResult = $this->Read( $sTable, $sId . "," . $sParentId, $sId . "=" . $iParentId );

        // Nothing found?
		if ( !count( $aResult ) )
    		return( -1 );

        // Return ParentId
        return( $aResult[0][ $sParentId ] );
	}

	/** Liefert alle ChildIDs zu einer ID
		Stufe 0: Liefert die direkten Childs
		Stufe 1: Liefert auch die Childs der Childs
		Stufe 2: Liefert auch die Childs der Childs der Childs ...
		Stufe -1: Liefert alle Childs
		@param $sTable
		@param $sIDField
		@param $sParentIDField
		@param $sFilter
		@param $sID
		@param $iStufen
		@return String mit allen Child-IDs mit Beistrich getrennt. Kann im select fr eine IN Klausel verwendet werden.
	*/
	function GetChildIDs( $sTable, $sIDField, $sParentIDField, $sFilter, $sID, $iStufen = -1 )
	{
	    $sBuffer = "$sIDField <> $sParentIDField AND $sParentIDField in ( $sID )";
	    if ( !IsNull( $sFilter ) )
	        $sBuffer .= " AND $sFilter";

		$aResult = $this->Read( $sTable, $sIDField, $sBuffer );

		$aID = ArrayGetColumn( $aResult, $sIDField );
		if ( is_array( $aID ) )
			$sChildIDs = Implode( ",", $aID );
		else
			$sChildIDs = '';

		if ( strlen( $sChildIDs ) > 0 && $iStufen != 0 )
		{
			$sNewIDs = $this->GetChildIDs( $sTable, $sIDField, $sParentIDField, $sFilter, $sChildIDs, $iStufen - 1 );

			if ( strlen( $sNewIDs ) > 0 )
				$sChildIDs .= ",$sNewIDs";
		}

		return( $sChildIDs );
	}

	/** ???
		@param $sTable
		@param $sIDField
		@param $sParentIDField
		@param $sFilter
		@return ???
	*/
	function GetRootIDs( $sTable, $sIDField, $sParentIDField, $sFilter = '' )
	{
		$aResult = $this->Read( $sTable, $sIDField, Array2Statement( Array( $sFilter, "$sIDField=$sParentIDField" ) ) );
		$aID = ArrayGetColumn( $aResult, $sIDField );
		$sIDs = ArrayImplode( $aID, "," );
		return( $sIDs );
	}

	function GetIDs( $sTable, $sIDField, $sFilter = '' )
	{
		$aResult = $this->Read( $sTable, $sIDField, $sFilter );
		$aID = ArrayGetColumn( $aResult, $sIDField );
		$sIDs = ArrayImplode( $aID, "," );
		return( $sIDs );
	}

	function GetIDNamesToRoot( $sTable, $sIDField, $sParentIDField, $sNameField, $sIDs )
	{
		$aResult = $this->Read( $sTable, $sIDField . "," . $sParentIDField . "," . $sNameField, $sIDField . "=" . $sID );
		$sIDNames = $aResult[ 0 ][ $sNameField ];
		$sIDNames = $aResult[ 0 ][ $sNameField ] . ", " . $sIDNames;
	}

	function TraceBegin( $sFunction, $iTrace = Null )
	{
        global $oTimer;

        if ( !isset( $oTimer ) )
            return( false );

	    if ( $iTrace === null )
	        $iTrace = 0;
	    if ( $iTrace < $this->GetTrace() )
	        $iTrace = $this->GetTrace();

        if ( $iTrace > 0 )
            $oTimer->Start( $sFunction );
	}

	function TraceEnd( $sFunction, $sText = Null, $iTrace = Null )
	{
        global $oTimer;

        if ( !isset( $oTimer ) )
            return( false );

	    if ( $iTrace === null )
	        $iTrace = 0;
	    if ( $iTrace < $this->GetTrace() )
	        $iTrace = $this->GetTrace();

        if ( $iTrace > 0 )
        {
            $oTimer->Stop( $sFunction );
            $oTimer->Trace( $sFunction, $sText, $iTrace );
        }
	}

    /** Sets trace state
        @param $iValue 0-3, default 1
        @return void
    */
    function SetTrace( $iValue = null )
    {
        if ( $iValue === null )
            $iValue = 0;
        if ( $iValue < 0 || $iValue > 3 )
            $iValue = 0;

        $this->iTrace = $iValue;
    }

    /** Returns current trace state
        @return true/false
    */
    function GetTrace()
    {
        return( $this->iTrace );
    }

    /** Checks, if table exists
        @param $sTable name of table
        @return true/false
    */
    function TableExists( $sTable, $iTrace = null )
    {
        $fExists = false;

        $sQuery = 'SHOW TABLES LIKE \'' . $sTable . '\'';
        $this->TraceBegin( 'DB: TableExists: Query', $iTrace );
        $this->query( $sQuery );
        $this->TraceEnd( 'DB: TableExists: Query', $sQuery, $iTrace );
        while ( $this->next_record() )
        {
            foreach( $this->Record as $sName )
                if ( $sName == $sTable )
                {
                    $fExists = true;
                    break;
                }
        }
        $this->free();

        return( $fExists );
    }

    /** Creates a database table
        @param $sTable name of table
        @param $aFields array of fieldname = typestring
        @param $sPrimary fieldname of the primary field (optional)
        @return true/false
    */
    function TableCreate( $sTable, $aFields, $sPrimary = null, $aDefault = null, $iTrace = null )
    {
        // create table
        $sQuery = '';
        foreach( $aFields as $sField => $aField )
        {
            if ( $sQuery != '' )
                $sQuery .= ', ';
            $sQuery .= $sField . ' ' . $this->GetFieldType( $aField );

            // Set field collation
            if ( $this->sCollation !== null && $this->sCollation !== '' && $this->sVersion >= 4 )
                $sQuery .= ' COLLATE ' . $this->sCollation;
            // Use default value from module
            if ( $aDefault !== null && isset( $aDefault[ $sField ] ) && $aDefault[ $sField ] !== '' && $aDefault[ $sField ] !== null )
                $sQuery .= ' NOT NULL DEFAULT \'' . mysql_real_escape_string( $aDefault[ $sField ] ) . '\'';

            if ( $sPrimary !== null )
            {
                if ( $sField == $sPrimary )
                    $sQuery .= ' PRIMARY KEY';
            }
            elseif ( $aField[ 'type' ] == 'ID' )
                $sQuery .= ' PRIMARY KEY';
        }
        $sQuery = 'CREATE TABLE ' . $sTable . ' ( ' . $sQuery . ' ) ENGINE = MYISAM';
        if ( $this->sCollation !== null && $this->sCollation !== '' && $this->sVersion >= 4 )
            $sQuery .= ' COLLATE ' . $this->sCollation;
        $this->TraceBegin( 'DB: TableCreate: Query', $iTrace );
        $this->query( $sQuery );
        $this->TraceEnd( 'DB: TableCreate: Query', $sQuery, $iTrace );

        return( true );
    }

    /** Modifies the database table
        @param $sTable name of table
        @param $aFields array of fieldname = typestring
        @return true/false
    */
    function TableAlter( $sTable, $aFields, $fInsert = null, $fModify = null, $fDelete = null, $aDefault = null, $iTrace = null )
    {
        if ( $fInsert === null )
            $fInsert = true;
        if ( $fModify === null )
            $fModify = true;
        if ( $fDelete === null )
            $fDelete = true;

        // get all fields from database table
        $aFieldsDB = array();
        $sQuery = 'SHOW FIELDS FROM ' . $sTable;
        $this->TraceBegin( 'DB: TableAlter: Query', $iTrace );
        $this->query( $sQuery );
        $this->TraceEnd( 'DB: TableAlter: Query', $sQuery, $iTrace );
        while ( $this->next_record() )
            $aFieldsDB[ $this->Record[ 'Field' ] ] = $this->Record;
        $this->free();

        // delete all fields from table which are not in $aFields
        if ( $fDelete )
        {
            foreach( $aFieldsDB as $sField => $aField )
                if ( !isset( $aFields[ $sField ] ) )
                {
                    $sQuery = 'ALTER TABLE ' . $sTable . ' DROP COLUMN `' . $sField . '`';
                    $this->TraceBegin( 'DB: TableAlter: Query', $iTrace );
                    $this->query( $sQuery );
                    $this->TraceEnd( 'DB: TableAlter: Query', $sQuery, $iTrace );
                }
        }

        $sPrevious = '';
        foreach( $aFields as $sField => $aField )
        {
            if ( !isset( $aFieldsDB[ $sField ] ) )
            {
                if ( $fInsert )
                {
                    $sQuery = 'ALTER TABLE ' . $sTable . ' ADD `' . $sField . '` ' . $this->GetFieldType( $aField );
                    // Set field collation
                    if ( $this->sCollation !== null && $this->sCollation !== '' && $this->sVersion >= 4 )
                        $sQuery .= ' COLLATE ' . $this->sCollation;
                    // Use default value from module
                    if ( $aDefault !== null && isset( $aDefault[ $sField ] ) && $aDefault[ $sField ] !== '' && $aDefault[ $sField ] !== null )
                        $sQuery .= ' NOT NULL DEFAULT \'' . mysql_real_escape_string( $aDefault[ $sField ] ) . '\'';
                    if ( $sPrevious != '' )
                        $sQuery .= ' AFTER `' . $sPrevious . '`';
                    $this->TraceBegin( 'DB: TableAlter: Query', $iTrace );
                    $this->query( $sQuery );
                    $this->TraceEnd( 'DB: TableAlter: Query', $sQuery, $iTrace );
                }
            }
            else
            {
                if ( $fModify )
                {
                    $sQuery = 'ALTER TABLE ' . $sTable . ' MODIFY `' . $sField . '` ' . $this->GetFieldType( $aField );
                    // Set field collation
                    if ( $this->sCollation !== null && $this->sCollation !== '' && $this->sVersion >= 4 )
                        $sQuery .= ' COLLATE ' . $this->sCollation;
                    // use default value from module
                    if ( $aDefault !== null && isset( $aDefault[ $sField ] ) && $aDefault[ $sField ] !== '' && $aDefault[ $sField ] !== null )
                        $sQuery .= ' NOT NULL DEFAULT \'' . mysql_real_escape_string( $aDefault[ $sField ] ) . '\'';
                    $this->TraceBegin( 'DB: TableAlter: Query', $iTrace );
                    $this->query( $sQuery );
                    $this->TraceEnd( 'DB: TableAlter: Query', $sQuery, $iTrace );
                }
            }

            $sPrevious = $sField;
        }

        return( true );
    }

    function TableIndex( $sTable, $aFields, $iTrace = null )
    {
        $aPrimary = array();
        $aMultiple = array();
        $aIndex = array();
        $aUnique = array();
        foreach( $aFields as $sField => $aField )
    		switch( $aField[ 'type' ] )
            {
                case 'ID':
                    $aPrimary[] = $sField;
                break;

                case 'MULTIPLEID':
                    $aPrimary[] = $sField;
                    $aMultiple[] = $sField;
                break;

                case 'HIDDENID':
                case 'FOREIGNID':
                case 'FOREIGNNAME':
                    $aIndex[] = $sField;
                break;

                case 'NAME':
                	$aUnique[] = $sField;
               	break;
            }

        // create or update primary key
        if ( count( $aPrimary ) > 0 )
            $this->TableIndexCreate( $sTable, null, 'PRIMARY', $aPrimary );

        // create or update reverse multiple key (as unique) (used in m:n table)
        if ( count( $aMultiple ) > 1 )
        {
            $aMultiple = array_reverse( $aMultiple );
            $this->TableIndexCreate( $sTable, $aMultiple[ 0 ], 'UNIQUE', $aMultiple );
        }

        // add indexes
        foreach( $aIndex as $sField )
            $this->TableIndexCreate( $sTable, $sField, 'INDEX', array( $sField ) );

       	// add unique
        foreach( $aUnique as $sField )
            $this->TableIndexCreate( $sTable, $sField, 'UNIQUE', array( $sField ) );
    }

    function TableIndexCreate( $sTable, $sName, $sType, $aFields, $iTrace = null )
    {
        switch( $sType )
        {
            case 'PRIMARY':
                $sName = 'PRIMARY';
                $sCreate = 'PRIMARY KEY';
                $sDrop = 'PRIMARY KEY';
            break;

            case 'INDEX':
                $sCreate = 'INDEX ' . $sName;
                $sDrop = 'INDEX ' . $sName;
            break;

            case 'UNIQUE':
                $sCreate = 'UNIQUE ' . $sName;
                $sDrop = 'INDEX ' . $sName;
            break;

            case 'FULLTEXT':
                $sCreate = 'FULLTEXT ' . $sName;
                $sDrop = 'INDEX ' . $sName;
            break;

            default:
                die( 'TableIndexCreate: Invalid index type: ' . $sType );
            break;
        }

        // get current index
        if ( $this->sVersion >= 4 )
            // MySQL 4.x up
            $sQuery = 'SHOW INDEX FROM ' . $sTable . ' WHERE key_name = \'' . $sName . '\'';
        else
            // MySQL 3.x
            $sQuery = 'SHOW INDEX FROM ' . $sTable;

        $this->TraceBegin( 'DB: TableIndexCreate: Query', $iTrace );
        $this->query( $sQuery );
        $aResult = $this->Result();
        $this->TraceEnd( 'DB: TableIndexCreate: Query', $sQuery, $iTrace );

        // MySQL 3.x
        if ( $this->sVersion < 4 )
            foreach( array_keys( $aResult ) as $sKey )
                if ( $aResult[ $sKey ][ 'Key_name' ] != $sName )
                    unset( $aResult[ $sKey ] );

        if ( count( $aResult ) > 0 )
        {
            // check, if all fields exist in the index
            $aIndex = ArrayGetColumn( $aResult, 'Column_name' );
            $aCreate = array_diff( $aFields, $aIndex );

            if ( count( $aCreate ) > 0 )
            {
                // drop existing index and create new one
                $sQuery = 'ALTER TABLE ' . $sTable . ' DROP ' . $sDrop . ', ADD ' . $sCreate . ' ( ' . implode( ',', $aFields ) . ' )';
                $this->TraceBegin( 'DB: TableIndexCreate: Query', $iTrace );
                $this->query( $sQuery );
                $this->TraceEnd( 'DB: TableIndexCreate: Query', $sQuery, $iTrace );
            }
        }
        else
        {
            $sQuery = 'ALTER TABLE ' . $sTable . ' ADD ' . $sCreate . ' ( ' . implode( ',', $aFields ) . ' )';
            $this->TraceBegin( 'DB: TableIndexCreate: Query', $iTrace );
            $this->query( $sQuery );
            $this->TraceEnd( 'DB: TableIndexCreate: Query', $sQuery, $iTrace );
        }

        return( true );
    }


    function GetFieldType( $aField )
    {
        return( $this->GetFieldTypeX( $aField[ 'type' ], $aField[ 'values' ] ) );
    }

    function GetFieldTypeX( $sType, $sValues = null )
    {
		switch( $sType )
        {
            // normal primary key
            case 'ID':
                return( 'INT( 10 ) UNSIGNED NOT NULL AUTO_INCREMENT' );
            break;

            // used in m:n relations as primary keys
            // are not allowed together with an ID
            case 'MULTIPLEID':
                return( 'INT( 10 ) UNSIGNED NOT NULL' );
            break;

            case 'FOREIGNID':
            case 'HIDDENID':
                return( 'INT( 10 ) UNSIGNED' );
            break;

            case 'NAME':
            case 'FOREIGNNAME':
                return( 'VARCHAR( 255 )' );
            break;

            case 'TEXT':
                return( 'VARCHAR( 255 )' );
            break;

            case 'TEXTBOX':
                return( 'TEXT' );
            break;

            case 'IP':
                return( 'VARCHAR( 15 )' );
            break;

            case 'CHECKBOX':
                return( 'ENUM( \'y\', \'n\' )' );
            break;

            case 'RADIO':
                if ( !is_array( $sValues ) )
                    die( 'RADIO: values missing' );
                return( "ENUM( '" . implode( "','", $sValues ) . "' )" );
            break;

            case 'YESNO':
                return( 'ENUM( \'y\', \'n\' )' );
            break;

            case 'TRISTATE':
                return( 'ENUM( \'y\', \'n\' )' );
            break;

            case 'DISPLAY':
                if ( is_array( $sValues ) )
                    return( "ENUM( '" . implode( "','", $sValues ) . "' )" );
                else
                    return( 'ENUM( \'y\', \'n\' )' );
                break;

            case 'LANGUAGE':
                return( 'ENUM( \'de\', \'en\' )' );
            break;

            case 'GENDER':
                return( 'ENUM( \'m\', \'f\' )' );
            break;

            case 'PASSWORD':
                return( 'VARCHAR( 32 )' );
            break;

			case 'CREDITCARDNUMBER':
                die( 'getfieldtype: ' . $sType );
			break;

			case 'CREDITCARDDATE':
                die( 'getfieldtype: ' . $sType );
			break;

			case 'USERNAME':
                return( 'VARCHAR( 32 )' );
			break;

			case 'EMAIL':
                return( 'VARCHAR( 255 )' );
			break;

            case 'URL_TYP':
                return( 'VARCHAR( 255 )' );
            break;

			case 'URL':
                return( 'VARCHAR( 512 )' );
			break;

            case 'TARGET':
                return( 'VARCHAR( 255 )' );
			break;

			case 'DATE':
			    return( 'DATE' );
			break;

			case 'TIME':
			    return( 'TIME' );
			break;

			case 'DATETIME':
			    return( 'DATETIME' );
			break;

		    case 'PERCENT':
                return( 'FLOAT' );
	        break;

		    case 'INTEGER':
                return( 'INT( 10 )' );
	        break;

		    case 'FLOAT':
		    case 'REAL':
                return( 'FLOAT' );
	        break;

		    case 'DOUBLE':
                return( 'DOUBLE' );
	        break;

		    case 'CURRENCY':
                return( 'DOUBLE' );
	        break;

		    case 'IMAGE':
                return( 'VARCHAR( 255 )' );
	        break;

		    case 'COUNTRY':
                return( 'VARCHAR( 255 )' );
	        break;

            case 'FILE':
                return( 'VARCHAR( 255 )' );
            break;

            case 'UPLOAD':
                die( 'getfieldtype: ' . $sType );
            break;

            case 'SELECT':
                return( 'VARCHAR( 255 )' );
            break;

            default:
                die( 'getfieldtype: ' . $sType );
            break;
        }
    }
}
