<?php

defined( 'oPage' ) or die();

include_once $sRoot . 'include/debug.php';

function ErrorHandler( $iNumber, $sError, $sFilename, $sLine, $aVar = null )
{
    if ( $iNumber == E_NOTICE )
    {
        // continue on 'Undefined index' notice
        if ( substr( $sError, 0, 16 ) == 'Undefined index:' )
            return;

        // continue on 'Undefined offset' notice
        if ( substr( $sError, 0, 17 ) == 'Undefined offset:' )
            return;

        // continue on 'Undefined property' notice
        if ( substr( $sError, 0, 19 ) == 'Undefined property:' )
            return;
    }

    echo ErrorReport(  $iNumber, $sError, $sFilename, $sLine, $aVar );

    if ( $iNumber & ( E_ERROR | E_CORE_ERROR | E_COMPILE_ERROR | E_USER_ERROR ) )
    {
        ob_end_flush();

        Debug();
        print_r ( $aVar );
        die();
    }
}

function ErrorReport( $iNumber, $sError, $sFilename, $sLine, $aVar = null )
{
    switch( $iNumber )
    {
        case E_ERROR:
            $sLevel = 'Error';
        break;

        case E_WARNING:
            $sLevel = 'Warning';
        break;

        case E_NOTICE:
            $sLevel = 'Notice';
        break;

        default:
            $sLevel = 'Error ' . $iNumber;
        break;
    }

    $sBuffer = $sLevel . ': ' . $sError . "\n";
    $sBuffer .= $sFilename . ' Line ' . $sLine . "\n";
    $sBuffer .= "<br />\n";

    return( $sBuffer );
}

function ErrorHandlerFatal( $sBuffer )
{
    if ( preg_match( '(error</b>: (.+) in (.+) on line (.+)<br)', $sBuffer, $aRegs )  )
    {
        $sBuffer .= ErrorReport( E_ERROR, strip_tags( $aRegs[ 1 ] ), strip_tags( $aRegs[ 2 ] ), strip_tags( $aRegs[ 3 ] ) );
    }

    return( $sBuffer );
}

// display errors
ini_set( 'display_errors', '1' );

// no prepend and append string (maybe interferes with ErrerHandlerFatal
ini_set( 'error_prepend_string', null );
ini_set( 'error_append_string', null );

// no error log (may be changed later to something usefull)
// ini_set( 'error_log', null ); -> does not work with open_basedir restriction in effect.
ini_set( 'log_errors', '0' );

// config error reporting
ini_set( 'track_errors', '0' );
ini_set( 'html_errors', '0' );
ini_set( 'ignore_repeated_errors', '1' );
ini_set( 'ignore_repeated_source', '1' );
ini_set( 'warn_plus_overloading', '1' );

// set the error reporting level for this script
// error_reporting( E_USER_ERROR | E_USER_WARNING | E_USER_NOTICE );
// error_reporting( E_ALL | E_STRICT );
// error_reporting( E_ALL & ~E_NOTICE );
error_reporting( E_ALL );

// set to the user defined error handler
set_error_handler( 'ErrorHandler' );

// start output buffering for fatal errors
ob_start( 'ErrorHandlerFatal' );
