/*
 * form.js 0.6
 *
 * Functions to evaluate form input
 * Released as part of the oPage Web Content Management Framework
 * http://www.opage.at
 *
 * Released under the GNU GPL license
 * Author: Hannes Dorn <hannes@dorn.cc>
 *
 * Depends on: system.js
 *
 */

function FormSubmit( oElement )
{
    if ( oElement.form.onsubmit )
        if ( !oElement.form.onsubmit() )
            return( false );

    oElement.form.elements[ 'action' ].value = oElement.name;
    oElement.form.submit();

    return( true );
}

function FormTrim( oForm )
{
	for ( i = 0; i < oForm.elements.length; i++ )
        oForm.elements[ i ].value = oForm.elements[ i ].value.replace( /\s+$/, '' ).replace( /^\s+/, '' )
}

function FormCheckText( oField, sError )
{
    if ( oField.value != '' )
        return( true );

    if ( sError )
    {
        alert( sError );
        oField.focus();
    }

    return( false );
}

function FormTextCounter( oField, oFieldCounter, iMax )
{
    if ( oField.value.length > iMax )
        oField.value = oField.value.substr( 0, iMax );
    oFieldCounter.value = iMax - oField.value.length;
}

function FormCheckNumber( oField, sError )
{
    fError = false;
    if ( oField.value != '' )
    {
        for( i = 0; i < oField.value.length; i++ )
        {
            sChar = oField.value.charAt( i );
            if ( sChar < '0' || sChar > '9' )
                fError = true;
        }

        if ( fError == false )
            return( true );
    }

    if ( sError )
    {
        alert( sError );
        oField.focus();
    }

    return( false );
}

function FormCheckInteger( oField, sError )
{
    if ( oField.value == '' )
        return( true );

    // trim leading 0
    oField.value = oField.value.replace( /^0*/, '' );
    if ( oField.value == '' )
        oField.value = '0';

    if ( !isNaN( oField.value ) )
        if ( parseInt( oField.value ) == oField.value )
            return( true );

    if ( sError )
    {
        alert( sError );
        oField.focus();
    }

    return( false );
}

function FormCheckFloat( oField, sError )
{
    if ( oField.value == '' )
        return( true );

    // remove .
    oField.value = oField.value.replace( /\./, '' );

    // trim leading 0
    oField.value = oField.value.replace( /^0*/, '' );

    // trim 0 after ,
    if ( oField.value.indexOf( ',' ) >= 0 )
        oField.value = oField.value.substring( 0, oField.value.indexOf( ',' ) + 1 ) + oField.value.substring( oField.value.indexOf( ',' ) + 1, oField.value.length ).replace( /0*$/, '' );

    if ( oField.value == '' || oField.value.substring( 0, 1 ) == ',' )
        oField.value = '0' + oField.value;

    sValue = oField.value;
    sValue = sValue.replace( /\./, '' );
    sValue = sValue.replace( /,/, '.' );

    if ( !isNaN( sValue ) )
    {
        sValue = String( parseFloat( sValue ) );
        sValue = sValue.replace( /\./, ',' );

        if ( sValue == oField.value )
            return( true );
    }

    if ( sError )
    {
        alert( sError );
        oField.focus();
    }

    return( false );
}

function FormCheckCheckbox( oField, sError )
{
    if ( oField.length )
    {
        for( i = 0; i < oField.length; ++i )
            if ( oField[ i ].type == 'checkbox' )
                if ( oField[ i ].checked )
                    return( true );
    }
    else
        if ( oField.checked )
            return( true );

    if ( sError )
    {
        alert( sError );
        for( i = 0; i < oField.length; ++i )
            if ( oField[ i ].type == 'checkbox' )
            {
                oField[ i ].focus();
                break;
            }
    }

    return( false );
}

/* Returns true, if one checkbox is checked */
function FormCheckCheckboxes( oForm, sName )
{
	for ( i = 0; i < oForm.elements.length; i++ )
	    if ( oForm.elements[ i ].name.substr( 0, sName.length ) == sName && oForm.elements[ i ].type == 'checkbox' )
	        if ( FormCheckCheckbox( oForm.elements[ i ] ) )
	            return( true );
    return( false );
}

function FormSetCheckbox( oField, fChecked )
{
    if ( oField.length )
    {
        for( i = 0; i < oField.length; ++i )
            if ( oField[ i ].type == 'checkbox' )
                oField[ i ].checked = fChecked;
    }
    else
        oField.checked = fChecked;
}

function FormSetCheckboxes( oForm, sName, fChecked )
{
	for ( i = 0; i < oForm.elements.length; i++ )
	    if ( oForm.elements[ i ].name.substr( 0, sName.length ) == sName && oForm.elements[ i ].type == 'checkbox' )
	        FormSetCheckbox( oForm.elements[ i ], fChecked )
}

function FormCheckEmail( oField, sError )
{
    var fRegex = false;
    var fReturn = false;

    if( typeof( RegExp ) == 'function' )
    {
        var oRegex = new RegExp( 'abc' );
        if( oRegex.test( 'abc' ) == true )
        {
            fRegex = true;
        }
    }

    if ( fRegex == true )
    {
        var oRegex = new RegExp( '^([a-zA-Z0-9\\-\\.\\_]+)(\\@)([a-zA-Z0-9\\-\\.]+)(\\.)([a-zA-Z]{2,4})$' );
        if ( oRegex.test( oField.value ) )
            return( true );

        var oRegex = new RegExp( '^([a-zA-Z0-9\\-\\.\\_]+)(\\@)localhost$' );
        if ( oRegex.test( oField.value ) )
            return( true );
    }
    else
    {
        var sEmail = oField.value;
        if ( sEmail.search('@') >= 1 && sEmail.lastIndexOf('.') > sEmail.search('@') && sEmail.lastIndexOf('.') >= sEmail.length - 5 )
            return( true );
    }

    if ( sError )
    {
        alert( sError );
        oField.focus();
    }

    return( false );
}

function FormCheckListbox( oField, sError, iMin, iMax, sNothing )
{
    iSelected = 0;

    for( i = 0; i < oField.length; ++i )
        if ( oField.options[i].selected == true && oField.options[i].value != sNothing )
            iSelected++;

    if ( ( iSelected >= iMin || iMin == -1 ) && ( iSelected <= iMax || iMax == -1 ) && iSelected > 0 )
        return( true );

    if ( sError )
    {
        alert( sError );
        oField.focus();
    }

    return( false );
}

function FormCheckDropdown( oField, sError, sNothing )
{
    return( FormCheckListbox( oField, sError, 1, 1, sNothing ) );
}

function FormCheckDate( iDay, iMonth, iYear )
{
	iMonth = iMonth - 1;

	var oDate = new Date( iYear, iMonth, iDay );

	if ( ( oDate.getFullYear() != iYear ) || ( oDate.getMonth() != iMonth ) || ( oDate.getDate() != iDay ) )
		return false;

	return( true );
}

function FormCheckRadio( oField, sError )
{
    for( i = 0; i < oField.length; ++i )
        if ( oField[ i ].type == 'radio' )
            if ( oField[ i ].checked )
                return( true );

    if ( sError )
    {
        alert( sError );
        for( i = 0; i < oField.length; ++i )
            if ( oField[ i ].type == 'radio' )
            {
                oField[ i ].focus();
                break;
            }
    }

    return( false );
}

var sQuickselectName;
var sQuickselectValue;
var sQuickselectOnBlur;
var sQuickselectOnChange;

function FormQuickselect( sName, oEvent, iWidth, iOffsetLeft, iOffsetTop, sOnBlur, sOnChange, sPrefix )
{
	var fExact;

    if ( sPrefix == null )
        sPrefix = '';
    else
        sPrefix = sPrefix + '-';

    var oIs = SystemIs();
	var oSelNode = document.getElementById( sPrefix + sName );

    if ( iOffsetLeft != null && iOffsetTop != null )
        fExact = true;
    else
        fExact = false;

    if ( iWidth == null )
        iWidth = 50;
    if ( iOffsetLeft == null )
        iOffsetLeft = 53;
    if ( iOffsetTop == null )
        iOffsetTop = 38;
    else
        iOffsetTop += 16;

    sQuickselectName = sPrefix + sName;
    sQuickselectValue = oSelNode.value;
    sQuickselectOnBlur = sOnBlur;
    sQuickselectOnChange = sOnChange;

	if
    (
        ( oEvent.keyCode >= 48 && oEvent.keyCode <= 57 )
        || ( oEvent.keyCode >= 65 && oEvent.keyCode <= 90 )
        || oEvent.keyCode == 59 // Umlaut u
        || oEvent.keyCode == 192 // Umlaut 
        || oEvent.keyCode == 219 // Scharfes s
        || oEvent.keyCode == 222 // Umlaut a
    )
	{
        var oPosition;
        var oInput;

        oPosition = document.getElementById( sQuickselectName + '-quickselect_position' );
        if ( !oPosition )
            oPosition = document.getElementById( sQuickselectName + '#quickselect_position' );

        oInput = document.getElementById( sQuickselectName + '-quickselect_input' );
        if ( !oInput )
    		oInput = document.getElementById( sQuickselectName + '#quickselect_input' );

		if ( oInput == null )
		{
			var oNewNode;
			var oNode;

			oNewNode = document.createElement( 'INPUT' );
			oNewNode.type = 'text';
			oNewNode.id = sQuickselectName + '-quickselect_input';
			oNewNode.style.position = 'absolute';
			oNewNode.style.width = iWidth + 'px';
			oNewNode.style.height = 18 + 'px';
			oNewNode.style.paddingLeft = '0px';
			oNewNode.style.paddingRight = '0px';
			oNewNode.style.zIndex = 1;
			oNewNode.className = 'quickselect';

            oNode = document.getElementById( sQuickselectName + '-quickselect' );
            if ( !oNode )
                oNode = document.getElementById( sQuickselectName + '#quickselect' );

			oInput = oNode.appendChild ( oNewNode );
		}

		if ( fExact )
		{
            oInput.style.top = ( FormQuickselectGetTop( oPosition ) - iOffsetTop ) + 'px';
            oInput.style.left = ( FormQuickselectGetLeft( oPosition ) - iOffsetLeft ) + 'px';
        }
        else if ( FormQuickselectGetLeft( oPosition ) - iOffsetLeft > 0 && iOffsetLeft != 0 )
        {
            oInput.style.top = ( FormQuickselectGetTop( oPosition ) - 14 ) + 'px';
            oInput.style.left = ( FormQuickselectGetLeft( oPosition ) - iOffsetLeft ) + 'px';
        }
        else
        {
            oInput.style.top = ( FormQuickselectGetTop( oPosition ) - iOffsetTop ) + 'px';
            oInput.style.left = ( FormQuickselectGetLeft( oPosition ) + 1 ) + 'px';
        }

        oInput.value = '';
        oInput.style.visibility = 'visible';
        oInput.focus();

	    if ( oIs.is_ie6up )
	    {
    		if ( oEvent.keyCode > 90 )
    		{
    			switch ( oEvent.keyCode )
    			{
    				case 186:
    					// 
    					oInput.value = String.fromCharCode ( 252 );
                    break;

    				case 192:
    					// 
    					oInput.value = String.fromCharCode ( 246 );
                    break;

    				case 222:
    					// 
    					oInput.value = String.fromCharCode ( 228 );
                    break;
    			}
    		}
    		else
    		    oInput.value = String.fromCharCode( oEvent.keyCode ).toLowerCase();
        }

		oInput.onkeyup = FormQuickselect_KeyUp;
		oInput.onkeydown = FormQuickselect_KeyDown;
		oInput.onblur = FormQuickselect_onBlur;

		FormQuickselect_KeyUp( oEvent );
		oEvent.returnValue = false;

	    if ( oIs.is_opera )
            oInput.focus();

		return( false );
	}

	return( true );
}

function FormQuickselectGetTop( oObject )
{
    // Function to calculate position of an element.
    var iAmount;

    var sPosition;

    iAmount = oObject.offsetTop + oObject.offsetHeight;
    oObject = oObject.offsetParent;
    while ( oObject )
    {
        if ( window.getComputedStyle )
            sPosition = window.getComputedStyle( oObject, '' ).getPropertyValue( 'position' );
        else if ( oObject.currentStyle )
            sPosition = oObject.currentStyle.position;
        else if ( document.all )
            sPosition = oObject.style.position;

        if ( sPosition == 'absolute' )
            break;

        iAmount += oObject.offsetTop;
        oObject = oObject.offsetParent;
    }

    return( iAmount );
}

function FormQuickselectGetLeft( oObject )
{
    // Function for IE to calculate position of an element.
    var iAmount;

    iAmount = oObject.offsetLeft;
    oObject = oObject.offsetParent;
    while ( oObject )
    {
        if ( window.getComputedStyle )
            sPosition = window.getComputedStyle( oObject, '' ).getPropertyValue( 'position' );
        else if ( oObject.currentStyle )
            sPosition = oObject.currentStyle.position;
        else if ( document.all )
            sPosition = oObject.style.position;

        if ( sPosition == 'absolute' )
            break;

        iAmount += oObject.offsetLeft;
        oObject = oObject.offsetParent;
    }
    return( iAmount );
}

function FormQuickselect_KeyUp( oEvent )
{
    var sName = sQuickselectName;
	var oTxtNode;

    oTxtNode = document.getElementById( sName + '-quickselect_input' );
	if ( !oTxtNode )
    	oTxtNode = document.getElementById( sName + '#quickselect_input' );

	var oSelNode = document.getElementById( sName );
	var sText;
	var i;

	sText = oTxtNode.value.toLowerCase();

	if ( !oEvent && document.all )
	    oEvent = event;

	if
    (
        ( oEvent.keyCode >= 48 && oEvent.keyCode <= 57 )
        || ( oEvent.keyCode >= 65 && oEvent.keyCode <= 90 )
        || oEvent.keyCode == 8
        || oEvent.keyCode == 59 // Umlaut u
        || oEvent.keyCode == 192 // Umlaut o
        || oEvent.keyCode == 219 // Scharfes s
        || oEvent.keyCode == 222 // Umlaut a
    )
	{
		if ( oSelNode.options.length > 0 )
		{
			for ( i = 0; i < oSelNode.options.length; i++ )
			{
				if ( sText == oSelNode.options[ i ].text.substr( 0, sText.length ).toLowerCase() )
				{
					oSelNode.selectedIndex = i;
					break ;
				}
			}
		}
	}
}

function FormQuickselect_KeyDown( oEvent )
{
    var sName = sQuickselectName;
	var oSelNode = document.getElementById( sName );

	if ( document.all )
	    oEvent = event;

	if ( oEvent.keyCode == 38 )
	{
		// cursor up!
		oSelNode.selectedIndex -= 1;
		if ( oSelNode.selectedIndex < 0 )
			oSelNode.selectedIndex = 0;
	}
	else if ( oEvent.keyCode == 40 )
        // cursor down!
		oSelNode.selectedIndex += 1;

	return( true );
}

function FormQuickselect_onBlur()
{
	var oTxtNode;

    oTxtNode = document.getElementById( sQuickselectName + '-quickselect_input' );
	if ( !oTxtNode )
    	oTxtNode = document.getElementById( sQuickselectName + '#quickselect_input' );

	oTxtNode.style.visibility = 'hidden';

    if ( sQuickselectOnBlur != null )
        eval( sQuickselectOnBlur );

	var oSelNode = document.getElementById( sQuickselectName );

    if ( sQuickselectOnChange != null && sQuickselectValue != oSelNode.value )
        eval( sQuickselectOnChange );

    if ( oSelNode.onchange != null && sQuickselectValue != oSelNode.value )
        oSelNode.onchange();
}

//
// Liefert den ausgewhlten Wert einer Radio Button Gruppe
// written by Hannes Dorn
//
function FormRadioGetSelected( Ctrl )
{
    for( i = 0; i < Ctrl.length; ++i )
        if ( Ctrl[ i ].checked )
            return( Ctrl[ i ].value );

    return( '' );
}

//
// Liefert den ersten ausgewhlten Wert einer Option Box
// written by Hannes Dorn
//
function FormListboxGetSelected( oCtrl )
{
    for( i = 0; i < oCtrl.length; ++i )
        if ( oCtrl.options[i].selected == true )
            return( oCtrl.options[i].value );

    return( '' );
}


















//
// Setzt den Wert einer Option Box auf selected
// written by Hannes Dorn
//
function DropdownSetSelected( Ctrl, Value )
{
    for( i = 0; i < Ctrl.length; ++i )
        if ( Ctrl.options[i].value == Value )
        {
            Ctrl.options[i].selected = true;
            return( true );
        }

    return( false );
}
