<?php

defined( 'oPage' ) or die();

class CParameter extends CModul
{
    /** Array, in dem die Key=Value Wertpaare gespeichert werden
    */
    var $aValues = array();

    /** Constructor
        Initialisiert zuerst die Basisklasse.
        Setzt den Namen der Tabelle
    */
    function CParameter( $sName = null )
    {
        $this->SetDefault( 'parameter' );

        // Zuerst die Basisklasse initialisieren
        $this->CModul( $sName );

        // Sortierung
        $this->SetOrder( 'name' );

        $this->SetFieldUseAdminList( 'id', true );
        $this->SetFieldUseAdminList( 'name', true );
        $this->SetFieldUseAdminList( 'value', true );
        $this->SetFieldUseAdminList( 'title', true );
    }

    /** Initialisiert die Liste mit den Feldern
        Folgende Felder werden untersttzt:
        DATA.id
        DATA.name
        DATA.value
        DATA.title (mehrsprachig)
        DATA.description (mehrsprachig)
        return nix
    */
    function SetFields()
    {
        $this->SetField( 'id', 'ID', Null, 'parameter_id' );
        $this->SetField( 'name' );
        $this->SetField( 'value', Null, true );
        $this->SetField( 'title', Null, true );
        $this->SetField( 'description', 'TEXTBOX', true );

        CModul::SetFields();
    }

    function AdminListProperties( &$oAdmin, $sName )
    {
        global $oParameter;

        parent::AdminListProperties( $oAdmin, $sName );

        switch( $sName )
        {
            case 'name':
                $oAdmin->SetWidth( $sName, 40 );
            break;

            case 'value':
                $oAdmin->SetWidth( $sName, 20 );
            break;

            case 'title':
                $oAdmin->SetWidth( $sName, 40 );
            break;
        }
    }

    function AdminProperties( &$oAdmin, $sName )
    {
        global $oParameter;

        parent::AdminProperties( $oAdmin, $sName );

        switch( $sName )
        {
            case 'name':
                $oAdmin->SetWidth( $sName, 40 );
            break;

            case 'title':
                $oAdmin->SetWidth( $sName, 135 );
                break;

            case 'value':
                $oAdmin->SetWidth( $sName, 135 );
                break;

            case 'description':
                $oAdmin->SetWidth( $sName, 135 );
                $oAdmin->SetHeight( $sName, 5 );
                break;
        }
    }

    /** Liest alle Datenstze aus der Datenbank und kopiert die Daten um.
        Damit die Parameter ber den Namen schneller identifiziert werden knnen,
        wird das Result-Array traversiert.
        @return nix
    */
	function QueryData( $fAppend = Null, $aData = null )
    {
        // read all parameters
        CModul::QueryData();

        // init value array
        $this->aValues = array();

        // copy all entries
        foreach( $this->aResult as $sKey => $aRow )
            $this->aValues[ $aRow[ 'name' ] ] = $aRow[ 'value' ];
    }

    /** Liefert zum angegebenen Name Key den Parameter. Ist der Parameter nicht gesetzt, wird der Defaultwert retourniert.
        @param $sName Name des Parameters
        @param $sDefault Defaultwert fr den Parameter
        @return String Wert des Parameters
    */
    function Get( $sName, $sDefault = null )
    {
        if ( $sDefault === null )
            $sDefault = '';

        if ( !IsKey( $this->aValues, $sName ) )
        {
            // Wenn's den Parameter noch nicht gibt, dann in die Datenbank einfgen
            $iCount = $this->Count( $this->GetField( 'name' ) . "='$sName'" );
            if ( $iCount == 0 )
            {
                $aRow[ 'name' ] = $sName;
                $aRow[ 'value' ] = '';
                $this->Insert( $aRow );
            }
        }

        $sValue = $this->aValues[ $sName ];

        if ( IsNull( $sValue ) )
            $sValue = $sDefault;

        return( $sValue );
    }

    /** Aktualisiert einen Wert
        Ist der Parameter in der Datenbank noch nicht vorhanden, wird er eingefgt.
        @param $sName Name des Parameters
        @param $sValue Wert fr den Parameter
        @return true/false
    */
    function Set( $sName, $sValue )
    {
        $aRow[ 'name' ] = $sName;
        $aRow[ 'value' ] = $sValue;

        // Wenn's den Parameter noch nicht gibt, dann in die Datenbank einfgen
        $iCount = $this->Count( $this->GetField( 'name' ) . "='$sName'" );
        if ( $iCount > 0 )
            $this->Update( $aRow, 'name' );
        else
            $this->Insert( $aRow );

        $this->aValues[ $sName ] = $sValue;

        return( true );
    }

    /** Updates a value in all languages
        @param $sName Name of the parameter
        @param $sValue Value of the parameter
        @return true/false
    */
    function SetAll( $sName, $sValue )
    {
        global $oApp;

        $aRow[ 'name' ] = $sName;
        $aRow[ 'value' ] = $sValue;

        // Save current language postfix
        $sPostfix = $this->GetLanguagePostfix();

        // Check, if a record exists
        $iCount = $this->Count( $this->GetField( 'name' ) . "='$sName'" );

        // Loop through each content language
        $iMax = $oApp->GetLanguageCount( true );
        for( $i = 0; $i < $iMax; $i++ )
        {
            // Get language number
            $iLanguage = $oApp->GetLanguageIdByIndex( $i, true );

            // Set language postfix
            $this->SetLanguagePostfix( $oApp->GetLanguagePostfix( $iLanguage, true ), true );

            // Insert or update the record
            if ( $iCount > 0 )
                $this->Update( $aRow, 'name' );
            else
                $this->Insert( $aRow );
        }

        // Restore original language postfix
        $this->SetLanguagePostfix( $sPostfix );

        // Update new value in memory
        $this->aValues[ $sName ] = $sValue;

        return( true );
    }
}
