<?php

defined( 'oPage' ) or die();

class CNumber
{
    /** Name of the charset, default iso-8859-1
        windows-1252
        iso-8859-1
        utf-8
    */
    var $sCharset = 'iso-8859-1';

    /** Current language
    */
    var $sLanguage;

    /** Constructor
        @param $sLanguage de, en
        @return nix
    */
    function CNumber( $sLanguage = null, $sCharset = null )
    {
        $this->SetLanguage( $sLanguage );

        if ( $sCharset !== null )
            $this->sCharset = $sCharset;
    }

    /** Formatiert den Wert nach dem angegeben Format
        @param $iValue Wert, der formatiert werden soll
        @param $iFormat Integer Wert fr das Format, Default = 0, kann aber auch ein sprinft() Format String sein
        @return String

    */
    function Format( $dValue, $Format = 0 )
    {
        // PHP does bank rounding
        // round( 1.005, 2 ) = 1.00
        $dValue += 0.00001;

		if ( is_integer( $Format ) )
        {
            switch( $Format )
            {
                // 1.100,00 bzw. 1,100.00
                case 0:
                    switch( $this->sLanguage )
                    {
                        case 'en':
                            $sBuffer = number_format( $dValue, 2, '.', ',' );
                        break;

                        default:
                            $sBuffer = number_format( $dValue, 2, ',', '.' );
                        break;
                    }
                break;

                // 1.100 bzw. 1,100
                case 1:
                    switch( $this->sLanguage )
                    {
                        case 'en':
                            $sBuffer = number_format( $dValue, 0, '.', ',' );
                        break;

                        default:
                            $sBuffer = number_format( $dValue, 0, ',', '.' );
                        break;
                    }
                break;

                // 1.100,0000 bzw. 1,100.0000
                case 2:
                    switch( $this->sLanguage )
                    {
                        case 'en':
                            $sBuffer = number_format( $dValue, 4, '.', ',' );
                        break;

                        default:
                            $sBuffer = number_format( $dValue, 4, ',', '.' );
                        break;
                    }
                break;

                // 1100.00
                case 3:
                    $sBuffer = number_format( $dValue, 2, '.', '' );
                break;

                // 1100.0000
                case 4:
                    $sBuffer = number_format( $dValue, 4, '.', '' );
                break;

                // 1100,00 bzw. 1100.00
                case 5:
                    switch( $this->sLanguage )
                    {
                        case 'en':
                            $sBuffer = number_format( $dValue, 2, '.', '' );
                        break;

                        default:
                            $sBuffer = number_format( $dValue, 2, ',', '' );
                        break;
                    }
                break;

                // 10
                case 6:
                    $sBuffer = number_format( $dValue, 0, '', '' );
                break;

                // 0.1 -> 10 (for percent)
                case 7:
                    $sBuffer = number_format( $dValue * 100, 0, '', '' );
                break;

                // 1.100,000 bzw. 1,100.000
                case 8:
                    switch( $this->sLanguage )
                    {
                        case 'en':
                            $sBuffer = number_format( $dValue, 3, '.', ',' );
                        break;

                        default:
                            $sBuffer = number_format( $dValue, 3, ',', '.' );
                        break;
                    }
                break;

                // 0.1234 -> 10.34 (for percent)
                case 9:
                    $sBuffer = number_format( $dValue * 100, 2, '.', '' );
                break;

                // 1.100 bzw. 1,100
                case 10:
                    switch( $this->sLanguage )
                    {
                        case 'en':
                            $sBuffer = number_format( $dValue, 0, '.', ',' ) . '.-';
                        break;

                        default:
                            $sBuffer = number_format( $dValue, 0, ',', '.' ) . ',-';
                        break;
                    }
                break;

                default:
                    die( 'CNumber: Format: Ungltiges Format' );
                break;
            }
        }
        else
            $sBuffer = sprintf( $Format, $dValue );

        // If charset is utf-8, convert template from Windows to UTF-8
        if ( $this->sCharset == 'utf-8' )
            $sBuffer = StringWindowsToUtf8( $sBuffer );

        return( $sBuffer );
    }

    /** Sets the language
    	@param $sLanguage short text for the language: {de|en}, default de
    */
    function SetLanguage( $sLanguage = null )
    {
        if ( $sLanguage === null || $sLanguage === '' )
        {
            $sLanguage = 'de';
        }

        $this->sLanguage = $sLanguage;
    }
}

if ( !function_exists( 'StringWindowsToUtf8' ) )
{
    /*
        This structure encodes the difference between ISO-8859-1 and Windows-1252,
        as a map from the UTF-8 encoding of some ISO-8859-1 control characters to
        the UTF-8 encoding of the non-control characters that Windows-1252 places
        at the equivalent code points.
        Aidan Kehoe <php-manual@parhasard.net>
    */
    global $aCp1252ToUtf8;
    $aCp1252ToUtf8 = array
    (
        "\xc2\x80" => "\xe2\x82\xac", /* EURO SIGN */
        "\xc2\x82" => "\xe2\x80\x9a", /* SINGLE LOW-9 QUOTATION MARK */
        "\xc2\x83" => "\xc6\x92",     /* LATIN SMALL LETTER F WITH HOOK */
        "\xc2\x84" => "\xe2\x80\x9e", /* DOUBLE LOW-9 QUOTATION MARK */
        "\xc2\x85" => "\xe2\x80\xa6", /* HORIZONTAL ELLIPSIS */
        "\xc2\x86" => "\xe2\x80\xa0", /* DAGGER */
        "\xc2\x87" => "\xe2\x80\xa1", /* DOUBLE DAGGER */
        "\xc2\x88" => "\xcb\x86",     /* MODIFIER LETTER CIRCUMFLEX ACCENT */
        "\xc2\x89" => "\xe2\x80\xb0", /* PER MILLE SIGN */
        "\xc2\x8a" => "\xc5\xa0",     /* LATIN CAPITAL LETTER S WITH CARON */
        "\xc2\x8b" => "\xe2\x80\xb9", /* SINGLE LEFT-POINTING ANGLE QUOTATION */
        "\xc2\x8c" => "\xc5\x92",     /* LATIN CAPITAL LIGATURE OE */
        "\xc2\x8e" => "\xc5\xbd",     /* LATIN CAPITAL LETTER Z WITH CARON */
        "\xc2\x91" => "\xe2\x80\x98", /* LEFT SINGLE QUOTATION MARK */
        "\xc2\x92" => "\xe2\x80\x99", /* RIGHT SINGLE QUOTATION MARK */
        "\xc2\x93" => "\xe2\x80\x9c", /* LEFT DOUBLE QUOTATION MARK */
        "\xc2\x94" => "\xe2\x80\x9d", /* RIGHT DOUBLE QUOTATION MARK */
        "\xc2\x95" => "\xe2\x80\xa2", /* BULLET */
        "\xc2\x96" => "\xe2\x80\x93", /* EN DASH */
        "\xc2\x97" => "\xe2\x80\x94", /* EM DASH */
        "\xc2\x98" => "\xcb\x9c",     /* SMALL TILDE */
        "\xc2\x99" => "\xe2\x84\xa2", /* TRADE MARK SIGN */
        "\xc2\x9a" => "\xc5\xa1",     /* LATIN SMALL LETTER S WITH CARON */
        "\xc2\x9b" => "\xe2\x80\xba", /* SINGLE RIGHT-POINTING ANGLE QUOTATION*/
        "\xc2\x9c" => "\xc5\x93",     /* LATIN SMALL LIGATURE OE */
        "\xc2\x9e" => "\xc5\xbe",     /* LATIN SMALL LETTER Z WITH CARON */
        "\xc2\x9f" => "\xc5\xb8"      /* LATIN CAPITAL LETTER Y WITH DIAERESIS*/
    );

    function StringWindowsToUtf8( $sBuffer )
    {
       global $aCp1252ToUtf8;
       return( strtr( utf8_encode( $sBuffer ), $aCp1252ToUtf8 ) );
    }
}
