<?php

define( 'oPage', 1 );

global $sRoot;
global $oFactory;
global $oHook;
global $oApp;
global $oPlugins;
global $oDb;

$sRoot = dirname( dirname( __FILE__ ) ) . '/';

include_once $sRoot . 'include/tools.php';

// Set memory limit to 128 MB if it's lower
if ( Something2Byte( ini_get( 'memory_limit' ) ) < 134217728 )
{
    ini_set( 'memory_limit', '128M' );
}

// include custom config
include_once $sRoot . 'custom/config.php';

// create application object
include_once $sRoot . 'include/app.php';
$oApp = new CApp( $aConfig );

// unset config
unset( $aConfig );

// Create Factory
include_once $sRoot . 'include/factory.php';
if ( file_exists( $sRoot . 'custom/factory.php' ) )
{
    include_once $sRoot . 'custom/factory.php';
    $oFactory = New CFactoryX();
}
else
{
    $oFactory = New CFactory();
}

// Create the global hook system
$oHook = $oFactory->Hook();

// Set default charset
ini_set( 'default_charset', $oApp->GetCharset() );

// Set charset in header
header( 'Content-Type: text/html; charset=' . $oApp->GetCharset() );
header( 'X-UA-Compatible: IE=edge,chrome=1' );
header( 'imagetoolbar: no' );

// Set locale
// For windows codes see
// Country/Region Strings: http://msdn.microsoft.com/en-us/library/cdax410z%28VS.71%29.aspx
// Language Strings: http://msdn.microsoft.com/en-us/library/39cwe7zf%28VS.71%29.aspx
switch( $oApp->GetLanguageShortText() )
{
    case 'de':
        // DOH 2010.02.10 do not use LC_ALL, which includes LC_NUMERIC, 1.234 * 1 will result in 1,234
        setlocale( LC_COLLATE, 'de_DE@euro', 'de_DE', 'german' );
        setlocale( LC_CTYPE, 'de_DE@euro', 'de_DE', 'german' );
        setlocale( LC_MONETARY, 'de_DE@euro', 'de_DE', 'german' );
        setlocale( LC_TIME, 'de_DE@euro', 'de_DE', 'german' );
    break;

    case 'en':
    default:
        // DOH 2010.02.10 do not use LC_ALL, which includes LC_NUMERIC, 1.234 * 1 will result in 1,234
        setlocale( LC_COLLATE, 'en_US', 'english' );
        setlocale( LC_CTYPE, 'en_US', 'english' );
        setlocale( LC_MONETARY, 'en_US', 'english' );
        setlocale( LC_TIME, 'en_US', 'english' );
    break;
}

// Disable register globals
ini_set( 'register_globals', '0' );

// Disable magic quotes (This feature has been DEPRECATED as of PHP 5.3.0 and REMOVED as of PHP 5.4.0.)
ini_set( 'magic_quotes_gpc', '0' );

// Rename session cookie to something else, than PHPSESSID
ini_set( 'session.name', 'opagesid' );

// DOH 2010.03.29 If system is under phpunit test -> do not touch error handling, phpunit will do
if ( !$oApp->IsPhpUnit() )
{
    // Use debugging and error handling only on development and testing servers
    if ( $oApp->GetDebug() )
    {
        include_once $sRoot . 'include/debug.php';
        include_once $sRoot . 'include/error.php';
    }
    else
    {
        // display errors: nope
        ini_set( 'display_errors', '0' );
        ini_set( 'display_startup_errors', '0' );

        // no prepend and append string (maybe interferes with ErrerHandlerFatal
        ini_set( 'error_prepend_string', null );
        ini_set( 'error_append_string', null );

        // no error log (may be changed later to something usefull)
        // ini_set( 'error_log', null ); -> does not work with open_basedir restriction in effect.
        ini_set( 'log_errors', '0' );

        // config error reporting
        ini_set( 'track_errors', '0' );
        ini_set( 'html_errors', '0' );
        ini_set( 'ignore_repeated_errors', '1' );
        ini_set( 'ignore_repeated_source', '1' );
        ini_set( 'warn_plus_overloading', '0' );

        // disable error reporting
        error_reporting( 0 );
    }
}

// create the global plugin system
if ( !isset( $oPlugins ) )
{
    $oPlugins = $oFactory->Plugins();
    $oPlugins->Find();
}

if ( $oApp->IsDevelopment() && $oApp->GetoPage() == 'pro' )
{
    $oTimer = $oFactory->Timer();
    $oTimer->SetTraceTime( $oApp->GetParameter( 'tracetime', 0.1 ) );

    $iTrace = $oApp->GetParameter( 'trace' );
    if ( $iTrace === null || $iTrace === '' )
    {
        $iTrace = 0;
    }
    if ( $iTrace > 0 )
    {
        $oTimer->Start( 'page' );
    }
}
else
{
    $iTrace = 0;
}

// Connect to database
if ( !isset( $oDb ) )
{
    $oDb = $oFactory->Db();
    $oDb->SetTrace( $iTrace );
}

if ( !defined( 'oPage install' ) && !defined( 'oPage error' ) )
{
    // Read Parameters
    if ( !isset( $oParameter ) )
    {
        $oParameter = $oFactory->Modul( 'CParameter' );
        if ( $oParameter !== null )
        {
            $oParameter->SetLanguagePostfix( $oApp->GetLanguagePostfix( -1, true ) );
            $oParameter->QueryData();
        }
    }

    // Create Counter object
    if ( !isset( $oCounter ) && $oApp->GetoPage() != 'base' )
    {
        $oCounter = $oFactory->Modul( 'CCounter' );
    }

    // Include custom opage script
    $sCustom = 'custom/';
    if ( file_exists( $sRoot . 'custom/opage.php' ) )
    {
        include_once $sRoot . 'custom/opage.php';
    }
    if ( $oApp->GetAdmin() )
    {
        if ( file_exists( $sRoot . 'admin/custom/opage.php' ) )
        {
            include_once $sRoot . 'admin/custom/opage.php';
        }
    }
}
