<?php

defined( 'oPage' ) or die();

/** Plugins class for including plugins into oPage
*/
class CPlugins
{
    // REGION member variables
    //

    /** Plugin path
        @default 'plugin/'
    */
    var $sPath = 'plugin/';

    /** List of available plugins
        key: plugin name
        values: reference to the plugin object
    */
    var $aList = array();

    //
    // END REGION

    /**
     * Looks into the plugin directory for available plugins
     */
    function Find()
    {
        global $sRoot;

        if ( !is_dir( $sRoot . $this->sPath ) )
            return( false );

        // Read plugin directory
        $oDir = dir( $sRoot . $this->sPath );

        // Retrieve directories
        while ( false !== ( $sPlugin = $oDir->read() ) )
        {
            if ( $sPlugin == '..' )
            {
                continue;
            }
            if ( $sPlugin == '.' )
            {
                continue;
            }

            // Is it a directory?
            if ( !is_dir( $sRoot . $this->sPath . $sPlugin ) )
            {
                continue;
            }

            // Create Plugin path
            $sPath = $this->sPath . $sPlugin . '/';

            // Insert Plugin into list
            $this->Add( $sPlugin, $sPath );
        }

        // Close directory
        $oDir->close();
    }

    /**
     * Setup plugins
     * @param &$oSetup reference to the setup object
     */
    function Setup( &$oSetup )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->Setup( $oSetup );
        }
    }

    /**
     * Setup modul
     * @param &$oObject reference to the module object
     * @param &$sClass
     */
    function SetupModulData( &$oObject, $sClass )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->SetupModulData( $oObject, $sClass );
        }
    }

    /**
     * Factory extension for the plugin
     * @param $sClass name of the wanted class
     * @param $sCreate reference for the name of the class, which should be created
     * @param $sName reference for the name of the module
     * @param $sInclude reference for the filename of the module
     */
    function FactoryModul( $sClass, &$sCreate, &$sName, &$sInclude )
    {
        // Call setup and return after first plugin, which provides the module information
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            if ( $oPlugin->FactoryModul( $sClass, $sCreate, $sName, $sInclude ) )
            {
                return( true );
            }
        }

        return( false );
    }

    /**
     * Factory extension for the plugin
     * @param $oObject reference of the instanced
     * @param $sClass name of the wanted class
     */
    function FactoryModulConfigure( &$oObject, $sClass )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->FactoryModulConfigure( $oObject, $sClass );
        }
    }

    /**
     * Factory extension for the plugin
     * @param $sClass name of the wanted class
     * @param $sCreate reference for the name of the class, which should be created
     * @param $sName reference for the name of the module
     * @param $sInclude reference for the filename of the module
     */
    function FactoryControl( $sClass, &$sCreate, &$sName, &$sInclude )
    {
        // Call setup and return after first plugin, which provides the module information
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            if ( $oPlugin->FactoryControl( $sClass, $sCreate, $sName, $sInclude ) )
            {
                return( true );
            }
        }

        return( false );
    }

    /**
     * Factory extension for the plugin
     * @param $oObject reference of the instanced
     * @param $sClass name of the wanted class
     */
    function FactoryControlConfigure( &$oObject, $sClass )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->FactoryControlConfigure( $oObject, $sClass );
        }
    }

    /**
     * Hook entry point for Admin
     * @param CWebpageNormal &$oWebpage webpage object
     * @param string &$sUrl a reference to the url for the filename, which should be loaded
     */
    function Admin( &$oWebpage, &$sUrl )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->Admin( $oWebpage, $sUrl );
        }
    }

    /**
     * Hook entry point for AdminNavigation
     * @param CWebpage &$oWebpage webpage object
     * @param CModul $oModule reference to module navigation object
     */
    function AdminNavigation( &$oWebpage, &$oModule, &$iFeatureRights, &$fGlobalNavigation )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->AdminNavigation( $oWebpage, $oModule, $iFeatureRights, $fGlobalNavigation );
        }
    }

    /**
     * Hook entry point for AdminWebpageBasis
     * @param CWebpageBasis &$oWebpage reference to the page object
     * @param string $sPart ('constructor', 'begin', 'end')
     */
    function AdminWebpageBasis( &$oWebpage, $sPart )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->AdminWebpageBasis( $oWebpage, $sPart );
        }
    }

    /**
     * Hook entry point for AdminWebpageNormal
     * @param CWebpageNormal &$oWebpage reference to the page object
     * @param string $sPart ('constructor', 'begin', 'end')
     */
    function AdminWebpageNormal( &$oWebpage, $sPart )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->AdminWebpageNormal( $oWebpage, $sPart );
        }
    }

    /**
     * Hook entry point for AdminWebpageContent
     * @param CWebpageContent &$oWebpage reference to the page object
     * @param string $sPart ('constructor', 'begin', 'end')
     */
    function AdminWebpageContent( &$oWebpage, $sPart )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->AdminWebpageContent( $oWebpage, $sPart );
        }
    }

    /**
     * Hook entry point for AdminList
     * @param CWebpageX &$oWebpage reference to the page object
     * @param &$oModul reference to the module
     * @param &$oAdmin reference to the admin control
     */
    function AdminList( &$oWebpage, &$oModul, &$oAdmin )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->AdminList( $oWebpage, $oModul, $oAdmin );
        }
    }

    /**
     * Hook entry point for AdminDetail
     * @param CWebpageX &$oWebpage reference to the page object
     * @param CModul &$oModul reference to the module
     * @param CAdmin &$oAdmin reference to the admin control
     */
    function AdminDetail( &$oWebpage, &$oModul, &$oAdmin )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->AdminDetail( $oWebpage, $oModul, $oAdmin );
        }
    }

    /** Hook entry point for Webpage
        @param CWebpage &$oWebpage reference to the page object
        @param string $sPart ('constructor', 'begin', 'end')
    */
    function Webpage( &$oWebpage, $sPart )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->Webpage( $oWebpage, $sPart );
        }
    }

    /** Hook entry point for WebpageBasis
        @param CWebpageBasis &$oWebpage reference to the page object
        @param string $sPart ('constructor', 'begin', 'end')
    */
    function WebpageBasis( &$oWebpage, $sPart )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->WebpageBasis( $oWebpage, $sPart );
        }
    }

    /**
     * Hook entry point for WebpageX
     * @param CWebpageX &$oWebpage reference to the page object
     * @param string $sPart ('constructor', 'begin', 'end')
     */
    function WebpageX( &$oWebpage, $sPart )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            if ( $oPlugin->GetWebpageXRun() )
            {
                $oPlugin->WebpageX( $oWebpage, $sPart );
            }
        }
    }

    /**
     * Hook entry point for Index
     * @param CWebpageX &$oWebpage reference to the page object
     */
    function Index( &$oWebpage )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->Index( $oWebpage );
        }
    }

    /**
     * Add a plugin to the plugin list
     * @param string $Plugin name of the plugin
     * @param string $sPath path to the plugin
     */
    function Add( $sPlugin, $sPath )
    {
        global $sRoot;
        global $oFactory;

        // Plugin: include file
        if ( !file_exists( $sRoot . $sPath . 'plugin.php' ) )
        {
            return( null );
        }
        include_once $sRoot . 'include/plugin.php';
        include_once $sRoot . $sPath . 'plugin.php';

        // Plugin: create instance
        $sClass = 'C' . ucfirst( $sPlugin ) . 'Plugin';
        $oPlugin = new $sClass();

        // Set plugin path
        $oPlugin->SetPath( $sPath );

        // Set plugin name
        $oPlugin->SetName( $sPlugin );

        // Configure plugin
        $oFactory->_PluginConfigure( $oPlugin, $sClass );

        // Store reference of the plugin
        $this->aList[ $sPlugin ] = &$oPlugin;

        return( true );
    }

    /**
     * Returns the plugin object for a plugin
     * @param $sPlugin name of the plugin
     * @return Reference to the plugin object or null
     */
    function &Get( $sPlugin )
    {
		$oPlugin = null;
        $rPlugin = &$oPlugin;

        if ( !isset( $this->aList[ $sPlugin ] ) )
            return( $rPlugin );

        $rPlugin = &$this->aList[ $sPlugin ];

        return( $rPlugin );
    }

    /**
     * Checks, if a plugin is available
     * @param $sPlugin name of the plugin
     * @return true/false
     */
    function Exists( $sPlugin )
    {
        if ( !isset( $this->aList[ $sPlugin ] ) )
        {
            return( false );
        }

        return( true );
    }

    /**
     * Sets plugin path
     */
    function SetPath( $sValue )
    {
        $this->sPath = $sValue;
    }

    /**
     * Gets plugin path
     */
    function GetPath()
    {
        return( $this->sPath );
    }

    /**
     * Hook entry point for Template Resource loading
     * @param &$oTemplate reference to the template object
     */
    function ResourceRead( &$oTemplate, &$sResource, $sLanguage, $sName = null )
    {
        foreach( array_keys( $this->aList ) as $sKey )
        {
            $oPlugin = &$this->aList[ $sKey ];
            $oPlugin->ResourceRead( $oTemplate, $sResource, $sLanguage, $sName );
        }
    }
}
