<?php

defined( 'oPage' ) or die();

/** Liefert zu bestimmten Ereignissen einen Meldungstext
	@autor Hannes Dorn DOH
	@history 08.09.2001 DOH Erste Version
*/
class CResource
{
    /** Name of the charset, default iso-8859-1
        windows-1252
        iso-8859-1
        utf-8
    */
    var $sCharset = 'iso-8859-1';

    /** Current language
    */
    var $sLanguage;

	/** Modulspezifische Informationen */
    var $aMessage;
    var $aError;

    /** Konstruktor
    	Initializes the Arrays with messages in the desired language
    	@param $sLanguage:
    	    0 or de: german
    	    1 or en: english
    	    2 or fr: french
    	    3 or es: spanish
    	    4 or it: italian
    	@return void
    */
    function CResource( $sLanguage = null, $sCharset = null )
    {
        if ( $sLanguage === null )
            $sLanguage = 'de';
        if ( is_int( $sLanguage ) )
        {
            if ( $sLanguage == 0 )
                $sLanguage = 'de';
            elseif ( $sLanguage == 1 )
                $sLanguage = 'en';
        }
        $this->sLanguage = $sLanguage;

        if ( $sCharset !== null )
            $this->sCharset = $sCharset;

        switch( $this->sLanguage )
        {
            // german
            case 'de':
            case 0:
                $this->aError[ 'file_copy' ] = 'Die Datei \'%s\' konnte nicht nach \'%s\' kopiert werden.';
                $this->aError[ 'file_delete' ] = 'Die Datei \'%s\' konnte nicht gelscht werden.';
                $this->aError[ 'dir_notfound' ] = 'Das Verzeichnis \'%s\' wurde nicht gefunden.';
                $this->aError[ 'image_resize' ] = 'Die Gre des Bildes \'%s\' konnte nicht angepat werden.';
                $this->aError[ 'image_convert' ] = 'Das Bild \'%s\' konnte nicht nach \'%s\' konvertiert werden.';
                $this->aError[ 'data_invalid_id' ] = 'Die ID \'%s\' von \'%s\' ist ungltig.';
                $this->aError[ 'data_missing' ] = 'Die Daten \'%s\' in \'%s\' fehlen.';
                $this->aError[ 'database_insert' ] = 'Der Datensatz konnte nicht in \'%s\' eingefgt werden.';
                $this->aError[ 'database_update' ] = 'Der Datensatz \'%s\' in \'%s\' konnte nicht aktualisiert werden.';
                $this->aError[ 'database_delete' ] = 'Der Datensatz \'%s\' in \'%s\' konnte nicht gelscht werden.';
                $this->aError[ 'database_delete_all' ] = 'Die Datenstze in \'%s\' konnten nicht gelscht werden.';
                $this->aError[ 'database_import' ] = 'Die Datenstze konnten nicht importiert werden.';

                $this->aMessage[ 'database_insert' ] = 'Der Datensatz wurde erfolgreich in \'%s\' eingefgt.';
                $this->aMessage[ 'database_update' ] = 'Der Datensatz \'%s\' in \'%s\' wurde erfolgreich aktualisiert.';
                $this->aMessage[ 'database_delete' ] = 'Der Datensatz \'%s\' in \'%s\' wurde erfolgreich gelscht.';
                $this->aMessage[ 'database_delete_all' ] = 'Die Datenstze von \'%s\' wurden erfolgreich gelscht.';
                $this->aMessage[ 'database_import' ] = 'Die Datenstze wurden erfolgreich importiert.';

                $this->aError[ 'data_invalid' ] = 'Bitte berprfen Sie die angegebenen Daten.';
                $this->aError[ 'data_notloaded' ] = 'Die Daten konnten nicht geladen werden.';
                $this->aMessage[ 'data_loaded' ] = 'Die Daten wurden erfolgreich geladen.';
                $this->aError[ 'data_notsaved' ] = 'Die Daten konnten nicht gespeichert werden.';
                $this->aMessage[ 'data_saved' ] = 'Die Daten wurden erfolgreich gespeichert.';
                $this->aError[ 'data_notdeleted' ] = 'Die Daten konnten nicht gelscht werden.';
                $this->aMessage[ 'data_deleted' ] = 'Die Daten wurden erfolgreich gelscht.';

                $this->aError[ 'data_notfound' ] = 'Es wurden keine Daten gefunden.';
                $this->aError[ 'data_max' ] = 'Sie knnen nur maximal %s Eintrge einfgen.';

                $this->aError[ 'message_notsent' ] = 'Die Nachricht wurde nicht versendet.';
                $this->aMessage[ 'message_sent' ] = 'Die Nachricht wurde erfolgreich versendet.';
                $this->aMessage[ 'message_sentto' ] = 'Die Nachricht wurde erfolgreich an \'%s\' versendet.';
            break;

            case 'en':
            case 1:
            default:
                $this->aError[ 'file_copy' ] = 'The File \'%s\' could not be copied to \'%s\'.';
                $this->aError[ 'file_delete' ] = 'The File \'%s\' could not be deleted.';
                $this->aError[ 'dir_notfound' ] = 'The Directory \'%s\' could not be found.';
                $this->aError[ 'image_resize' ] = 'The Image \'%s\' could not be resized.';
                $this->aError[ 'image_convert' ] = 'The Image \'%s\' could not be converted into \'%s\'.';
                $this->aError[ 'data_invalid_id' ] = 'The ID \'%s\' for \'%s\' is invalid.';
                $this->aError[ 'data_missing' ] = 'The Data \'%s\' for \'%s\' is missing.';
                $this->aError[ 'database_insert' ] = 'The record for \'%s\' could not be inserted.';
                $this->aError[ 'database_update' ] = 'The record \'%s\' for \'%s\' could not be updated.';
                $this->aError[ 'database_delete' ] = 'The record \'%s\' for \'%s\' could not be deleted.';
                $this->aError[ 'database_delete_all' ] = 'Not all records of \'%s\' could be deleted.';
                $this->aError[ 'database_import' ] = 'Import was not successfull.';

                $this->aMessage[ 'database_insert' ] = 'The record for \'%s\' has been inserted.';
                $this->aMessage[ 'database_update' ] = 'The record \'%s\' for \'%s\' has been updated.';
                $this->aMessage[ 'database_delete' ] = 'The record \'%s\' for \'%s\' has been deleted.';
                $this->aMessage[ 'database_delete_all' ] = 'All records of \'%s\' have been deleted.';
                $this->aMessage[ 'database_import' ] = 'Import was successfull.';

                $this->aError[ 'data_invalid' ] = 'Please check the supplied data.';
                $this->aError[ 'data_notloaded' ] = 'Data could not be loaded.';
                $this->aMessage[ 'data_loaded' ] = 'Data loaded successfully.';
                $this->aError[ 'data_notdeleted' ] = 'Data could not be deleted.';
                $this->aMessage[ 'data_deleted' ] = 'Data deleted successfully.';
                $this->aError[ 'data_notsaved' ] = 'Data could not be saved.';
                $this->aMessage[ 'data_saved' ] = 'Data stored successfully.';

                $this->aError[ 'data_notfound' ] = 'Data could not be found.';
                $this->aError[ 'data_max' ] = 'You can only insert %s records maximum.';

                $this->aError[ 'message_notsent' ] = 'The message could not be sent.';
                $this->aMessage[ 'message_sent' ] = 'The message has been sent.';
                $this->aMessage[ 'message_sentto' ] = 'The message has been sent to \'%s\'.';
            break;
        }
    }

    /** Error
        Liefert eine Fehlermeldung
        @param sError Name der Fehlermeldung
        @param sValue1 1. Parameter, optional
        @param sValue2 2. Parameter, optional
        @return String
    */
    function Error( $sError, $sValue1 = '', $sValue2 = '' )
    {
        if ( !isset( $this->aError[ $sError ] ) )
            die( "CResource: Error: Invalid Errorname $sError" );

        $sBuffer = $this->aError[ $sError ];

        // If charset is utf-8, convert message from Windows to UTF-8
        if ( $this->sCharset == 'utf-8' )
            $sBuffer = StringWindowsToUtf8( $sBuffer );

        return( sprintf( $sBuffer, $sValue1, $sValue2 ) );
    }

    /** Message
        Liefert eine Meldung
        @param sMessage Name der Meldung
        @param sValue1 1. Parameter, optional
        @param sValue2 2. Parameter, optional
        @return String
    */
    function Message( $sMessage, $sValue1 = '', $sValue2 = '' )
    {
        if ( !isset( $this->aMessage[ $sMessage ] ) )
            die( "CResource: Message: Invalid Messagename $sMessage" );

        $sBuffer = $this->aMessage[ $sMessage ];

        // If charset is utf-8, convert template from Windows to UTF-8
        if ( $this->sCharset == 'utf-8' )
            $sBuffer = StringWindowsToUtf8( $sBuffer );

        return( sprintf( $sBuffer, $sValue1, $sValue2 ) );
    }
}

if ( !function_exists( 'StringWindowsToUtf8' ) )
{
    /*
        This structure encodes the difference between ISO-8859-1 and Windows-1252,
        as a map from the UTF-8 encoding of some ISO-8859-1 control characters to
        the UTF-8 encoding of the non-control characters that Windows-1252 places
        at the equivalent code points.
        Aidan Kehoe <php-manual@parhasard.net>
    */
    global $aCp1252ToUtf8;
    $aCp1252ToUtf8 = array
    (
        "\xc2\x80" => "\xe2\x82\xac", /* EURO SIGN */
        "\xc2\x82" => "\xe2\x80\x9a", /* SINGLE LOW-9 QUOTATION MARK */
        "\xc2\x83" => "\xc6\x92",     /* LATIN SMALL LETTER F WITH HOOK */
        "\xc2\x84" => "\xe2\x80\x9e", /* DOUBLE LOW-9 QUOTATION MARK */
        "\xc2\x85" => "\xe2\x80\xa6", /* HORIZONTAL ELLIPSIS */
        "\xc2\x86" => "\xe2\x80\xa0", /* DAGGER */
        "\xc2\x87" => "\xe2\x80\xa1", /* DOUBLE DAGGER */
        "\xc2\x88" => "\xcb\x86",     /* MODIFIER LETTER CIRCUMFLEX ACCENT */
        "\xc2\x89" => "\xe2\x80\xb0", /* PER MILLE SIGN */
        "\xc2\x8a" => "\xc5\xa0",     /* LATIN CAPITAL LETTER S WITH CARON */
        "\xc2\x8b" => "\xe2\x80\xb9", /* SINGLE LEFT-POINTING ANGLE QUOTATION */
        "\xc2\x8c" => "\xc5\x92",     /* LATIN CAPITAL LIGATURE OE */
        "\xc2\x8e" => "\xc5\xbd",     /* LATIN CAPITAL LETTER Z WITH CARON */
        "\xc2\x91" => "\xe2\x80\x98", /* LEFT SINGLE QUOTATION MARK */
        "\xc2\x92" => "\xe2\x80\x99", /* RIGHT SINGLE QUOTATION MARK */
        "\xc2\x93" => "\xe2\x80\x9c", /* LEFT DOUBLE QUOTATION MARK */
        "\xc2\x94" => "\xe2\x80\x9d", /* RIGHT DOUBLE QUOTATION MARK */
        "\xc2\x95" => "\xe2\x80\xa2", /* BULLET */
        "\xc2\x96" => "\xe2\x80\x93", /* EN DASH */
        "\xc2\x97" => "\xe2\x80\x94", /* EM DASH */
        "\xc2\x98" => "\xcb\x9c",     /* SMALL TILDE */
        "\xc2\x99" => "\xe2\x84\xa2", /* TRADE MARK SIGN */
        "\xc2\x9a" => "\xc5\xa1",     /* LATIN SMALL LETTER S WITH CARON */
        "\xc2\x9b" => "\xe2\x80\xba", /* SINGLE RIGHT-POINTING ANGLE QUOTATION*/
        "\xc2\x9c" => "\xc5\x93",     /* LATIN SMALL LIGATURE OE */
        "\xc2\x9e" => "\xc5\xbe",     /* LATIN SMALL LETTER Z WITH CARON */
        "\xc2\x9f" => "\xc5\xb8"      /* LATIN CAPITAL LETTER Y WITH DIAERESIS*/
    );

    function StringWindowsToUtf8( $sBuffer )
    {
       global $aCp1252ToUtf8;
       return( strtr( utf8_encode( $sBuffer ), $aCp1252ToUtf8 ) );
    }
}
