<?php

chdir( dirname( __FILE__ ) );
include_once '../../include/opage.php';
include_once $sRoot . 'include/yaml/spyc.php';

if ( $oApp->GetInterface() != 'cli' )
{
    die();
}

class CApplication
{
    var $sVersion = '0.1';

    // define possible parameters
    var $aParameter = array
    (
        'data:',
        'list::',
        'listlong::',
        'dump:',
        'load:',
        'clean:',
        'trunc:',
        'truncload:',

        'help::',
        'version::'
    );

    // actual options
    var $aOption;

    function &Init()
    {
        // Get provided options
        $this->aOption = getopt( '', $this->aParameter );

        return( $this );
    }

    function Run( $aOption = null )
    {
        if ( $aOption == null )
        {
            $aOption = &$this->aOption;
        }

        // Display help
        if ( isset( $aOption[ 'help' ] ) || count( $aOption ) == 0 || empty( $aOption ) )
        {
            fwrite( STDOUT, $this->Help() );
            return( false );
        }

        // Show version
        if ( isset( $aOption[ 'version' ] ) )
        {
            fwrite( STDOUT, $this->Version() );
            return( true );
        }

        $oAction = new CDataDumpAndLoad();

        // Show a list of all modules
        if ( isset( $aOption[ 'list' ] ) )
        {
            $oAction->ListModules( 'name' );
            return( true );
        }
        // Show a list of all modules
        if ( isset( $aOption[ 'listlong' ] ) )
        {
            $oAction->ListModules();
            return( true );
        }

        foreach( $aOption as $sOption => $sValue )
        {
            switch( $sOption )
            {
                case 'data':
                    $oAction->SetDataPath( $sValue . '/' );
                break;

                case 'dump':
                    $oAction->Dump( $sValue );
                break;

                case 'load':
                    $oAction->Load( $sValue );
                break;

                case 'clean':
                    $oAction->Clean( $sValue );
                break;

                case 'trunc':
                    $oAction->Trunc( $sValue );
                break;

                case 'truncload':
                    $oAction->Trunc( $sValue );
                    $oAction->Load( $sValue );
                break;
            }
        }

        return( true );
    }

    function Help()
    {
        $sBuffer = '';
        $sBuffer .= "DataDumpAndLoad {$this->sVersion} by Hannes Dorn\n";
        $sBuffer .= "\n";
        $sBuffer .= "Usage: data [switches]\n";
        $sBuffer .= "       data [switches] <modulename>\n";
        $sBuffer .= "\n";
        $sBuffer .= "  --data <datadir>        Set data directory, default _data\n";
        $sBuffer .= "  --list                  List all modules (module name only)\n";
        $sBuffer .= "  --listlong              List all modules (module name and title)\n";
        $sBuffer .= "\n";
        $sBuffer .= "  --dump <module>         Dump data from a module\n";
        $sBuffer .= "  --load <module>         Load data for a module\n";
        $sBuffer .= "  --clean <module>        Delete local module data\n";
        $sBuffer .= "  --trunc <module>        Empty database table\n";
        $sBuffer .= "  --truncload <module>    Empty database table and load data\n";
        $sBuffer .= "\n";
        $sBuffer .= "  --help                  Prints this usage information.\n";
        $sBuffer .= "  --version               Prints the version and exits.\n";
        return( $sBuffer );
    }

    function Version()
    {
        $sBuffer = '';
        $sBuffer .= "DataDumpAndLoad {$this->sVersion} by Hannes Dorn\n";
        return( $sBuffer );
    }
}

class CDataDumpAndLoad
{
    var $sDataPath;

    function CDataDumpAndLoad( $sDataPath = null )
    {
        if ( $sDataPath === null || $sDataPath === '' )
        {
            $sDataPath = '_data/';
        }
        $this->SetDataPath( $sDataPath );
    }

    function SetDataPath( $sValue )
    {
        global $sRoot;

        // Check data directory
        if ( is_file( $sRoot . $sValue ) || !is_dir( $sRoot . $sValue ) )
        {
            die( "Invalid directory {$sValue}" );
        }

        $this->sDataPath = $sValue;
    }

    function ListModules( $fLong = null )
    {
        global $oFactory;

        if ( $fLong == null )
        {
            $fLong = true;
        }

        $oModule = $oFactory->Modul( 'CModule' );
        $oModule->ClearFilters();
        $oModule->SetView();
        $oModule->QueryData();
        foreach( $oModule->GetResult() as $aRow )
        {
            if ( $fLong )
            {
                printf( "%-10s%s\n", $aRow[ 'name' ], $aRow[ 'title' ] );
            }
            else
            {
                fwrite( STDOUT, $aRow[ 'name' ] . "\n" );
            }
        }
    }

    function Dump( $sModule )
    {
        global $sRoot;
        /* @var $oFactory CFactoryX */
        global $oFactory;
        /* @var $oDb CDb */
        global $oDb;

        // Create module
        /* @var $oModule CModul */
        $oModule = $oFactory->Modul( $sModule );
        if ( $oModule == null )
        {
            fwrite( STDOUT, "{$sModule}: Module not found.\n" );
            return( false );
        }

        // Create the filename
        $sFilename = $this->sDataPath . $oModule->GetName() . '.yaml';

        // Open the file
        $hFile = fopen( $sRoot . $sFilename, 'w' );

        // Get number of records for % calculation
        $iRecords = $oDb->CountAll( $oModule->GetTable( 1 ), $oModule->GetField( 'id' ) );

        // Write data into yaml file
        // Do not use $oModule->QueryData() and GetResult(). These return an array with internal fieldnames
        $sClear = str_repeat( chr( 8 ), 8 );
        fwrite( STDOUT, "{$sModule}: Dumping {$iRecords} records into {$sFilename}...   0.00%" . $sClear );
        for( $i = 0; $i < $iRecords; $i = $i + 500 )
        {
            $aRows = $oDb->Read( $oModule->GetTable( 1 ), null, null, null, $i . ',500' );
            fwrite( $hFile, Spyc::YAMLDump( $aRows, true ) );
            $this->_CopyStaticFilesToData( $oModule, $aRows );
            fwrite( STDOUT, sprintf( '%7.2f%%', $i / $iRecords * 100 ) . $sClear );
        }
        fwrite( STDOUT, "done.   \n" );

        // Close the file
        fclose( $hFile );

        return( true );
    }

    function Load( $sModule )
    {
        global $sRoot;
        global $oFactory;
        global $oDb;

        // Create module
        $oModule = $oFactory->Modul( $sModule );
        if ( $oModule == null )
        {
            fwrite( STDOUT, "{$sModule}: Module not found.\n" );
            return( false );
        }

        // Check data directory
        if ( is_file( $sRoot . $this->sDataPath ) || !is_dir( $sRoot . $this->sDataPath ) )
        {
            die( "Invalid directory {$this->sDataPath}" );
        }

        // Create the filename
        $sFilename = $this->sDataPath . $oModule->GetName() . '.yaml';

        // Check data directory
        if ( !is_file( $sRoot . $sFilename ) )
        {
            die( "Invalid file {$sFilename}" );
        }

        // Open the file
        $hFile = fopen( $sRoot . $sFilename, 'r' );

        // Get filesize for % calculation
        $iSize = filesize( $sRoot . $sFilename );

        // Load data from file into database
        $sClear = str_repeat( chr( 8 ), 8 );
        fwrite( STDOUT, "{$sModule}: Loading {$sFilename}...   0.00%" . $sClear );
        $sRecord = '';
        $iRead = 0;
        while( !feof( $hFile ) )
        {
            $sLine = fgets( $hFile );
            $iRead += strlen( $sLine );

            // new record?
            if ( rtrim( $sLine, "\n\r" ) == '---' || feof( $hFile ) )
            {
                // Do we have data?
                if ( $sRecord <> '' )
                {
                    $aRows = Spyc::YAMLLoadString( $sRecord );
                    // do not user $oModule->Insert() -> this would modify some data
                    $oDb->Insert( $oModule->GetTable( 1 ), $aRows );
                    $this->_CopyStaticFilesToDb( $oModule, $aRows );
                    fwrite( STDOUT, sprintf( '%7.2f%%', $iRead / $iSize * 100 ) . $sClear );
                }

                $sRecord = '';
            }

            $sRecord .= $sLine;
        }
        fwrite( STDOUT, "done.   \n" );

        return( true );
    }

    function Clean( $sModule )
    {
        global $sRoot;
        global $oFactory;

        // Create module
        $oModule = $oFactory->Modul( $sModule );
        if ( $oModule == null )
        {
            fwrite( STDOUT, "{$sModule}: Module not found.\n" );
            return( false );
        }

        // Check data directory
        if ( is_file( $sRoot . $this->sDataPath ) || !is_dir( $sRoot . $this->sDataPath ) )
        {
            die( "Invalid directory {$this->sDataPath}" );
        }

        // Create the filename
        $sFilename = $this->sDataPath . $oModule->GetName() . '.yaml';

        // Delete file
        fwrite( STDOUT, "{$sModule}: Deleting {$sFilename}..." );
        unlink( $sRoot . $sFilename );
        fwrite( STDOUT, " done.\n" );

        // Delete static content
        DirectoryDelete( $this->sDataPath . $oModule->GetName() . '/' );

        return( true );
    }

    function Trunc( $sModule )
    {
        global $oFactory;

        // Create module
        $oModule = $oFactory->Modul( $sModule );
        if ( $oModule == null )
        {
            fwrite( STDOUT, "{$sModule}: Module not found.\n" );
            return( false );
        }
        $oModule->ClearFilters();
        $oModule->SetView();

        // Truncate table
        fwrite( STDOUT, "{$sModule}: Truncating..." );
        // Use $oModule->Delete() to delete also the according static files in the filesystem
        $oModule->Delete();
        fwrite( STDOUT, " done.\n" );

        return( true );
    }

    function _CopyStaticFilesToData( $oModule, $aRows = null )
    {
        $this->_CopyStaticFiles( $oModule, $aRows, 'data' );
    }

    function _CopyStaticFilesToDb( $oModule, $aRows = null )
    {
        $this->_CopyStaticFiles( $oModule, $aRows, 'db' );
    }

    function _CopyStaticFiles( $oModule, $aRows, $sMode )
    {
        global $sRoot;

        if ( $aRows === null )
        {
            $aRows = $oModule->GetResult();
        }

        // Get list of fields with static content
        $aFields = $oModule->GetFieldList( array( 'FILE', 'IMAGE' ) );

        // Get paths to static files
        foreach( $aFields as $sKey => $sField )
        {
            $aFields[ $sField ] = $oModule->GetFieldPath( $sField );
            unset( $aFields[ $sKey ] );
        }

        // Copy files to _data/...
        foreach( $aRows as $aRow )
        {
            foreach( $aFields as $sField => $sPath )
            {
                if ( $aRow[ $sField ] !== null && $aRow[ $sField ] !== '' )
                {
                    switch( $sMode )
                    {
                        case 'data':
                            $sSource = $sRoot . $sPath . $aRow[ $sField ];
                            $sDestination = $sRoot . $this->sDataPath . $oModule->GetName() . '/' . $sField . '/' . $aRow[ $sField ];
                        break;

                        case 'db':
                            $sSource = $sRoot . $this->sDataPath . $oModule->GetName() . '/' . $sField . '/' . $aRow[ $sField ];
                            $sDestination = $sRoot . $sPath . $aRow[ $sField ];
                        break;

                        default:
                            die( 'invalid mode' );
                        break;
                    }

                    if ( is_file( $sSource ) )
                    {
                        if ( !is_dir( dirname( $sDestination ) ) )
                        {
                            mkdir( dirname( $sDestination ), 0777, true );
                        }
                        copy( $sSource, $sDestination );
                    }
                }
            }
        }
    }
}

$oApplication = new CApplication();
if ( !$oApplication->Init()->Run() )
{
    exit( 1 );
}
