<?php

defined( 'oPage' ) or die();

/** CSession
    Normal PHP sessions are file based.
    PHP locks the file for the whole time the script is running.
    If a users calls the same script twice or some other scripts in the same session (e.g. via frames or ajax), PHP locks the session file and serialises all calls.
    But file sessions are limited to one webserver.
    MySQL sessions store the session data in the central database, so more then one webserver can be used.
    But MySQL sessions need to implement session locking!

    Here a similar aproach is implemented by using a global lock for the session id (get_lock()).
    AJAX requires a more finer locking mechanism as detailed in http://thwartedefforts.org/2006/11/11/race-conditions-with-ajax-and-php-sessions/.
*/

class CSession
{
    var $sRoot;
    var $sId;

    function CSession( $sId = null, $fInit = null )
    {
        global $oApp;

        if ( $fInit === null )
            $fInit = true;

        // Session Handler setzen
        Session_Set_Save_Handler
        (
            array( $this, '_Open' ),
            array( $this, '_Close' ),
            array( $this, '_Read' ),
            array( $this, '_Write' ),
            array( $this, '_Destroy' ),
            array( $this, '_GarbageCollector' )
        );

        // PHP5: session_write_close needs to be called before the database connection is closed
        // so the session_write_close function is registered as shutdown function
        register_shutdown_function( 'session_write_close' );

        // Achtung: Es werden alle Sessions in ein und derselben Tabelle gespeichert.
        // Mit Root wird das Web festgelegt, fr das die Benutzerkennung gltig ist.
        $this->sRoot = $oApp->GetRoot();

        if ( $fInit )
            $this->Init( $sId );
    }

    function Init( $sId = null )
    {
        global $HTTP_COOKIE_VARS;

        // if ( ini_get( 'session.use_only_cookies' ) == 0 )
        //    ini_set( 'session.use_only_cookies', '1' );

        // first, check if we have a correct session id
        // we have to check the cookie, cause session_id() returns the id only after session_start() has been called

        // if we do not have a session id, check if we have one in a cookie or somewhere else
        if ( $sId == '' )
            // DOH 04.10.2006 Do not use $_COOKIE, it is affected by magic_quotes_gpc settings in php.ini
            if ( PHP_VERSION < '4.1.0' )
                $sId = $HTTP_COOKIE_VARS[ session_name() ];
            else
                $sId = $_COOKIE[ session_name() ];

        // validate session id
        if ( $sId != '' && $sId !== null )
        {
            if ( abs( crc32( substr( $sId, 0, 32 ) ) ) != substr( $sId, 32 ) )
                $sId = null;
        }

        // generate new session id
        if ( $sId == '' )
        {
            // generate session id
            mt_srand( (double) microtime() * 1000000 );
            $sId = time();
            $sId .= ' ' . mt_rand( 0, 999999 );
            $sId .= ' ' . $_SERVER[ 'REMOTE_ADDR' ];
            $sId = md5( $sId );
            $sId .= abs( crc32( $sId ) );
        }

        // if we have a session id, set it
        $this->sId = $sId;
        if ( PHP_VERSION < '4.1.0' )
        {
            if ( $sId != $HTTP_COOKIE_VARS[ session_name() ] );
                session_id( $sId );
        }
        else
            if ( $sId != $_COOKIE[ session_name() ] )
                session_id( $sId );

        // start session
        session_start();

        if ( PHP_VERSION < '4.1.0' )
            // 2009.02.24 DOH   wtf is this doing?
            //                  ini_get( 'register_globals' ) returns === false, when there is no register_globals setting
            //                  ini_get( 'register_globals' ) returns == true, when the register_globals is set to true
            if ( ini_get( 'register_globals' ) === false || ini_get( 'register_globals' ) )
            {
                global $aSessionFields;
                if ( !session_is_registered( 'aSessionFields' ) )
                    session_register( 'aSessionFields' );
            }

        return( true );
    }

    function SetValue( $sName, $sValue )
    {
        global $HTTP_SESSION_VARS;

        $sName = $this->sRoot . '::' . $sName;
        if ( PHP_VERSION < '4.1.0' )
        {
            // 2009.02.24 DOH   wtf is this doing?
            //                  ini_get( 'register_globals' ) returns === false, when there is no register_globals setting
            //                  ini_get( 'register_globals' ) returns == true, when the register_globals is set to true
            if ( ini_get( 'register_globals' ) === false || ini_get( 'register_globals' ) )
            {
                global $aSessionFields;
                $aSessionFields[ $sName ] = $sValue;
            }
            $HTTP_SESSION_VARS[ 'aSessionFields' ][ $sName ] = $sValue;
        }
        else
        {
            if ( $sValue === null )
                unset( $_SESSION[ 'aSessionFields' ][ $sName ] );
            else
                $_SESSION[ 'aSessionFields' ][ $sName ] = $sValue;
        }
    }

    function GetValue( $sName )
    {
        global $HTTP_SESSION_VARS;

        $sName = $this->sRoot . '::' . $sName;

        if ( PHP_VERSION < '4.1.0' )
        {
            // 2009.02.24 DOH   wtf is this doing?
            //                  ini_get( 'register_globals' ) returns === false, when there is no register_globals setting
            //                  ini_get( 'register_globals' ) returns == true, when the register_globals is set to true
            if ( ini_get( 'register_globals' ) === false || ini_get( 'register_globals' ) )
            {
                global $aSessionFields;
                return( $aSessionFields[ $sName ] );
            }
            return( $HTTP_SESSION_VARS[ 'aSessionFields' ][ $sName ] );
        }
        else
            return( $_SESSION[ 'aSessionFields' ][ $sName ] );
    }

    function SetRoot( $sValue )
    {
        global $oApp;
        $this->sRoot = $oApp->GetRoot() . $sValue;
    }

    function _Open( $sSavePath, $sSessionName )
    {
    	return( true );
    }

    function _Close()
    {
        global $oDb;

        // Release session lock
        $oDb->Unlock( $this->sRoot . '::' . 'session', $this->sId );

    	return( true );
    }

    function _Read( $sId )
    {
    	global $oDb;

        // Get a lock, timeout after 60 seconds
        if ( !$oDb->Lock( $this->sRoot . '::' . 'session', $sId ) )
           die( 'Can not get session lock!' );

        $sTable = 'session';
        $sFields = 'data';
        $sFilter = 'id=\'' . addslashes( $sId ) . '\'';
        $sOrder = '';
    	$aResult = $oDb->Read( $sTable, $sFields, $sFilter, $sOrder );
        if ( ArrayCount( $aResult < 1 ) )
            return( false );

        if ( IsNull( $aResult[0][ 'data' ] ) )
            return( '' );
        else
            return( $aResult[0][ 'data' ] );
    }

    function _Write( $sId, $sData )
    {
    	global $oDb;

        $aRow = array
        (
            'id' => $sId,
            'last' => time(),
            'data' => $sData
        );
        $aRows[] = $aRow;

        $sTable = 'session';
        $sFields = 'count( id ) as count';
        $sFilter = "id='" . addslashes( $sId ) . "'";
    	$aResult = $oDb->Read( $sTable, $sFields, $sFilter );
        if ( $aResult[ 0 ][ 'count' ] == 0 )
            $iRows = $oDb->Insert( 'session', $aRows );
        else
            $iRows = $oDb->Update( 'session', $aRows, 'id' );

    	return( true );
    }

    function _Destroy( $sId )
    {
    	global $oDb;

    	$sId = addslashes( $sId );

        // Sessions lschen
        return( $oDb->Delete( 'session', "id='" . $sId . "'" ) );
    }

    function _GarbageCollector( $iLifeTime )
    {
    	global $oDb;

        // Alle Sessions lschen
        // return( $oDb->Delete( 'session', 'last < ' . ( time() - $iLifeTime ) ) );

        return( $oDb->Delete( 'session', 'last < ' . ( time() - 10800 ) ) );
    }

	function Setup()
	{
        global $oApp;
        global $oDb;

        $aFields = array();

        $aField = array
        (
            'multilanguage' => false,
            'field' => 'id',
            'prefix' => true,
            'values' => null,
            'type' => 'TEXT'
        );
        $aFields[ 'id' ] = $aField;

        $aField = array
        (
            'multilanguage' => false,
            'field' => 'last',
            'prefix' => true,
            'values' => null,
            'type' => 'INTEGER'
        );
        $aFields[ 'last' ] = $aField;

        $aField = array
        (
            'multilanguage' => false,
            'field' => 'data',
            'prefix' => true,
            'values' => null,
            'type' => 'TEXTBOX'
        );
        $aFields[ 'data' ] = $aField;

	    // create table
	    if ( !$oDb->TableExists( 'session' ) )
	        $oDb->TableCreate( 'session', $aFields, 'id' );
	    else
	        $oDb->TableAlter( 'session', $aFields );

	    return( true );
	}
}
