<?php

defined( 'oPage' ) or die();

/** Setup class, used to install, setup and oPage
*/
class CSetup
{
    // REGION member variables
    //

    /** Buffer for setup messages
    */
    var $sContent;

    /** Array with module names for the modul menu
    */
    var $aModules = array();

    /** Array with module names for the tools menu
    */
    var $aTools = array();

    /** Array with System page entries
        This are page entries usualy not used in any menu
    */
    var $aPages = array();

    /** Array with Module page entries
        This are page entries usualy used in any modules menu
        The additional pages are inserted after the main entries
    */
    var $aPagesModules = array();
    var $aPagesModulesAdditional = array();

    /** Array with Tool page entries
        This are page entries usualy used in any tools menu
        The additional pages are inserted after the main entries
    */
    var $aPagesTools = array();
    var $aPagesToolsAdditional = array();

    /** Array with Content page entries
        This are page entries usualy used in the frontend
    */
    var $aPagesContent = array();

    //
    // END REGION

    /** Constructor: init setup
    */
    function CSetup()
    {
        global $sRoot;
        global $oApp;
        global $oPlugins;

        // REGION set default modules (listed in tools menu)
        //
        $this->AddTool( 'CModule' );
        $this->AddTool( 'CPage' );
        $this->AddTool( 'CPageModul' );
        $this->AddTool( 'CPageTools' );

        // add counter only for std and pro
        switch( $oApp->GetoPage() )
        {
            case 'std':
            case 'pro':
                $this->AddTool( 'CCounter' );
            break;
        }

        $this->AddTool( 'CParameter' );
        $this->AddTool( 'CUser' );

        if ( $oApp->GetoPage() == 'pro' )
        {
            // rights modules (depending on used feature
            if ( $oApp->GetFeature( 'user', 'rights' ) == 1 )
            {
                $this->AddTool( 'CUserRight' );
                $this->AddTool( 'CUserRightUser' );
            }
            if ( $oApp->GetFeature( 'user', 'rights' ) == 2 )
            {
                $this->AddTool( 'CUserGroup' );
                $this->AddTool( 'CUserGroupUser' );
                $this->AddTool( 'CUserRight' );
                $this->AddTool( 'CUserRightGroup' );
            }
            if ( $oApp->GetFeature( 'user', 'client' ) == 1 )
            {
                $this->AddTool( 'CUserClient' );
                $this->AddTool( 'CUserClientUser' );
            }
        }

        //
        // END REGION

        // REGION set default modules (listed in modul menu)
        //

        // Call Setup to let the plugins add themselvs to the modules and tools
        $oPlugins->Setup( $this );

        //
        // END REGION

        // REGION set default pages
        //

        // System pages
        $this->AddPage( 'system', 'a_nopage', null, null, 'Kein Seiten-Link', 'Administration - Kein Seiten-Link', null, null, null, null, null, true );
        $this->AddPage( 'system', 'admin', 'admin/index.php', null, 'Administration', 'Administration' );
        $this->AddPage( 'system', 'a_install', 'admin/install.php', 'content', 'Installation', 'Administration - Installation' );
        $this->AddPage( 'system', 'a_setup', 'admin/setup.php', 'content', 'Setup', 'Administration - Setup' );
        $this->AddPage( 'system', 'a_test', 'admin/test.php', 'content', 'Test', 'Administration - Test' );

        $this->AddPage( 'system', 'a_logon', 'admin/custom/pages/logon/logon.php', null, 'Anmelden', 'Administration - Anmelden' );
        $this->AddPage( 'system', 'a_navigation', 'admin/custom/pages/navigation/navigation.php', null, 'Navigation', 'Administration - Navigation' );
        $this->AddPage( 'system', 'a_logo', 'admin/custom/pages/logo.php', null, 'Logo', 'Administration - Logo' );

        // add preview only for std and pro
        switch( $oApp->GetoPage() )
        {
            case 'std':
            case 'pro':
                $this->AddPage( 'system', 'a_preview', 'admin_std/custom/pages/preview/preview.php', null, 'Vorschau', 'Administration - Vorschau' );
            break;
        }
        $this->AddPage( 'system', 'a_download', 'admin/custom/pages/download/download.php', null, 'Download', 'Administration - Download' );

        $this->AddPage( 'system', 'nopage', null, null, 'Kein Seiten-Link', null, null, null, null, true );
        $this->AddPage( 'system', 'home', 'index.php', null, 'Home', null, null, null, null, true );
        $this->AddPage( 'system', 'download', 'include/pages/download/download.php', null, 'Download' );
        $this->AddPage( 'system', 'popup', 'include/pages/popup/popup.php', null, 'Popup' );
        $this->AddPage( 'system', 'redirect', 'include/pages/redirect/redirect.php', null, 'Weiterleitung' );

        // Modul pages (additional)
        $this->AddPage( 'modulesadditional', 'a_editor', 'admin/custom/pages/editor/editor.php', 'content', 'Editor', 'Administration - Editor', 99998 );
        $this->AddPage( 'modulesadditional', 'a_editor_template', 'admin/custom/pages/editor/template.php', 'content', 'Templates', 'Administration - Templates', 99999 );

        // Tools pages (additional)
        $this->AddPage( 'toolsadditional', 'a_home', 'admin/custom/pages/home/home.php', 'content', 'Home', 'Administration - Home', 0 );
        $this->AddPage( 'toolsadditional', 'a_logoff', 'admin/custom/pages/logoff/logoff.php', '_top', 'Abmelden', 'Administration - Abmelden', 99999 );

        //
        // END REGION
    }

    /** Install procedure: setup oPage from scratch
    */
    function Install( $sInstall = null )
    {
        if ( $sInstall == '' || $sInstall == 'directory' )
            $this->Directory();

        if ( $sInstall == '' || $sInstall == 'db' )
            $this->Db();
        if ( $sInstall == '' || $sInstall == 'session' )
            $this->Session();
        if ( $sInstall == '' || $sInstall == 'modul' )
        {
            $this->Modul();
            $this->Page();
        }
        if ( $sInstall == '' || $sInstall == 'parameter' )
            $this->Parameter();
        if ( $sInstall == '' || $sInstall == 'user' )
            $this->User();

        return( true );
    }

    /** Setup procedure: setup oPage modules only
    */
    function Setup( $sModul = null )
    {
        $this->Directory();
        $this->Modul( $sModul );
        $this->Page();

        return( true );
    }

    /** Test procedure: make some basic tests
    */
    function Test( $sTest = null )
    {
        if ( $sTest == '' || $sTest == 'parameter' )
            $this->Parameter();
        if ( $sTest == '' || $sTest == 'email' )
            $this->Email();
        if ( $sTest == '' || $sTest == 'image' )
            $this->Image();

        return( true );
    }

    /** Directory: creates directory structure
    */
    function Directory()
    {
        $this->sContent .= '<h1>Directory structure</h1>';

        if ( !$this->_DirectoryCreate( 'cache/' ) )
            return( false );
        if ( !$this->_IndexRedirect( 'cache/index.php' ) )
            return( false );
        if ( !$this->_HtaccessDeny( 'cache/.htaccess' ) )
            return( false );

        if ( !$this->_DirectoryCreate( 'plugin/' ) )
            return( false );
        if ( !$this->_IndexRedirect( 'plugin/index.php' ) )
            return( false );

        if ( !$this->_DirectoryCreate( 'static/' ) )
            return( false );
        if ( !$this->_IndexRedirect( 'static/index.php' ) )
            return( false );

        if ( !$this->_DirectoryCreate( 'temp/' ) )
            return( false );
        if ( !$this->_IndexRedirect( 'temp/index.php' ) )
            return( false );
        if ( !$this->_HtaccessDeny( 'temp/.htaccess' ) )
            return( false );

        return( true );
    }

    /** Directory: creates a directory
        @param $sPath subpath below $sRoot, including trailing '/'
    */
    function _DirectoryCreate( $sPath )
    {
        global $sRoot;

        if ( !is_dir( $sRoot . $sPath ) )
        {
            if ( !mkdir( $sRoot . $sPath, 0777 ) )
            {
                $this->sContent .= '<p class="texterror">' . $sPath . ' failed</p>' . "\n";
                return( false );
            }
            $this->sContent .= '<p>' . $sPath . ' created</p>' . "\n";
        }
        else
            $this->sContent .= '<p>' . $sPath . ' ok</p>' . "\n";

        return( true );
    }

    /** HtaccessDeny: creates a htaccess file to deny access
        @param $sFilename subpath below $sRoot, eg. 'cache/.htaccess'
    */
    function _HtaccessDeny( $sFilename )
    {
        global $sRoot;

        if ( !is_file( $sRoot . $sFilename ) )
        {
            if ( !FileHtaccessDeny( $sRoot . $sFilename ) )
            {
                $this->sContent .= '<p class="texterror">' . $sFilename . ' failed</p>' . "\n";
                return( false );
            }
            else
                $this->sContent .= '<p>' . $sFilename . ' created</p>' . "\n";
        }
        else
            $this->sContent .= '<p>' . $sFilename . ' ok</p>' . "\n";

        return( true );
    }

    /** IndexRedirect: creates a index.php file to redirect to the homepage
        @param $sFilename subpath below $sRoot, eg. 'cache/index.php'
    */
    function _IndexRedirect( $sFilename )
    {
        global $sRoot;

        if ( !is_file( $sRoot . $sFilename ) )
        {
            if ( !FileIndexRedirect( $sRoot . $sFilename, $sRoot ) )
            {
                $this->sContent .= '<p class="texterror">' . $sFilename . ' failed</p>' . "\n";
                return( false );
            }
            else
                $this->sContent .= '<p>' . $sFilename . ' created</p>' . "\n";
        }
        else
            $this->sContent .= '<p>' . $sFilename . ' ok</p>' . "\n";

        return( true );
    }

    /** Db: checks the database connection
    */
    function Db()
    {
        global $oDb;
        global $oApp;

        $this->sContent .= '<h1>Testing database connection</h1>';

        // Test, if database is accessible
        if ( !$oDb->Test() )
        {
            Header( 'Location: ' . $oApp->GetRoot() . 'admin/custom/error/error.php?error=db_connect' );
            exit();
        }

        $this->sContent .= '<p>ok</p>' . "\n";
        return( true );
    }

    /** Session: setup session table in the database (or check, if table is ok)
    */
    function Session()
    {
        global $oApp;
        global $oFactory;

        $this->sContent .= '<h1>Setup session table</h1>';

        // Setup session table
        $oObject = $oFactory->Session( null, false );
        if ( !$oObject->Setup() )
        {
            $this->sContent .= '<p class="texterror">failed</p>' . "\n";
            return( false );
        }

        $this->sContent .= '<p>ok</p>' . "\n";
        return( true );
    }

    /** Modul: setup module tables in the database and add page entries for this modules
        loads custom modul list
        calls setup for each modul
        adds result to $sContent
        $oDb must be created first
    */
    function Modul( $sModulFilter = null )
    {
        global $oFactory;
        global $oPlugins;
        global $oApp;

        $this->sContent .= '<h1>Setup Modules</h1>';

        // Create module table (currently only used in setup)
        // todo: find something usefull for this table or get rid of it
        // todo: maybe display some statistiks on the home page of the backend
        $oModul = $oFactory->Modul( 'CModule' );
        $oModul->Setup();
        $oModul->SetupData();

        // get all module entries from the table and call setup for each of them
        // todo: get max value (but not 99999) from the database
        $iSortTools = 0;
        $iSortModul = 0;

        // combine tools and modules entries
        $aModules = array_merge( $this->aTools, $this->aModules );
        // get the entries from the database
        $oModul->QueryData();
        foreach( $oModul->GetResult() as $aRow )
            $aModules[ $aRow[ 'name' ] ] = $aRow[ 'name' ];

        // Empty rights array
        $aRights = array();

        // now run setup
        foreach( $aModules as $sModul )
        {
            if ( $sModul == $sModulFilter || $sModulFilter == '' )
            {
                $this->sContent .= '<h2>' . $sModul . '</h2>' . "\n";

                $oObject = $oFactory->Modul( $sModul );
                if ( !is_object( $oObject ) )
                    $this->sContent .= '<p class="texterror">class not found</p>' . "\n";
                elseif ( !method_exists( $oObject, 'Setup' ) )
                    $this->sContent .= '<p class="texterror">method Setup() not found</p>' . "\n";
                elseif ( $oObject->GetSetup() == false )
                    $this->sContent .= '<p class="texterror">method Setup() disabled</p>' . "\n";
                elseif ( $oObject->Setup() )
                {
                    $this->_ModulInsert( $oModul, $sModul );

                    $this->sContent .= '<p>table \'' . $oObject->GetTable( 1 ) . '\' ok</p>' . "\n";

                    // Setup general module data
                    $oObject->SetupData();

                    // Call Setup to let the plugins add data
                    $oPlugins->SetupModulData( $oObject, $sModul );

                    $this->sContent .= '<p>table \'' . $oObject->GetTable( 1 ) . '\' data ok</p>' . "\n";

                    // Update Fulltext index
                    if ( method_exists( $oObject, 'FulltextUpdate' ) )
                    {
                        if ( $oObject->FulltextUpdate() )
                            $this->sContent .= '<p>table \'' . $oObject->GetTable( 1 ) . '\' fulltext ok</p>' . "\n";
                    }

                    // Create page entries for the module
                    $aPages = array();
                    if ( $oObject->GetSetupPages() )
                    {
                        // is the current module in tools
                        if ( $this->IsTool( $sModul ) )
                        {
                            $iSortTools = $oObject->SetupPages( $aPages, $iSortTools );
                            foreach( $aPages as $sPage => $aPage )
                                $this->AddPage( 'tools', $sPage, $aPage[ 'path' ], $aPage[ 'target' ], $aPage[ 'title' ], $aPage[ 'title_display' ], $aPage[ 'sort' ], $aPage[ 'title_navigation' ], $aPage[ 'title_menu' ], $aPage[ 'page_content' ], $aPage[ 'page_view' ], $aPage[ 'page_admin' ] );
                        }
                        // no, its a modul module
                        else
                        {
                            $iSortModul = $oObject->SetupPages( $aPages, $iSortModul );
                            foreach( $aPages as $sPage => $aPage )
                                $this->AddPage( 'modules', $sPage, $aPage[ 'path' ], $aPage[ 'target' ], $aPage[ 'title' ], $aPage[ 'title_display' ], $aPage[ 'sort' ], $aPage[ 'title_navigation' ], $aPage[ 'title_menu' ], $aPage[ 'page_content' ], $aPage[ 'page_view' ], $aPage[ 'page_admin' ] );
                        }
                    }
                    else
                    {
                        $oObject->SetupPages( $aPages );
                        foreach( $aPages as $sPage => $aPage )
                            $this->AddPage( 'system', $sPage, $aPage[ 'path' ], $aPage[ 'target' ], $aPage[ 'title' ], $aPage[ 'title_display' ], $aPage[ 'sort' ], $aPage[ 'title_navigation' ], $aPage[ 'title_menu' ], $aPage[ 'page_content' ], $aPage[ 'page_view' ], $aPage[ 'page_admin' ] );
                    }

                    // Create rights
                    $sName = $oObject->GetName();
                    $aRights[ $sName . '#admin' ] = array( 'title' => ucwords( str_replace( '_', ' ', $sName ) ) . ' - Admin' );
                }
                else
                    $sContent .= '<p class="texterror">table \'' . $oObject->GetTable( 1 ) . '\' error</p>' . "\n";
            }
        }

        // Insert rights
        if ( $oApp->GetFeature( 'user', 'rights' ) == 1 || $oApp->GetFeature( 'user', 'rights' ) == 2 )
        {
            // Check rights in the database
            $oObject = $oFactory->Modul( 'CUserRight' );
            foreach( $aRights as $sName => $aRight )
                $this->_UserRightInsert( $oObject, $sName, $aRight );
        }
    }

    /** Modul: Inserts a module into the module table
    */
    function _ModulInsert( &$oModul, $sName )
    {
        $this->sContent .= '<h2>Checking module entry ' . $sName . '</h2>';

        if ( $oModul->Count( $oModul->GetField( 'name' ) . '= \'' . $sName . '\'' ) == 0 )
        {
            $aValues = array();
            $aValues[ 'name' ] = $sName;
            $aValues[ 'display' ] = 'y';

            $oModul->Insert( $aValues );
            $this->sContent .= '<p>created</p>' . "\n";
        }
        else
            $this->sContent .= '<p>ok</p>' . "\n";
    }

    /** Page: setup page table entries in the database
        checks, if important page entries are available
        missing entries are created
        adds result to $sContent
        $oDb must be created first
    */
    function Page()
    {
        global $oFactory;

        // check page entries
        $this->sContent .= '<h1>Checking page entries</h1>';

        // insert page entries into the table
        $oObject = $oFactory->Modul( 'CPage' );
        foreach( $this->aPages as $sName => $aPage )
            $this->_PageInsert( $oObject, $sName, $aPage );
        foreach( $this->aPagesModules as $sName => $aPage )
            $this->_PageInsert( $oObject, $sName, $aPage );
        foreach( $this->aPagesModulesAdditional as $sName => $aPage )
            $this->_PageInsert( $oObject, $sName, $aPage );
        foreach( $this->aPagesTools as $sName => $aPage )
            $this->_PageInsert( $oObject, $sName, $aPage );
        foreach( $this->aPagesToolsAdditional as $sName => $aPage )
            $this->_PageInsert( $oObject, $sName, $aPage );
        foreach( $this->aPagesContent as $sName => $aPage )
            $this->_PageInsert( $oObject, $sName, $aPage );

        // insert menu entries into the database
        $oObject = $oFactory->Modul( 'CPageModul' );
        foreach( $this->aPagesModules as $sName => $aPage )
            $this->_PageMenuInsert( $oObject, $sName, $aPage );
        foreach( $this->aPagesModulesAdditional as $sName => $aPage )
            $this->_PageMenuInsert( $oObject, $sName, $aPage );

        $oObject = $oFactory->Modul( 'CPageTools' );
        foreach( $this->aPagesTools as $sName => $aPage )
            $this->_PageMenuInsert( $oObject, $sName, $aPage );
        foreach( $this->aPagesToolsAdditional as $sName => $aPage )
            $this->_PageMenuInsert( $oObject, $sName, $aPage );
    }

    /** Page: Inserts a page entry into the page table
    */
    function _PageInsert( &$oModul, $sName, $aPage )
    {
        $this->sContent .= '<h2>Checking page entry ' . $sName . '</h2>';
        if ( $oModul->Count( $oModul->GetField( 'name' ) . '= \'' . $sName . '\'' ) == 0 )
        {
            if ( !isset( $aPage[ 'name' ] ) )
                $aPage[ 'name' ] = $sName;
            if ( !isset( $aPage[ 'title_display' ] ) )
                $aPage[ 'title_display' ] = $aPage[ 'title' ];
            if ( !isset( $aPage[ 'title_navigation' ] ) )
                $aPage[ 'title_navigation' ] = $aPage[ 'title' ];
            if ( !isset( $aPage[ 'title_menu' ] ) )
                $aPage[ 'title_menu' ] = $aPage[ 'title' ];

            $oModul->Insert( $aPage );
            $this->sContent .= '<p>created</p>' . "\n";
        }
        else
            $this->sContent .= '<p>ok</p>' . "\n";
    }

    /** Page: Inserts a page entry into the pagemenu table
    */
    function _PageMenuInsert( &$oModul, $sName, $aPage )
    {
        $this->sContent .= '<h2>Checking menu entry ' . $sName . '</h2>';

        if ( $oModul->Count( $oModul->GetField( 'name' ) . '= \'' . $sName . '\'' ) == 0 )
        {
            if ( !isset( $aPage[ 'name' ] ) )
                $aPage[ 'name' ] = $sName;
            if ( !isset( $aPage[ 'display' ] ) )
                $aPage[ 'display' ] = 'y';
            if ( !isset( $aPage[ 'parent_name' ] ) )
                // get "a_page" from "a_page_modul"
                if ( substr_count( $aPage[ 'name' ], '_' ) > 1 )
                    $aPage[ 'parent_name' ] = substr( $aPage[ 'name' ], 0, strrpos( $aPage[ 'name' ], '_' ) );

            $oModul->Insert( $aPage );
            $this->sContent .= '<p>created</p>' . "\n";
        }
        else
            $this->sContent .= '<p>ok</p>' . "\n";
    }

    /** Parameter: insert some basic entries
    */
    function Parameter()
    {
        global $oApp;
        global $oFactory;

        $this->sContent .= '<h1>Checking parameters</h1>';

        $oObject = $oFactory->Modul( 'CParameter' );
        $oObject->SetLanguagePostfix( $oApp->GetLanguagePostfix( -1, true ) );
        $oObject->QueryData();

        $this->sContent .= '<h2>Checking parameter email</h2>';
        if ( $oObject->Get( 'email' ) == '' )
            $oObject->Set( 'email', 'office@example.com' );
        if ( $oObject->Get( 'email' ) == 'office@example.com' )
            $this->sContent .= '<p class="texterror">change parameter email to a real email adress!' . "\n";

        $this->sContent .= '<h2>Checking parameter email_name</h2>';
        if ( $oObject->Get( 'email_name' ) == '' )
            $oObject->Set( 'email_name', 'Your name goes here' );
        if ( $oObject->Get( 'email_name' ) == 'Your name goes here' )
            $this->sContent .= '<p class="texterror">change parameter email_name to your real name!' . "\n";

        $this->sContent .= '<h2>Checking parameter rows</h2>';
        if ( $oObject->Get( 'rows' ) == '' )
            $oObject->Set( 'rows', '20' );

        $this->sContent .= '<h2>Checking parameter counter_mode</h2>';
        if ( $oObject->Get( 'counter_mode' ) == '' )
            $oObject->Set( 'counter_mode', 'm' );

        $this->sContent .= '<p>ok</p>' . "\n";
    }

    /** User: create the default admin user (if it not exists)
    */
    function User()
    {
        global $oApp;
        global $oFactory;

        $this->sContent .= '<h1>Checking users</h1>';

        // setup user data
        $aUsers = array();
        $aUsers[ 'admin' ] = array( 'password' => 'admin', 'fullname' => 'Administrator' );

        // Check usernames in the database and update $aUser with ID
        $oObject = $oFactory->Modul( 'CUser' );
        foreach( array_keys( $aUsers ) as $sName )
            $this->_UserInsert( $oObject, $sName, $aUsers[ $sName ] );

        if ( $oApp->GetFeature( 'user', 'rights' ) == 1 || $oApp->GetFeature( 'user', 'rights' ) == 2 )
        {
            // setup rights data
            $aRights = array();
            $aRights[ 'admin#admin' ] = array( 'title' => 'Admin' );

            // Check rights in the database and update $aRight with ID
            $oObject = $oFactory->Modul( 'CUserRight' );
            foreach( array_keys( $aRights ) as $sName )
                $this->_UserRightInsert( $oObject, $sName, $aRight[ $sName ] );
        }

        if ( $oApp->GetFeature( 'user', 'rights' ) == 1 )
        {
            // Assign the rights to the user
            $oObject = $oFactory->Modul( 'CUserUserRight' );
            foreach( $aUsers as $sUser => $aUser )
                foreach( $aRights as $sRight => $aRight )
                    $this->_UserUserRightInsert( $oObject, $sUser, $sRight, $aUser[ 'id' ], $aRight[ 'id' ] );
        }

        if ( $oApp->GetFeature( 'user', 'rights' ) == 2 )
        {
            // setup group data
            $aGroups = array();
            $aGroups[ 'admin' ] = array( 'title' => 'Admin' );

            // Check groups in the database and update $aGroup with ID
            $oObject = $oFactory->Modul( 'CUserGroup' );
            foreach( array_keys( $aGroups ) as $sName )
                $this->_GroupInsert( $oObject, $sName, $aGroups[ $sName ] );

            // Assign the group to the user
            $oObject = $oFactory->Modul( 'CUserUserGroup' );
            foreach( $aGroups as $sGroup => $aGroup )
                foreach( $aUsers as $sUser => $aUser )
                    $this->_UserUserGroupInsert( $oObject, $sUser, $sGroup, $aUser[ 'id' ], $aGroup[ 'id' ] );

            // Assign the right to the group
            $oObject = $oFactory->Modul( 'CUserGroupRight' );
            foreach( $aGroups as $sGroup => $aGroup )
                foreach( $aRights as $sRight => $aRight )
                    $this->_UserGroupRightInsert( $oObject, $sGroup, $sRight, $aGroup[ 'id' ], $aRight[ 'id' ] );
        }
    }

    /** User: Inserts an entry into the table
    */
    function _UserInsert( &$oModul, $sName, &$aUser )
    {
        $this->sContent .= '<h2>Checking user entry ' . $sName . '</h2>';

        $oModul->ClearFilters();
        $oModul->SetFilter( 'username', $sName );
        $oModul->QueryData();

        if ( $oModul->IsData() )
        {
            $aUser[ 'id' ] = $oModul->Get( 'id' );
            $this->sContent .= '<p>ok</p>' . "\n";
        }
        else
        {
            if ( !isset( $aUser[ 'username' ] ) )
                $aUser[ 'username' ] = $sName;
            if ( !isset( $aUser[ 'fullname' ] ) )
                $aUser[ 'fullname' ] = $aUser[ 'username' ];
            if ( !isset( $aUser[ 'display' ] ) )
                $aUser[ 'display' ] = 'y';

            $oModul->Insert( $aUser );
            $this->sContent .= '<p>created</p>' . "\n";
        }
    }

    /** User: Right: Inserts an entry into the table
    */
    function _UserRightInsert( &$oModul, $sName, &$aRight )
    {
        $this->sContent .= '<h3>Checking right entry ' . $sName . '</h3>';

        $oModul->ClearFilters();
        $oModul->SetFilter( 'name', $sName );
        $oModul->QueryData();

        if ( $oModul->IsData() )
        {
            $aRight[ 'id' ] = $oModul->Get( 'id' );
            $this->sContent .= '<p>ok</p>' . "\n";
        }
        else
        {
            if ( !isset( $aRight[ 'name' ] ) )
                $aRight[ 'name' ] = $sName;
            if ( !isset( $aRight[ 'title' ] ) )
                $aRight[ 'title' ] = $aRight[ 'name' ];
            if ( !isset( $aRight[ 'display' ] ) )
                $aRight[ 'display' ] = 'y';

            $oModul->Insert( $aRight );
            $this->sContent .= '<p>created</p>' . "\n";
        }
    }

    /** User: UserRight: Inserts an entry into the table
    */
    function _UserUserRightInsert( &$oModul, $sUser, $sRight, $iUser, $iRight )
    {
        $this->sContent .= '<h3>Checking right ' . $sRight . ' for user ' . $sUser . '</h3>';

        $oModul->SetFilter( 'id', $iUser );
        $oModul->SetFilter( 'rightx', $iRight );
        $oModul->QueryData();

        if ( $oModul->IsData() )
            $this->sContent .= '<p>ok</p>' . "\n";
        else
        {
            $aValues = array();
            $aValues[ 'id' ] = $iUser;
            $aValues[ 'rightx' ] = $iRight;

            $oModul->Insert( $aValues );
            $this->sContent .= '<p>created</p>' . "\n";
        }
    }

    /** User: Group: Inserts an entry into the table
    */
    function _GroupInsert( &$oModul, $sName, &$aGroup )
    {
        $this->sContent .= '<h3>Checking group entry ' . $sName . '</h3>';

        $oModul->ClearFilters();
        $oModul->SetFilter( 'name', $sName );
        $oModul->QueryData();

        if ( $oModul->IsData() )
        {
            $aGroup[ 'id' ] = $oModul->Get( 'id' );
            $this->sContent .= '<p>ok</p>' . "\n";
        }
        else
        {
            if ( !isset( $aGroup[ 'name' ] ) )
                $aGroup[ 'name' ] = $sName;
            if ( !isset( $aGroup[ 'title' ] ) )
                $aGroup[ 'title' ] = $aGroup[ 'name' ];
            if ( !isset( $aGroup[ 'display' ] ) )
                $aGroup[ 'display' ] = 'y';

            $oModul->Insert( $aGroup );
            $this->sContent .= '<p>created</p>' . "\n";
        }
    }

    /** User: UserGroup: Inserts a user into the group
    */
    function _UserUserGroupInsert( &$oModul, $sUser, $sGroup, $iUser, $iGroup )
    {
        $this->sContent .= '<h3>Checking group ' . $sGroup . ' for user ' . $sUser . '</h3>';

        $oModul->SetFilter( 'id', $iUser );
        $oModul->SetFilter( 'groupx', $iGroup );
        $oModul->QueryData();

        if ( $oModul->IsData() )
            $this->sContent .= '<p>ok</p>' . "\n";
        else
        {
            $aValues = array();
            $aValues[ 'id' ] = $iUser;
            $aValues[ 'groupx' ] = $iGroup;

            $oModul->Insert( $aValues );
            $this->sContent .= '<p>created</p>' . "\n";
        }
    }

    /** User: GroupRight: Assigns a right to a group
    */
    function _UserGroupRightInsert( &$oModul, $sGroup, $sRight, $iGroup, $iRight )
    {
        $this->sContent .= '<h3>Checking group ' . $sGroup . ' for user ' . $sGroup . '</h3>';

        $oModul->SetFilter( 'id', $iGroup );
        $oModul->SetFilter( 'rightx', $iRight );
        $oModul->QueryData();

        if ( $oModul->IsData() )
            $this->sContent .= '<p>ok</p>' . "\n";
        else
        {
            $aValues = array();
            $aValues[ 'id' ] = $iGroup;
            $aValues[ 'rightx' ] = $iRight;

            $oModul->Insert( $aValues );
            $this->sContent .= '<p>created</p>' . "\n";
        }
    }

    /** Email: test email sending
    */
    function Email()
    {
        global $oApp;
        global $sRoot;
        global $oFactory;
        global $oParameter;

        $this->sContent .= '<h1>Checking email</h1>';

        // try to send an e-mail
        $oMail = $oFactory->Control( 'CMail' );
        $oMail->SetFrom( $oParameter->Get( 'email' ), $oParameter->Get( 'email_name' ) );
        $oMail->SetTo( $oParameter->Get( 'email'), $oParameter->Get( 'email_name' ) );
        $sTemplate = 'admin/custom/setup/email.tpl';
        if ( !$oMail->Send( $sRoot . $sTemplate ) )
        {
            Header( 'Location: ' . $oApp->GetRoot() . 'admin/custom/pages/error/error.php?error=email_send' );
            exit();
        }

        $this->sContent .= '<p>ok</p>' . "\n";
    }

    /** Image: test image processing
    */
    function Image()
    {
        global $sRoot;
        global $oApp;
        global $oFactory;

        $this->sContent .= '<h1>Checking image resize</h1>';

        // try to copy and resize the test image
        $sSource = 'images/opage.gif';
        $sImage = 'temp/temp.gif';
        // use FileCopy here to get a new file without read-only attribute
        if ( !FileCopy( $sRoot . $sSource, $sRoot . $sImage ) )
        {
            Header( 'Location: ' . $oApp->GetRoot() . 'admin/custom/pages/error/error.php?error=image_copy' );
            exit();
        }
        $oImage = $oFactory->Image( $sRoot . $sImage );
        if ( !$oImage->Resize( 1, 1, true ) )
        {
            unlink( $sRoot . $sImage );
            Header( 'Location: ' . $oApp->GetRoot() . 'admin/custom/pages/error/error.php?error=image_resize' );
            exit();
        }

        // get image size
        $aSize = GetImageSize( $sRoot . $sImage );

        // remove image
        unlink( $sRoot . $sImage );

        // check, if image has been really resized.
        if ( $aSize[ 0 ] != 1 || $aSize[ 1 ] != 1 )
        {
            Header( 'Location: ' . $oApp->GetRoot() . 'admin/custom/pages/error/error.php?error=image_resize' );
            exit();
        }

        $this->sContent .= '<p>ok</p>' . "\n";
    }

    /** Get Setup result
    */
    function GetContent()
    {
        return( $this->sContent );
    }

    /** AddModul adds a module to aModules array
        @param $sModul class name of the module
        @return void
    */
    function AddModul( $sModul )
    {
        $this->aModules[ $sModul ] = $sModul;
    }
    function IsModul( $sModul )
    {
        return( isset( $this->aModules[ $sModul ] ) );
    }

    /** AddTool adds a module to aTools array
        @param $sModul class name of the module
        @return void
    */
    function AddTool( $sModul )
    {
        $this->aTools[ $sModul ] = $sModul;
    }
    function IsTool( $sModul )
    {
        return( isset( $this->aTools[ $sModul ] ) );
    }

    /** AddPage adds a page to aPages array
        @return void
    */
    function AddPage( $sType, $sName, $sPath, $sTarget = null, $sTitle = null, $sTitleDisplay = null, $sSort = null, $sTitleNavigation = null, $sTitleMenu = null, $fContent = null, $fView = null, $fAdmin = null )
    {
        $aPages = null;

        switch( $sType )
        {
            case 'system':
                $aPages = &$this->aPages;
            break;

            case 'modules':
                $aPages = &$this->aPagesModules;
            break;
            case 'modulesadditional':
                $aPages = &$this->aPagesModules;
            break;

            case 'tools':
                $aPages = &$this->aPagesTools;
            break;
            case 'toolsadditional':
                $aPages = &$this->aPagesTools;
            break;

            case 'content':
                $aPages = &$this->aPagesContent;
            break;

            default:
                die( 'invalid AddPage type: ' . $sType . ' (' . __FILE__ . ', line ' . __LINE__ . ')' );
            break;
        }

        $aPage = array
        (
            'path' => $sPath,
            'target' => $sTarget,
            'title' => $sTitle,
            'title_display' => $sTitleDisplay,
            'sort' => $sSort,
            'title_navigation' => $sTitleNavigation,
            'title_menu' => $sTitleMenu,
            'page_content' => ( $fContent ? 'y' : 'n' ),
            'page_view' => ( $fView ? 'y' : 'n' ),
            'page_admin' => ( $fAdmin ? 'y' : 'n' )
        );

        if ( isset( $aPages[ $sName ] ) )
            $this->sContent .= '<p class="texterror">Page: Overwriting \'' . $sName . '\' with ' . $aPage[ 'path' ] . '</p>' . "\n";

        $aPages[ $sName ] = $aPage;
    }

    function SetContent( $sValue )
    {
        $this->sContent = $sValue;
    }
    function AddContent( $sValue )
    {
        $this->sContent .= $sValue;
    }
    function Error( $sValue )
    {
        $this->AddContent( '<p class="texterror">' . $sValue . '</p>' . "\n" );
    }
}
