<?php

defined( 'oPage' ) or die();

include_once $sRoot . 'include/tpl/tpl.php';
include_once $sRoot . 'include/tools.php';

class CTemplate extends CTpl
{
    /** Name of template, modul, etc.
        Used for loading templated specific resources
        Used for creating cache filename
    */
    var $sName;

    /** Resource file which will be loaded after the default resource, but before the custom resource files
    */
    var $sResource;

    function CTemplate( $sFilename, $sBlock = null, $aParameter = null, $sLanguage = null, $sName = null, $sResource = null )
    {
        global $oApp;
        global $oFactory;

        if ( $sLanguage === null || $sLanguage === '' )
        {
            $sLanguage = $oApp->GetLanguageShortText();
        }

        $this->sName = $sName;
        $this->sResource = $sResource;

        $this->oDate = $oFactory->Date();
        $this->oNumber = $oFactory->Number();

        // set current charset
        $this->sCharset = $oApp->GetCharset();

        // currently templates and resourcefiles are windows-1252
        $this->sTemplateCharset = 'windows-1252';

        // set root
        $this->SetRoot( $oApp->GetFileRoot() );

        // set cache directory
        $this->SetCacheDirectory( 'cache/templates/' );

        // Check, if oPage is in production mode.
        if ( ( $oApp->IsProduction() && $oApp->GetFeature( 'templates', 'caching' ) != 0 ) || $oApp->GetFeature( 'templates', 'caching' ) == 1 )
        {
            $this->SetCache( true );
        }

        $this->CTpl( $sFilename, $sBlock, $aParameter, $sLanguage, $sResource );

        // Globale Variablen
        $this->Assign( 'global.browser.name', $oApp->GetBrowser() );
        $this->Assign( 'global.browser.version', $oApp->GetBrowserVersion() );
        $this->Assign( 'global.clientip', $oApp->GetClientIP() );
        $this->Assign( 'global.servername', $oApp->GetServerName() );
        $this->Assign( 'global.queryseparator', $oApp->GetQuerySeparator() );
        $this->Assign( 'global.paramseparator', $oApp->GetParamSeparator() );
        $this->Assign( 'global.http', $oApp->GetHttp() );
        $this->Assign( 'global.https', $oApp->GetHttps() );
        $this->Assign( 'global.root', $oApp->GetRoot() );
        $this->Assign( 'global.root_css', $oApp->GetRoot() );
        $this->Assign( 'global.root_image', $oApp->GetRoot() );
        $this->Assign( 'global.root_javascript', $oApp->GetRoot() );
        $this->Assign( 'global.path', $oApp->PathLink( '' ) );
        $this->Assign( 'global.dir', $oApp->GetDir() );
        $this->Assign( 'global.charset', $oApp->GetCharset() );
        $this->Assign( 'global.language', $oApp->GetLanguageShortText() );
        $this->Assign( 'global.language_id', $oApp->GetLanguage() );
        $this->Assign( 'global.language_full', $oApp->GetLanguageText() );
        $this->Assign( 'global.language_postfix', $oApp->GetLanguagePostfix() );
        $this->Assign( 'global.editor_fontformats', 'h4;h5;h6;p' );

        // Set globals from config
        $aGlobals = $oApp->GetFeature( 'global' );
        if ( is_array( $aGlobals ) )
        {
            foreach( $aGlobals as $sName => $sValue )
            {
                $this->Assign( 'global.' . $sName, $sValue );
            }
        }

        $sScriptname = $oApp->GetScriptName();
        $aParameter = $oApp->GetParameters();
        $this->Assign( 'global.parameter', $aParameter );
        $sParameter = HttpParam( $aParameter, $oApp->GetQuerySeparator(), $oApp->GetParamSeparator() );
        $this->Assign( 'global.requesturi', $sScriptname . $sParameter );
        $this->Assign( 'global.scriptname', $sScriptname );
        $this->Assign( 'global.parameters', substr( $sParameter, 1 ) );
    }

    /** make path relative
        @param &$sFilename
        @return true/false
    */
    function ReadFilenamePrepare( &$sFilename )
    {
        global $sRoot;

        if ( substr( $sFilename, 0, 2 ) != '..' )
        {
            $sFilename = $sRoot . $sFilename;
        }

        return( true );
    }

    /** prepares a filename for the cachefile
        make path relative, or something else
        file is only used, if this function returns true
        @param &$sFilename
        @return true/false
    */
    function CacheFilenamePrepare( $sBlock, &$sFilename )
    {
        global $oApp;
        global $oClient;

        if ( isset( $this->sName ) )
        {
            if ( $sBlock != '' )
            {
                $sBlock .= '_';
            }

            $sBlock .= $this->sName;
        }

        if ( $oApp->GetFeature( 'user', 'client' ) == 1 )
        {
            if ( !$oApp->GetAdmin() )
            {
                if ( $oClient->IsData() )
                {
                    if ( $sBlock <> '' )
                    {
                        $sBlock .= '_';
                    }

                    $sBlock .= $oClient->Get( 'number' );
                }
            }
        }

        return( parent::CacheFilenamePrepare( $sBlock, $sFilename ) );
    }

    function CacheRead( $sFilename, $iTemplateLastModified )
    {
        global $oApp;

        // When cachecreate is set, then recreate the template cache file.
        if ( $oApp->GetParameter( 'cachecreate' ) !== null )
        {
            return( false );
        }

        return( parent::CacheRead( $sFilename, $iTemplateLastModified ) );
    }

    /** Creates an array with all res.variables used in the template
        @param $sFilename filename of the template
        @param $sTemplate buffer with template content
        @return string with template content with replaced resource strings.
    */
    function ResourceReplace( $sFilename, &$sTemplate, $sResource = null )
    {
        global $oApp;
        global $sRoot;
        global $oPlugins;

        // load resource files

        // System resource file: include/resource/{language}.res
        // System resource file: include_std/resource/{language}.res
        // System resource file: include_pro/resource/{language}.res
        // System resource file: include/resource/{modulname}#{language}.res
        // System resource file: include_std/resource/{modulname}#{language}.res
        // System resource file: include_pro/resource/{modulname}#{language}.res
        // System admin resource file: admin/custom/resource/{language}.res
        // System admin resource file: admin_std/custom/resource/{language}.res
        // System admin resource file: admin_pro/custom/resource/{language}.res
        // System admin resource file: admin/custom/resource/{modulname}#{language}.res
        // System admin resource file: admin_std/custom/resource/{modulname}#{language}.res
        // System admin resource file: admin_pro/custom/resource/{modulname}#{language}.res
        // Plugin resource file: plugin/{pluginname}/resource/{language}.res
        // Plugin resource file: plugin/{pluginname}/resource/{modulname}#{language}.res
        // Resource file from function parameter $sResource
        // Custom resource file: custom/resource/{language}.res
        // Custom resource file: custom/resource/{modulname}#{language}.res
        // Custom admin resource file: custom/admin/custom/resource/{language}.res
        // Custom admin resource file: custom/admin/custom/resource/{modulname}#{language}.res
        // Template resource file: {filename}#{language}.res

        // skip resource replace if language is not set
        if ( isset( $this->sLanguage ) )
        {
            // load global resources
            $sResourceFilename = $sRoot . 'include/resource/' . $this->sLanguage . '.res';
            $this->ResourceRead( $sResourceFilename, $sResource );
            $sResourceFilename = $sRoot . 'include_std/resource/' . $this->sLanguage . '.res';
            $this->ResourceRead( $sResourceFilename, $sResource );
            $sResourceFilename = $sRoot . 'include_pro/resource/' . $this->sLanguage . '.res';
            $this->ResourceRead( $sResourceFilename, $sResource );
            if ( isset( $this->sName ) )
            {
                $sResourceFilename = $sRoot . 'include/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                $sResourceFilename = $sRoot . 'include_std/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                $sResourceFilename = $sRoot . 'include_pro/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
            }

            // Load admin Resources
            if ( $oApp->GetAdmin() )
            {
                $sResourceFilename = $sRoot . 'admin/custom/resource/' . $this->sLanguage  . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                $sResourceFilename = $sRoot . 'admin_std/custom/resource/' . $this->sLanguage  . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                $sResourceFilename = $sRoot . 'admin_pro/custom/resource/' . $this->sLanguage  . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                if ( isset( $this->sName ) )
                {
                    $sResourceFilename = $sRoot . 'admin/custom/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                    $this->ResourceRead( $sResourceFilename, $sResource );
                    $sResourceFilename = $sRoot . 'admin_std/custom/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                    $this->ResourceRead( $sResourceFilename, $sResource );
                    $sResourceFilename = $sRoot . 'admin_pro/custom/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                    $this->ResourceRead( $sResourceFilename, $sResource );
                }
            }

            // Include standard resource files from plugins
            // plugin/{pluginname}/resource/{language}.res
            if ( is_object( $oPlugins ) )
            {
                $oPlugins->ResourceRead( $this, $sResource, $this->sLanguage, $this->sName );
            }

            // $sFilename: relative or absolut path
            // relative: plugin/{pluginname}/resource/{language}.res
            // absolut: d:\htdocs\website\plugin/{pluginname}/resource/{language}.res
            // -> make all relative
            $sBuffer = substr( str_replace( '\\', '/', realpath( $sFilename ) ), strlen( $sRoot ) );

            // if template filename starts with plugin, load plugin resources
/*
            if ( substr( $sBuffer, 0, 7 ) == 'plugin/' )
            {
                $iPos = strpos( $sBuffer, '/', 7 );
                $sResourceFilename = $sRoot . substr( $sBuffer, 0, $iPos + 1 ) . 'resource/' . $this->sLanguage  . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                if ( isset( $this->sName ) )
                {
                    $sResourceFilename = $sRoot . substr( $sBuffer, 0, $iPos + 1 ) . 'resource/' . $this->sName . '#' . $this->sLanguage  . '.res';
                    $this->ResourceRead( $sResourceFilename, $sResource );
                }
            }
            // if template filename starts with custom/plugin, load plugin resources
            elseif ( substr( $sBuffer, 0, 14 ) == 'custom/plugin/' )
            {
                $iPos = strpos( $sBuffer, '/', 14 );
                $sResourceFilename = $sRoot . substr( $sBuffer, 7, $iPos - 7 + 1 ) . 'resource/' . $this->sLanguage  . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                if ( isset( $this->sName ) )
                {
                    $sResourceFilename = $sRoot . substr( $sBuffer, 7, $iPos - 7 + 1 ) . 'resource/' . $this->sName . '#' . $this->sLanguage  . '.res';
                    $this->ResourceRead( $sResourceFilename, $sResource );
                }
            }
 */
        }

        // load resources
        if ( $this->sResource !== null && $this->sResource !== '' )
        {
            $this->ResourceRead( $this->sResource, $sResource );
        }

        if ( isset( $this->sLanguage ) )
        {
            // Load custom Resources
            $sResourceFilename = $sRoot . 'custom/resource/' . $this->sLanguage . '.res';
            $this->ResourceRead( $sResourceFilename, $sResource );
            if ( isset( $this->sName ) )
            {
                $sResourceFilename = $sRoot . 'custom/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
            }

            // Load custom admin Resources
            if ( $oApp->GetAdmin() )
            {
                $sResourceFilename = $sRoot . 'custom/admin/resource/' . $this->sLanguage  . '.res';
                $this->ResourceRead( $sResourceFilename, $sResource );
                if ( isset( $this->sName ) )
                {
                    $sResourceFilename = $sRoot . 'custom/admin/resource/' . $this->sName . '#' . $this->sLanguage . '.res';
                    $this->ResourceRead( $sResourceFilename, $sResource );
                }
            }
        }

        return( parent::ResourceReplace( $sFilename, $sTemplate, $sResource ) );
    }

    /** Prepare resources: convert string buffer into array
        @param $sResource string buffer with resources: 1. line: name, 2. line: text
        @return array() with value/name pairs
    */
    function _ResourcePrepare( $sResource )
    {
        $aRes = parent::_ResourcePrepare( $sResource );

        // Find all 'title_' resource entries and $slook, if list_title_{fieldname} exists
        foreach( array_keys( $aRes ) as $sName )
        {
            if ( substr( $sName, 0, 6 ) == 'title_' )
            {
                if ( !array_key_exists( 'list_' . $sName, $aRes ) )
                {
                    $aRes[ 'list_' . $sName ] = $aRes[ $sName ];
                }
            }
        }

        return( $aRes );
    }

    /** Creates the basename of a block
        The following extensions are removed
            #header, #footer
            #begin, #end
            #yes, #no
            #notempty
            #total
            =, <>0, <>1
        @param $sBlock name of the block
        @return basename of the block
    */
    function BlockBasename( $sBlock )
    {
        if ( substr( $sBlock, -7 ) == '#header' )
        {
            $sBlock = substr( $sBlock, 0, -7 );
        }
        if ( substr( $sBlock, -7 ) == '#footer' )
        {
            $sBlock = substr( $sBlock, 0, -7 );
        }

        if ( substr( $sBlock, -6 ) == '#begin' )
        {
            $sBlock = substr( $sBlock, 0, -6 );
        }
        if ( substr( $sBlock, -4 ) == '#end' )
        {
            $sBlock = substr( $sBlock, 0, -4 );
        }

        if ( substr( $sBlock, -4 ) == '#yes' )
        {
            $sBlock = substr( $sBlock, 0, -4 );
        }
        if ( substr( $sBlock, -3 ) == '#no' )
        {
            $sBlock = substr( $sBlock, 0, -3 );
        }

        if ( substr( $sBlock, -9 ) == '#notempty' )
        {
            $sBlock = substr( $sBlock, 0, -9 );
        }
        if ( substr( $sBlock, -6 ) == '#total' )
        {
            $sBlock = substr( $sBlock, 0, -6 );
        }

        $sBlock = strtok( $sBlock, '=<>' );

        return( $sBlock );
    }

    /** Parses a block
        @param $sName name of the block
        @param $sBlock name of the block which should receive the content
        @param $fLegacy make a user warning, if this block is detected
        @return void
    */
    function &Parse( $sName = null, $sBlock = null, $fLegacy = null )
    {
        parent::Parse( $sName, $sBlock );

        if ( $fLegacy )
        {
            if ( $this->IsBlock( $sName ) )
            {
                trigger_error( 'Legacy block detected: ' . $sName . ' in ' . $this->GetFilename(),  E_USER_NOTICE );
            }
        }

        $oThis = &$this;
        return( $oThis );
    }

	function FormatValue( $sType, $sFormat, $sValue )
	{
	    global $oFactory;

	    if ( $sType != 'custom' )
	    {
	        return( parent::FormatValue( $sType, $sFormat, $sValue ) );
	    }

        if ( substr( $sFormat, 0, 7 ) == 'number ' )
        {
            $sFormat = substr( $sFormat, 7 );
            return( $this->oNumber->Format( $sValue, (int)$sFormat ) );
        }
        elseif ( substr( $sFormat, 0, 5 ) == 'date ' )
        {
            $sFormat = substr( $sFormat, 5 );
            return( $this->oDate->Format( (int)$sFormat, $sValue ) );
        }
        else
        {
            return( $this->oNumber->Format( $sValue, (int)$sFormat ) );
        }
    }

    function DoubleEncode( $sValue )
    {
        global $oApp;

        // use single encode if QuerySeparator = ?
        if ( $oApp->GetQuerySeparator() == '?' )
        {
            return( urlencode( $sValue ) );
        }

        return( urlencode( urlencode( $sValue ) ) );
    }

    function SetLanguage( $sValue = null )
    {
        parent::SetLanguage( $sValue );

        // Set language (set by parent::SetLanguage()) to used controls
        $this->oDate->SetLanguage( $this->sLanguage );
        $this->oNumber->SetLanguage( $this->sLanguage );

        $this->Assign( 'global.datetime', $this->oDate->GetDateTime() );
        $this->Assign( 'global.date', $this->oDate->Format( 0 ) );
        $this->Assign( 'global.time', $this->oDate->Format( 1 ) );
        $this->Assign( 'global.weekday', $this->oDate->Format( 12 ) );
    }
}
