<?php

/** CTpl
    Based on XTPL by Barnabs Debreceni [cranx@users.sourceforge.net], (c) 2000
    Rewritten by Hannes Dorn (hannes@dorn.cc), (c) 2002
    Reduced all unnecessary features.
    Optimized replacing of vars.
    Removed scanning of globals.

    Semantic:

    Block:
    <!-- BEGIN: block -->
    <!-- END: block -->

    Or-Blocks:
    Gets parsed, when one of its block names getting parsed (on the same level)
    <!-- BEGIN: block1|block2|#header -->
    <!-- END: block1|block2|#header -->
    <!-- BEGIN: block1|block2|#begin -->
    <!-- END: block1|block2|#begin -->
    <!-- BEGIN: block1|block2 -->
    <!-- END: block1|block2 -->
    <!-- BEGIN: block1|block2|#end -->
    <!-- END: block1|block2|#end -->
    <!-- BEGIN: block1|block2|#footer -->
    <!-- END: block1|block2|#footer -->

    And-Blocks:
    Gets parsed, when all of its block names have been parsed (on the same level)
    <!-- BEGIN: block1&block2&#header -->
    <!-- END: block1&block2&#header -->
    <!-- BEGIN: block1&block2&#begin -->
    <!-- END: block1&block2&#begin -->
    <!-- BEGIN: block1&block2 -->
    <!-- END: block1&block2 -->
    <!-- BEGIN: block1&block2&#end -->
    <!-- END: block1&block2&#end -->
    <!-- BEGIN: block1&block2&#footer -->
    <!-- END: block1&block2&#footer -->

    File inclusion:
    {FILE "filename.tpl" parameter="value"}
    --> Includes file into main template
    --> replaces {PARAM.parameter} with value

    Variable replacment:
    Single Value:
    {variable}
    Array of Values:
    {variable.value}

    You can also add default values for parameters in the included template file
    <!-- BEGIN: PARAM -->
    parameter="value"
    parameter2="value2"
    <!-- END: PARAM -->

    Resource Replacement:
    Resource File with Language Strings:
    title_title
    Titel
    If resource file exists, every RES.title_title is replaced by Titel

    Each variable can be used in different flavours:
    {variable}          Original Value
    {variable#xml}      XML encoded Value
    {variable#xmlplain} XML encoded text value, no newlines
    {variable#encoded}  Url encoded Value
    {variable#doubleencoded} Url Encoded Value (2x) (for pathinfo parameters) (this can be overruled to a single encoded version by overwriting DoubleEncode() in a derived class)
    {variable#doubleencodedx} Url Encoded Value (2x) (for pathinfo parameters) (this has to be always doubleencoded)
    {variable#plain}    Plain text value, no newlines
    {variable#slashed}  Plain text with slashes previous '
    {variable#text}     Plain text value with newlines
    {variable#html}     HTML encoded text
    {variable#htmlplain} HTML encoded plain text
    {variable#htmlplainslashed} HTML encoded plain text with slashed '
    {variable#nbsp}     Plain text, blanks are replace by &nbsp;
    {variable#_}        Plain text without "; blanks, /, #, etc. are replaced by _
    {variable#id}       Plain only with a-z, A-Z, 0-9, _, :, ., -
    {variable#utf8}     UTF-8 encoded value
    {variable#utf8plain} UTF-8 encoded Plain text value, no newlines
    {variable#utf8xml}  UTF-8 encoded value, XML encoded
    {variable#utf8plainxml} UTF-8 encoded Plain text value, no newlines, xml encoded

    Its possible to insert newlines at word breaks by supplying a line length
    eg: {variable#text76}    Max. Line length 76

    For each variable can be a format string provided (like used in printf).
    Be carefull when using this, there are no type checks applied and you may get wrong results.
    #xxx comes last
    |xxx comes before #xxx
    {variable|format}
    {variable|format#xml}
    etc.
    Since printf does not supply formats for date/times, you can just add printf, date or number left to the format.
    {variable|%4d} is the same as
    {variable|printf %4d}
    {variable|date H:i:s}
    {variable|strftime %H:%M:%S}
    {variable|number 2, } is equal to number_format( value, 2, ',', ' ' )
    {variable|size KB} or {variable|size MB}
    {variable|size XX 2, } can be used to format the size value
    {variable|list /} splits variable into <option value="value">value</option> entries separated by /
    {variable|limit XX} limits the field to XX chars.
    As it is obvious, you can't use # and | in format strings.

    tpl.align#number
    e.g.: text1{tpl.align#12}text2
    will insert as many spaces needed to align text2 to column 12 (useful in text emails)
    e.g.: text1{tpl.align#3}text2
    will insert just one spase because text1 is longer then 3 chars
*/

class CTpl
{
    /** Data and output charset, default iso-8859-1
        windows-1252
        iso-8859-1
        utf-8
    */
    var $sCharset = 'iso-8859-1';

    /** Template and resource charset, default iso-8859-1
        windows-1252
        iso-8859-1
        utf-8
    */
    var $sTemplateCharset = 'iso-8859-1';

    /** Filename of main template
    */
    var $sFilename;

    /** Main Block
    */
    var $sBlock;

    /** Current Language (de, en)
    */
    var $sLanguage;

    /** real path of the root of the web site (default this is set to document root)
    */
    var $sRoot;

    /** Characters for variable, resource and block names
        {name}, {name.name}, {name.name=asdf}, {name.name#xml}, ...
    */
    var $sBlockNames = 'A-Za-z0-9_=<>!#|&\-';
    var $sVariableNames = 'A-Za-z0-9_\\.\\ =<>!#:|&%,\\\\\\/\\~-';    // note the \\\\ this means \\, but php requries \\\\
    var $sResourceNames = 'A-Za-z0-9_\\.#\\ ';

    /** Stores any Blocks and it's Sub-Blocks of a Template
        $aBlock[ 'main' ]
        $aBlock[ 'main.table' ]
        $aBlock[ 'main.table.row' ]
        $aBlock[ 'main.table2' ]
        $aBlock[ 'main.table2.row' ]

        $aBlock[ 'main' ][ 'template' ] = Template

        $aBlock[ 'main' ][ 'var' ] = array() of variables used in this block.
        $aBlock[ 'main' ][ 'var#xml' ] = array() of xml encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#xmlplain' ] = array() of xml encoded text variables used in this block.
        $aBlock[ 'main' ][ 'var#encoded' ] = array() of url encoded variables used in this block.
        $aBlock[ 'main' ][ 'var_encoded' ] = array() of url encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#doubleencoded' ] = array() of url 2x encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#doubleencodedx' ] = array() of url 2x encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#plain' ] = array() of plain text variables used in this block.
        $aBlock[ 'main' ][ 'var_plain' ] = array() of plain text variables used in this block.
        $aBlock[ 'main' ][ 'var#slashed' ] = array() of variables with slashes added used in this block.
        $aBlock[ 'main' ][ 'var#text' ] = array() of text variables used in this block.
        $aBlock[ 'main' ][ 'var#html' ] = array() of html encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#htmlplain' ] = array() of html encoded plain text variables used in this block.
        $aBlock[ 'main' ][ 'var#htmlplainslashed' ] = array() of html encoded plain text variables with slashed ' used in this block
        $aBlock[ 'main' ][ 'var#nbsp' ] = array() of variables where space is replaced with &nbsp; used in this block.
        $aBlock[ 'main' ][ 'var#_' ] = array() of variables where space (and others) are replaced with _ used in this block.
        $aBlock[ 'main' ][ 'var#id' ] = array() of variables consisting only of a-z, A-Z, 0-9, _, :, ., -
        $aBlock[ 'main' ][ 'var#utf8' ] = array() of utf8 encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#utf8xml' ] = array() of utf8 encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#utf8plain' ] = array() of utf8 plain encoded variables used in this block.
        $aBlock[ 'main' ][ 'var#utf8plainxml' ] = array() of utf8 plain encoded variables used in this block.

        $aBlock[ 'main' ][ 'var#length' ] = array() of length of variables used in this block, eg. 76 for var#text76
        $aBlock[ 'main' ][ 'var#format' ] = array() of date format spezifications

        $aBlock[ 'main' ][ 'content' ] = Parsed content
        $aBlock[ 'main' ][ 'block' ] = array() of subblocks
        $aBlock[ 'main' ][ 'var_block' ] = array() of subblocks, but without #..., =..., <..., >...,
        $aBlock[ 'main' ][ 'or_block' ] = array() of block names to be parsed when this block gets parsed or null
        $aBlock[ 'main' ][ 'and_block' ] = array() of block names to be parsed when all the blocks in this and_block have been parsed or null
        $aBlock[ 'main' ][ 'parsed' ] = true, if block is parsed, reseted by reset
        $aBlock[ 'main' ][ 'andor' ] = array of blockname, contains all blocknames separated by & or | in it's name
    */
    var $aBlock = array();

    /** Defines, if parsed templates should be cached
        This saves time when opening the template
        Cache looks only for changes in main template!
        If you change a sub template, the cache will not be recreated!
    */
    var $fCache = false;

    /** Defines where to store cached templates
        This directory must exist and have to be possible to be created and must be writeable!
        This directory is relative to the current script path.
    */
    var $sCacheDirectory;

    /** Stores values for variables
    */
    var $aVariable = array();
    var $aLocal = array();

    /** Creates the template object
        Reads the main template and all subtemplates
        Reads the PARAM section and Replaces all parameters with its values
        @param $sFilename filename of the template
        @param $sBlock main block, default 'main'
        @param $aParamter name/value array of parameters, default null
        @param $sLanguage de/en
        @return void
    */
    function CTpl( $sFilename = null, $sBlock = null, $aParameter = null, $sLanguage = null )
    {
        if ( $sBlock === null )
        {
            $sBlock = 'main';
        }

        // Store language (de, en, etc.)
        $this->SetLanguage( $sLanguage );

        // Store Main Block name in instance
        $this->sBlock = $sBlock;

        // Set root path, if not set already
        if ( !isset( $this->sRoot ) )
        {
            $this->sRoot = $_SERVER[ 'DOCUMENT_ROOT' ] . '/';
        }

        if ( !isset( $this->sCacheDirectory ) )
        {
            $this->sCacheDirectory = 'cache/';
        }

        // If we have a filename, load template and init
        if ( $sFilename ==! null )
        {
            $this->Init( $sFilename, $aParameter );
        }

        $this->Assign( 'global.now', time() );
        $this->Assign( 'global.sessionname', session_name() );
        $this->Assign( 'global.sessionid', session_id() );

        if ( !$sUploadMaxFilesize = ini_get( 'upload_max_filesize' ) )
        {
            $sUploadMaxFilesize = '5M';
        }
        $sUploadMaxFilesize = $this->Something2Byte( $sUploadMaxFilesize );
        if ( $sPostMaxSize = ini_get('post_max_size' ) )
        {
            $sPostMaxSize = $this->Something2Byte( $sPostMaxSize );
            if ( $sPostMaxSize < $sUploadMaxFilesize )
            {
                $sUploadMaxFilesize = $sPostMaxSize;
            }
        }
        $this->Assign( 'global.upload_max_filesize', $sUploadMaxFilesize );
    }

    /** Init
        @param $sFilename
    */
    function Init( $sFilename, $aParameter = null )
    {
        // Extend filename with extension, if extension is missing
        if ( substr( $sFilename, -4 ) != '.tpl' )
        {
            $sFilename .= '.tpl';
        }

        if ( !file_exists( $sFilename ) )
        {
            error_log( "CTemplate: File not found: '$sFilename'", 0 );
            return( false );
        }
        $this->sFilename = $sFilename;

        $fCreate = true;
        if ( $this->GetCache() )
        {
            $sCacheFilename = $this->CacheFilename( $sFilename, $this->sBlock );
            $iLastModified = filemtime( $sFilename );
            $fCreate = !$this->CacheRead( $sCacheFilename, $iLastModified );
        }

        if ( $fCreate )
        {
            // Write a file to show, that we are creating a new cache file.
            if ( $this->GetCache() )
            {
                $File = fopen( $sCacheFilename . '_creating', 'w' );
                fputs( $File, 'creating cachefile' );
                fclose( $File );
                chmod( $sCacheFilename . '_creating', 0666 );
            }

            // read Template (recursion!)
            $sBuffer = $this->Read( $sFilename, $aParameter );

            // Replace resource placeholders with values
            $this->ResourceReplace( $sFilename, $sBuffer );

            // Convert template into output charset
            $sBuffer = StringConvert( $sBuffer, $this->sTemplateCharset, $this->sCharset );

            // split Template into blocks
            $this->Block( $sBuffer, $this->sBlock );

            // Cache speichern?
            if ( $this->GetCache() )
            {
                $this->CacheWrite( $sCacheFilename );
            }
        }

        $sBasename = basename( $this->sFilename );
        $this->Assign( 'local.template', $this->sFilename, true );
        $this->Assign( 'local.template_dirname', dirname( $this->sFilename ), true );
        $this->Assign( 'local.template_basename', $sBasename, true );
        $this->Assign( 'local.template_basename_filename', substr( $sBasename, 0, strrpos( $sBasename, '.' ) ), true );
        $this->Assign( 'local.template_basename_extension', substr( $sBasename, strrpos( $sBasename, '.' ) + 1 ), true );

        return( true );
    }

    /** Returns the filename of the main template
        @param void
        @return string
    */
    function GetFilename()
    {
        return( $this->sFilename );
    }

    /** Reads the Template file (rekursive)
        @param $sFilename Filename of the Template
        @param $aParamter name/value array of parameters, default null
        @return String One huge Buffer with all the Templates read
    */
    function Read( $sFilename, $aParameter = null )
    {
        // Clear Stat Cache, open file, read file, close file.
    	clearstatcache();

        if ( !file_exists( $sFilename ) )
        {
            error_log( "CTpl: Read: File not found: '$sFilename'", 0 );
            return( false );
        }

    	if ( !( $pFile = fopen( $sFilename, 'r' ) ) )
    	{
            die( "CTpl: Read: File could not be read: '$sFilename'" );
    	}

        if ( filesize( $sFilename ) == 0 )
        {
            $sTemplate = '';
        }
        else
        {
            // 2009.04.30 DOH   trim first line (intend looks better in sourcecode, since first line inhertis inted from {FILE} command in parent template
            $sTemplate = ltrim( fread( $pFile, filesize( $sFilename ) ) );
        }
    	fclose( $pFile );

        // look for default values for parameters
        $i = strpos( $sTemplate, '<!-- BEGIN: PARAM -->' );
        if ( $i !== false )
        {
            $i += 22;
            $j = strpos( $sTemplate, '<!-- END: PARAM -->', $i );
            if ( $j !== FALSE )
            {
                $sDefaults = trim( substr( $sTemplate, $i, $j - $i ) );
                $aDefaults = explode( "\n", $sDefaults );
                foreach( $aDefaults as $sDefault )
                {
                    $sDefault = trim( $sDefault );

                    if ( !IsNull( $sDefault ) )
                    {
                        $aDefault = explode( '=', $sDefault, 2 );
                        $aDefault[ 0 ] = trim( $aDefault[ 0 ] );
                        $aDefault[ 1 ] = substr( trim( $aDefault[ 1 ] ), 1, -1 );

                        // use the default only if no parameter value is supplied
                        if ( !isset( $aParameter[ $aDefault[ 0 ] ] ) )
                        {
                            // replace {PARAM.xxxx} in default value with parameter value, if supplied
                            if ( is_array( $aParameter ) )
                            {
                                foreach( $aParameter as $sKey => $sValue )
                                {
                                    $aDefault[ 1 ] = str_replace( '{PARAM.' . $sKey . '}', $sValue, $aDefault[ 1 ] );
                                }
                            }

                            $aParameter[ $aDefault[ 0 ] ] = $aDefault[ 1 ];
                        }
                    }
                }

                // Remove PARAM block
                $sTemplate = substr( $sTemplate, $j + 19 );
            }
        }

        // replace parameters with parameters
        if ( is_array( $aParameter ) )
        {
            foreach( $aParameter as $sKey => $sValue )
            {
                if ( strpos( $sValue, chr( 123 ) . 'PARAM.' ) !== false )
                {
                    foreach( $aParameter as $sSubName => $sSubValue )
                    {
                        $aParameter[ $sKey ] = str_replace( '{PARAM.' . $sSubName . '}', $sSubValue, $aParameter[ $sKey ] );
                    }
                }
            }
        }

        // replace parameters with values
        if ( is_array( $aParameter ) )
        {
            foreach( $aParameter as $sKey => $sValue )
            {
                $sTemplate = str_replace( '{PARAM.' . $sKey . '}', $sValue, $sTemplate );
            }
        }

        // remove remaining {PARAM.xxx} entries
        if ( strpos( $sTemplate, chr( 123 ) . 'PARAM.' ) !== false )
        {
            $sTemplate = preg_replace( '(\{PARAM\.([' . $this->sVariableNames . ']+?)\})', '', $sTemplate );
        }

        // Look for {FILE "filename.tpl" parameter1="value" parameter2="value"} and replace it with content of file (rekursion...)
        $iStart = 0;
        while( ( $iStart = strpos( $sTemplate, '{FILE', $iStart ) ) !== false )
        {
            // Look for '}'
            $iStop = strpos( $sTemplate, '"}', $iStart );
            if ( $iStop !== false )
            {
                // contains "filename.tpl" parameter1="value" parameter2="value"
                $sBuffer = trim( substr( $sTemplate, $iStart + 6, $iStop - $iStart - 6 ) );

                // split into parts
                $aBuffer = explode( '"', $sBuffer );

                // get filename
                $sFilename = $aBuffer[ 1 ];

                // prepare parts
                $aParameter = array();
                $iMax = count( $aBuffer );
                for ( $i = 0; $i < $iMax; $i += 2 )
                {
                    $sKey = trim( substr( $aBuffer[ $i ], 0, -1 ) );
                    if ( $i >= 2 && $sKey <> '' )
                    {
                        $aParameter[ $sKey ] = $aBuffer[ $i + 1 ];
                    }
                }

                // make relative path, etc...
                if ( $this->ReadFilenamePrepare( $sFilename ) )
                {
                    $sBuffer = $this->Read( $sFilename, $aParameter );
                }
                else
                {
                    $sBuffer = '';
                }

                // Stop marks end of FILE-Tag
                $iStop += 2;

                // Insert Buffer2 into Buffer replacing FILE-Tag
                $sTemplate = substr( $sTemplate, 0, $iStart ) . $sBuffer . substr( $sTemplate, $iStop );

                // Set Start to end of Buffer2
                $iStart += strlen( $sBuffer );
            }
            else
            {
                $iStart += 8;
            }
        }

        // Look for {TIMESTAMP "filename.tpl"} and replace it with the timestamp of the file
        $iStart = 0;
        while( ( $iStart = strpos( $sTemplate, '{TIMESTAMP', $iStart ) ) !== false )
        {
            // Look for '}'
            $iStop = strpos( $sTemplate, '"}', $iStart );
            if ( $iStop !== false )
            {
                // contains "filename.tpl"
                $sBuffer = trim( substr( $sTemplate, $iStart + 11, $iStop - $iStart - 11 ) );

                // split into parts
                $aBuffer = explode( '"', $sBuffer );

                // get filename
                $sFilename = $aBuffer[ 1 ];

                // make relative path, etc...
                $sBuffer = '';
                if ( $this->ReadFilenamePrepare( $sFilename ) )
                {
                    if ( file_exists( $sFilename ) )
                    {
                        $sBuffer = filemtime( $sFilename );
                    }
                }

                // Stop marks end of FILE-Tag
                $iStop += 2;

                // Insert Buffer2 into Buffer replacing FILE-Tag
                $sTemplate = substr( $sTemplate, 0, $iStart ) . $sBuffer . substr( $sTemplate, $iStop );

                // Set Start to end of Buffer2
                $iStart += strlen( $sBuffer );
            }
            else
            {
                $iStart += 11;
            }
        }

        return( trim( $sTemplate ) );
    }

    /** prepares a filename of an include statement within a template
        make path relative, or something else
        file is only included, if this function returns true
        @param &$sFilename
        @return true/false
    */
    function ReadFilenamePrepare( &$sFilename )
    {
        $sFilename = $sRoot . $sFilename;
        return( true );
    }

    /** Extracts the Blocks
        @param $sTemplate Buffer with Template content
        @param $sBlock Name of Block to start with
        @return void

        Looks for Block starts.
        Replaces subblocks with {_BLOCK_.block.subblock}
        Stores content of each block in an array like 'block', 'block.subblock', ...
    */
    function Block( $sTemplate, $sBlock )
    {
    	$iLevel = 0;
	    $aName = array();

	    $aBuffer = explode( '<!-- ', $sTemplate );
	    foreach( $aBuffer as $sValue )
	    {
		    // if ( eregi( '(BEGIN:|END:)[[:blank:]]*([' . $this->sBlockNames . ']+)[[:blank:]]*-->(.*)', $sValue, $aRessource ) )
		    if ( preg_match( '/(BEGIN:|END:)\s*([' . $this->sBlockNames . ']+)\s*-->(.*)/s', $sValue, $aRessource ) )
		    {
    			// $aRessource[ 1 ] = BEGIN or END
    			// $aRessource[ 2 ] = block name
    			// $aRessource[ 3 ] = kinda content
    			if ( $aRessource[ 1 ] == 'BEGIN:' )
    			{
    				$sParent = implode( '.', $aName );

                    // insert block into parent's subblock list
                    $this->aBlock[ $sParent ][ 'block' ][] = $aRessource[ 2 ];

                    // insert block into parent's var_block list
                    $sName = $this->BlockBasename( $aRessource[ 2 ] );
                    $this->aBlock[ $sParent ][ 'var_block' ][ $sName ] = $sName;

                    // add one level - array( "main", "table", "row" )
    				$aName[ ++$iLevel ] = $aRessource[ 2 ];

                    // make block name (main.table.row)
    				$sName = implode( '.', $aName );

                    // Store Block in Array
                    $this->aBlock[ $sName ][ 'template' ] .= $aRessource[ 3 ];
    				$this->aBlock[ $sParent ][ 'template' ] .= "{_BLOCK_.$sName}";

    				// Split Blockname by '|'
    				$aOr = explode( '|', $aRessource[ 2 ] );
    				if ( count( $aOr ) > 1 )
    				{
                        foreach( $aOr as $sOr )
                        {
                            if ( $sOr != '#begin' && $sOr != '#end' && $sOr != '#header' && $sOr != '#footer' )
                            {
            				    $this->aBlock[ $sName ][ 'andor' ][] = $sParent . '.' . $sOr;
                                $this->aBlock[ $sParent . '.' . $sOr ][ 'or_block' ][] = $sName;
                                // insert and block into var_block from parent block
                                $sAnd = $this->BlockBasename( $sOr );
                                $this->aBlock[ $sParent ][ 'var_block' ][ $sOr ] = $sOr;
                            }
                        }
    				}

    				// Split Blockname by '&'
    				$aAnd = explode( '&', $aRessource[ 2 ] );
    				if ( count( $aAnd ) > 1 )
    				{
                        foreach( $aAnd as $sAnd )
                        {
                            if ( $sAnd != '#begin' && $sAnd != '#end' && $sAnd != '#header' && $sAnd != '#footer' )
                            {
            				    $this->aBlock[ $sName ][ 'andor' ][] = $sParent . '.' . $sAnd;
                                $this->aBlock[ $sParent . '.' . $sAnd ][ 'and_block' ][] = $sName;
                                // insert and block into var_block from parent block
                                $sAnd = $this->BlockBasename( $sAnd );
                                $this->aBlock[ $sParent ][ 'var_block' ][ $sAnd ] = $sAnd;
                            }
                        }
    				}
    			}
                else
                {
                    if ( $aRessource[ 1 ] == 'END:' )
                    {
    				    unset( $aName[ $iLevel-- ] );
    				    $sName = implode( '.', $aName );
                        $this->aBlock[ $sName ][ 'template' ] .= $aRessource[ 3 ];
                    }
                }
            }
	        else
	        {
	            // no block delimiters found
	            $sName = implode( '.', $aName );
                $this->aBlock[ $sName ][ 'template' ] .= '<!-- ' . $sValue;
    		}
        }

        // Look for used variables
        foreach( array_keys( $this->aBlock ) as $sName )
        {
            $this->BlockFindVars( $this->aBlock[ $sName ] );
        }
    }

    /** Creates the basename of a block
        @param $sBlock name of the block
        @return basename of the block
    */
    function BlockBasename( $sBlock )
    {
        // Return the first token of the blockname
        return( strtok( $sBlock, '#=<>' ) );
    }

    /** Creates an array with all variables used in the block
        @param &$aBuffer Reference to a buffer array
        @return void
    */
    function BlockFindVars( &$aBlock )
    {
        // Look for {variable}
	    preg_match_all( '/\{([' . $this->sVariableNames . ']+)}/', $aBlock[ 'template' ], $aVariables );

        if ( !is_array( $aVariables[ 1 ] ) )
        {
            return;
        }

	    foreach( $aVariables[ 1 ] as $sVariable )
	    {
	        // ignore variables beginning with tpl.
            if ( substr( $sVariable, 0, 4 ) == 'tpl.' )
            {
                continue;
            }

            $i = strrpos( $sVariable, '#' );
            if ( $i !== false )
            {
                $sType = substr( $sVariable, $i );
                if ( $sType == '#xml' || $sType == '#xmlplain' || $sType == '#encoded' || $sType == '#doubleencoded' || $sType == '#doubleencodedx' || substr( $sType, 0, 6 ) == '#plain' || $sType == '#slashed' || substr( $sType, 0, 5 ) == '#text' || substr( $sType, 0, 5 ) == '#html' || $sType == '#nbsp' || $sType == '#_' || $sType == '#id' || $sType == '#utf8' || $sType == '#utf8xml' || $sType == '#utf8plain' || $sType == '#utf8plainxml' )
                {
                    $sVar = substr( $sVariable, 0, $i );
                }
                else
                {
                    $sVar = $sVariable;
                    $sType = '';
                }
            }
            else
            {
                if ( substr( $sVariable, -8 ) == '_encoded' )
                {
                    $sVar = substr( $sVariable, 0, -8 );
    	            $sType = substr( $sVariable, -8 );
    	        }
                elseif ( substr( $sVariable, -6 ) == '_plain' )
                {
                    $sVar = substr( $sVariable, 0, -6 );
    	            $sType = substr( $sVariable, -6 );
    	        }
                else
                {
                    $sVar = $sVariable;
                    $sType = '';
                }
            }

            // Longest entry must go first' Otherwise #html will split #htmlplain into #html as type and plain as length!
            $sLength = Null;
            if ( substr( $sType, 0, 17 ) == '#htmlplainslashed' )
	        {
                $sLength = substr( $sType, 17 );
	            $sType = substr( $sType, 0, 17 );
            }
            elseif ( substr( $sType, 0, 10 ) == '#htmlplain' )
	        {
                $sLength = substr( $sType, 10 );
	            $sType = substr( $sType, 0, 10 );
            }
            elseif ( substr( $sType, 0, 6 ) == '#plain' or substr( $sType, 0, 6 ) == '_plain' )
	        {
                $sLength = substr( $sType, 6 );
	            $sType = substr( $sType, 0, 6 );
            }
            elseif ( substr( $sType, 0, 5 ) == '#text' )
	        {
                $sLength = substr( $sType, 5 );
	            $sType = substr( $sType, 0, 5 );
            }
            elseif ( substr( $sType, 0, 5 ) == '#html' )
	        {
                $sLength = substr( $sType, 5 );
	            $sType = substr( $sType, 0, 5 );
            }

            if ( $sLength !== null )
            {
                $aBlock[ 'var#length' ][ $sVar . $sType ] = $sLength;
            }

            // store variable name (including format)
            $aBlock[ 'var' . $sType ][ $sVar ] = $sVar;

            // extract format string (if any) and save it
            $i = strrpos( $sVar, '|' );
            if ( $i !== false )
            {
                $sName = $sVar;
                $sFormat = substr( $sName, $i + 1 );
                $sVar = substr( $sName, 0, $i );

                if ( substr( $sFormat, 0, 5 ) == 'date ' )
                {
                    $sType = 'date';
                    $sFormat = substr( $sFormat, 5 );
                }
                elseif ( substr( $sFormat, 0, 9 ) == 'strftime ' )
                {
                    $sType = 'strftime';
                    $sFormat = substr( $sFormat, 9 );
                }
                elseif ( substr( $sFormat, 0, 7 ) == 'number ' )
                {
                    $sType = 'number';
                    $sFormat = substr( $sFormat, 7 );
                }
                elseif ( substr( $sFormat, 0, 7 ) == 'printf ' )
                {
                    $sType = 'printf';
                    $sFormat = substr( $sFormat, 7 );
                }
                elseif ( substr( $sFormat, 0, 5 ) == 'size ' )
                {
                    $sType = 'size';
                    $sFormat = substr( $sFormat, 5 );
                }
                elseif ( substr( $sFormat, 0, 7 ) == 'custom ' )
                {
                    $sType = 'custom';
                    $sFormat = substr( $sFormat, 7 );
                }
                elseif ( substr( $sFormat, 0, 5 ) == 'list ' )
                {
                    $sType = 'list';
                    $sFormat = substr( $sFormat, 5 );
                }
                elseif ( substr( $sFormat, 0, 6 ) == 'limit ' )
                {
                    $sType = 'limit';
                    $sFormat = substr( $sFormat, 6 );
                }
                else
                {
                    $sType = 'printf';
                }

                $aBlock[ 'var#format' ][] = array( 'name' => $sName, 'var' => $sVar, 'type' => $sType, 'format' => $sFormat );
            }
        }

        return;
    }

    /** Assign some values to Variables
        @param $Name either a name or an array of Name/Value pairs
        @param $Value either a single value or an array of Values, default = ''
        @return $this
    */
    function &Assign( $Name, $Value = null, $fLocal = null )
    {
        if ( $fLocal === null )
        {
            $fLocal = false;
        }

    	if ( is_array( $Name ) )
    	{
    	    foreach( $Name as $sName => $sValue )
    	    {
    	        if ( $sValue === null )
    	        {
        			unset( $this->aVariable[ $sName ] );
    		        if ( $fLocal )
    		        {
            			unset( $this->aLocal[ $sName ] );
    		        }
    	        }
    	        else
    	        {
        			$this->aVariable[ $sName ] = $sValue;
    		        if ( $fLocal )
    		        {
            			$this->aLocal[ $sName ] = $sValue;
    		        }
                }
            }
  		}
    	else
    	{
    	    if ( is_array( $Value ) )
    	    {
    	        foreach( $Value as $sName => $sValue )
    	        {
    	            if ( $sValue === null )
    	            {
                		unset( $this->aVariable[ $Name . '.' . $sName ] );
        		        if ( $fLocal )
        		        {
                    		unset( $this->aLocal[ $Name . '.' . $sName ] );
        		        }
                    }
                    else
                    {
                		$this->aVariable[ $Name . '.' . $sName ] = $sValue;
        		        if ( $fLocal )
        		        {
                    		$this->aLocal[ $Name . '.' . $sName ] = $sValue;
        		        }
                    }
                }
    	    }
    	    else
    	    {
    	        if ( $Value === null )
    	        {
            		unset( $this->aVariable[ $Name ] );
    		        if ( $fLocal )
    		        {
            			unset( $this->aLocal[ $Name ] );
    		        }
                }
                else
                {
            		$this->aVariable[ $Name ] = $Value;
    		        if ( $fLocal )
    		        {
            			$this->aLocal[ $Name ] = $Value;
    		        }
                }
            }
    	}

        $oThis = &$this;
        return( $oThis );
    }

    /** Appends a value to an existing variable
        @param $sName
        @param $sValue
    */
    function &Append( $sName, $sValue )
    {
        $this->aVariable[ $sName ] .= $sValue;
        $oThis = &$this;
        return( $oThis );
    }

    /** Checks if a variable is set
        @param $sName
        @return true/false
    */
    function IsAssigned( $sName )
    {
        if ( !is_array( $this->aVariable ) )
        {
            return( false );
        }

        if ( phpversion() < '4.1' )
        {
            foreach( array_keys( $this->aVariable ) as $sKey )
            {
                if ( $sName == $sKey )
                {
                    return( true );
                }
            }
            return( false );
        }

        return( array_key_exists( $sName, $this->aVariable ) );
    }

    /** Parse a Block (which means replace variables by value and store content)
        @param $sName Name of block to be parsed
        @param $sBlock Name of block, where content is stored (used to merge content of blocks of the same level)
        @return true/false false, if block was not found.
    */
    function &Parse( $sName = null, $sBlock = null )
    {
        if ( $sName === null )
        {
            $sName = 'main';
        }

        // if block is not set, then exit right away
        if ( !isset( $this->aBlock[ $sName ] ) )
        {
            $oThis = &$this;
            return( $oThis );
        }

        // if block has a template parse the template
        if ( isset( $this->aBlock[ $sName ][ 'template' ] ) )
        {
            if ( $sBlock === null )
            {
                $sBlock = $sName;
            }

            // prepare formated values
            if ( isset( $this->aBlock[ $sName ][ 'var#format' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#format' ] as $aVariable )
                {
                    if ( $this->aVariable[ $aVariable[ 'var' ] ] != '' )
                    {
                        $this->aVariable[ $aVariable[ 'name' ] ] = $this->FormatValue( $aVariable[ 'type' ], $aVariable[ 'format' ], $this->aVariable[ $aVariable[ 'var' ] ] );
                    }
                }
            }

            // get block contents
        	$sBuffer = $this->aBlock[ $sName ][ 'template' ];

            // replace all vars of block with values
            if ( isset( $this->aBlock[ $sName ][ 'var#xml' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#xml' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringXmlEncode( $this->aVariable[ $sVariable ] );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#xml}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#xmlplain' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#xmlplain' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringXmlEncode( StringPlain( $this->aVariable[ $sVariable ] ) );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#xmlplain}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#encoded' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#encoded' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = urlencode( $this->aVariable[ $sVariable ] );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#encoded}', $sValue, $sBuffer );
                }
            }
            if ( isset( $this->aBlock[ $sName ][ 'var_encoded' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var_encoded' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = urlencode( $this->aVariable[ $sVariable ] );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '_encoded}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#doubleencoded' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#doubleencoded' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = $this->DoubleEncode( $this->aVariable[ $sVariable ] );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#doubleencoded}', $sValue, $sBuffer );
                }
            }
            if ( isset( $this->aBlock[ $sName ][ 'var#doubleencodedx' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#doubleencodedx' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = self::DoubleEncode( $this->aVariable[ $sVariable ] );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#doubleencodedx}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#plain' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#plain' ] as $sVariable )
                {
                    $sLength = $this->aBlock[ $sName ][ 'var#length' ][ $sVariable . '#plain' ];
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringPlain( $this->aVariable[ $sVariable ] );
                        if ( $sLength > 0 )
                        {
                            $sValue = wordwrap( $sValue, $sLength );
                        }
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#plain' . $sLength . '}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var_plain' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var_plain' ] as $sVariable )
                {
                    $sLength = $this->aBlock[ $sName ][ 'var#length' ][ $sVariable . '_plain' ];
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringPlain( $this->aVariable[ $sVariable ] );
                        if ( $sLength > 0 )
                        {
                            $sValue = wordwrap( $sValue, $sLength );
                        }
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '_plain' . $sLength . '}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#slashed' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#slashed' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = addslashes( $this->aVariable[ $sVariable ] );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#slashed}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#text' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#text' ] as $sVariable )
                {
                    $sLength = $this->aBlock[ $sName ][ 'var#length' ][ $sVariable . '#text' ];
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringHtml2Text( $this->aVariable[ $sVariable ] );
                        if ( $sLength > 0 )
                        {
                            $sValue = wordwrap( $sValue, $sLength );
                        }
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#text' . $sLength . '}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#html' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#html' ] as $sVariable )
                {
                    $sLength = $this->aBlock[ $sName ][ 'var#length' ][ $sVariable . '#html' ];
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringHtml2Text( $this->aVariable[ $sVariable ] );
                        if ( $sLength > 0 )
                        {
                            $sValue = wordwrap( $sValue, $sLength );
                        }
                        $sValue = StringText2Html( $sValue );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#html' . $sLength . '}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#htmlplain' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#htmlplain' ] as $sVariable )
                {
                    $sLength = $this->aBlock[ $sName ][ 'var#length' ][ $sVariable . '#htmlplain' ];
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringPlain( $this->aVariable[ $sVariable ] );
                        if ( $sLength > 0 )
                        {
                            $sValue = wordwrap( $sValue, $sLength );
                        }
                        $sValue = htmlspecialchars( $sValue, ENT_QUOTES );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#htmlplain' . $sLength . '}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#htmlplainslashed' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#htmlplainslashed' ] as $sVariable )
                {
                    $sLength = $this->aBlock[ $sName ][ 'var#length' ][ $sVariable . '#htmlplainslashed' ];
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = StringPlain( $this->aVariable[ $sVariable ] );
                        if ( $sLength > 0 )
                        {
                            $sValue = wordwrap( $sValue, $sLength );
                        }
                        $sValue = addslashes( htmlspecialchars( $sValue ) );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#htmlplainslashed' . $sLength . '}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#nbsp' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#nbsp' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        $sValue = str_replace( ' ', '&nbsp;', StringPlain( $this->aVariable[ $sVariable ] ) );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#nbsp}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#_' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#_' ] as $sVariable )
                {
                    $sValue = $this->aVariable[ $sVariable ];
                    if ( $sValue != '' )
                    {
                        $sValue = $this->PrepareValue_( $sValue );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#_}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#id' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#id' ] as $sVariable )
                {
                    $sValue = $this->aVariable[ $sVariable ];
                    if ( $sValue != '' )
                    {
                        $sValue = $this->PrepareValueId( $sValue );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#id}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#utf8' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#utf8' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        if ( $this->sCharset == 'utf-8' )
                        {
                            $sValue = $this->aVariable[ $sVariable ];
                        }
                        elseif ( $this->sCharset == 'windows-1252' )
                        {
                            $sValue = StringWindowsToUtf8( $this->aVariable[ $sVariable ] );
                        }
                        else
                        {
                            $sValue = utf8_encode( $this->aVariable[ $sVariable ] );
                        }
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#utf8}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#utf8xml' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#utf8xml' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        if ( $this->sCharset == 'utf-8' )
                        {
                            $sValue = $this->aVariable[ $sVariable ];
                        }
                        elseif ( $this->sCharset == 'windows-1252' )
                        {
                            $sValue = StringWindowsToUtf8( $this->aVariable[ $sVariable ] );
                        }
                        else
                        {
                            $sValue = utf8_encode( $this->aVariable[ $sVariable ] );
                        }
                        $sValue = htmlspecialchars( $sValue, ENT_QUOTES );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#utf8xml}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#utf8plain' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#utf8plain' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        if ( $this->sCharset == 'utf-8' )
                        {
                            $sValue = StringPlain( $this->aVariable[ $sVariable ] );
                        }
                        elseif ( $this->sCharset == 'windows-1252' )
                        {
                            $sValue = StringWindowsToUtf8( StringPlain( $this->aVariable[ $sVariable ] ) );
                        }
                        else
                        {
                            $sValue = utf8_encode( StringPlain( $this->aVariable[ $sVariable ] ) );
                        }
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#utf8plain}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var#utf8plainxml' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var#utf8plainxml' ] as $sVariable )
                {
                    $sValue = '';
                    if ( $this->aVariable[ $sVariable ] != '' )
                    {
                        if ( $this->sCharset == 'utf-8' )
                        {
                            $sValue = StringPlain( $this->aVariable[ $sVariable ] );
                        }
                        elseif ( $this->sCharset == 'windows-1252' )
                        {
                            $sValue = StringWindowsToUtf8( StringPlain( $this->aVariable[ $sVariable ] ) );
                        }
                        else
                        {
                            $sValue = utf8_encode( StringPlain( $this->aVariable[ $sVariable ] ) );
                        }
                        $sValue = htmlspecialchars( $sValue, ENT_QUOTES );
                    }
                    $sBuffer = str_replace( '{' . $sVariable . '#utf8plainxml}', $sValue, $sBuffer );
                }
            }

            if ( isset( $this->aBlock[ $sName ][ 'var' ] ) )
            {
                foreach( $this->aBlock[ $sName ][ 'var' ] as $sVariable )
                {
                    if ( substr( $sVariable, 0, 8 ) == '_BLOCK_.' )
                    {
                        // Create name of SubBlock
                        $sSubBlock = substr( $sVariable, 8 );

                        // Get content of SubBlock
                        $sBuffer = str_replace( '{' . $sVariable . '}', $this->aBlock[ $sSubBlock ][ 'content' ], $sBuffer );

                        // reset SubBlock
                        $this->Reset( $sSubBlock );

                        // reset empty and/or blocks (they may never get reseted if they are not part of the content of the parent block but exist (see Block()).
                        if ( isset( $this->aBlock[ $sSubBlock ][ 'andor' ] ) )
                        {
                            foreach( $this->aBlock[ $sSubBlock ][ 'andor' ] as $sPart )
                            {
                                if ( $this->aBlock[ $sPart ][ 'content' ] == '' )
                                {
                                    $this->Reset( $sPart );
                                }
                            }
                        }
                    }
                    else
                    {
                        $sValue = $this->aVariable[ $sVariable ];
                        $sBuffer = str_replace( '{' . $sVariable . '}', $sValue, $sBuffer );
                    }
                }
            }

            $this->aBlock[ $sBlock ][ 'content' ] .= $sBuffer;
        }

        $this->aBlock[ $sName ][ 'parsed' ] = true;

        // or_block parsen
        if ( isset( $this->aBlock[ $sName ][ 'or_block' ] ) && isset( $this->aBlock[ $sName ][ 'or_block' ] ) )
        {
            foreach( $this->aBlock[ $sName ][ 'or_block' ] as $sPart )
            {
                if ( $this->aBlock[ $sPart ][ 'content' ] == '' )
                {
                    $this->Parse( $sPart );
                }
            }
        }

        // and_block parsen
        if ( isset( $this->aBlock[ $sName ][ 'and_block' ] ) && isset( $this->aBlock[ $sName ][ 'and_block' ] ) )
        {
            foreach( $this->aBlock[ $sName ][ 'and_block' ] as $sPart )
            {
                if ( $this->aBlock[ $sPart ][ 'content' ] == '' )
                {
                    // check blocks if number of unparsed blocks is 1
                    $i = 0;

                    // cut of parent block names
                    $sParent = substr( $sPart, 0, strrpos( $sPart, '.' ) );
                    $sPart = substr( $sPart, strrpos( $sPart, '.' ) + 1 );

    				// Split Blockname by '&'
    				$aAnd = explode( '&', $sPart );
    				if ( count( $aAnd ) > 1 )
    				{
                        foreach( $aAnd as $sAnd )
                        {
                            if ( $sAnd == '#begin' || $sAnd == '#end' || $sAnd == '#header' || $sAnd == '#footer' )
                            {
                                $i++;
                            }
                            elseif ( $this->IsParsed( $sParent . '.' . $sAnd ) )
                            {
                                $i++;
                            }
                        }

                        if ( count( $aAnd ) == $i )
                        {
                            $this->Parse( $sParent . '.' . $sPart );
                        }
                    }
                }
            }
        }

        $oThis = &$this;
        return( $oThis );
	}

    /**
     * Prepare value for var#_
     * @param string $sValue value to be prepared
     * @return string prepared string
     */
    function PrepareValue_( $sValue )
    {
        $sValue = str_replace( '%', '', $sValue );
        $sValue = StringPlain( $sValue );
        // Replace Umlaute with utf8_encoded version (is replaced in all browsers, but not in IE8)
        $sValue = str_replace( array( '', '', '', '', '', '', '' ), array( '%C3%84', '%C3%96', '%C3%9C', '%C3%A4', '%C3%B6', '%C3%BC', '%C3%9F' ), $sValue );
        $sValue = iconv( $this->sCharset, 'ascii//TRANSLIT', $sValue );
        // 2010-11-24 DOH   '.' must be removed or replaced, or the string will be part of the ScriptName(), which is done wrong by apache
        $sValue = str_replace( array( '#', '$', '\'', '(', ')', ',', ':', ';', '?', '[', ']', '>', '<', '"' ), '', $sValue );
        $sValue = str_replace( array( ' ', '|', '/', '@', '=', '+', '*', '.' ), '-', $sValue );
        $sValue = str_replace( '&', '&amp;', $sValue );

        return( $sValue );
    }

    /**
     * Prepare value for var#id
     * Only allow the following chars: a-z, A-Z, 0-9, _, :, ., -
     * @param string $sValue value to be prepared
     * @return string prepared string
     */
    function PrepareValueId( $sValue )
    {
        $sValue = str_replace( '%', '', $sValue );
        $sValue = StringPlain( $sValue );
        // Replace Umlaute
        $sValue = str_replace( array( '', '', '', '', '', '', '' ), array( 'Ae', 'Oe', 'Ue', 'ae', 'oe', 'ue', 'ss' ), $sValue );
        $sValue = iconv( 'windows-1252', 'ascii//TRANSLIT', $sValue );
        // 2010-11-24 DOH   '.' must be removed, or the string will be part of the ScriptName(), which is done wrong by apache
        $sValue = str_replace( array( '#', '$', '\'', '(', ')', ',', ';', '?', '[', ']', '>', '<', '"' ), '', $sValue );
        $sValue = str_replace( array( ' ', '|', '/', '@', '=', '+', '*', '&' ), '-', $sValue );

        return( $sValue );
    }

	function FormatValue( $sType, $sFormat, $sValue )
	{
	    switch( $sType )
	    {
	        case 'date':
                $sValue = date( $sFormat, $sValue );
            break;

	        case 'strftime':
                $sValue = strftime( $sFormat, $sValue );
            break;

	        case 'number':
	            if ( !isset( $sFormat[ 0 ] ) )
	                $sFormat[ 0 ] = '2';
	            if ( !isset( $sFormat[ 1 ] ) )
	                $sFormat[ 1 ] = ',';
	            if ( !isset( $sFormat[ 2 ] ) )
	                $sFormat[ 2 ] = null;

                $sValue = number_format( $sValue, $sFormat[ 0 ], $sFormat[ 1 ], $sFormat[ 2 ] );
	        break;

	        case 'printf':
	            $sValue = sprintf( $sFormat, $sValue );
	        break;

	        case 'size':
	            if ( $sFormat == 'KB' )
	            {
	                $sValue = round( $sValue / 1024, 0 );

                    if ( strlen( $sFormat ) > 2 )
                    {
                        $sValue = number_format( $sValue, $sFormat[ 3 ], $sFormat[ 4 ], $sFormat[ 5 ] );
                    }
	            }
	            elseif ( $sFormat == 'MB' )
	            {
	                $sValue = round( $sValue / 1048576, 0 );

                    if ( strlen( $sFormat ) > 2 )
                    {
                        $sValue = number_format( $sValue, $sFormat[ 3 ], $sFormat[ 4 ], $sFormat[ 5 ] );
                    }
	            }
	            elseif ( substr( $sFormat, 0, 2 ) == 'XX' )
	            {
	                $sSuffix = '';
                    if ( $sValue >= 1099511627776 )
                    {
                        $sValue = round( $sValue / 1099511627776, 2 );
                        $sSuffix = 'TB';
                    }
                    elseif ( $sValue >= 1073741824 )
                    {
                        $sValue = round( $sValue / 1073741824, 2 );
                        $sSuffix = 'GB';
                    }
                    elseif ( $sValue >= 1048576 )
                    {
                        $sValue = round( $sValue / 1048576, 2 );
                        $sSuffix = 'MB';
                    }
                    elseif ( $sValue >= 1024 )
                    {
                        $sValue = round( $sValue / 1024, 2 );
                        $sSuffix = 'KB';
                    }
                    else
                    {
                        if ( $sValue == 1 )
                        {
                            $sSuffix = 'Byte';
                        }
                        else
                        {
                            $sSuffix = 'Bytes';
                        }
                    }

                    if ( strlen( $sFormat ) > 2 )
                    {
                        $sValue = number_format( $sValue, $sFormat[ 3 ], $sFormat[ 4 ], $sFormat[ 5 ] );
                    }

                    $sValue .= ' ' . $sSuffix;
	            }
	            break;

	        case 'list':
	            $aValues = explode( $sFormat, $sValue );
	            $sValue = '';
	            foreach( $aValues as $sItem )
	            {
	                $sItem = trim( $sItem );
	                $sValue .= '<option value="' . str_replace( '"', '&quot;', $sItem ) . '">' . $sItem . '</option>' . "\n";
                }
            break;

            case 'limit':
                $iLimit = $sFormat;
                if ( $iLimit > 0 )
                {
                    $sValue = substr( strip_tags( $sValue ), 0, $iLimit + 1 );
                    if ( strlen( $sValue ) > $iLimit )
                    {
                        $sValue = substr( $sValue, 0, $iLimit - 3 ) . '...';
                    }
                }
            break;
        }

	    return( $sValue );
    }

    /** Returns parsed content
        @param $sName Name of block, if not set, default block (set in constructor) is used.
        @return String
    */
    function Text( $sName = null )
    {
	    if ( $sName === null )
	    {
    		$sName = $this->sBlock;
	    }

        $sContent = &$this->aBlock[ $sName ][ 'content' ];

        // search for {tpl.xxxx}
        $iStart = strpos( $sContent, '{tpl.' );
        while( $iStart !== false )
        {
            $iEnd = strpos( $sContent, '}', $iStart );
            if ( $iEnd === false )
            {
                break;
            }

            $sVariable = substr( $sContent, $iStart + 5, $iEnd - $iStart - 5 );

            $sSpaces = '';
            if ( substr( $sVariable, 0, 6 ) == 'align#' )
            {
                $iLength = substr( $sVariable, 6 );
                $iLeft = strrpos( substr( $sContent, 0, $iStart ), "\n" );
                if ( $iLeft > 0 )
                {
                    $iLeft++;
                }

                if ( $iStart - $iLeft < $iLength - 1 )
                {
                    $sSpaces = str_repeat( ' ', $iLength - ( $iStart - $iLeft ) - 1 );
                }
                else
                {
                    $sSpaces = ' ';
                }
            }

            $sContent = substr( $sContent, 0, $iStart ) . $sSpaces . substr( $sContent, $iEnd + 1 );

            // to a avoid confusion of notepad++ plugin functionlist, use chr(123) instead '{'
            $iStart = strpos( $sContent, chr( 123 ) . 'tpl.' );
        }

	    return( $sContent );
    }

    /** echo content
     *	@param $sName string blockname, default null
     */
    function Output( $sName = null )
    {
        echo $this->Text( $sName );
    }

    function SetRoot( $sValue )
    {
        $this->sRoot = $sValue;
    }
    function GetRoot()
    {
        return( $this->sRoot );
    }

    /**
     * Cleans the cache directory
     * @param string $sClient
     * @return void
     */
    function CacheClean( $sClient = null )
    {
        $sFilter = '';

        if ( $sClient != null )
        {
            $sFilter .= '_' . preg_quote( $sClient ) . '$';
        }

        if ( $sFilter != '' )
        {
            $sFilter = '(' . $sFilter . ')';
        }

        $this->_CleanDirectory( $this->sCacheDirectory, $sFilter );
    }

    /**
     * Deletes all files recursively from the directory
     * @param string $sDirectory path to the directory
     * @param string $sFilter regular expression
     * @return void
     */
    function _CleanDirectory( $sDirectory, $sFilter )
    {
        $hDirectory = opendir( $this->sRoot . $sDirectory );
        if ( !$hDirectory )
        {
            return;
        }

        // Loop through the directory
        while( ( $sEntry = readdir( $hDirectory ) ) !== false )
        {
            // current directory
            if ( $sEntry == '.' )
            {
                continue;
            }
            // parrent directory
            if ( $sEntry == '..' )
            {
                continue;
            }
            // empty directory marker (needed by mercurial)
            if ( $sEntry == '.emptydir' )
            {
                continue;
            }

            // create full path and filename
            $sFullname = $this->sRoot . $sDirectory . $sEntry;

            // dive into subdirectories
            if ( is_dir( $sFullname ) )
            {
                $this->_CleanDirectory( $sDirectory . $sEntry . DIRECTORY_SEPARATOR, $sFilter );
                continue;
            }

            // check if filter is in filename
            if ( $sFilter === null )
            {
                unlink( $sFullname );
            }
            elseif ( preg_match( $sFilter, basename( $sFullname ) ) == 1 )
            {
                unlink( $sFullname );
            }
        }
        closedir( $hDirectory );
    }

    /** Creates the filename for the cachefile
        @param $sFilename Filename of Template (relative)
        @param $sBlock Name of the main block
        @return String with Cache Filename
    */
    function CacheFilename( $sFilename, $sBlock )
    {
        if ( $sBlock == 'main' )
        {
            $sBlock = '';
        }

        // Create Cache Directory
        $sDirectory = $this->GetRoot() . $this->GetCacheDirectory();

        // Prepare cache directory name
        $this->CacheDirectoryPrepare( $sBlock, $sDirectory );

        // create cachefilename
        $this->CacheFilenamePrepare( $sBlock, $sFilename );

        // Append path from filename^
        $this->MkDirR( $sDirectory . dirname( $sFilename ), 0777 );

        return( $sDirectory . $sFilename );
    }

    /** prepares a directory name for the cachefile
        make path relative, or something else
        file is only used, if this function returns true
        @param &$sFilename
        @return true/false
    */
    function CacheDirectoryPrepare( $sBlock, &$sFilename )
    {
        return( true );
    }

    /** prepares a filename for the cachefile
        make path relative, or something else
        file is only used, if this function returns true

        @param &$sFilename
        @return true/false
    */
    function CacheFilenamePrepare( $sBlock, &$sFilename )
    {
        // Generate cache filename (must be extended with language because of resources)
        if ( isset( $this->sLanguage ) )
        {
            if ( $sBlock <> '' )
            {
                $sBlock = '_' . $sBlock;
            }

            $sBlock = '#' . $this->sLanguage . $sBlock;;
        }

        // generate filename
        // $sFilename = md5( realpath( $sFilename ) . $sBlock );
        $sFilename = str_replace( "\\", '/', str_replace( "\\\\", '/', realpath( $sFilename ) ) );
        $sFilename = substr( $sFilename, strlen( realpath( $this->GetRoot() ) ) + 1 );
        $sFilename .= $sBlock;

        return( true );
    }

    /** Looks for a cache file. If exists, read the cache file and return true
        Otherwise return false
        @param $sFilename Filename of the cache file
        @param $iTemplateLastModified Filetime of template file
        @return true/false
    */
    function CacheRead( $sFilename, $iTemplateLastModified )
    {
        // When cache not exists -> create cache
        if ( !file_exists( $sFilename ) )
        {
            return( false );
        }
        if ( filesize( $sFilename ) == 0 )
        {
            return( false );
        }

        // When template is newer then cache -> create cache
        $iLastModified = filemtime( $sFilename );
        if ( $iTemplateLastModified > $iLastModified )
        {
            // Look if not someone else is already creating the cache file
            if ( !file_exists( $sFilename . '_creating' ) )
            {
                return( false );
            }
            else
            {
                // When creating file is oldern then 2 minutes, assume that something went wrong
                // and create a new cachefile
                $iLastModified = filemtime( $sFilename . '_creating' );
                if ( time() > $iLastModified + 120 )
                {
                    return( false );
                }
            }
        }

        // Clear Stat Cache, open file, read file, close file.
    	clearstatcache();
    	if ( !( $pFile = fopen( $sFilename, 'r' ) ) )
    	{
            die( "CTpl: CacheRead: File not found: '$sFilename'" );
    	}
        $sBuffer = fread( $pFile, filesize( $sFilename ) );
    	fclose( $pFile );

        $this->aBlock = unserialize( $sBuffer );

        return( true );
    }

    /** Writes the contents of $this->aBlocks into a file
        returns true, if everything is ok. Otherwise false.
        @param $sFilename Filename of Template (relative)
        @return true/false
    */
    function CacheWrite( $sFilename )
    {
        // Open file
        $File = fopen( $sFilename, 'w' );

        // Serialize Block array and write it
        fputs( $File, serialize( $this->aBlock ) );

        // Close file
        fclose( $File );

        // Filesystemrechte setzen
        chmod( $sFilename, 0666 );

        // delete _creating file
        if ( file_exists( $sFilename . '_creating' ) )
        {
            unlink( $sFilename . '_creating' );
        }

        return( true );
    }

    function SetCache( $fValue = null )
    {
        if ( $fValue === null )
        {
            $fValue = true;
        }

        $this->fCache = $fValue;
    }

    function GetCache()
    {
        return( $this->fCache );
    }

    function SetCacheDirectory( $sValue )
    {
        $this->sCacheDirectory = $sValue;
    }

    function GetCacheDirectory()
    {
        return( $this->sCacheDirectory );
    }

    function Get( $sName )
    {
        return( $this->aVariable[ $sName ] );
    }

    function GetVars()
    {
        return( $this->aVariable );
    }

    function SetVars( $aValue )
    {
        $this->aVariable = $aValue;

	    foreach( $this->aLocal as $sName => $sValue )
	    {
			$this->aVariable[ $sName ] = $sValue;
	    }
    }

    function GetVarblocks( $sName )
    {
        if ( isset( $this->aBlock[ $sName ][ 'var_block' ] ) )
        {
            return( $this->aBlock[ $sName ][ 'var_block' ] );
        }
        else
        {
            return( array() );
        }
    }

    function IsBlock( $sName )
    {
    	return( isset( $this->aBlock[ $sName ] ) );
    }

    function IsParsed( $sName )
    {
    	return( $this->aBlock[ $sName ][ 'parsed' ] );
    }

    /** Reset Block content
        @param $sName Name of Block
        @return true/false
    */
    function Reset( $sName = Null )
    {
        if ( $sName === null )
        {
            $sName = 'main';
        }

        if ( !$this->IsBlock( $sName ) )
        {
            return( false );
        }

        $this->aBlock[ $sName ][ 'content' ] = '';
        $this->aBlock[ $sName ][ 'parsed' ] = false;

        return( true );
    }

    /** Set Block content
        @param $sName Name of Block
        @param $sBuffer Content of Block
        @return true/false
    */
    function SetBlock( $sName, $sBuffer )
    {
        if ( !$this->IsBlock( $sName ) )
        {
            return( false );
        }

        $this->aBlock[ $sName ][ 'content' ] = $sBuffer;

        return( true );
    }

    /** Creates an array with all res.variables used in the template
        @param $sTemplate Buffer with Template content
        @param $sResource Buffer with already read resources
        @return true/false
    */
    function ResourceReplace( $sFilename, &$sTemplate, $sResource = null )
    {
        // create resource filename
        $sResourceFilename = $sFilename;

        // remove extension
        $i = strrpos( $sResourceFilename, '.' );
        if ( $i !== false )
        {
            $sResourceFilename = substr( $sResourceFilename, 0, $i );
        }

        // add language extension
        if ( isset( $this->sLanguage ) )
        {
            $sResourceFilename .= '#' . $this->sLanguage;
        }

        // append resource extension
        $sResourceFilename .= '.res';

        // load resources
        $this->ResourceRead( $sResourceFilename, $sResource );

        // prepare resources
        $aRes = $this->_ResourcePrepare( $sResource );

        // replace resources in template
        $sTemplate = $this->_ResourceReplace( $sTemplate, $aRes );

        return( true );
    }

    /** Reads resources from resource file and appends it to buffer
        @param sFilename filename of resource file
        @param sBuffer reference to buffer to append resources
        @return true/false
    */
    function ResourceRead( $sFilename, &$sBuffer )
    {
        // check if file exists
        if ( !file_exists( $sFilename ) )
        {
            return( false );
        }

        // use file_get_contents if available, should be faster (see PHP documentation)
        if ( function_exists( 'file_get_contents' ) )
        {
            $sBuffer .= file_get_contents( $sFilename );
        }
        else
        {
            // open resource file
        	if ( !( $pFile = fopen( $sFilename, 'r' ) ) )
        	{
                die( "CTpl: ResourceRead: File could not be read: '$sFilename'" );
        	}

            // read whole file and append it to buffer
            $iFilesize = filesize( $sFilename );
            if ( $iFilesize > 0 )
            {
                $sBuffer .= fread( $pFile, $iFilesize );
            }

            // close file
        	fclose( $pFile );
        }

    	return( true );
    }

    /** Prepare resources: convert string buffer into array
        @param $sResource string buffer with resources: 1. line: name, 2. line: text
        @return array() with value/name pairs
    */
    function _ResourcePrepare( $sResource )
    {
        // create resource array
        $aBuffer = explode( "\n", str_replace( "\n\n", "\n", str_replace( "\r", "\n", $sResource ) ) );

        $aRes = array();
        $iMax = count( $aBuffer );
        for( $i = 0; $i < $iMax; $i++ )
        {
            $aBuffer[ $i ] = trim( $aBuffer[ $i ] );
            if ( $aBuffer[ $i ] != '' )
            {
            	$aRes[ $aBuffer[ $i ] ] = trim( $aBuffer[ ++$i ] );
            }
        }

        return( $aRes );
    }

    /** Replace resource entries in the template
        @param $sTemplate buffer with template content
        @param $aRes array with name/value pairs
        @return string with replace resources
    */
    function _ResourceReplace( $sTemplate, $aRes )
    {
        // Find all Resource Vars in Template
        preg_match_all( '/\{RES\.([' . $this->sResourceNames . ']+?)}/', $sTemplate, $aVariables );
        $aVariables = $aVariables[ 1 ];

        // Replace found Vars with values from Resource
        foreach( $aVariables as $sVariable )
        {
            $aVariable = explode( '#', $sVariable );

            $sValue = $aRes[ $aVariable[ 0 ] ];
            if ( $aVariable[ 1 ] == 'encoded' )
            {
                $sValue = urlencode( $sValue );
            }
            elseif ( $aVariable[ 1 ] == 'doubleencoded' )
            {
                $sValue = $this->DoubleEncode( $sValue );
            }
            elseif ( $aVariable[ 1 ] == 'doubleencodedx' )
            {
                $sValue = CTpl::DoubleEncode( $sValue );
            }
            elseif ( $aVariable[ 1 ] == 'slashed' )
            {
                $sValue = addslashes( $sValue );
            }
            elseif ( $aVariable[ 1 ] == 'plain' )
            {
                $sValue = StringPlain( $sValue );
            }
            elseif ( $aVariable[ 1 ] == 'text' )
            {
                $sValue = StringHtml2Text( $sValue );
            }
            elseif ( $aVariable[ 1 ] == 'html' )
            {
                $sValue = StringHtml2Text( $sValue );
                $sValue = StringText2Html( $sValue );
            }

            $sTemplate = str_replace( '{RES.' . $sVariable . '}', $sValue, $sTemplate );
        }

        return( $sTemplate );
    }

    function DoubleEncode( $sValue )
    {
        return( urlencode( urlencode( $sValue ) ) );
    }

    function Something2Byte( $sSize = 0 )
    {
        if ( !$sSize )
        {
            return( 0 );
        }

        $aSize[ 'GB' ] = 1073741824;
        $aSize[ 'Gb' ] = 1073741824;
        $aSize[ 'G' ] = 1073741824;
        $aSize[ 'g' ] = 1073741824;
        $aSize[ 'MB' ] = 1048576;
        $aSize[ 'Mb' ] = 1048576;
        $aSize[ 'M' ] = 1048576;
        $aSize[ 'm' ] = 1048576;
        $aSize[ 'KB' ] = 1024;
        $aSize[ 'Kb' ] = 1024;
        $aSize[ 'K' ] = 1024;
        $aSize[ 'k' ] = 1024;

        foreach( $aSize as $sKey => $iBytes )
        {
            if ( ( strlen( $sSize ) > strlen( $sKey ) ) && ( substr( $sSize, strlen( $sSize ) - strlen( $sKey ) ) == $sKey ) )
            {
                $sSize = substr( $sSize, 0, strlen( $sSize ) - strlen( $sKey ) ) * $iBytes;
                break;
            }
        }

        return( $sSize );
    }

    function MkDirR( $sPath, $iMode = null )
    {
        // Check if directory already exists
        if ( is_dir( $sPath ) || empty( $sPath) )
        {
            return( true );
        }

        // Ensure a file does not already exist with the same name
        if ( is_file( $sPath ) )
        {
            trigger_error( 'MkDirR() File exists with the same name', E_USER_WARNING );
            return( false );
        }

        // Create the directory tree
        $sPathNext = substr( $sPath, 0, strrpos( $sPath, '/' ) );
        if ( $this->MkDirR( $sPathNext, $iMode ) )
        {
            if ( !file_exists( $sPath . '/.' ) )
            {
                if ( !mkdir( $sPath, $iMode ) )
                {
                    return( false );
                }

                if ( !chmod( $sPath, $iMode ) )
                {
                    return( false );
                }
            }
        }

        return( true );
    }

    /** Sets the language
    	@param $sLanguage short text for the language: {de|en}, default de
    */
    function SetLanguage( $sValue = null )
    {
        if ( $sValue === null || $sValue === '' )
        {
            $sValue = 'de';
        }

        $this->sLanguage = $sValue;
    }
}

/** Converts German Sonderzeichen from HTML to Text
    @parm $sBuffer String
    @return $sBuffer
*/
function StringHtml2TextSonderzeichen( $sBuffer )
{
    $sBuffer = str_replace( '&auml;', '', $sBuffer );
    $sBuffer = str_replace( '&Auml;', '', $sBuffer );
    $sBuffer = str_replace( '&ouml;', '', $sBuffer );
    $sBuffer = str_replace( '&Ouml;', '', $sBuffer );
    $sBuffer = str_replace( '&uuml;', '', $sBuffer );
    $sBuffer = str_replace( '&Uuml;', '', $sBuffer );
    $sBuffer = str_replace( '&szlig;', '', $sBuffer );

    return( $sBuffer );
}

/** Entfernt alle Sonderzeichen, Tags, etc. aus einen String
    @param $sBuffer String
    @return String ohne \r, \n und ohne HTML Tags, " und ' werden durch &quot; ersetzt
*/
function StringPlain( $sBuffer )
{
    $sBuffer = StringHtml2Text( $sBuffer );
    $sBuffer = str_replace( "\r", '', $sBuffer );
    $sBuffer = str_replace( "-\n", '-', $sBuffer );
    $sBuffer = str_replace( "\n", ' ', $sBuffer );

// 2007.10.08 DOH plain is supposed to be plain text without newlines!
//    $sBuffer = str_replace( "\"", '&quot;', $sBuffer );
//    $sBuffer = str_replace( "'", '&quot;', $sBuffer );

    return( $sBuffer );
}

/** Entfernt alle Sonderzeichen, Tags, etc. aus einen String
    Ersetzt <br> durch CRLF
    @param $sBuffer String
    @return String
*/
function StringHtml2Text( $sBuffer )
{
    // replace <a href="mailto:firstname.lastname@server.com">blah blah</a> with blah blah (mailto:firstname.lastname@server.com)
    // replace <a href="mailto:firstname.lastname@server.com">firstname.lastname@server.com</a> with mailto:firstname.lastname@server.com
    // replace <a href="http://www.xyz.com/test">blah blah</a> with blah blah (http://www.xyz.com/test)
    // replace <a href="http://www.xyz.com/test">www.xyz.com/test</a> with http://www.xyz.com/test
    $sBuffer = preg_replace_callback( '/<a href=\"(mailto:)([a-z][\w\-\.]+@[a-z][\w\d\-\.]+)\"[^>]*>([^<]*)<\/a>/i', 'StringHtml2TextCallback', $sBuffer );
    $sBuffer = preg_replace_callback( '/<a href=\"([http|ftp|https]+:\/\/)([a-z][\w\-]*\.[a-z][\w\-]*\.[\w\-.\/\?&=%+:;#]+[\w\/\?&=%+:;#])\"[^>]*>([^<]*)<\/a>/i', 'StringHtml2TextCallback', $sBuffer );

    $sBuffer = str_replace( "\r", '', $sBuffer );
    $sBuffer = str_replace( array( "<br>\n", "<br/>\n", "<br />\n" ), "\n", $sBuffer );
    $sBuffer = str_replace( array( '<br>', '<br/>', '<br />' ), "\n", $sBuffer );

    $sBuffer = strip_tags( $sBuffer );

    $sBuffer = StringHtml2TextSonderzeichen( $sBuffer );

    $sBuffer = str_replace( '&gt;', '>', $sBuffer );
    $sBuffer = str_replace( '&lt;', '<', $sBuffer );
    $sBuffer = str_replace( '&quot;', '"', $sBuffer );
    $sBuffer = str_replace( '&amp;', '&', $sBuffer );
    $sBuffer = str_replace( '&nbsp;', ' ', $sBuffer );

    return( $sBuffer );
}

function StringHtml2TextCallback( $aMatches )
{
    // www.xyz.com == text within <a></a>
    if ( $aMatches[ 1 ] . $aMatches[ 2 ] == $aMatches[ 3 ] || $aMatches[ 2 ] == $aMatches[ 3 ] )
    {
        // http://www.xyz.com
        return( $aMatches[ 1 ] . $aMatches[ 2 ] );
    }
    else
    {
        // text (http://www.xyz.com)
        return( $aMatches[ 3 ] . ' (' . $aMatches[ 1 ] . $aMatches[ 2 ] . ')' );
    }
}

/** Wandelt eine String nach HTML um
    @param $sBuffer String
    @return String
*/
function StringText2Html( $sBuffer )
{
    $sBuffer = str_replace( '&', '&amp;', $sBuffer );

    $sBuffer = StringText2HtmlSonderzeichen( $sBuffer );

    $sBuffer = str_replace( '>', '&gt;', $sBuffer );
    $sBuffer = str_replace( '<', '&lt;', $sBuffer );
    $sBuffer = str_replace( '"', '&quot;', $sBuffer );

    $sBuffer = str_replace( "\n", '<br />', $sBuffer );
    $sBuffer = str_replace( "\r", '', $sBuffer );

    return( $sBuffer );
}

/** Converts German Sonderzeichen from Text to HTML
    @parm $sBuffer String
    @return $sBuffer
*/
function StringText2HtmlSonderzeichen( $sBuffer )
{
    $sBuffer = str_replace( '', '&auml;', $sBuffer );
    $sBuffer = str_replace( '', '&Auml;', $sBuffer );
    $sBuffer = str_replace( '', '&ouml;', $sBuffer );
    $sBuffer = str_replace( '', '&Ouml;', $sBuffer );
    $sBuffer = str_replace( '', '&uuml;', $sBuffer );
    $sBuffer = str_replace( '', '&Uuml;', $sBuffer );
    $sBuffer = str_replace( '', '&szlig;', $sBuffer );

    return( $sBuffer );
}

/** Encodes buffer for xml output
    @param $sBuffer string
    @return string
*/
function StringXmlEncode( $sBuffer )
{
    // non-printable chars must be removed!
    // #x0 - #x8 (ASCII 0 - 8)
    // #xB - #xC (ASCII 11 - 12)
    // #xE - #x1F (ASCII 14 - 31)
    $sBuffer = str_replace
    (
        array( chr( 0 ), chr( 1 ) , chr( 2 ), chr( 3 ), chr( 4 ), chr( 5 ), chr( 6 ), chr( 7 ), chr( 8 ), chr( 11 ), chr( 12 ), chr( 14 ), chr( 15 ), chr( 16 ), chr( 17 ), chr( 18 ), chr( 19 ), chr( 20 ), chr( 21 ), chr( 22 ), chr( 23 ), chr( 24 ), chr( 25 ), chr( 26 ), chr( 27 ), chr( 28 ), chr( 29 ), chr( 30 ), chr( 31 ) ),
        '',
        $sBuffer
    );

    $sBuffer = str_replace
    (
        array( '', '', '"', '', '', '', '', '', '', '', '<', '>' ),
        array( '-', '-', '&#34;', '&#252;', '&#220;', '&#246;', '&#214;', '&#228;', '&#196;', '&#223;', '&lt;', '&gt;' ),
        $sBuffer
    );

    $sBuffer = preg_replace
    (
        array( '/\&([a-z\d\#]+)\;/i', '/\&/', '/\#\|\|([a-z\d\#]+)\|\|\#/i', '/([^a-zA-Z\d\s\<\>\&\;\.\:\=\"\-\/\%\?\!\'\(\)\[\]\{\}\$\#\+\,\@_])/e' ),
        array( '#||\\1||#', '&amp;', '&\\1;', "'&#'.ord('\\1').';'" ),
        $sBuffer
    );

    return( $sBuffer );
}

if ( !function_exists( 'StringConvert' ) )
{
/** Convert a string from one charset into another. Currently only iso-8859-15, windows-1252 and utf-8 are supported
    @param $sBuffer string
    @param $sFrom charset
    @param $sTo charset
    @return string
*/
    function StringConvert( $sBuffer, $sFrom, $sTo )
    {
        switch( $sFrom )
        {
            case 'windows-1252':
                switch( $sTo )
                {
                    case 'windows-1252':
                    break;

                    case 'utf-8':
                        $sBuffer = StringWindowsToUtf8( $sBuffer );
                    break;

                    case 'iso-8859-15':
                    break;
                }
            break;

            case 'utf-8':
                switch( $sTo )
                {
                    case 'windows-1252':
                        $sBuffer = utf8_decode( $sBuffer );
                    break;

                    case 'utf-8':
                    break;

                    case 'iso-8859-15':
                        $sBuffer = utf8_decode( $sBuffer );
                    break;
                }
            break;

            case 'iso-8859-15':
                switch( $sTo )
                {
                    case 'windows-1252':
                    break;

                    case 'utf-8':
                        $sBuffer = utf8_encode( $sBuffer );
                    break;

                    case 'iso-8859-15':
                    break;
                }
            break;
        }

        return( $sBuffer );
    }
}

if ( !function_exists( 'StringWindowsToUtf8' ) )
{
    /*
        This structure encodes the difference between ISO-8859-1 and Windows-1252,
        as a map from the UTF-8 encoding of some ISO-8859-1 control characters to
        the UTF-8 encoding of the non-control characters that Windows-1252 places
        at the equivalent code points.
        Aidan Kehoe <php-manual@parhasard.net>
    */
    global $aCp1252ToUtf8;
    $aCp1252ToUtf8 = array
    (
        "\xc2\x80" => "\xe2\x82\xac", /* EURO SIGN */
        "\xc2\x82" => "\xe2\x80\x9a", /* SINGLE LOW-9 QUOTATION MARK */
        "\xc2\x83" => "\xc6\x92",     /* LATIN SMALL LETTER F WITH HOOK */
        "\xc2\x84" => "\xe2\x80\x9e", /* DOUBLE LOW-9 QUOTATION MARK */
        "\xc2\x85" => "\xe2\x80\xa6", /* HORIZONTAL ELLIPSIS */
        "\xc2\x86" => "\xe2\x80\xa0", /* DAGGER */
        "\xc2\x87" => "\xe2\x80\xa1", /* DOUBLE DAGGER */
        "\xc2\x88" => "\xcb\x86",     /* MODIFIER LETTER CIRCUMFLEX ACCENT */
        "\xc2\x89" => "\xe2\x80\xb0", /* PER MILLE SIGN */
        "\xc2\x8a" => "\xc5\xa0",     /* LATIN CAPITAL LETTER S WITH CARON */
        "\xc2\x8b" => "\xe2\x80\xb9", /* SINGLE LEFT-POINTING ANGLE QUOTATION */
        "\xc2\x8c" => "\xc5\x92",     /* LATIN CAPITAL LIGATURE OE */
        "\xc2\x8e" => "\xc5\xbd",     /* LATIN CAPITAL LETTER Z WITH CARON */
        "\xc2\x91" => "\xe2\x80\x98", /* LEFT SINGLE QUOTATION MARK */
        "\xc2\x92" => "\xe2\x80\x99", /* RIGHT SINGLE QUOTATION MARK */
        "\xc2\x93" => "\xe2\x80\x9c", /* LEFT DOUBLE QUOTATION MARK */
        "\xc2\x94" => "\xe2\x80\x9d", /* RIGHT DOUBLE QUOTATION MARK */
        "\xc2\x95" => "\xe2\x80\xa2", /* BULLET */
        "\xc2\x96" => "\xe2\x80\x93", /* EN DASH */
        "\xc2\x97" => "\xe2\x80\x94", /* EM DASH */
        "\xc2\x98" => "\xcb\x9c",     /* SMALL TILDE */
        "\xc2\x99" => "\xe2\x84\xa2", /* TRADE MARK SIGN */
        "\xc2\x9a" => "\xc5\xa1",     /* LATIN SMALL LETTER S WITH CARON */
        "\xc2\x9b" => "\xe2\x80\xba", /* SINGLE RIGHT-POINTING ANGLE QUOTATION*/
        "\xc2\x9c" => "\xc5\x93",     /* LATIN SMALL LIGATURE OE */
        "\xc2\x9e" => "\xc5\xbe",     /* LATIN SMALL LETTER Z WITH CARON */
        "\xc2\x9f" => "\xc5\xb8"      /* LATIN CAPITAL LETTER Y WITH DIAERESIS*/
    );

    function StringWindowsToUtf8( $sBuffer )
    {
       global $aCp1252ToUtf8;
       return( strtr( utf8_encode( $sBuffer ), $aCp1252ToUtf8 ) );
    }
}
