<?php

defined( 'oPage' ) or die();

class CWebpage_Base
{
    /** Modul oPage
    */
    var $oPage;

    /** Template engine
    */
    var $oTpl;

    /** Template filename
    */
    var $sTemplate;

    /** Allow caching of content in the browser
    */
    var $fCacheInBrowser = true;

    /** trim content before sending it to client
    */
    var $fTrim = false;

    function CWebpage_Base()
    {
        global $oFactory;
        global $oPlugins;

        $this->oPage = $oFactory->Modul( 'CPage' );

        // Hook for plugins
        $oPlugins->Webpage( $this, 'constructor' );
    }

    function Init()
    {
        global $oPlugins;

		$this->oPage->QueryData();

        if ( !$this->oPage->IsData() )
        {
            die( 'CWebpage_Base: QueryData: invalid page: ' . $this->oPage->GetFilterValue( 'name' ) );
        }

        // Hook for plugins
        $oPlugins->Webpage( $this, 'init' );

        return( true );
    }

    function Begin()
    {
        global $oPlugins;

        if ( !$this->GetCacheInBrowser() && !headers_sent() )
        {
            $this->Begin_Header();
        }
        $this->Begin_Prepare();

        // Hook for plugins
        $oPlugins->Webpage( $this, 'begin' );
    }

    function Begin_Header()
    {
        header( 'Expires: Tue, 21 Nov 1972 05:00:00 GMT' );                 // Datum der Vergangenheit
        header( 'Last-Modified: ' . gmdate( 'D, d M Y H:i:s' ) . ' GMT' );  // immer gendert
        header( 'Cache-Control: no-cache, must-revalidate' );               // HTTP/1.1
        header( 'Pragma: no-cache' );                                       // HTTP/1.0
    }

    function Begin_Prepare()
    {
        /* @var $oApp CApp */
        global $oApp;
        /* @var $oFactory CFactory */
        global $oFactory;

        // Seiten-Template anlegen
		$this->oTpl = $oFactory->Template( $oApp->Pathlink( $this->GetTemplate() ) );

        $sScriptname = $oApp->GetScriptname();
        $aParameter = $oApp->GetParameters();
        $this->Assign( 'query', $aParameter );

        // Unset a lot of parameters not good for back urls, etc.
        unset( $aParameter[ session_name() ] );
        unset( $aParameter[ 'submit' ] );
        unset( $aParameter[ 'submit_logon' ] );
        unset( $aParameter[ 'submit_logoff' ] );
        unset( $aParameter[ 'timezone' ] );
        unset( $aParameter[ 'datetime' ] );
        unset( $aParameter[ 'language' ] );

        // Unset all empty form parameters, admin_field and admin_action entries
        foreach( $aParameter as $sName => $sValue )
        {
            if ( $sValue === '' || $sValue === null )
                unset( $aParameter[ $sName ] );
            if ( substr( $sName, 0, 11 ) == 'admin_field' )
                unset( $aParameter[ $sName ] );
            if ( substr( $sName, 0, 12 ) == 'admin_action' )
                unset( $aParameter[ $sName ] );
        }

        // Prepare Back Uri
        $this->Assign( 'global.backuri', $oApp->GetRoot() . $sScriptname . HttpParam( $aParameter ) );
        $sQuery = HttpParam( $aParameter, $oApp->GetQuerySeparator(), $oApp->GetParamSeparator() );
        $this->Assign( 'global.backurix', $oApp->GetRoot() . $sScriptname . $sQuery );
        if ( $sQuery === null || $sQuery === '' )
            $sQuery = $oApp->GetQuerySeparator();
        $this->Assign( 'global.backuriy', $oApp->GetRoot() . $sScriptname . $sQuery );

        // Prepare Print Uri
        unset( $aParameter[ 'back' ] );
        $aParameter[ 'print' ] = '1';
        // Prepare Print Uri (new)
        $this->assign( 'global.printuri', $oApp->GetRoot() . $sScriptname . HttpParam( $aParameter ) );
        $this->assign( 'global.printurix', $oApp->GetRoot() . $sScriptname . HttpParam( $aParameter, $oApp->GetQuerySeparator(), $oApp->GetParamSeparator() ) );

        // Assign page values
        $this->Assign( 'page', $this->oPage->aResult[ 0 ] );

        // if page is in print mode, do not index
        // 2010-11-20 DOH    if page is called not via main url, do not index
        $sUrl = $oApp->GetProtocol() . $oApp->GetHostname();
        if ( $oApp->GetPrint() || ( $oApp->GetHttp() != $sUrl && $oApp->GetHttps() != $sUrl ) )
        {
            $this->Assign( 'page.robots', 'noindex,nofollow' );
        }
        else
        {
            $this->Assign( 'page.robots', 'index,follow' );
        }
    }

    function End()
    {
        global $oPlugins;

        global $oTimer;
        global $iTrace;

        // Hook for plugins
        $oPlugins->Webpage( $this, 'end' );

        $sContent = $this->End_Prepare();
        $this->PrintContent( $sContent );

        if ( isset( $oTimer ) && $iTrace > 0 )
        {
            $oTimer->Stop( 'page' );
            $oTimer->Trace( 'page', Null, $iTrace );
        }

        return( true );
    }

    function End_Prepare()
    {
        global $sRoot;

        // Assign page values again to reflect updates
        $this->Assign( 'page', $this->oPage->aResult[ 0 ] );

        // Parse IE css blocks
        foreach( array( 'default_ie', 'default_ie9', 'default_ie8', 'default_ie7', 'default_ie6' ) as $sName )
        {
            if ( file_exists( $sRoot . 'custom/' . $sName . '.css' ) )
            {
                $this->Parse( 'main.opage_' . $sName );
            }
        }

        // Seite parsen
        $this->Parse();

        // Content holen
        $sContent = $this->oTpl->Text();

        // Remove empty attributes, but not alt="", summary="" and title=""
        $sContent = preg_replace( '/(action|alt|summary|title|value)\s?=\s?\"\"/i', '$1=" "', $sContent );
        $sContent = preg_replace( '/alt\s?=\s?\"\"/i', 'alt=" "', $sContent );
        $sContent = preg_replace( '/\s\w*\s?=\s?\"\"/m', '', $sContent );
        $sContent = preg_replace( '/(action|alt|summary|title|value)\s?=\s?\"\s\"/i', '$1=""', $sContent );

        // trim content
        if ( $this->GetTrim() )
        {
            // remove spaces from beginning of lines, remove empty lines
            $sContent = preg_replace( '/^\s*/m', '', $sContent );
            // reduce mulitple spaces to one
            $sContent = preg_replace( '/[ \t][ \t]+/', ' ', $sContent );
        }

        return( $sContent );
    }

    function PrintContent( &$sContent )
    {
        $this->PrintContentBuffer();

        echo $sContent;
        flush();

        return( true );
    }

    function PrintContentBuffer()
    {
        // flush any existing output buffer (see include/error.php)
        if ( function_exists( 'ob_get_level' ) )
            while( ob_get_level() > 0 )
               ob_end_flush();
    }

    /** Return a reference to the page module object
        @param void
        @return reference to page module object
    */
    function &Page()
    {
        $oPage =& $this->oPage;
        return( $oPage );
    }

    function SetId( $sPage )
    {
        $this->oPage->SetFilter( 'name', $sPage );
    }
    function GetId()
    {
        return( $this->oPage->GetFilterValue( 'name' ) );
    }

    function GetTitle()
    {
        return( $this->oPage->Get( 'title' ) );
    }
    function SetTitle( $sValue )
    {
        $this->oPage->Set( 'title', $sValue );
        return( true );
    }

    function AddTitle( $sValue, $sDelimiter = null, $fPrepend = null )
    {
        if ( $sValue === '' || $sValue === null )
            return( false );
        if ( $sDelimiter === null )
            $sDelimiter = ' - ';
        if ( $fPrepend === null )
            $fPrepend = false;

        $sTitle = $this->GetTitle();
        if ( $sTitle !== '' && $sTitle !== null )
            $sTitle = $fPrepend ? $sDelimiter . $sTitle : $sTitle . $sDelimiter;
        $sTitle = $fPrepend ? $sValue . $sTitle : $sTitle . $sValue;

        $this->SetTitle( $sTitle );

        return( true );
    }

    function SetCacheInBrowser( $fValue )
    {
        $this->fCacheInBrowser = $fValue;
    }
    function GetCacheInBrowser()
    {
        return( $this->fCacheInBrowser );
    }

    function SetTrim( $fValue )
    {
        $this->fTrim = $fValue;
    }
    function GetTrim()
    {
        return( $this->fTrim );
    }

    function SetTemplate( $sValue )
    {
		$this->sTemplate = $sValue;
    }
    function GetTemplate()
    {
        return( $this->sTemplate );
    }

    function SetTemplateParameter( $aValue )
    {
        die( 'Invalid function call: CWebpage::SetTemplateParameter();' );
    }
    function GetTemplateParameter()
    {
        if ( !is_object( $this->oTpl ) )
        {
            return( null );
        }
        return( $this->oTpl->GetVars() );
    }

    function &Template()
    {
        $oTpl = &$this->oTpl;
        return( $oTpl );
    }

    function &Assign( $sName, $sValue )
    {
        $this->oTpl->Assign( $sName, $sValue );
        $oThis = &$this;
        return( $oThis );
    }

    function &Append( $sName, $sValue )
    {
        $this->oTpl->Append( $sName, $sValue );
        $oThis = &$this;
        return( $oThis );
    }

    function &Parse( $sBlock = Null )
    {
        $this->oTpl->Parse( $sBlock );
        $oThis = &$this;
        return( $oThis );
    }

    function AddNavigation( &$oNavigation, $sId, $sParent = '', $sQuery = '', $iHeight = Null, $sType = Null, $iWidth = Null, $fForce = Null )
    {
        global $oDb;

        if ( $sId === '' || $sId === null )
            return( false );

        $sTable = $this->oPage->GetTable();
        $sFields = $this->oPage->GetFields();
        $sFilter = $this->oPage->GetField( 'name' ) . "='" . addslashes( $sId ) . "'";
        $sOrder = '';
		$aResult = $oDb->Read( $sTable, $sFields, $sFilter, $sOrder );
        if ( ArrayCount( $aResult ) == 0 )
        {
            die( "CWebpage::AddNavigation: Ungltiger Page Eintrag '$sId'" );
            return( false );
        }

        foreach( $aResult as $sKey => $aRow )
        {
            if ( $sQuery === '' || $sQuery === null )
                $sName = $oNavigation->Add( $aRow[ 'name' ], $aRow[ 'path' ], $aRow[ 'target' ], $aRow[ 'title_' . $oNavigation->GetType() ], $sParent, $iHeight, $iWidth, $sType, Null, Null, Null, $fForce, $aRow[ 'onclick' ] );
            else
            {
                if ( strpos( $aRow[ 'path' ], '?' ) === false )
                    $sName = $oNavigation->Add( $aRow[ 'name' ], $aRow[ 'path' ] . "?$sQuery", $aRow[ 'target' ], $aRow[ 'title_' . $oNavigation->GetType() ], $sParent, $iHeight, $iWidth, $sType, Null, Null, Null, $fForce, $aRow[ 'onclick' ] );
                else
                    $sName = $oNavigation->Add( $aRow[ 'name' ], $aRow[ 'path' ] . "&$sQuery", $aRow[ 'target' ], $aRow[ 'title_' . $oNavigation->GetType() ], $sParent, $iHeight, $iWidth, $sType, Null, Null, Null, $fForce, $aRow[ 'onclick' ] );
            }
        }

        return( $sName );
    }

    function UserCheck( $sRedirect )
    {
        global $oUser;

        $sUsername = $oUser->GetUser( 'username' );

        if ( $sUsername === null || $sUsername === '' )
        {
            header( 'Location: ' . $sRedirect );
            exit;
        }

        return( true );
    }

    function RightCheck( $sModul, $sRight, $sRedirect )
    {
        global $oApp;
        global $oUser;

        if ( $oApp->GetFeature( 'user', 'rights' ) > 0 )
            if ( !$oUser->Right( $sModul, $sRight ) )
            {
                if ( $sRedirect === null )
                {
                    return( false );
                }

                header( 'Location: ' . $sRedirect );
                exit;
            }

        return( true );
    }
}

switch( $oApp->GetoPage() )
{
    case 'base':
    default:
        class CWebpage extends CWebpage_Base
        {
            function CWebpage()
            {
                parent::CWebpage_Base();
            }
        }
    break;

    case 'std':
        include_once $sRoot . 'include_std/webpage.php';
        class CWebpage extends CWebpage_Std
        {
            function CWebpage()
            {
                parent::CWebpage_Std();
            }
        }
    break;

    case 'pro':
        include_once $sRoot . 'include_std/webpage.php';
        include_once $sRoot . 'include_pro/webpage.php';
        class CWebpage extends CWebpage_Pro
        {
            function CWebpage()
            {
                parent::CWebpage_Pro();
            }
        }
    break;
}
