<?php

defined( 'oPage' ) or die();

class CContent extends CModul
{
    /** Number of possible groupX fields
    */
    var $iFieldGroup = 10;

    function CContent( $sName = null )
    {
        // Create admin page
        $this->SetShowPageAdmin( true );
        // Set path to admin templates
        $this->SetAdminPath( 'plugin/content/admin/content/' );

        // Set default name
        $this->SetDefault( 'content' );

        // Initialize base class
        $this->CModul( $sName );

        // Set default filters
        $this->SetView( 'active' );
        $this->SetFilter( 'display', 'n', '<>' );

        // Set default sort orders
        $this->SetOrder( 'section_id' );
        $this->SetOrder( 'parent_id' );
        $this->SetOrder( 'sort' );
        $this->SetOrder( 'date', 'desc' );
        $this->SetOrder( 'date_from', 'desc' );
        $this->SetOrder( 'date_till' );

        // Disable unused default fields
        $this->SetFieldUse( 'section_id', false );
        $this->SetFieldUse( 'category_id', false );
        $this->SetFieldUse( 'category_name', false );
        $this->SetFieldUse( 'parent_id', false );
        for ( $i = 1; $i <= $this->iFieldGroup; ++$i )
            $this->SetFieldUse( 'group' . $i, false );
        $this->SetFieldUse( 'date', false );
        $this->SetFieldUse( 'more', false );
        $this->SetFieldUse( 'image_text', false );
        $this->SetFieldUse( 'image_medium', false );
        $this->SetFieldUse( 'author', false );
        $this->SetFieldUse( 'author_email', false );
        $this->SetFieldUse( 'filename_title', false );
        $this->SetFieldUse( 'filename_name', false );
        $this->SetFieldUse( 'filename_file', false );
        $this->SetFieldUse( 'url_title', false );
        $this->SetFieldUse( 'url', false );
        $this->SetFieldUse( 'target', false );
        $this->SetFieldUse( 'sort2', false );

        // Enabled default fields for listview in the administration interface
        $this->SetFieldUseAdminList( 'id' );
        $this->SetFieldUseAdminList( 'title_display' );
        $this->SetFieldUseAdminList( 'text' );
        $this->SetFieldUseAdminList( 'sort' );
        $this->SetFieldUseAdminList( 'display' );

        // Enable automatic translation of urls to hyperlinks
        $this->SetTextPrepareUrlsStrict();
    }

    function SetFields()
    {
        $this->SetField( 'id', 'ID' );
        $this->SetField( 'section_id', 'FOREIGNID' );
        $this->SetField( 'category_id', 'FOREIGNID' );
        $this->SetField( 'category_name', 'FOREIGNNAME' );
        $this->SetField( 'parent_id', 'FOREIGNID' );
        for ( $i = 1; $i <= $this->iFieldGroup; ++$i )
            $this->SetField( 'group' . $i, 'YESNO' );
        $this->SetField( 'clearfix', 'YESNO' );
        $this->SetField( 'template', 'SELECT' );
        $this->SetField( 'date', 'DATETIME' );
        $this->SetField( 'title_display', null, true );
        $this->SetField( 'title', null, true );
        $this->SetField( 'image_text', null, true );
        $this->SetField( 'image_small', 'IMAGE' );
        $this->SetField( 'image_medium', 'IMAGE' );
        $this->SetField( 'image_big', 'IMAGE' );
        $this->SetField( 'text', 'TEXTBOX', true );
        $this->SetField( 'text_ishtml', 'YESNO', true );
        $this->SetField( 'more', null, true );
        $this->SetField( 'author' );
        $this->SetField( 'author_email', 'EMAIL' );
        $this->SetField( 'date_from', 'DATETIME' );
        $this->SetField( 'date_till', 'DATETIME' );
        $this->SetField( 'filename_title', null, true );
        $this->SetField( 'filename_name', null, true );
        $this->SetField( 'filename_file', 'FILE', true );
        $this->SetField( 'url_title', null, true );
        $this->SetField( 'url', 'URL' );
        $this->SetField( 'target', 'TARGET' );
        $this->SetField( 'sort', 'INTEGER' );
        $this->SetField( 'sort2', 'INTEGER' );
        $this->SetField( 'display', 'DISPLAY', true );

        parent::SetFields();
    }

    function AdminList( &$oAdmin )
    {
        global $oApp;

        if ( $this->GetFieldUse( 'parent_id' ) && !$this->GetFieldUseAdminList( 'section_id' ) )
        {
            $this->SetFieldUseAdminList( 'parent_id', false );

            $sParent = $oApp->GetParameter( 'parent_id' );
            if ( $sParent === '' )
                $sParent = null;
            $this->SetFilter( 'parent_id', $sParent );
        }

        parent::AdminList( $oAdmin );
    }

    function AdminListProperties( &$oAdmin, $sName )
    {
        parent::AdminListProperties( $oAdmin, $sName );

        switch( $sName )
        {
            case 'parent_id':
                if ( $this->GetFieldUse( 'title' ) && $this->GetFieldUse( 'name' ) )
                    $aParent = $this->GetList( array( 'title', 'name', 'id' ), null, '', null, array( '0' => ' ', '2a' => '(', '2b' => ')', '3a' => '[', '3b' => ']' ) );
                else
                    $aParent = $this->GetList( array( 'title', 'id' ), null, '', null, array( '0' => ' ', '2a' => '[', '2b' => ']' ) );
                $oAdmin->SetOptions( $sName, $aParent );
            break;

            case 'template':
                $oAdmin->SetOptions( $sName, $this->GetFieldAdminOptions( $sName ) );
            break;

            case 'date':
                $oAdmin->SetDefault( $sName, null );
            break;

            case 'title_display':
                $oAdmin->SetReadonly( $sName, true );
                $oAdmin->SetMaxlength( $sName, $this->Parameter( 'title_maxlength', 255 ) );
                $oAdmin->SetSyncMaster( $sName, 'title' );
            break;

            case 'text':
                $oAdmin->SetWidth( $sName, 30 );
                $oAdmin->SetLimit( $sName, 150 );
            break;

            case 'sort2':
                $oAdmin->SetWidth( $sName, 5 );
                $oAdmin->SetMaxlength( $sName, 10 );
            break;
        }
    }

    function Admin( &$oAdmin )
    {
        global $oApp;

        parent::Admin( $oAdmin );

        if ( $this->GetFieldUse( 'parent_id' ) )
            $oAdmin->SetDefault( 'parent_id', $oApp->GetParameter( 'parent_id' ) );
    }

    function AdminProperties( &$oAdmin, $sName )
    {
        parent::AdminProperties( $oAdmin, $sName );

        switch( $sName )
        {
            case 'parent_id':
                if ( $this->GetFieldUse( 'title' ) && $this->GetFieldUse( 'name' ) )
                    $aParent = $this->GetList( array( 'title', 'name', 'id' ), null, '', null, array( '0' => ' ', '2a' => '(', '2b' => ')', '3a' => '[', '3b' => ']' ) );
                else
                    $aParent = $this->GetList( array( 'title', 'id' ), null, '', null, array( '0' => ' ', '2a' => '[', '2b' => ']' ) );
                $oAdmin->SetOptions( $sName, $aParent );
            break;

            case 'template':
                $oAdmin->SetOptions( $sName, $this->GetFieldAdminOptions( $sName ) );
                $oAdmin->SetWidth( $sName, $this->GetFieldWidthAdmin( $sName, 130 ) );
            break;

            case 'date':
                $oAdmin->SetDefault( $sName, null );
            break;

            case 'title_display':
                $oAdmin->SetMaxlength( $sName, $this->Parameter( 'title_maxlength', 255 ) );
                $oAdmin->SetSyncMaster( $sName, 'title' );
            break;

            case 'title':
                $oAdmin->SetMaxlength( $sName, $this->Parameter( 'title_maxlength', 255 ) );
                $oAdmin->SetWidth( $sName, $this->GetFieldWidthAdmin( $sName, 40 ) );
            break;

            case 'text':
                $oAdmin->SetWidth( $sName, $this->GetFieldWidthAdmin( $sName, 108 ) );
                $oAdmin->SetHeight( $sName, 32 );
            break;

            case 'text_ishtml':
                $oAdmin->SetDefault( $sName, 'y' );
            break;

            case 'image_text':
                $oAdmin->SetWidth( $sName, 10 );
            break;

            case 'author':
                $oAdmin->SetWidth( $sName, 30 );
            break;

            case 'author_email':
                $oAdmin->SetWidth( $sName, 30 );
            break;

            case 'url_title':
                $oAdmin->SetWidth( $sName, $this->GetFieldWidthAdmin( $sName, 40 ) );
            break;

            case 'url':
                $oAdmin->SetWidth( $sName, $this->GetFieldWidthAdmin( $sName, 35 ) );
            break;

            case 'target':
                $oAdmin->SetWidth( $sName, $this->GetFieldWidthAdmin( $sName, 20 ) );
            break;

            case 'sort2':
                $oAdmin->SetWidth( $sName, 5 );
                $oAdmin->SetMaxlength( $sName, 10 );
            break;
        }
    }

    function PrepareRow( &$aRow )
    {
        global $oApp;

        foreach( $this->GetFieldList() as $sName )
        {
            switch( $sName )
            {
                case 'title':
                case 'image_text':
                case 'author':
                    $this->PrepareText( $aRow, $sName );
                break;

                case 'text':
                    if ( $aRow[ $sName ] !== null && $aRow[ $sName ] !== '' )
                    {
                        if ( strpos( $aRow[ $sName ], '<p' ) === false )
                        {
                            if ( strpos( $aRow[ $sName ], '<div' ) === false && strpos( $aRow[ $sName ], '<table' ) === false )
                            {
                                $aRow[ $sName ] = '<p>' . $aRow[ $sName ] . '</p>';
                            }
                        }
                    }

                    $this->PrepareText( $aRow, $sName, $aRow[ $sName . '_ishtml' ] == 'y' );
                break;

                case 'image_small':
                case 'image_medium':
                case 'image_big':
                    $this->PrepareImage( $aRow, $sName );
                break;

                case 'filename_file':
                    $this->PrepareFile( $aRow, $sName );
                break;

	            case 'date':
	            case 'date_from':
	            case 'date_till':
	                $this->PrepareDate( $aRow, $sName );
                break;

                case 'url':
                    $this->PrepareUrl( $aRow, $sName, null, null, null, $oApp->GetRoot() );
                break;
            }
        }

        // Use url as url_title, when url_title is missing
        if ( $this->GetFieldUse( 'url_title' ) && $this->GetFieldUse( 'url' ) && $aRow[ 'url_title' ] == '' )
		{
			$sValue = $aRow[ 'url' ];
			if ( substr( $sValue, 0, 7 ) == 'http://' )
            {
				$sValue = substr( $sValue, 7 );
            }
			if ( substr( $sValue, -1, 1 ) == '/' )
            {
				$sValue = substr( $sValue, 0, -1 );
            }
			$aRow[ 'url_title' ] = $sValue;
		}

        // This is a module specific action: Replace {image_xxx} with img tag.
        if ( $this->GetFieldUse( 'text' ) )
        {
            $this->PrepareImageText( $aRow, 'image_small', 'text' );
            $this->PrepareImageText( $aRow, 'image_medium', 'text' );
            $this->PrepareImageText( $aRow, 'image_big', 'text' );
        }

        return( parent::PrepareRow( $aRow ) );
    }

    /** Replaces {image_xxxx} with HTML code and clear the field.
        If {image_xxxx} is found, and image_xxxx has a valid value, {image_xxxx} gets replaced by an img tag and image_small is set to null
        @param &$aRow name/value pairs
        @param $sNameImage name of the image field, default = null ('image_small')
        @param $sNameText name of the text field, default = null ('text')
        @return true
    */
    function PrepareImageText( &$aRow, $sNameImage = null, $sNameText = null )
    {
        global $oApp;

        if ( $sNameImage === null )
            $sNameImage = 'image_small';
        if ( $sNameText === null )
            $sNameText = 'image_text';

        // Is the text field used?
        if ( !$this->GetFieldUse( $sNameText ) )
            return( true );

        // Does the text field contain {image_xxxx}?
        if ( strpos( $aRow[ $sNameText ], '{' . $sNameImage . '}' ) === false )
            return( true );

        // Is the image field used and does it have a value?
        if ( !$this->GetFieldUse( $sNameImage ) || $aRow[ $sNameImage ] === null || $aRow[ $sNameImage ] === '' )
        {
            $aRow[ $sNameText ] = str_replace( '{' . $sNameImage . '}', '', $aRow[ $sNameText ] );
            return( true );
        }

        // replace {image_small} with HTML code
        $aRow[ $sNameText ] = str_replace( '{' . $sNameImage . '}', '<img src="' . $oApp->GetRoot() . $aRow[ $sNameImage ] . '" width="' . $aRow[ $sNameImage . '#width' ] . '" height="' . $aRow[ $sNameImage. '#height' ] . '" border="0" alt="" style="width: ' . $aRow[ $sNameImage. '#widthunits' ] . '; height: ' . $aRow[ $sNameImage. '#heightunits' ] . '; max-width: 100%;" />', $aRow[ $sNameText ] );

        // clear image_small
        $aRow[ $sNameImage ] = null;

        return( true );
    }
}

