<?php

defined( 'oPage' ) or die();

/** Plugin class
*/
class CSectionPlugin extends CPlugin
{
    /**	activate url rewrite mode
        'asdf/' is rewritten to 'index.php?subsection=asdf'

        2009.08.27 DOH Changes need to be done accordingly in .htaccess and plugin/section/controller.php (Comment below, _Fillnavigation). Don't forget to upload .htaccess!

        Apache:
        .htacess should contain:

        options +followsymlinks
        AcceptPathInfo On

        # Make sure that directory listings are disabled
        Options -Indexes

        # make sure nobody gets the htaccess, README, COPYING or VERSION files
        <Files ~ "^([\._]ht|README$|VERSION$|COPYING$)">
            Order allow,deny
            Deny from all
        </Files>

        # Enable URL rewriting
        RewriteEngine On

        # Not all installations will require the following line. If you do,
        # change it to the path to your directory relative to your document root.
        #RewriteBase /

        # passthru empty pages
        RewriteRule ^$ - [last]

        # passthru existing files and directories
        RewriteCond %{REQUEST_FILENAME} -f [or]
        RewriteCond %{REQUEST_FILENAME} -d
        RewriteRule .* - [last]

        # append missing / to GETs
        RewriteCond %{REQUEST_METHOD} GET
        RewriteRule ^(.+[^/])$ %{REQUEST_URI}/ [nocase,redirect=permanent]

        # route section names
        # http://localhost/website/section/subsection/...
        # --> http://localhost/website/section_-_subsection/...
        RewriteRule ^([a-z0-9_-]+)/([a-z0-9_-]+)/(.*)?$ ./$1_-_$2/$3 [nocase,next]

        # http://localhost/website/section_-_subsection/...
        # --> http://localhost/website/index.php/subsection=section_-_subsection/...
        RewriteRule ^([a-z0-9_-]+)/(.*)?$ ./index.php/subsection=$1/$2 [nocase,last]

        @default: false
    */
    var $fUrlRewrite = false;

    var $sSection;
    var $aSection = array();

    var $sSubSection;
    var $aSubSection = array();

    /**
     * If SubSectionMode is true, listing of sections is limited to current section
     * and subsections of the current section
     * @var bool $fSubSectionMode default false
     */
    var $fSubSectionMode = false;

    var $iLevel;

    /** Use page title as base title:
        true: "sectiontitle - pagetitle"
        false: "sectiontitle"
        default: true;
    */
    var $fUsePageTitle = true;

    /** Setup plugins
        @var &$oSetup reference to the setup object
    */
    function Setup( &$oSetup )
    {
        $oSetup->AddTool( 'CSection' );
        $oSetup->AddTool( 'CSectionAdmins' );
        $oSetup->AddTool( 'CSectionViews' );
    }

    /**
     * Setup modul
     * @param CModul &$oModul reference to the module object
     * @param string &$sClass
     */
    function SetupModulData( &$oObject, $sClass )
    {
        switch( $sClass )
        {
            case 'CSection':
                $aRow[] = array
                (
                    'id' => 1,
                    'name' => 'Home',
                    'page_name' => 'home',
                    'use_page_title' => 'n',
                    'title' => 'Home',
                    'title_navigation' => 'Home',
                    'title_menu' => 'Home',
                    'title_content' => 'Home',
                    'navigation_hide' => 'n',
                    'menu_hide' => 'n',
                    'admin_navigation_hide' => 'n',
                    'admin_menu_hide' => 'n',
                    'display' => 'y'
                );
                $oObject->SetupData( $aRow );
            break;

            default:
                return( false );
            break;
        }

        return( true );
    }

    /**
     * Factory extension for the plugin
     * @param string $sClass name of the wanted class
     */
    function FactoryModul( $sClass, &$sCreate, &$sName, &$sInclude )
    {
        switch( $sClass )
        {
            case 'CSection':
                $sCreate = $sClass;
                $sInclude = $this->sPath . 'modul/section.php';
            break;

            case 'CSectionAdmins':
                $sCreate = 'CMenu';
                if ( $sName === null )
                {
                    $sName = 'section_admins';
                }
            break;

            case 'CSectionViews':
                $sCreate = 'CMenu';
                if ( $sName === null )
                {
                    $sName = 'section_views';
                }
            break;

            default:
                return( false );
            break;
        }

        return( true );
    }

    /** Factory extension for the plugin
        @param $oObject reference of the instanced
        @param $sClass name of the wanted class
    */
    function FactoryModulConfigure( &$oObject, $sClass )
    {
        global $oApp;

        $oObject->SetScriptname( $this->GetSubsectionAsParameter( $oApp->GetScriptName() ) );

        // set subsection name as parameter postfix
        $oObject->SetParameterPostfix( $this->GetSubSection() );

        switch( $sClass )
        {
            case 'CGallery':
                $oObject->SetFieldUseAdminList( 'section_id' );
            break;

            case 'CPage':
                $oObject->SetFieldUseAdminList( 'page_content' );
                $oObject->SetFieldUseAdminList( 'page_view' );
                $oObject->SetFieldUseAdminList( 'page_admin' );
            break;

            case 'CSection':
                $oObject->AddSub( $oObject->GetClassGroup() . 'Admins', 'id', 'section_id' );
                $oObject->AddSub( $oObject->GetClassGroup() . 'Views', 'id', 'section_id' );
                $oObject->AddSub( $sClass, 'id', 'parent_id' );
                $oObject->SetFieldUseAdminList( 'parent_id' );
                $oObject->SetFieldUseAdminList( 'name' );
                $oObject->SetFieldUseAdmin( 'page_name' );
                $oObject->SetFieldUseAdmin( 'parameter' );
                $oObject->SetFieldUseAdmin( 'use_page_title' );
                $oObject->SetFieldUseAdmin( 'admin_navigation_hide' );
                $oObject->SetFieldUseAdmin( 'admin_menu_hide' );
                $oObject->SetFieldUseAdmin( 'navigation_hide' );
                $oObject->SetFieldUseAdmin( 'menu_hide' );
            break;

            case 'CSectionAdmins':
                // Set new default name (used as subadmin name)
                $oObject->SetDefault( 'menu_admins' );
                $oObject->SetSetupPages( false );
                $oObject->SetFieldUseAdminList( 'section_id' );
                $oObject->SetFieldUseAdminList( 'parameter' );
                $oObject->SetFieldUseAdminList( 'use_page_title' );
                $oObject->SetPageAdmin( true );
            break;

            case 'CSectionViews':
                // Set new default name (used as subadmin name)
                $oObject->SetDefault( 'menu_views' );
                $oObject->SetSetupPages( false );
                $oObject->SetFieldUseAdminList( 'section_id' );
                $oObject->SetFieldUseAdminList( 'parameter' );
                $oObject->SetFieldUseAdminList( 'template_targetname' );
                $oObject->SetFieldUse( 'parent_name', false );
                $oObject->SetPageView( true );
            break;

            default:
                return( false );
            break;
        }

        return( true );
    }

    /** Factory extension for the plugin
        @param $oObject reference of the instanced module
        @param $sClass name of the wanted class
    */
    function FactoryControlConfigure( &$oObject, $sClass )
    {
        global $oApp;

        $oObject->SetScriptname( $this->GetSubsectionAsParameter( $oApp->GetScriptName() ) );

        switch( $sClass  )
        {
            case 'CPager':
                if ( !$oApp->GetAdmin() )
                {
                    if ( $this->fUrlRewrite )
                    {
                        $oObject->SetParameterExclude( 'subsection' );
                    }
                }
            break;
        }

        return( true );
    }

    /**
     * This is called from the main page
     * @param $oWebpage CWebpage webpage object
     */
    function Index( &$oWebpage )
    {
        global $sRoot;
        global $oFactory;
        global $oPlugins;

        // Create section view module
        $oSectionViews = $oFactory->Modul( 'CSectionViews' );

        // Get all views associated with the currenct subsection
        $oSectionViews->SetFilter( 'section_id', $this->GetSubSectionId() );
        $oSectionViews->QueryData();
        if ( !$oSectionViews->IsData() )
        {
            return( false );
        }

        // Create page modul
        $oPage = $oFactory->Modul( 'CPage' );

        // Process all views
        foreach( $oSectionViews->GetResult() as $aSectionView )
        {
            if ( $aSectionView[ 'name' ] === null || $aSectionView[ 'name' ] === '' )
            {
                continue;
            }

            // Get the view
            $oPage->SetFilter( 'name', $aSectionView[ 'name' ] );
            $oPage->QueryData();
            if ( !$oPage->IsData() )
            {
                trigger_error( 'invalid page ' . $aSectionView[ 'name' ], E_USER_ERROR );
                continue;
            }
            $aPage = $oPage->GetRow();

            // Get controller
            $oPlugin = null;
            if ( substr( $aSectionView[ 'name' ], 0, 7 ) == 'plugin_' )
            {
                $sPlugin = substr( $aSectionView[ 'name' ], 7 );
                $iSub = strpos( $sPlugin, '_' );
                if ( $iSub !== false )
                {
                    $sPlugin = substr( $sPlugin, 0, $iSub );
                }
                $oPlugin = $oPlugins->Get( $sPlugin );
            }
            if ( $oPlugin == null )
            {
                $oPlugin = &$this;
            }

            // Include and run the view class
            $sBlock = $aSectionView[ 'template_targetname' ];
            if ( $sBlock === '' || $sBlock === null )
            {
                $sBlock = $oPlugin->GetBlock();
            }

            // Process parameter field
            $aParameter = null;
            if ( $aSectionView[ 'parameter' ] !== '' && $aSectionView[ 'parameter' ] !== null )
            {
                parse_str( $aSectionView[ 'parameter' ], $aParameter );

                foreach( array_keys( $aParameter ) as $sKey )
                {
                    if ( $aParameter[ $sKey ] === '' || $aParameter[ $sKey ] === null )
                    {
                        unset( $aParameter[ $sKey ] );
                    }
                }
            }

            $oWebpage->Assign( $sBlock, $oPlugin->RunView( $aPage[ 'path' ], $aParameter ) );
            $oWebpage->Parse( 'main.' . $sBlock );
        }
    }

    /**
     * Hook entry point for Admin
     * @param CWebpageNormal &$oWebpage webpage object
     * @param string &$sUrl a reference to the url for the filename, which should be loaded
     */
    function Admin( &$oWebpage, &$sUrl )
    {
        global $oApp;
        global $oFactory;
        global $oUser;

        if ( $sUrl !== null && $sUrl !== '' )
        {
            return( false );
        }

        if ( $this->GetSubSection() === null || $this->GetSubSection() === '' )
        {
            return( false );
        }

        // create SectionAdmins module
        $oSectionAdmins = $oFactory->Modul( 'CSectionAdmins' );
        if ( $oSectionAdmins === null )
        {
            return( false );
        }

        $oSectionAdmins->SetFieldPrefix( $oSectionAdmins->GetTable( 1 ) . '.' );
        // join with Page module
        $oPage = $oFactory->Modul( 'CPage' );
        $oPage->SetFieldPrefix( $oPage->GetTable( 1 ) . '.' );
        $oSectionAdmins->AddTable( $oPage->GetTable( 1 ) );
        $oSectionAdmins->AddJoin( $oPage->GetTable( 1 ), 'inner join', $oSectionAdmins->GetField( 'name' ) . '=' . $oPage->GetField( 'name' ) );
        // add fields from joined tables
        $oSectionAdmins->SetField( 'page_path', null, null, $oPage->GetField( 'path' ), false );
        $oSectionAdmins->SetField( 'page_title', null, null, $oPage->GetField( 'title' ), false );
        $oSectionAdmins->SetField( 'page_title_navigation', null, null, $oPage->GetField( 'title_navigation' ), false );
        $oSectionAdmins->SetField( 'page_title_menu', null, null, $oPage->GetField( 'title_menu' ), false );
        $oSectionAdmins->SetField( 'page_target', null, null, $oPage->GetField( 'target' ), false );
        // set order by
        $oSectionAdmins->SetOrder( 'sort' );

        $oSectionAdmins->SetFilter( 'section_id', $this->GetSubSectionId() );
        $oSectionAdmins->SetLimit( 1 );
        $oSectionAdmins->QueryData();
        if ( !$oSectionAdmins->IsData() )
        {
            return( false );
        }

        $aRow = $oSectionAdmins->GetRow();

        if ( $aRow[ 'page_path' ] == '' )
        {
            return( false );
        }

        // Query: section, subsection and other parameters
        // use separate arrays for aParameter and aAdminParameter to get the values in order
        $aParameter = array();
        parse_str( $aRow[ 'parameter' ], $aAdminParameter );
        if ( !isset( $aAdminParameter[ 'section' ] ) )
        {
            $aParameter[ 'section' ] = $this->GetSection();
        }
        else
        {
            $aParameter[ 'section' ] = $aAdminParameter[ 'section' ];
            unset( $aAdminParameter[ 'section' ] );
        }
        if ( !isset( $aAdminParameter[ 'subsection' ] ) )
        {
            $aParameter[ 'subsection' ] = $this->GetSubSection();
        }
        else
        {
            $aParameter[ 'subsection' ] = $aAdminParameter[ 'subsection' ];
            unset( $aAdminParameter[ 'subsection' ] );
        }
        foreach( array_keys( $aAdminParameter ) as $sKey )
        {
            $aParameter[ $sKey ] = $aAdminParameter[ $sKey ];
        }
        foreach( array_keys( $aParameter ) as $sKey )
        {
            if ( IsNull( $aParameter[ $sKey ] ) )
            {
                unset( $aParameter[ $sKey ] );
            }
        }

        // Query: now the query parameters can be generated
        $sQuery = HttpParam( $aParameter, ( strpos( $aRow[ 'page_path' ], '?' ) === false ? $oApp->GetQuerySeparator() : $oApp->GetParamSeparator() ), $oApp->GetParamSeparator() );
        $sUrl = $aRow[ 'page_path' ] . $sQuery;
    }

    /**
     * Hook entry point for AdminNavigation
     * @param CWebpage &$oWebpage webpage object
     * @param CModul $oModule reference to module navigation object
     */
    function AdminNavigation( &$oWebpage, &$oModule, &$iFeatureRights, &$fGlobalNavigation )
    {
        global $oApp;
        global $oFactory;
        global $oUser;

        $sSection = $this->GetSection();
        if ( $sSection !== null && $sSection !== '' )
        {
            // do not use global navigation
            $fGlobalNavigation = false;

            // create SectionAdmins module
            $oSectionAdmins = $oFactory->Modul( 'CSectionAdmins' );
            $oSectionAdmins->SetFieldPrefix( $oSectionAdmins->GetTable( 1 ) . '.' );
            // join with Page module
            $oPage = $oFactory->Modul( 'CPage' );
            $oPage->SetFieldPrefix( $oPage->GetTable( 1 ) . '.' );
            $oSectionAdmins->AddTable( $oPage->GetTable( 1 ) );
            $oSectionAdmins->AddJoin( $oPage->GetTable( 1 ), 'inner join', $oSectionAdmins->GetField( 'name' ) . '=' . $oPage->GetField( 'name' ) );
            // add fields from joined tables
            $oSectionAdmins->SetField( 'page_path', null, null, $oPage->GetField( 'path' ), false );
            $oSectionAdmins->SetField( 'page_title', null, null, $oPage->GetField( 'title' ), false );
            $oSectionAdmins->SetField( 'page_title_navigation', null, null, $oPage->GetField( 'title_navigation' ), false );
            $oSectionAdmins->SetField( 'page_title_menu', null, null, $oPage->GetField( 'title_menu' ), false );
            $oSectionAdmins->SetField( 'page_target', null, null, $oPage->GetField( 'target' ), false );
            // set order by
            $oSectionAdmins->SetOrder( 'sort' );

            // create Section module
            $oSection = $oFactory->Modul( 'CSection' );
            $oSection->ClearOrder( 'title' );
            $oSection->SetFilter( 'admin_menu_hide', array( null, 'n' ) );

            $oModule->FillNavigation( $oSection, $oSectionAdmins, 'name', $sSection, null, $iFeatureRights, null, 0, null, 'name' );
        }
    }

    /** Hook entry point for AdminList
        @param &$oWebpage reference to the page object
        @param &$oModul reference to the module
    */
    function AdminList( &$oWebpage, &$oModul, &$oAdmin )
    {
        global $oApp;
        global $oFactory;

        if ( !$oModul->GetFieldUse( 'section_id' ) )
        {
            return( false );
        }

        if ( $this->GetSubSection() === null || $this->GetSubSection() === '' )
        {
            return( false );
        }
        $sSubSectionId = $this->GetSubSectionId();

        // prepend section title to web page title
        $oWebpage->AddTitle( $this->aSubSection[ 'title_navigation' ], ': ', true );

        // set section name as parameter postfix
        $oModul->SetParameterPostfix( $this->aSubSection[ 'name' ] );

        if ( $this->GetSubSectionMode() )
        {
            // Create section module
            $oSection = $oFactory->Modul( 'CSection' );

            // ID Liste erstellen
            if ( $oSection->GetFieldUse( 'parent_id' ) )
            {
                $aId = array_keys( $oSection->GetList( null, '( ' . $oSection->GetField( 'parent_id' ) . '=' . AddSlashes( $sSubSectionId ) . ' OR ' . $oSection->GetField( 'id' ) . '=' . AddSlashes( $sSubSectionId ) . ' )', '' ) );
            }
            else
            {
                $aId = array_keys( $oSection->GetList( null, '( ' . $oSection->GetField( 'id' ) . '=' . AddSlashes( $sSubSectionId ) . ' )', '' ) );
            }

            if ( count( $aId ) > 0 )
            {
                $oModul->SetFilter( 'section_id', '(' . implode( ',', $aId ) . ')', 'in', false );
            }
            else
            {
                $oModul->SetFilter( 'section_id', '' );
            }
        }
        else
        {
            $oModul->SetFilter( 'section_id', $sSubSectionId );
            $oModul->SetFieldUseAdminList( 'section_id', false );
        }

        return( true );
    }

    /** Hook entry point for AdminDetail
        @param &$oWebpage reference to the page object
        @param &$oModul reference to the module
    */
    function AdminDetail( &$oWebpage, &$oModul, &$oAdmin )
    {
        global $oApp;
        global $oFactory;

        if ( !$oModul->GetFieldUse( 'section_id' ) )
        {
            return( false );
        }

        if ( $this->GetSubSection() === null || $this->GetSubSection() === '' )
        {
            return( false );
        }
        $sSubSectionId = $this->GetSubSectionId();

        // append section title to web page title
        $oWebpage->AddTitle( $this->aSubSection[ 'title' ], ' ' );

        // set section name as parameter postfix
        $oModul->SetParameterPostfix( $this->aSubSection[ 'name' ] );

        if ( $this->GetSubSectionMode() )
        {
            // Create section module
            $oSection = $oFactory->Modul( 'CSection' );

            // ID Liste erstellen
            if ( $oSection->GetFieldUse( 'parent_id' ) )
            {
                $aId = array_keys( $oSection->GetList( null, '( ' . $oSection->GetField( 'parent_id' ) . '=' . AddSlashes( $sSubSectionId ) . ' OR ' . $oSection->GetField( 'id' ) . '=' . AddSlashes( $sSubSectionId ) . ' )', '' ) );
            }
            else
            {
                $aId = array_keys( $oSection->GetList( null, '( ' . $oSection->GetField( 'id' ) . '=' . AddSlashes( $sSubSectionId ) . ' )', '' ) );
            }

            if ( count( $aId ) > 0 )
            {
                $oModul->SetFilter( 'section_id', '(' . implode( ',', $aId ) . ')', 'in', false );
            }
            else
            {
                $oModul->SetFilter( 'section_id', '' );
            }
        }
        else
        {
            $oModul->SetFilter( 'section_id', $sSubSectionId );
            // $oModul->SetFieldUseAdmin( 'section_id', false );
        }

        if ( $oModul->GetFieldUseAdmin( 'section_id' ) )
        {
            $sParentSubSection = $oApp->GetParameter( 'parent_subsection' );
            if ( $this->GetSubSectionMode() && ( $sParentSubSection !== null && $sParentSubSection !== '' ) )
            {
                $oModul->SetDefaultValue( 'section_id', $sParentSubSection );
            }
            else
            {
                $oModul->SetDefaultValue( 'section_id', $sSubSectionId );
            }
        }

        return( true );
    }

    /** Hook entry point for Webpage
        @param &$oWebpage reference to the page object
        @param $sPart ('constructor', 'begin', 'end')
    */
    function Webpage( &$oWebpage, $sPart )
    {
        global $oApp;
        global $oUser;

        switch( $sPart )
        {
            case 'constructor':
                // Get parameter value section
                $this->SetSection( $oApp->GetParameter( 'section' ) );

                // Get parameter value subsection
                $sDefault = 'Home';
                if ( $oApp->GetAdmin() )
                {
                    if ( $oUser->GetUser() == 'admin' )
                    {
                        $sDefault = '';
                    }
                }
                $this->SetSubSection( $oApp->GetParameter( 'subsection', $sDefault ) );

                // Get parameter value subsectionmode
                if ( $oApp->GetParameter( 'subsectionmode', '0' ) == '1' )
                {
                    $this->SetSubSectionMode( true );
                }
            break;

            case 'init':
            break;

            case 'begin':
            break;

            case 'end':
            break;
        }
    }

    /** Hook entry point for AdminWebpageBasis
        @param &$oWebpage reference to the page object
        @param $sPart ('constructor', 'begin', 'end')
    */
    function AdminWebpageBasis( &$oWebpage, $sPart )
    {
        global $oApp;
        global $oFactory;
        global $oUser;

        switch( $sPart )
        {
            case 'constructor':
                // Read data for subsection (and maybe update subsection name and section name)
                if ( $this->QuerySubSectionData() )
                {
                    // Read data for section (and maybe update section name)
                    $this->QuerySectionData();
                }
            break;

            case 'begin':
                // Assign parameters to page template
                // Note: This is the backend, in the frontend, section is an array
                $oWebpage->Assign( 'section', $oApp->GetParameter( 'section' ) );
                $oWebpage->Assign( 'subsection', $oApp->GetParameter( 'subsection' ) );
                $oWebpage->Assign( 'subsectionmode', $oApp->GetParameter( 'subsectionmode' ) );
            break;

            case 'end':
            break;
        }
    }

    /** Hook entry point for AdminWebpageContent
        @param &$oWebpage reference to the page object
        @param $sPart ('constructor', 'begin', 'end')
    */
    function AdminWebpageContent( &$oWebpage, $sPart )
    {
        global $sRoot;
        global $oApp;
        global $oFactory;
        global $oUser;

        switch( $sPart )
        {
            case 'constructor':
            break;

            case 'begin':
                // create navigation object
                $oNavigation = $oFactory->Control( 'CNavigation' );

                // Show 'Administration' only for user admin
                if ( $oUser->GetUser() == 'admin' )
                {
                    $oWebpage->AddNavigation( $oNavigation, 'admin' );
                }

                // Create section modul
                $oSection = $oFactory->Modul( 'CSection' );

                // fill top section menu
                $oSection->SetFilter( 'parent_id', null );
                $oSection->SetFilter( 'admin_navigation_hide', array( null, 'n' ) );
                // $oSection->FillNavigation( $oNavigation, 'admin/index.php', null, null, 'title_navigation', true, null, null, 'name' );
                $oSection->QueryData();
                $aResult = $oSection->GetResult();
                foreach( $aResult as $aRow )
                {
                    // Query: section, subsection and other parameters
                    // use separate arrays for aParameter and aSectionParameter to get the values in order
                    $aParameter = array();
                    parse_str( $aRow[ 'parameter' ], $aSectionParameter );
                    if ( !isset( $aSectionParameter[ 'section' ] ) )
                    {
                        $aParameter[ 'section' ] = $aRow[ 'name' ];
                    }
                    else
                    {
                        $aParameter[ 'section' ] = $aSectionParameter[ 'section' ];
                        unset( $aSectionParameter[ 'section' ] );
                    }
                    if ( !isset( $aSectionParameter[ 'subsection' ] ) )
                    {
                        $aParameter[ 'subsection' ] = $aRow[ 'name' ];
                    }
                    else
                    {
                        $aParameter[ 'subsection' ] = $aSectionParameter[ 'subsection' ];
                        unset( $aSectionParameter[ 'subsection' ] );
                    }
                    if ( $aParameter[ 'section' ] == $aParameter[ 'subsection' ] )
                    {
                        unset( $aParameter[ 'section' ] );
                    }
                    foreach( array_keys( $aSectionParameter ) as $sKey )
                    {
                        $aParameter[ $sKey ] = $aSectionParameter[ $sKey ];
                    }
                    foreach( array_keys( $aParameter ) as $sKey )
                    {
                        if ( IsNull( $aParameter[ $sKey ] ) )
                        {
                            unset( $aParameter[ $sKey ] );
                        }
                    }

                    $sQuery = HttpParam( $aParameter, $oApp->GetQuerySeparator(), $oApp->GetParamSeparator() );
                    $oNavigation->Add( $aRow[ 'name' ], 'admin/index.php' . $sQuery, null, $aRow[ 'title_navigation' ] );
                }

                // mark active
                $oNavigation->SetActive( $this->GetSection() );

                // Create template
                $oTemplate = $this->Template( 'admin/content.tpl', 'admin_sectionnavigationtop' );

                // output navigation
                $oNavigation->OutputTpl( $oTemplate );

                // Get the output
                $oTemplate->Parse();
                $oWebpage->Append( 'navigationtop', $oTemplate->Text() );
            break;

            case 'end':
            break;
        }
    }

    /** Hook entry point for WebpageX
        @param &$oWebpage reference to the page object
        @param $sPart ('constructor', 'begin_pre', 'begin', 'end')
    */
    function WebpageBasis( &$oWebpage, $sPart )
    {
        global $oApp;
        global $oFactory;

        switch( $sPart )
        {
            case 'constructor':
                // Read data for subsection (and maybe update subsection name)
                if ( !$this->QuerySubSectionData() )
                {
                    header( 'HTTP/1.1 404 Not Found' );
                    header( 'Location: ' . $oApp->GetRoot() );
                    exit;
                }

                // Read data for section (and maybe update section name)
                if ( !$this->QuerySectionData() )
                {
                    header( 'HTTP/1.1 404 Not Found' );
                    header( 'Location: ' . $oApp->GetRoot() );
                    exit;
                }

                // Calc subsection level
                $this->QuerySubsectionLevel();
            break;

            case 'init':
                // Create or update page title (this must be in 'init')

                // Clear page title, if it should not be used
                if ( !$this->fUsePageTitle )
                {
                    $oWebpage->SetTitle( '' );
                }

                // Prepend page title with section title, when section title is != page title
                if ( $oWebpage->GetTitle() != $this->aSection[ 'title' ] )
                {
                    $oWebpage->AddTitle( $this->aSection[ 'title' ], null, true );
                }

                // Prepend page title with subsection title
                if ( strtolower( $this->aSubSection[ 'name' ] ) != strtolower( $this->aSection[ 'name' ] ) )
                {
                    $oWebpage->AddTitle( $this->aSubSection[ 'title' ], null, true );
                }
            break;

            case 'begin':
                // Note: This is the frontend, in the backend, section is a string
                $oWebpage->Assign( 'section', $this->aSection );
                $oWebpage->Assign( 'subsection', $this->aSubSection );

                // 2010-11-20 DOH   overwrite global.scriptname set in CTemplate
                $oWebpage->Assign( 'global.scriptname', $this->GetSubsectionAsParameter( $oApp->GetScriptName() ) );
            break;

            case 'end':
            break;
        }
    }

    function SetSection( $sSection, $sDefault = null )
    {
        if ( $sSection === null || $sSection === '' )
        {
            $this->sSection = $sDefault;
            return;
        }
        $this->sSection = $sSection;
    }

    function GetSection()
    {
        return( strtolower( $this->sSection ) );
    }

    function GetSectionId()
    {
        if ( !isset( $this->aSection[ 'id' ] ) )
        {
            trigger_error( 'section id not set', E_USER_ERROR );
            return( null );
        }
        return( $this->aSection[ 'id' ] );
    }

    function SetSubSection( $sSection, $sDefault = null )
    {
        if ( $sSection === null || $sSection === '' )
        {
            $this->sSubSection = $sDefault;
            return;
        }
        $this->sSubSection = $sSection;
    }

    function GetSubSection()
    {
      if ( $this->sSubSection === null || $this->sSubSection === '' )
      {
        return( $this->GetSection() );
      }
      return( strtolower( $this->sSubSection ) );
    }

    function GetSubSectionId()
    {
        if ( !isset( $this->aSubSection[ 'id' ] ) )
        {
           return( $this->GetSectionId() );
        }
        return( $this->aSubSection[ 'id' ] );
    }

    /**
     * Set subsection mode
     * @param bool $fValue true/false, default null (true)
     * @return void
     */
    function SetSubSectionMode( $fValue = null )
    {
        $this->fSubSectionMode = $fValue;
    }

    /**
     * Get current value for SubSectionMode
     * @param void
     * @return true/false
     */
    function GetSubSectionMode()
    {
        return( $this->fSubSectionMode );
    }

    /**
     * Reads subsection data from database and sets $aSubSection
     * @return true/false
     */
    function QuerySubSectionData()
    {
        global $oApp;
        global $oFactory;

        $this->aSubSection = array();

        $sSubSection = $this->GetSubSection();
        if ( $sSubSection === '' || $sSubSection === null )
        {
            return( false );
        }

        $oSection = $oFactory->Modul( 'CSection' );

        $this->aSubSection = $this->QuerySectionByName( $sSubSection, $oSection );
        if ( $this->aSubSection === null )
        {
            return( false );
        }

        // Evaluate 'parameter' field: subsection=
        if ( !$oApp->GetAdmin() )
        {
            parse_str( $this->aSubSection[ 'parameter' ], $aParameter );

            // If subsection is set
            if ( isset( $aParameter[ 'subsection' ] ) && $aParameter[ 'subsection' ] != '' )
            {
                $this->aSubSection = $this->QuerySectionByName( $aParameter[ 'subsection' ], $oSection );
                if ( $this->aSubSection === null )
                {
                    return( false );
                }
            }
        }

        $oSection->PrepareRow( $this->aSubSection );

        return( true );
    }

    /**
     * Reads section data from database and sets $aSection
     * @return true/false
     */
    function QuerySectionData()
    {
        global $oFactory;

        $this->aSection = array();

        /** var CSection $oSection */
        $oSection = $oFactory->Modul( 'CSection' );

        // If parent_id is not used, use subsection data
        if ( !$oSection->GetFieldUse( 'parent_id' ) )
        {
            $this->sSection = $this->sSubSection;
            $this->aSection = $this->aSubSection;
            return( true );
        }

        $oSection->SetLimitRows( 1 );

        // Get section data for section
        $sSection = $this->GetSection();

        // Get root section from subsection, if no root section is set
        if ( $sSection === '' || $sSection === null )
        {
            $iId = $oSection->QueryRoot( $this->GetSubSectionId() );
            $oSection->SetFilter( 'id', $iId );
        }
        else
        {
            // Get section data for main section
            $oSection->SetFilter( 'name', $sSection );
            $oSection->SetFilter( 'parent_id', null );
        }

        $oSection->QueryData();
        if ( !$oSection->IsData() )
        {
            return( false );
        }
        $this->aSection = $oSection->GetRow();

        $this->SetSection( $this->aSection[ 'name' ] );

        $oSection->PrepareRow( $this->aSection );

        return( true );
    }

    /**
     * Sets the section level depending on the current subsection
     * @return true/false
     */
    function QuerySubsectionLevel()
    {
        global $oFactory;

        $this->iLevel = 1;

        /** var CSection $oSection */
        $oSection = $oFactory->Modul( 'CSection' );

        if ( !$oSection->GetFieldUse( 'parent_id' ) )
        {
            return( true );
        }

        // Get current parent id
        $iParentId = $this->aSubSection[ 'parent_id' ];

        // as long as a parent exists
        while( $iParentId !== null )
        {
            // increase level
            $this->iLevel++;

            // read parent
            $oSection->SetFilter( 'id', $iParentId );
            $oSection->QueryData();
            $iParentId = $oSection->Get( 'parent_id' );
        }

        return( true );
    }

    /** Returns the level of the current subsection. Top level sections have level 1
        @param none
        @return integer
    */
    function GetLevel()
    {
        return( $this->iLevel );
    }

    /**
        @param $oNavigation control
        @param $sValue id, etc.
        @param $sField 'id' or 'parent_id', gets compared to value, default 'id'
        @param $iLevel starting level, default 1
        @param $fRecursive = null (default false)
        @param $sParameter additional parameter
        @param $iLevelMax max level, default null
    */
    function FillNavigation( &$oNavigation, $sValue, $sField = null, $iLevel = null, $fRecursive = null, $sParameter = null, $iLevelMax = null )
    {
        global $oFactory;

        // Create section modul
        $oSection = $oFactory->Modul( 'CSection' );

        // set filter to hide all items, which should not be displayed in the navigation
        $oSection->SetFilter( $oNavigation->GetType() . '_hide', array( null, 'n' ) );

        // Create page modul
        $oPage = $oFactory->Modul( 'CPage' );

        $this->_FillNavigation( $oNavigation, $oSection, $oPage, $sValue, $sField, $iLevel, $fRecursive, null, $sParameter, $iLevelMax );
    }

    /**
        @param $oNavigation control
        @param $oSection modul
        @param $oPage modul
        @param $sValue id, etc.
        @param $sField 'id' or 'parent_id', gets compared to value, default 'id'
        @param $iLevel starting level, default 1
        @param $fRecursive = null (default false)
        @param $sParent parent value (used in recursion to get a unique name like 'section1.section2')
        @param $sParameter additional parameter
        @param $iLevelMax max level, default null
    */
    function _FillNavigation( &$oNavigation, &$oSection, &$oPage, $sValue, $sField = null, $iLevel = null, $fRecursive = null, $sParent = null, $sParameter = null, $iLevelMax = null )
    {
        global $oApp;

        if ( $sField === null )
        {
            $sField = 'id';
        }
        if ( $iLevel === null )
        {
            $iLevel = 1;
        }
        if ( $fRecursive === null )
        {
            $fRecursive = false;
        }

        // get current page data
        $aPage = $oPage->GetRow();

        // query sections
        $oSection->SetFilter( $sField, $sValue );
        $oSection->QueryData();
        $oSection->ClearFilter( $sField );

        foreach( $oSection->GetResult() as $aSection )
        {
            if ( $aSection[ 'page_name' ] === null || $aSection[ 'page_name' ] === '' )
            {
                continue;
            }

            // check if last page is the same as current page, otherwise load data for current page
            if ( $aPage[ 'name' ] != $aSection[ 'page_name' ] )
            {
                $oPage->SetFilter( 'name', $aSection[ 'page_name' ] );
                $oPage->QueryData();
                if ( !$oPage->IsData() )
                {
                    die( 'invalid page in ' . __FILE__ . ' line ' . __LINE__ );
                }
                $aPage = $oPage->GetRow();
            }

            // Name
            $sName = $aSection[ 'name' ];
            // use lower case names (section names are also used lower case), so make them compareable.
            $sName = strtolower( $sName );

            // Default title: Use Section Title
            $sTitle = $aSection[ 'title_' . $oNavigation->GetType() ];
            // If forced, use page title
            if ( $oSection->GetFieldUse( 'use_page_title' ) )
            {
                if ( $aSection[ 'use_page_title' ] == 'y' )
                {
                    $sTitle = $aPage[ 'title_' . $oNavigation->GetType() ];
                }
            }

            // Path
            if ( $aPage[ 'path' ] !== null && $aPage[ 'path' ] !== '' )
            {
                // Query: section, subsection and other parameters
                // use separate arrays for aParameter and aSectionParameter to get the values in order
                $aParameter = array();

                // Parse paramters from $sParameter
                parse_str( $sParameter, $aParameter );

                // Parse paramters from section
                parse_str( $aSection[ 'parameter' ], $aSectionParameter );

                if ( isset( $aSectionParameter[ 'section' ] ) )
                {
                    $aParameter[ 'section' ] = $aSectionParameter[ 'section' ];
                    unset( $aSectionParameter[ 'section' ] );
                }

                if ( !isset( $aSectionParameter[ 'subsection' ] ) )
                {
                    $aParameter[ 'subsection' ] = $aSection[ 'name' ];
                }
                else
                {
                    $aParameter[ 'subsection' ] = $aSectionParameter[ 'subsection' ];
                    unset( $aSectionParameter[ 'subsection' ] );
                }

                foreach( array_keys( $aSectionParameter ) as $sKey )
                {
                    $aParameter[ $sKey ] = $aSectionParameter[ $sKey ];
                }
                foreach( array_keys( $aParameter ) as $sKey )
                {
                    if ( IsNull( $aParameter[ $sKey ] ) )
                    {
                        unset( $aParameter[ $sKey ] );
                    }
                }

                $sPath = $aPage[ 'path' ];

                // for index.php the default subsection is Home -> clear subsection to avoid duplicate contend issues
                if ( strcasecmp( $aParameter[ 'subsection' ], 'home' ) == 0 && $sPath == 'index.php' )
                {
                    unset( $aParameter[ 'subsection' ] );
                    $sPath = '.';
                }

                // do url rewriting?
                if ( $this->fUrlRewrite )
                {
                    // just for root pages where the path is 'index.php' (which is the default page)
                    // and the only parameter is subsection
                    // and only Section and Subsection or just Section

                    if ( $sPath == 'index.php' && $aParameter[ 'subsection' ] != '' && substr_count( $aParameter[ 'subsection' ], '_-_' ) <= 3 )
                    {
                        $sQuery = str_replace( '_-_', '/', $aParameter[ 'subsection' ] ) . '/';
                        unset( $aParameter[ 'subsection' ] );
                        $sQueryParameter = HttpParam( $aParameter, $oApp->GetQuerySeparator(), $oApp->GetParamSeparator() );
                    }
                    else
                    {
                        $sQuery = $sPath;
                        $sQueryParameter = HttpParam( $aParameter, ( strpos( $sPath, '?' ) === false ? $oApp->GetQuerySeparator() : $oApp->GetParamSeparator() ), $oApp->GetParamSeparator() );
                    }
                }
                else
                {
                    $sQuery = $sPath;
                    $sQueryParameter = HttpParam( $aParameter, ( strpos( $sPath, '?' ) === false ? $oApp->GetQuerySeparator() : $oApp->GetParamSeparator() ), $oApp->GetParamSeparator() );
                }

                // Insert into control
                $sFullname = $oNavigation->Add( $sName, $sQuery, $aPage[ 'target' ], $sTitle, $sParent, Null, Null, Null, $sQueryParameter, Null, Null, Null, $aPage[ 'onclick' ] );
            }
            else
            {
                // Insert into control
                $sFullname = $oNavigation->Add( $sName, $aPage[ 'path' ], $aPage[ 'target' ], $sTitle, $sParent, Null, Null, Null, $sParameter, Null, Null, Null, $aPage[ 'onclick' ] );
            }

            // echo "Fullname: " . $sFullname . "<br>";

            // Rekursiv fr weitere Sektions
            if ( $fRecursive && $oSection->GetFieldUse( 'parent_id' ) && ( $iLevel < $iLevelMax || $iLevelMax == null ) )
            {
                $this->_FillNavigation( $oNavigation, $oSection, $oPage, $aSection[ 'id' ], 'parent_id', $iLevel + 1, $fRecursive, $sFullname, $sParameter, $iLevelMax );
            }
        }
    }

    /**
     * Creates a list of sections from the bottom section to the top section
     * @param CNavigation &$oBreadcrumbs navigation control
     * @param int $iLevel start level (from top)
     * @param int $iLevelMax max level (from top)
     * @return void
     */
    function FillBreadcrumbs( &$oBreadcrumbs, $iLevel = null, $iLevelMax = null )
    {
        global $oFactory;

        if ( $iLevel === null )
        {
            $iLevel = 1;
        }
        if ( $iLevel < 1 )
        {
            $iLevel = 1;
        }

        // Create section modul
        $oSection = $oFactory->Modul( 'CSection' );

        // Create page modul
        $oPage = $oFactory->Modul( 'CPage' );

        // Get current subsection
        $iSubSection = $this->GetSubSectionId();

        // Create a list of sections
        $aSections = array();

        // Now find the way to the main section
        $oSection->SetLimitRows( 1 );
        while( $iSubSection !== null )
        {
            // Get current section data
            $oSection->SetFilter( 'id', $iSubSection );
            $oSection->QueryData();
            if ( !$oSection->IsData() )
            {
                die( 'invalid section in ' . __FILE__ . ' line ' . __LINE__ );
            }
            $aSections[ $oSection->Get( 'name' ) ] = $oSection->Get( 'id' );

            // get ID from parent
            $iSubSection = $oSection->Get( 'parent_id' );
        }
        $oSection->SetLimitRows( 0 );
        $oSection->ClearFilter( 'id' );

        if ( count( $aSections ) == 0 )
        {
            return;
        }

        // Remove top sections
        $aSections = array_reverse( $aSections );
        if ( $iLevel === null )
        {
            $iLevel = 1;
        }
        if ( $iLevelMax === null )
        {
            $iLevelMax = count( $aSections );
        }
        $aSections = array_slice( $aSections, $iLevel - 1, $iLevelMax - $iLevel + 1 );
        $aSections = array_reverse( $aSections );

        // Now find the way to the main section
        foreach( $aSections as $iSection )
        {
            $this->_FillNavigation( $oBreadcrumbs, $oSection, $oPage, $iSection, null, null, false );
        }

        // insert home on top
        if ( $iLevel == 1 )
        {
            $sName = array_pop( array_keys( $aSections ) );

            // if last section was not home
            if ( strtolower( $sName ) != 'home' )
            {
                // get id from home
                $oSection->SetFilter( 'name', 'home' );
                $oSection->SetLimitRows( 1 );
                $oSection->QueryData();
                if ( !$oSection->IsData() )
                {
                    die( 'invalid section in ' . __FILE__ . ' line ' . __LINE__ );
                }
                $oSection->SetLimitRows( 0 );
                $oSection->ClearFilter( 'name' );

                // insert home
                $this->_FillNavigation( $oBreadcrumbs, $oSection, $oPage, $oSection->Get( 'id' ), null, null, false );
            }
        }
    }

    function GetPrevNext( $iId, &$iPrevId, &$iNextId )
    {
        global $oFactory;

        // Get current section data
        $oSection = $oFactory->Modul( 'CSection' );
        $oSection->SetFilter( 'id', $iId );
        $oSection->QueryData();
        $oSection->ClearFilter( 'id' );

        // Get all sections with the same parent
        $oSection->SetFilter( 'parent_id', $oSection->Get( 'parent_id' ) );
        $oSection->QueryData();
        $oSection->ClearFilter( 'parent_id' );

        // Get result
        $aSubSections = $oSection->GetResult();

        // Find neighbours of current section
        $iPrevId = null;
        $iNextId = null;
        $iLastId = null;

        $iMax = count( $aSubSections );
        for( $i = 0; $i < $iMax; ++$i )
        {
            // Set previous ID
            if ( $aSubSections[ $i ][ 'id' ] == $iId )
            {
                $iPrevId = $iLastId;
            }

            // Set next ID
            if ( $iLastId == $iId )
            {
                $iNextId = $aSubSections[ $i ][ 'id' ];
                break;
            }

            // Save current ID
            $iLastId = $aSubSections[ $i ][ 'id' ];
        }
    }

    function SetUrlRewrite( $fValue = null, $fForce = null )
    {
        global $oApp;

        if ( $fValue === null )
        {
            $fValue = true;
        }
        if ( $fForce === null )
        {
            $fForce = false;
        }

        // When to deactivate, no further checks needed
        if ( !$fValue )
        {
            $this->fUrlRewrite = false;
            return;
        }

        // When forced, no further checks needed
        if ( $fForce )
        {
            $this->fUrlRewrite = true;
            return;
        }

        // check for apache or IIS with rewrite capabilities
        // Apache: mod_rewrite
        // IIS: ISAPI_rewrite (http://www.isapirewrite.com/)
        if ( $oApp->IsApache() )
        {
            if ( $oApp->IsApacheModule( 'mod_rewrite' ) )
            {
                $this->fUrlRewrite = true;
                return;
            }
        }

        if ( $oApp->IsIis() )
        {
            if ( isset( $_SERVER[ 'HTTP_X_REWRITE_URL' ] ) )
            {
                $this->fUrlRewrite = true;
                return;
            }
        }

        $this->fUrlRewrite = false;

    }

    function GetUrlRewrite()
    {
        return( $this->fUrlRewrite );
    }

    /** Use page title as base title:
        true: "sectiontitle - pagetitle"
        false: "sectiontitle"
        default: true;
    */
    function SetUsePageTitle( $fValue = null )
    {
        if ( $fValue == null )
        {
            $fValue = true;
        }

        $this->fUsePageTitle = $fValue;
    }
    function GetUsePageTitle()
    {
        return( $this->fUsePageTitle );
    }

    /** Adds the subsection parameter depending on UrlRewrite
        section/subsection/subsubsection/ or
        index.php/subsection=section_-_subsection_-_subsubsection or
        index.php?subsection=section_-_subsection_-_subsubsection
        @param $sScriptname path and filename of the script
        @param $sParameter existing parameter string, but without the query separator, default null
        @return string
    */
    function GetSubsectionAsParameter( $sScriptname, $sParameter = null )
    {
        global $oApp;

        if ( null === $sParameter )
        {
            $sParameter = '';
        }

        // Get current subsection
        if ( $this->sSubSection !== null && $this->sSubSection !== '' )
        {
            $sSection = $this->sSubSection;
        }
        else
        {
            $sSection = $this->sSection;
        }

        // only for the root script
        if ( 'index.php' == $sScriptname && $this->GetUrlRewrite() )
        {
            $sPath = str_replace( '_-_', '/', $sSection ) . '/';
        }
        else
        {
            $sPath = $sScriptname . HttpParam( array( 'subsection' => $sSection ), $oApp->GetQuerySeparator(), $oApp->GetParamSeparator() );
        }

        if ( '' != $sParameter )
        {
            if ( '' == $sPath || $sScriptname == $sPath )
            {
                $sPath .= $oApp->GetQuerySeparator();
            }
            else
            {
                $sPath .= $oApp->GetParamSeparator();
            }
            $sPath .= $sParameter;
        }

        return( $sPath );
    }

    /**
     * Reads data for a section
     * @param string $sSection name of the section
     * @param CSection $oSection section module, default null
     * @return array section data or null
     */
    function QuerySectionByName( $sSection, $oSection = null )
    {
        global $oFactory;

        if ( !isset( $oSection ) )
        {
            $oSection = $oFactory->Modul( 'CSection' );
        }
        $oSection->SetLimitRows( 1 );

        $oSection->SetFilter( 'name', $sSection );
        $oSection->QueryData();
        if ( !$oSection->IsData() )
        {
            return( null );
        }

        return( $oSection->GetRow() );
    }
}
